#!/bin/bash

LOGFILE=mwcap_sdk_install.log
HOWTOFILE=howto.txt
MODUEL_INSTALL_DIR=/usr/local/share/ProCaptureSDK
MODUEL_INCLUDE_DIR=/usr/include/ProCaptureSDK
CURRENT_DIR=`pwd`
SDK_DIR=`pwd`
SRC_DIR="`pwd`/src"
BIN_DIR="`pwd`/bin"
SCRIPTS_DIR="`pwd`/scripts"

LibdpkgStatus=1
XCbStatus=1
GStatus=1
ASoundStatus=1
QtStatus=1
YasmStatus=1
X264Status=1
FFmpegStatus=1
COMPARE_STRING="Status: install ok installed"

buildfailed_name=""
libraryname=""

cat /dev/null > "$SDK_DIR/$LOGFILE"
cat /dev/null > "$SDK_DIR/$HOWTOFILE"

check_syslib()
{
	printf "check libdpkg\t ....... \r"
	binname=`sudo find /usr/bin/ -iname dpkg 2>/dev/null`
	if [ "$binname" = "" ];
	then
		printf "check libdpkg\t ....... [false]\n"
		echo "check over !"
		echo "Please install libdpkg-dev first ! You can see howto.txt to install the library"

		echo "" >> "$SDK_DIR/$HOWTOFILE"
		echo "Warning : Your system can't find dpkg, Please install dpkg first !" >> "$SDK_DIR/$HOWTOFILE"
		echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
		echo "sudo apt-get install libdpkg-dev" >> "$SDK_DIR/$HOWTOFILE"
		echo "" >> "$SDK_DIR/$HOWTOFILE"
		
		LibdpkgStatus=0
		exit 1
	else
		LibdpkgStatus=1
		printf "check libdpkg\t ....... [true]\n"
	fi
	
	printf "check g++\t ....... \r"
	PACKAGE_STATUS=`sudo dpkg -s g++ 2>/dev/null | grep Status:`
	if [ "$PACKAGE_STATUS" = "$COMPARE_STRING" ];
	then
		printf "check g++\t ....... [true]\n"		
		GStatus=1
	else
		printf "check g++\t ....... [false]\n"
		libraryname=${libraryname}"g++ "
		GStatus=0
	fi
}

check_systype()
{
	CHECK_NAME="Distributor ID:	Ubuntu"

	systype_name=`sudo lsb_release -i 2>/dev/null | grep Distributor`

	if [ "$systype_name" != "$CHECK_NAME" ];
	then
		echo "Error Install SDK failed: Your system is not ubuntu !"
		exit 1
	fi
}

check_libasound2()
{
	cd $SDK_DIR
	printf "check libasound\t ....... \r"
	for include_name in /usr/include /usr/local/include
	do
		alsa_name=`sudo find $include_name -iname alsa 2>/dev/null`
		if [ "$alsa_name" != "" ];
		then
			cd $alsa_name
			asound_name=`sudo find $alsa_name -iname asoundlib.h 2>/dev/null`
			if [ "$asound_name" != "" ];
			then
				ASoundStatus=1
				cd $SDK_DIR
				break;
			else
				cd $SDK_DIR
				ASoundStatus=0
				continue
			fi
		else
			ASoundStatus=0
		fi
	done

	if [ "$ASoundStatus" == "1" ];
	then
		for lib_name in /usr/lib /usr/local/lib
		do
			libasound_name=`sudo find $lib_name -iname libasound.so.2 2>/dev/null`
			if [ "$libasound_name" != "" ];
			then
				printf "check libasound\t ....... [true]\n"
				ASoundStatus=1
				return 0
			fi
		done
	fi
	
	printf "check libasound\t ....... [false]\n"
	libraryname=${libraryname}"libasound2 "
	ASoundStatus=0
}


check_xcb()
{
	cd $SDK_DIR
	printf "check libxcb\t ....... \r"
	for include_name in /usr/include /usr/local/include
	do
		xcb_dirname=`sudo find $include_name -iname xcb 2>/dev/null`
		if [ "$xcb_dirname" != "" ];
		then
			cd $xcb_dirname
			for xcb_name in xcb.h xproto.h xv.h shm.h
			do
				result=`sudo find $xcb_dirname -iname $xcb_name 2>/dev/null`
				if [ "$result" != "" ];
				then
					XCbStatus=1
					continue
				else
					cd $SDK_DIR
					XCbStatus=0
					break;
				fi
			done
			cd $SDK_DIR
		else
			XCbStatus=0
			continue
		fi

		if [ "$XCbStatus" == "0" ];
		then
			XCbStatus=0
			continue
		else
			XCbStatus=1
			break;
		fi
	done

	if [ "$XCbStatus" == "1" ];
	then
		for lib_name in /usr/lib /usr/local/lib
		do
			for libxcb_name in libxcb.so.1 libxcb-xv.so.0 libxcb-shm.so.0
			do
				result=`sudo find $lib_name -iname $libxcb_name 2>/dev/null`
				if [ "$result" != "" ];
				then
					XCbStatus=1
					continue
				else
					XCbStatus=0
					break
				fi
			done

			if [ "$XCbStatus" == "1" ];
			then
				printf "check libxcb\t ....... [true]\n"
				XCbStatus=1
				return 0
			else
				continue
			fi
		done
	fi
	
	printf "check libxcb\t ....... [false]\n"
	libraryname=${libraryname}"libxcb "
	XCbStatus=0
}


qmake_path=""
check_qt()
{
	printf "check qt5\t ....... \r"
	QtStatus=1

	for lib_path in /usr/lib /usr/local/lib
	do
		for libqt_name in libQt5Core.so* libQt5Gui.so* libQt5Widgets.so*	
		do
			qtlibpath=`sudo find $lib_path -iname $libqt_name 2>/dev/null`
			if [ "$qtlibpath" = "" ];
			then
				QtStatus=0
				break;
			else
				QtStatus=1
			fi
		done
		
		if [ "$QtStatus" = "1" ];
		then
			break;
		fi
	done
	
	if [ "$QtStatus" = 0 ];
	then
		printf "check qt5\t ....... [false]\n"
		libraryname=${libraryname}"qt5 "
		QtStatus=0
		return 0
	fi
	
	for search_path in /usr/local/sbin /usr/local/bin /usr/sbin /usr/bin /sbin /bin /usr/games /usr/local/games /usr/lib
	do
		qmake_path=`sudo find $search_path -iname qmake | grep 5 2>/dev/null`
		if [ "$qmake_path" = "" ];
		then
			continue
		else
			rcc_path=`sudo find $search_path -iname rcc | grep 5 2>/dev/null`
			if [ "$rcc_path" = "" ];
			then
				qmake_path=""
				continue
			else
				QtStatus=1
				printf "check qt5\t ....... [true]\n"
				return 0
			fi
		fi
	done

	printf "check qt5\t ....... [false]\n"
	libraryname=${libraryname}"qt5 "
	QtStatus=0
	return 0
}


check_ffmpeg()
{
	printf "check yasm\t ....... \r"
	FFmpegStatus=1
	name=yasm
	PACKAGE_STATUS=`sudo dpkg -s yasm 2>/dev/null | grep Status:`
	if [ "$PACKAGE_STATUS" = "$COMPARE_STRING" ];
	then 
		YasmStatus=1
		printf "check yasm\t ....... [true]\n"
	else
		FFmpegStatus=0
		YasmStatus=0
		printf "check yasm\t ....... [false]\n"
		libraryname=${libraryname}"yasm "
		echo "" >> "$SDK_DIR/$HOWTOFILE"
		echo "Please install yasm first !" >> "$SDK_DIR/$HOWTOFILE"
		echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
 		echo "sudo apt-get install yasm " >> "$SDK_DIR/$HOWTOFILE"
		echo "" >> howto.txt
	fi

	printf "check libx264\t ....... \r"
	PACKAGE_STATUS=`sudo dpkg -s libx264-dev 2>/dev/null | grep Status:`
	if [ "$PACKAGE_STATUS" = "$COMPARE_STRING" ];
	then
		X264Status=1
		printf "check libx264\t ....... [true]\n"
	else
		FFmpegStatus=0
		X264Status=0
		printf "check libx264\t ....... [false]\n"
		libraryname=${libraryname}"x264 "
		echo "" >> "$SDK_DIR/$HOWTOFILE"
		echo "Please install libx264 first !" >> "$SDK_DIR/$HOWTOFILE"
		echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
		echo "sudo apt-get install libx264-dev" >> "$SDK_DIR/$HOWTOFILE"
		echo "" >> "$SDK_DIR/$HOWTOFILE"
	fi

	printf "check ffmpeg\t ....... \r"
	path=""
	for search_path in /usr/lib /usr/local/lib
	do
		for libffmpeg_name in libavformat.so* libavcodec.so* libavutil.so* libswresample.so* libswscale.so*	
		do
			path=`sudo find $search_path -iname "$libffmpeg_name" 2>/dev/null`
			if [ "$path" = "" ];
			then
				path=""
				FFmpegStatus=0
				break;
			else
				FFmpegStatus=1
			fi
		done

		if [ "$FFmpegStatus" = 1 ];
		then
			break;
		fi
	done

	if [ "$FFmpegStatus" = 0 ];
	then
		printf "check ffmpeg\t ....... [false]\n"
		libraryname=${libraryname}"ffmpeg "
		return 0
	fi
	
	FFmpegPath=`sudo find /usr/ -iname libavformat | grep include 2>/dev/null`
	if [ "$FFmpegPath" = "" ];
	then
		FFmpegPath=""
		FFmpegStatus=0
		printf "check ffmpeg\t ....... [false]\n"
		libraryname=${libraryname}"ffmpeg "
		return 0
	else
		FFmpegPath=${FFmpegPath%/include/libavformat}
	fi
	
	if [ -n "$FFmpegPath" ];
	then
		for temp in $FFmpegPath
		do
			FFmpegPath=$temp
		done
	fi
	cd $SRC_DIR/mwcap-ffmpeg-muxer
	`sed -i "/FFMPEGDIR=/c FFMPEGDIR=$FFmpegPath" Makefile`
	FFmpegStatus=1
	printf "check ffmpeg\t ....... [true]\n"
}

build_tools()
{
	cd $SDK_DIR/tools

	printf "build %-25s ....... \r" "captureexpress"
	cd captureexpress
	if [ "$(getconf WORD_BIT)" = '32' ] && [ "$(getconf LONG_BIT)" = '64' ]; then
		XTAG=x64
	else
		XTAG=i686
	fi
	$qmake_path SDKDIR=$SDK_DIR PLATFORM=$XTAG >> "$SDK_DIR/$LOGFILE" 2>&1
	make -j4 >> "$SDK_DIR/$LOGFILE" 2>&1

	if [ ! -f "$SDK_DIR/bin/captureexpress" ];
	then
		printf "build %-25s ....... [ignore]\n" "captureexpress"
		buildfailed_name=${buildfailed_name}"captureexpress "
	else
		printf "build %-25s ....... [true]\n" "captureexpress"
	fi
	cd ..

	printf "build %-25s ....... \r" "deviceproperty"
	cd deviceproperty
	$qmake_path SDKDIR=$SDK_DIR >> "$SDK_DIR/$LOGFILE" 2>&1
	make -j4 >> "$SDK_DIR/$LOGFILE" 2>&1
	if [ ! -f "$SDK_DIR/bin/deviceproperty" ];
	then
		printf "build %-25s ....... [ignore]\n" "deviceproperty"
		buildfailed_name=${buildfailed_name}"deviceproperty "
	else
		printf "build %-25s ....... [true]\n" "deviceproperty"
	fi
	cd $SDK_DIR

	cd $SDK_DIR/src
	printf "build %-25s ....... \r" "libmwcapture"
	cd libmwcapture
	make >> "$SDK_DIR/$LOGFILE" 2>&1
	if [ ! -f "$SDK_DIR/bin/libmwcapture.a" ];
	then
		printf "build %-25s ....... [ignore]\n" "libmwcapture.a"
		buildfailed_name=${buildfailed_name}"libmwcapture.a "
	else
		printf "build %-25s ....... [true]\n" "libmwcapture.a"
	fi

	if [ ! -f "$SDK_DIR/bin/libmwcapture.so" ];
	then
		printf "build %-25s ....... [ignore]\n" "libmwcapture.so"
		buildfailed_name=${buildfailed_name}"libmwcapture.so "
	else
		printf "build %-25s ....... [true]\n" "libmwcapture.so"
	fi
	cd $SDK_DIR
}

build_src()
{	
	cd $SRC_DIR
	for filename in mw*
	do
		cd $filename
		if [ "$filename" = "mwcap-ffmpeg-muxer" ];
		then
			cd ..
			continue
		fi

		if [ "$filename" = "libmwcapture" -o "$filename" = "mwcap-hdcp-set" ];
		then
			cd ..
			continue
		fi
		
		if [ "$filename" = "mwcap-info" -o "$filename" = "mwcap-control" ];
		then
			cd ..
			continue
		fi

		if [ "$filename" = "mwcap-lowlatency-gui" -o "$filename" = "mwcap-multiview" ];
		then
			printf "build %-25s ....... \r" "$filename"
			$qmake_path >> "$SDK_DIR/$LOGFILE" 2>&1
			make -j4 >> "$SDK_DIR/$LOGFILE" 2>&1
			
			if [ ! -f "$SDK_DIR/bin/$filename" ];
			then
				printf "build %-25s ....... [ignore]\n" "$filename"
				buildfailed_name=${buildfailed_name}"$filename "
			else
				printf "build %-25s ....... [true]\n" "$filename"
				`sed  -i 's?'../../inc'?'$MODUEL_INCLUDE_DIR/inc'?g' $filename.pro`
				`sed  -i 's?'../../lib'?'\ '?g' $filename.pro`			
				`sed  -i 's?'-L\ ../../bin'?'\ '?g' $filename.pro`	
			fi	
			
		else
			printf "build %-25s ....... \r" "$filename"
			make >> "$SDK_DIR/$LOGFILE" 2>&1

			if [ ! -f "$SDK_DIR/bin/$filename" ];
			then
				printf "build %-25s ....... [ignore]\n" "$filename"
				buildfailed_name=${buildfailed_name}"$filename "
			else
				printf "build %-25s ....... [true]\n" "$filename"
				if [ "$filename" == "mwcap-upgrade" ];
				then
					cd ..
					continue
				else
					`sed  -i 's?'-I../../inc'?'-I$MODUEL_INCLUDE_DIR/inc'?g' makefile`
					`sed  -i 's?'-I../../lib'?'\ '?g' makefile`
					`sed  -i 's?'-L\ ../../bin'?'\ '?g' makefile`
				fi
			fi
		fi
		cd ..
	done

	cd $SDK_DIR
}

build_test()
{
	cd $SDK_DIR/test
	for filename in benchmark mwcap-v4l2
	do
		printf "build %-25s ....... \r" "$filename"
		cd $filename
		make >> "$SDK_DIR/$LOGFILE" 2>&1

		if [ ! -f "$SDK_DIR/bin/$filename" ];
		then
			printf "build %-25s ....... [ignore]\n" "$filename"
			buildfailed_name=${buildfailed_name}"$filename "
		else
			printf "build %-25s ....... [true]\n" "$filename"	
		fi	
	
		if [ "$filename" == "benchmark" ];		
		then
			`sed  -i 's?'-I../../inc'?'-I$MODUEL_INCLUDE_DIR/inc'?g' makefile`
			`sed  -i 's?'-I../../lib'?'\ '?g' makefile`			
			`sed  -i 's?'-L\ ../../bin'?'\ '?g' makefile`
		fi
		
		cd ..
	done

	cd $SDK_DIR
}

install_module()
{
	cd $CURRENT_DIR
	if [ ! -d "$MODUEL_INSTALL_DIR" ];
	then
		sudo mkdir -p $MODUEL_INSTALL_DIR
		sudo mkdir -p $MODUEL_INSTALL_DIR/bin
		sudo mkdir -p $MODUEL_INSTALL_DIR/scripts
		sudo mkdir -p $MODUEL_INSTALL_DIR/lib
	else
		for dir in $MODUEL_INSTALL_DIR/bin $MODUEL_INSTALL_DIR/scripts $MODUEL_INSTALL_DIR/lib
		do
			if [ ! -d "$dir" ];
			then	
				sudo mkdir -p $dir
			fi
		done
	fi
	
	sudo cp -avf $BIN_DIR/captureexpress $MODUEL_INSTALL_DIR/bin >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo cp -avf $BIN_DIR/deviceproperty $MODUEL_INSTALL_DIR/bin >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo cp -avf $BIN_DIR/libmwcapture.a $MODUEL_INSTALL_DIR/lib >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo cp -avf $BIN_DIR/libmwcapture.so $MODUEL_INSTALL_DIR/lib >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo cp -avf $SCRIPTS_DIR/mwcapsdk-uninstall.sh $MODUEL_INSTALL_DIR/scripts >> "$SDK_DIR/$LOGFILE" 2>&1

	cd $MODUEL_INSTALL_DIR/bin
	for binname in `ls $MODUEL_INSTALL_DIR/bin/`
	do
		if [ -x "$binname" ];
		then
			sudo ln -sf $MODUEL_INSTALL_DIR/bin/$binname /usr/bin >> "$SDK_DIR/$LOGFILE" 2>&1
		fi
	done
	
	sudo ln -sf $MODUEL_INSTALL_DIR/lib/libmwcapture.a /usr/lib >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo ln -sf $MODUEL_INSTALL_DIR/lib/libmwcapture.so /usr/lib >> "$SDK_DIR/$LOGFILE" 2>&1
	sudo ln -sf $MODUEL_INSTALL_DIR/scripts/mwcapsdk-uninstall.sh /usr/bin/ >> "$SDK_DIR/$LOGFILE" 2>&1

	cd $BIN_DIR
	rm captureexpress 2>/dev/null
	rm deviceproperty 2>/dev/null
	rm libmwcapture.a 2>/dev/null
	rm libmwcapture.so 2>/dev/null

	cd $CURRENT_DIR
	if [ ! -d "$MODUEL_INCLUDE_DIR" ];
	then
		sudo mkdir -p $MODUEL_INCLUDE_DIR
		sudo mkdir -p $MODUEL_INCLUDE_DIR/inc
	else
		if [ ! -d "$MODUEL_INCLUDE_DIR/inc" ];
		then	
			sudo mkdir -p $MODUEL_INCLUDE_DIR/inc
		fi
	fi
	
	for inc_name in `ls $SDK_DIR/inc`
	do
		if [ "$inc_name" = "dump.txt" ];
		then
			continue
		else
			sudo cp -avf $SDK_DIR/inc/$inc_name $MODUEL_INCLUDE_DIR/inc/ >> "$SDK_DIR/$LOGFILE" 2>&1
		fi
	done

	sudo cp -avf $SDK_DIR/lib/lib-mw-capture.h $MODUEL_INCLUDE_DIR/inc/ >> "$SDK_DIR/$LOGFILE" 2>&1
}


##install sdk
echo "Checking for required packages ... "
check_systype
check_syslib
check_libasound2
check_xcb
check_qt
#check_ffmpeg

if [ "$libraryname" != "" ];
then
	echo "follow libraries you need to install:"
	echo "$libraryname"
	echo "You can see howto.txt to install these libraries"
fi


if [ "$GStatus" = 0 ];
then
	echo "" >> "$SDK_DIR/$HOWTOFILE"
	echo "Warning : Your system can't find g++, Please install g++ first !" >> "$SDK_DIR/$HOWTOFILE"
	echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install g++" >> "$SDK_DIR/$HOWTOFILE"
	echo "" >> "$SDK_DIR/$HOWTOFILE"
	
	echo ""
	echo "Install failed ! You need to install g++"
	exit 1
fi

if [ "$XCbStatus" = 0 ];
then

	echo "" >> "$SDK_DIR/$HOWTOFILE"
	echo "Warning : Your system can't find libxcb, Please install libxcb first !" >> "$SDK_DIR/$HOWTOFILE"
	echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install libxcb1-dev" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install libxcb-xv0-dev" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install libxcb-shm0-dev" >> "$SDK_DIR/$HOWTOFILE"
	#echo ""
	echo "Following examples will build failed :" >> "$SDK_DIR/$HOWTOFILE"
	echo "mwcap-v4l2" >> "$SDK_DIR/$HOWTOFILE"
	echo "captureexpress" >> "$SDK_DIR/$HOWTOFILE"
	echo "deviceproperty" >> "$SDK_DIR/$HOWTOFILE"
	
	echo ""
	echo "Install failed ! You need to install libxcb"
	exit 1
fi 

if [ "$QtStatus" = 0 -o "$qmake_path" = "" ];
then
	echo "" >> "$SDK_DIR/$HOWTOFILE"
	echo "Warning : Your system can't find qt5, Please install qt5 first !" >> "$SDK_DIR/$HOWTOFILE"
	echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install qt5-default" >> "$SDK_DIR/$HOWTOFILE"
	#echo ""
	echo "Following examples will build failed :" >> "$SDK_DIR/$HOWTOFILE"
	echo "mwcap-lowlatency-gui" >> "$SDK_DIR/$HOWTOFILE"
	echo "mwcap-multiview" >> "$SDK_DIR/$HOWTOFILE"
	echo "captureexpress" >> "$SDK_DIR/$HOWTOFILE"
	echo "deviceproperty" >> "$SDK_DIR/$HOWTOFILE"

	echo ""
	echo "Install failed ! You need to install qt5"
	exit 1
fi

if [ "$ASoundStatus" = 0 ];
then
	echo "" >> "$SDK_DIR/$HOWTOFILE"
	echo "Warning : Your system can't find libasound2-dev, Please install libasound2-dev first !" >> "$SDK_DIR/$HOWTOFILE"
	echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
	echo "sudo apt-get install libasound2-dev" >> "$SDK_DIR/$HOWTOFILE"
	echo "Following examples will build failed :" >> "$SDK_DIR/$HOWTOFILE"
	echo "mwcap-multiview" >> "$SDK_DIR/$HOWTOFILE"
	echo "mwcap-osd" >> "$SDK_DIR/$HOWTOFILE"
	echo "benchmark" >> "$SDK_DIR/$HOWTOFILE"
	echo "" >> "$SDK_DIR/$HOWTOFILE"
fi

#if [ "$FFmpegStatus" = 0 ];
#then
#	echo "" >> "$SDK_DIR/$HOWTOFILE"
#	echo "Warning : Your system can't find ffmpeg, Please install ffmpeg first ! " >> "$SDK_DIR/$HOWTOFILE"
#	echo "You can use following commonds :" >> "$SDK_DIR/$HOWTOFILE"
#	echo "sudo wget http://ffmpeg.org/releases/ffmpeg-3.0.tar.bz2" >> "$SDK_DIR/$HOWTOFILE"
#	echo "tar -xf ffmpeg-3.0.tar.bz2" >> "$SDK_DIR/$HOWTOFILE"
#	echo "cd ffmpeg-3.0" >> "$SDK_DIR/$HOWTOFILE"
#	echo "./configure --enable-shared --enable-gpl --enable-libx264 --prefix=/usr/" >> "$SDK_DIR/$HOWTOFILE"
#	echo "make -j4" >> "$SDK_DIR/$HOWTOFILE"
#	echo "sudo make install" >> "$SDK_DIR/$HOWTOFILE"
#	#echo ""
#	echo "Following examples will build failed :" >> "$SDK_DIR/$HOWTOFILE"
#	echo "mwcap-ffmpeg-muxer" >> "$SDK_DIR/$HOWTOFILE"
#fi

#build tools and examples
echo ""
echo "Beginning build, please wait... "
build_tools
install_module

build_src
build_test

if [ "$buildfailed_name" != "" ];
then
	echo "follow examples build failed:"
	echo "$buildfailed_name"
	echo "You can see mwcap_sdk_install.log to figure out the cause"
fi

echo ""
echo "Beginning install, please wait... "
echo "cp $BIN_DIR/libmwcapture.a -> $MODUEL_INSTALL_DIR/lib/libmwcapture.a"
echo "cp $BIN_DIR/libmwcapture.so -> $MODUEL_INSTALL_DIR/lib/libmwcapture.so"
echo "cp $SCRIPTS_DIR/mwcapsdk-uninstall.sh -> $MODUEL_INSTALL_DIR/scripts/mwcapsdk-uninstall.sh"
echo "cp $SDK_DIR/inc -> $MODUEL_INCLUDE_DIR/inc"

existbin_name=""
binname=""
for binname in `ls $MODUEL_INSTALL_DIR/bin`
do
	if [ ! -x "$binname" ];
	then
		echo "cp $BIN_DIR/$binname -> $MODUEL_INSTALL_DIR/bin/$binname"
		existbin_name=${existbin_name}"$binname "
	fi
done

echo ""
echo "Install Successfully!"
echo ""
echo "If you want to uninstall the sdk you can use following commond:"
echo "mwcapsdk-uninstall.sh"
echo ""

if [ "$existbin_name" != "" ];
then
	echo "following exe files can be used in any directory:"
	for tempname in $existbin_name
	do
		echo "$tempname"
	done
fi

tempname=""
existbin_name=""
binname=""
cd $BIN_DIR
for binname in `ls $BIN_DIR`
do
	if [ -x "$binname" ];
	then
		existbin_name=${existbin_name}"$binname "
	fi
done

if [ "$existbin_name" != "" ];
then
	echo "following exe files can be used in $BIN_DIR:"
	for tempname in $existbin_name
	do
		echo "$tempname"
	done
fi

