#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "lib-mw-capture.h"
#include "video-device-info.h"
#include "video-audio-set.h"
#include "options.h"


int main(int argc, char *argv[])
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    //MW_RESULT mr;
    OPTION_RESULT opr;

    do
    {
        opr = ParseCommandLine(argc, argv);

        switch (opr)
        {
        case OPTION_INVALID_PARAMS:
        {
            int ind = getOptionInd(argc, argv);
            printf("Error:Invalid params \"%s\" for option \"%s\"\n",argv[optind -1], argv[ind]);
            return -1;
        }
            break;
        case OPTION_INVALID_OPTION:
        {
            return -1;
        }
            break;
        case OPTION_FAILED:
        {

        }
            break;
        case OPTION_ERROR_DEVICE_ID:
        {
            printf("Error:No device with device id %s\n",argv[optind -1]);
            return -1;
        }
        case OPTION_NO_DEVICE_OPTION:
        {
            printf("Error:Loss of essential device argument\n");
            return -1;
        }
        case OPTION_NOT_EXIST_DEVICE:
        {
            printf("Error:Device %s is not exist\n",argv[optind -1]);
            return -1;
        }
        default:
            break;
        }

        if ((g_dwCMD & (0x01 << OPTION_CMD_USAGE)) || g_dwCMD == 0)
        {
            PrintUsage();
            break;
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_LIST))
        {
            SelectAllVideoDevices();
            PrintSelectedVideoDevicesInfo();
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_INPUT_SET))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetVideoInputSource(videoDeviceInfo.szDevicePath, g_nVideoInputSource);
                if (!bRet)
                    break;

                char szInputType[32];
                switch (g_nVideoInputSource) {
                case 'a':
                    strcpy(szInputType, "auto");
                    break;
                case 0:
                    strcpy(szInputType, "hdmi");
                    break;
                case 1:
                    strcpy(szInputType, "vga");
                    break;
                case 2:
                    strcpy(szInputType, "sdi");
                    break;
                case 3:
                    strcpy(szInputType, "component");
                    break;
                case 4:
                    strcpy(szInputType, "cvbs");
                    break;
                case 5:
                    strcpy(szInputType, "yc");
                    break;
                default:
                    break;
                }

                printf("Set device %s input source to \"%s\" successfully\n",
                       videoDeviceInfo.szDevicePath,
                       szInputType);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s input source error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_INPUT_ASPECT))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szAspect[32];
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetVideoAspect(videoDeviceInfo.szDevicePath, g_nAspectX, g_nAspectY);
                if (!bRet)
                    break;

                if (g_nAspectX == 0 && g_nAspectY == 0)
                    strcpy(szAspect, "auto");
                else
                    sprintf(szAspect, "%d:%d", g_nAspectX, g_nAspectY);
                printf("Set device %s video input aspect to \"%s\" successfully\n",
                       videoDeviceInfo.szDevicePath,
                       szAspect
                       );
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s video input aspect %d:%d error\n",
                       videoDeviceInfo.szDevicePath,
                       g_nAspectX, g_nAspectY);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_INPUT_COLOR_FORMAT))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szColorFormat[32];
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetVideoColorFormat(videoDeviceInfo.szDevicePath, g_nVideoColorFormat);
                if (!bRet)
                    break;

                switch(g_nVideoColorFormat)
                {
                case 'a':
                    strcpy(szColorFormat, "auto");
                    break;
                case 1:
                    strcpy(szColorFormat, "RGB");
                    break;
                case 2:
                    strcpy(szColorFormat, "YUV BT.601");
                    break;
                case 3:
                    strcpy(szColorFormat, "YUV BT.709");
                    break;
                case 4:
                    strcpy(szColorFormat, "YUV BT.2020");
                    break;
                case 5:
                    strcpy(szColorFormat, "YUV BT.2020C");
                    break;
                default:
                    break;

                }

                printf("Set device %s color format to \"%s\" successfully\n",
                       videoDeviceInfo.szDevicePath,
                       szColorFormat);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s color format error\n",
                       videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_INPUT_QUANTIZATION))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szQuantization[32];
            switch(g_nVideoQuantization)
            {
            case 'a':
                strcpy(szQuantization, "auto");
                break;
            case 1:
                strcpy(szQuantization, "full");
                break;
            case 2:
                strcpy(szQuantization, "limited");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetVideoQuantization (videoDeviceInfo.szDevicePath, g_nVideoQuantization);
                if (!bRet)
                    break;

                printf("Set device %s quantization to \"%s\"successfully\n",
                       videoDeviceInfo.szDevicePath, szQuantization);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s quantization to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szQuantization);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_AUDIO_INPUT_SET))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetAudioInputSource(videoDeviceInfo.szDevicePath, g_nAudioInputSource);
                if (!bRet)
                    break;

                char szInputType[32];
                switch(g_nAudioInputSource)
                {
                case 'a':
                    strcpy(szInputType, "auto");
                    break;
                case 0:
                    strcpy(szInputType, "hdmi");
                    break;
                case 1:
                    strcpy(szInputType, "sdi");
                    break;
                case 2:
                    strcpy(szInputType, "line_in");
                    break;
                case 3:
                    strcpy(szInputType, "mic_in");
                    break;
                default:
                    break;
                }

                printf("Set device %s audio input source to \"%s\" successfully\n",
                       videoDeviceInfo.szDevicePath,
                       szInputType);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s audio input source error\n", videoDeviceInfo.szDevicePath);

        }

        else if (g_dwCMD & (0x01 << OPTION_CMD_EDID_READ))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            char szPath[256];
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                if (i > 0)
                    sprintf(szPath,"%s-%d", g_szReadEDIDPath, i);
                else
                    strcpy(szPath, g_szReadEDIDPath);

                bRet = ReadEDID(videoDeviceInfo.szDevicePath, szPath);
                if (!bRet)
                    break;

                printf("Save device %s EDID data to %s successfully\n", videoDeviceInfo.szDevicePath, szPath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Save device %s EDID data to %s error\n", videoDeviceInfo.szDevicePath, szPath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_EDID_WRITE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                if (strstr(videoDeviceInfo.channelInfo.szProductName, "HDMI") &&
                        strstr(videoDeviceInfo.channelInfo.szProductName, "AIO") &&
                        strstr(videoDeviceInfo.channelInfo.szProductName, "DVI"))
                {
                    printf("Device %s hasn't HDMI interface\n", videoDeviceInfo.szDevicePath);
                    break;
                }

                bRet = WriteEDID(videoDeviceInfo.szDevicePath, g_szWriteEDIDPath);
                if (!bRet)
                    break;

                printf("Write EDID data to device %s successfully\n", videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Write EDID data to device %s error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_LED_MODE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;

            char szLEDMode[32];
            switch(g_nLEDMode)
            {
            case 'a':
                strcpy(szLEDMode, "auto");
                break;
            case 0:
                strcpy(szLEDMode, "off");
                break;
            case 1:
                strcpy(szLEDMode, "on");
                break;
            case 2:
                strcpy(szLEDMode, "blink");
                break;
            case 3:
                strcpy(szLEDMode, "dbl_blink");
                break;
            case 4:
                strcpy(szLEDMode, "breath");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                bRet = SetLEDMode(videoDeviceInfo.szDevicePath, g_nLEDMode);
                if (!bRet)
                    break;

                printf("Set device %s LED mode to \"%s\"successfully\n", videoDeviceInfo.szDevicePath, szLEDMode);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s LED mode error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_TIMING_SET))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                bRet = SetTiming(videoDeviceInfo.szDevicePath, g_szSetTimingPath);
                if (!bRet)
                    break;

                printf("Set device %s timing successfully\n", videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s timing error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_TIMING_SAVE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                bRet = SaveTiming(videoDeviceInfo.szDevicePath, g_szSaveTimingPath);
                if (!bRet)
                    break;

                printf("Save device %s timing to \"%s\" successfully\n",
                       videoDeviceInfo.szDevicePath,
                       g_szSaveTimingPath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s timing error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_PIXEL_BRIGHTNESS))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetPixelBrightness(videoDeviceInfo.szDevicePath, g_nBrightness);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s brightness to %d successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nBrightness);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s brightness to %d error\n",
                       videoDeviceInfo.szDevicePath,
                       g_nBrightness);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_PIXEL_CONTRAST))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetPixelContrast(videoDeviceInfo.szDevicePath, g_nContrast);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s contrast to %d successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nContrast);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s contrast to %d error\n",
                       videoDeviceInfo.szDevicePath,
                       g_nContrast);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_PIXEL_HUE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetPixelContrast(videoDeviceInfo.szDevicePath, g_nHue);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s hue to %d successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nHue);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s hue to %d error\n",
                       videoDeviceInfo.szDevicePath,
                       g_nHue);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_PIXEL_SATURATION))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetPixelContrast(videoDeviceInfo.szDevicePath, g_nSaturation);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s saturation to %d successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nSaturation);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s saturation to %d error\n",
                       videoDeviceInfo.szDevicePath,
                       g_nSaturation);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_DEINTERLACE))
        {

            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;

            char szDeinterlaceMode[32];
            switch (g_nVideoOutputDeinterlace)
            {
            case 0:
            {
                strcpy(szDeinterlaceMode, "weave");
            }
                break;
            case 1:
            {
                strcpy(szDeinterlaceMode, "blend");
            }
                break;
            case 2:
            {
                strcpy(szDeinterlaceMode, "top_field");
            }
                break;
            case 3:
            {
                strcpy(szDeinterlaceMode, "bottom_field");
            }
                break;
            default:
            {
                strcpy(szDeinterlaceMode, "weave");
            }
                break;

            }


            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                nRet = SetVideoOutputDeinterlace(videoDeviceInfo.szDevicePath, g_nVideoOutputDeinterlace);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s deinterlace mode to \"%s\" successfully\n",
                           videoDeviceInfo.szDevicePath,
                           szDeinterlaceMode);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s deinterlace mode error\n", videoDeviceInfo.szDevicePath);

        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_RECT))
        {

            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;

            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                nRet = SetVideoOutputRect(videoDeviceInfo.szDevicePath, g_recVideoOutputRect);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s source rect successfully\n",
                           videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s source rect error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_ASPECT))
        {

            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;

            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                nRet = SetVideoOutputAspect(videoDeviceInfo.szDevicePath, g_nOutputAspectX, g_nOutputAspectY);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s output aspect to \"%d:%d\" successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nOutputAspectX,
                           g_nOutputAspectY);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s output aspect error\n", videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_LOWLATENCY))
        {

            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;

            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                nRet = SetVideoOutputLowlatency(videoDeviceInfo.szDevicePath, g_bLowlatency);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("%s device %s output lowlatency mode successfully\n",
                           g_bLowlatency == 0 ? "Close":"Open",
                           videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: %s device %s output lowlatency error\n",
                       g_bLowlatency == 0 ? "Close":"Open",
                       videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_COLOR_FORMAT))
        {

            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;

            char szColorFormat[32];
            switch(g_nOutputColorFormat)
            {
            case 0:
                strcpy(szColorFormat, "auto");
                break;
            case 1:
                strcpy(szColorFormat, "RGB");
                break;
            case 2:
                strcpy(szColorFormat, "YUV BT.601");
                break;
            case 3:
                strcpy(szColorFormat, "YUV BT.709");
                break;
            case 4:
                strcpy(szColorFormat, "YUV BT.2020");
                break;
            case 5:
                strcpy(szColorFormat, "YUV BT.2020C");
                break;
            default:
                break;

            }

            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);

                nRet = SetVideoOutputColorFormat(videoDeviceInfo.szDevicePath, g_nOutputColorFormat);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s color format to \"%s\" successfully\n",
                           videoDeviceInfo.szDevicePath,
                           szColorFormat
                           );
            }

            if(i != nVideoDeviceNum)
                printf("Error: Set device %s color format to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath,
                       szColorFormat);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_QUANTIZATION))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            char szQuantization[32];
            switch(g_nOutputQuantization)
            {
            case 0:
                strcpy(szQuantization, "auto");
                break;
            case 1:
                strcpy(szQuantization, "full");
                break;
            case 2:
                strcpy(szQuantization, "limited");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetVideoOutputQuantization (videoDeviceInfo.szDevicePath, g_nOutputQuantization);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s output quantization to \"%s\" successfully\n",
                           videoDeviceInfo.szDevicePath, szQuantization);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s output quantization to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szQuantization);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_SATURATION))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            char szSaturation[32];
            switch(g_nOutputSaturation)
            {
            case 0:
                strcpy(szSaturation, "auto");
                break;
            case 1:
                strcpy(szSaturation, "full");
                break;
            case 2:
                strcpy(szSaturation, "limited");
                break;
            case 3:
                strcpy(szSaturation, "extended_gamut");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetVideoOutputSaturation (videoDeviceInfo.szDevicePath, g_nOutputSaturation);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s output saturation to \"%s\" successfully\n",
                           videoDeviceInfo.szDevicePath, szSaturation);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s output saturation to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szSaturation);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_ASPECT_CONVERT))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            char szAspectConvert[32];
            switch(g_nOutputAspectConvert)
            {
            case 0:
                strcpy(szAspectConvert, "ignore");
                break;
            case 1:
                strcpy(szAspectConvert, "cropping");
                break;
            case 2:
                strcpy(szAspectConvert, "padding");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetVideoOutputAspectConvert(videoDeviceInfo.szDevicePath, g_nOutputAspectConvert);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s aspect convert mode to \"%s\" successfully\n",
                           videoDeviceInfo.szDevicePath, szAspectConvert);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s output aspect convert mode to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szAspectConvert);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_SET_AS_PRESET))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetAsPreset(videoDeviceInfo.szDevicePath);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Set device %s current settings as preset successfully\n",
                           videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s current settings as preset error\n",
                       videoDeviceInfo.szDevicePath);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_AUTO_SCAN))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szAutoScan[32];
            switch(g_nAutoScan)
            {
            case 0:
                strcpy(szAutoScan, "false");
                break;
            case 1:
                strcpy(szAutoScan, "true");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetVideoAutoScan(videoDeviceInfo.szDevicePath, g_nAutoScan);
                if (!bRet)
                    break;

                printf("Set device %s autoscan to \"%s\"successfully\n",
                       videoDeviceInfo.szDevicePath, szAutoScan);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s autoscan to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szAutoScan);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_LINK_WITH_VIDEO))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szLinkWithVideo[32];
            switch(g_nLinkWithVideo)
            {
            case 0:
                strcpy(szLinkWithVideo, "false");
                break;
            case 1:
                strcpy(szLinkWithVideo, "true");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetAudioLinkWithVideo(videoDeviceInfo.szDevicePath, g_nLinkWithVideo);
                if (!bRet)
                    break;

                printf("Set device %s link with video to \"%s\"successfully\n",
                       videoDeviceInfo.szDevicePath, szLinkWithVideo);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s link with video to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szLinkWithVideo);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_AUTO_SAMPL_PHASE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szAutoSamplPhase[32];
            switch(g_nAutoSamplPhase)
            {
            case 0:
                strcpy(szAutoSamplPhase, "false");
                break;
            case 1:
                strcpy(szAutoSamplPhase, "true");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetAutoSamplPhase(videoDeviceInfo.szDevicePath, g_nAutoSamplPhase);
                if (!bRet)
                    break;

                printf("Set device %s auto sampl phase to \"%s\"successfully\n",
                       videoDeviceInfo.szDevicePath, szAutoSamplPhase);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s auto sampl phase to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szAutoSamplPhase);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_AUTO_HALIGN))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            bool bRet;
            char szAutoHAlign[32];
            switch(g_nAutoHAlign)
            {
            case 0:
                strcpy(szAutoHAlign, "false");
                break;
            case 1:
                strcpy(szAutoHAlign, "true");
                break;
            default:
                break;
            }

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                bRet = SetAutoHAlign(videoDeviceInfo.szDevicePath, g_nAutoHAlign);
                if (!bRet)
                    break;

                printf("Set device %s auto halign to \"%s\"successfully\n",
                       videoDeviceInfo.szDevicePath, szAutoHAlign);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s auto halign to \"%s\" error\n",
                       videoDeviceInfo.szDevicePath, szAutoHAlign);
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_PHASE))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;
            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = SetPhase(videoDeviceInfo.szDevicePath, g_nPhase);
                if (nRet == -1)
                    break;
                else
                    printf("Set device %s phase to %d successfully\n",
                           videoDeviceInfo.szDevicePath,
                           g_nPhase);
            }

            if(i != nVideoDeviceNum)
                printf("Error: set device %s phase to \"%d\" error\n",
                       videoDeviceInfo.szDevicePath, g_nPhase);
        }
        /*else if (g_dwCMD & (0x01 << OPTION_CMD_VIDEO_OUTPUT_RELOAD_PRESET))
        {
            int nVideoDeviceNum = GetSelectDeviceCount();
            int i;
            int nRet;

            VIDEO_DEVICE_INFO videoDeviceInfo;
            for (i = 0; i < nVideoDeviceNum; i++)
            {
                GetDeviceInfoByIndex(i, &videoDeviceInfo);
                nRet = ReloadPreset(videoDeviceInfo.szDevicePath);
                if (nRet == -1)
                    break;

                if (nRet == 0)
                    printf("Device %s has no v4l2 stream\n",
                           videoDeviceInfo.szDevicePath);
                else
                    printf("Reload device %s settings successfully\n",
                           videoDeviceInfo.szDevicePath);
            }

            if(i != nVideoDeviceNum)
                printf("Error: Reload device %s settings error\n",
                       videoDeviceInfo.szDevicePath);
        }*/



    }while(false);

    MWCaptureExitInstance();
}


