
#include <stdio.h>
#include <getopt.h>
#include <unistd.h>
#include <regex.h>
#include "options.h"
#include "common.h"

static const char short_options[] = "hlv:a:p:o:e:m:t:";
static const struct option long_options[] =
{
    {"video-input",                 required_argument, NULL,    'v'},
    {"video-input-aspect",          required_argument, NULL,     0},
    {"video-input-color-format",    required_argument, NULL,     0},
    {"video-input-quantization",    required_argument, NULL,     0},


    {"audio-input",                 required_argument, NULL,    'a'},

    {"osd-enable",                  required_argument, NULL,    'o'},
    {"osd-load",                    required_argument, NULL,     0},

    {"video-auto-scan",                  required_argument, NULL,     0},
    {"audio-link-with-video",            required_argument, NULL,     0},

    {"edid-save",                   required_argument, NULL,    'e'},
    {"edid-load",                   required_argument, NULL,     0},

    {"led-mod",                     required_argument, NULL,    'm'},

    {"auto-sampl-phase",             required_argument, NULL,     0},
    {"auto-halign",                  required_argument, NULL,     0},
    {"set-phase",                  required_argument, NULL,     0},

    {"timing-save",                 required_argument, NULL,    't'},
    {"timing-set",                  required_argument, NULL,     0},

    {"pixel-brightness",            required_argument, NULL,    'p'},
    {"pixel-contrast",              required_argument, NULL,     0},
    {"pixel-hue",                   required_argument, NULL,     0},
    {"pixel-saturation",            required_argument, NULL,     0},

    {"video-output-deinterlace",    required_argument, NULL,     0},
    {"video-output-rect",           required_argument, NULL,     0},
    {"video-output-aspect",         required_argument, NULL,     0},
    {"video-output-lowlatency",     required_argument, NULL,     0},
    {"video-output-color-format",   required_argument, NULL,     0},
    {"video-output-quantization",   required_argument, NULL,     0},
    {"video-output-saturation",     required_argument, NULL,     0},
    {"video-output-aspect-convert", required_argument, NULL,     0},
    {"video-output-set-as-preset",  required_argument, NULL,     0},
    //{"video-output-reload-preset",   no_argument,       NULL,     0},


    {NULL,                  no_argument,       NULL,     0},
};

unsigned int g_dwCMD = 0;

int g_nVideoInputSource = -1;
int g_nAudioInputSource = -1;
int g_nAspectX = -1;
int g_nAspectY = -1;
int g_nVideoColorFormat = -1;
int g_nVideoQuantization = -1;
int g_nVideoSaturation = -1;
int g_nBrightness = -1;
int g_nContrast = -1;
int g_nHue = -1;
int g_nSaturation = -1;
char g_szReadEDIDPath[256] = {0};
char g_szWriteEDIDPath[256] = {0};
int g_nLEDMode = -1;
char g_szSetTimingPath[256] = {0};
char g_szSaveTimingPath[256] = {0};
int g_nVideoOutputDeinterlace = 0;
RECT g_recVideoOutputRect = {0};
int g_nOutputAspectX = -1;
int g_nOutputAspectY = -1;
BOOLEAN g_bLowlatency = 0;
int g_nOutputColorFormat = 0;
int g_nOutputQuantization = 0;
int g_nOutputSaturation = 0;
int g_nOutputAspectConvert = 0;

//add
int g_nAutoScan = -1;
int g_nLinkWithVideo = -1;

int g_nAutoSamplPhase = -1;
int g_nAutoHAlign = -1;
int g_nPhase = -1;

OPTION_RESULT ParseCommandLine(int argc, char *argv[])
{
    OPTION_RESULT opr = OPTION_OK;
    int nIdx;
    int c;
    while ((c = getopt_long(argc, argv, short_options, long_options, &nIdx)) != -1)
    {
        if (c == '?')
            return OPTION_INVALID_OPTION;


        switch (c)
        {
        case 0:
        {
            if (strcmp(long_options[nIdx].name, "video-input-aspect") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_INPUT_ASPECT);
                opr = ParseVideoAspectArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-input-color-format") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_INPUT_COLOR_FORMAT);
                opr = ParseVideoColorFormatArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-input-quantization") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_INPUT_QUANTIZATION);
                opr = ParseVideoQuantizationArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "pixel-contrast") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_PIXEL_CONTRAST);
                opr = ParseContrastArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "pixel-hue") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_PIXEL_HUE);
                opr = ParseHueArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "pixel-saturation") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_PIXEL_SATURATION);
                opr = ParseSaturationArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "edid-load") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_EDID_WRITE);
                opr = ParseWriteEDIDArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "timing-set") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_TIMING_SET);
                opr = ParseSetTimingArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-deinterlace") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_DEINTERLACE);
                opr = ParseVideoOutputDeinterlaceArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-rect") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_RECT);
                opr = ParseVideoOutputRectArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-aspect") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_ASPECT);
                opr = ParseVideoOutputAspectArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-lowlatency") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_LOWLATENCY);
                opr = ParseVideoOutputLowlatencyArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-color-format") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_COLOR_FORMAT);
                opr = ParseVideoOutputColorFormatArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-quantization") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_QUANTIZATION);
                opr = ParseVideoOutputQuantizationArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-saturation") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_SATURATION);
                opr = ParseVideoOutputSaturationArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-aspect-convert") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_ASPECT_CONVERT);
                opr = ParseVideoOutputAspectConvertArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-output-set-as-preset") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_SET_AS_PRESET);
                opr = ParseDeviceArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "video-auto-scan") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_AUTO_SCAN);
                opr = ParseVideoAutoScanArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "audio-link-with-video") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_LINK_WITH_VIDEO);
                opr = ParseAudioLinkWithVideoArgument(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "auto-sampl-phase") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_AUTO_SAMPL_PHASE);
                opr = ParseVideoAutoSamplPhase(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "auto-halign") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_AUTO_HALIGN);
                opr = ParseAudioAutoHAlign(optarg);
            }
            else if (strcmp(long_options[nIdx].name, "set-phase") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_PHASE);
                opr = ParsePhaseArgument(optarg);
            }
            /*else if (strcmp(long_options[nIdx].name, "video-output-reload-preset") == 0)
            {
                g_dwCMD |=(0x01 << OPTION_CMD_VIDEO_OUTPUT_RELOAD_PRESET);
            }*/
        }
            break;
        case 'h':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_USAGE);
        }
            break;
        case 'l':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_LIST);
        }
            break;
        //case 'd':
        //{
        //    g_dwCMD |= (0x01 << OPTION_CMD_DEVICE);
        //    opr = ParseDeviceArgument(optarg);

        //}
        //    break;
        case 'v':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_VIDEO_INPUT_SET);
            opr = ParseVideoSetArgument(optarg);
        }
            break;
        case 'a':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_AUDIO_INPUT_SET);
            opr = ParseAudioSetArgument(optarg);
        }
            break;
        case 'p':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_PIXEL_BRIGHTNESS);
            opr = ParseBrightnessArgument(optarg);
        }
            break;
        case 'e':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_EDID_READ);
            opr = ParseReadEDIDArgument(optarg);
        }
            break;
        case 'm':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_LED_MODE);
            opr = ParseLEDModeArgument(optarg);

        }
            break;
        case 't':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_TIMING_SAVE);
            opr = ParseSaveTimingArgument(optarg);
        }
            break;
        default:
            break;
        }

        if (opr != OPTION_OK)
            return opr;


        if (c != 'l' && c != 'h' && (c != 0 || strcmp(long_options[nIdx].name, "video-output-set-as-preset") != 0))
        {
            if (optind >= argc || argv[optind][0] == '-') {

                return OPTION_NO_DEVICE_OPTION;
            }

            optind++;

            opr = ParseDeviceArgument(argv[optind-1]);
            if (opr != OPTION_OK) {
                return opr;
            }
        }
    }

    /*
    if ((g_dwCMD & (0x01 << OPTION_CMD_DEVICE)) == 0 &&
            (g_dwCMD & (0x01 << OPTION_CMD_USAGE)) == 0 &&
            (g_dwCMD & (0x01 << OPTION_CMD_LIST)) == 0 && g_dwCMD != 0)
        opr = OPTION_NO_DEVICE_OPTION;
    */

    return opr;

}

void PrintUsage()
{
    printf("USAGE\n"
           "  mwcap-control [OPTIONS]\n\n"
           "DESCRIPTION\n"
           "  Set properties of Pro Capture cards.\n\n"
           "OPTIONS\n"
           "  -h                                            print usage (Default option)\n"
           "  -l                                            list all capture device information\n\n"
           "     DEVICE                                     DEVICE is used to specify a pro capture device, it \n"
           "                                                can be V4L2 device path (eg. /dev/video0) or capture\n"
           "                                                device ID (Board ID:Channel ID, eg. 0:0)\n\n"
           "  -v --video-input INPUT-TYPE DEVICE            set capture device video input source, INPUT-TYPE can\n"
           "                                                be: auto | hdmi | vga | sdi | component | cvbs | yc\n"
           "     --video-input-aspect ASPECT-RATIO DEVICE   set capture device input signal aspect. ASPECT-RATIO\n"
           "                                                can be a ratio(AspectX : AspectY eg. 16:9) or auto\n"
           "     --video-input-color-format COLOR-FORMAT DEVICE \n"
           "                                                set capture device input signal color format. COLOR-\n"
           "                                                FORMAT can be : auto | rgb | bt601 | bt709 | bt2020 |\n"
           "                                                bt2020c\n"
           "     --video-input-quantization QUANTIZATION DEVICE \n"
           "                                                set capture device input signal quantization. QUANT-\n"
           "                                                IZATION can be : auto | full | limited\n\n"
           "  -a --audio-input INPUT-TYPE DEVICE            set capture device audio input source, INPUT-TYPE can\n"
           "                                                be : auto | hdmi | sdi | line_in | mic_in\n"
           "  -e --edid-save FILE-NAME DEVICE               save EDID information to specified file\n"
           "     --edid-load FILE-NAME DEVICE               write the EDID information to Pro Capture card\n\n"
           "  -m --led LED-MODE DEVICE                      set capture device LED mode,  LED-MODE can be: auto |\n"
           "                                                off | on | blink | dbl_blink | breath\n\n"
           "  -t --timing-save FILE-PATH DEVICE             save video signal timing\n"
           "     --timing-set FILE-PATH DEVICE              set video signal timing\n"
           "\nOPTIONS FOR V4L2 STREAM\n"
           "  -p --pixel-brightness INT-VALUE DEVICE        set pixel's brightness, INT-VALUE must between -100\n"
           "                                                and 100\n"
           "     --pixel-contrast INT-VALUE DEVICE          set pixel's contrast, INT-VALUE must between 50 and\n"
           "                                                200\n"
           "     --pixel-hue INT-VALUE DEVICE               set pixel's hue, INT-VALUE must between -90 and 90\n"
           "     --pixel-saturation INT-VALUE DEVICE        set pixel's saturation, INT-VALUE must between 0\n"
           "                                                and 200\n\n"
           "     --video-output-deinterlace MODE DEVICE     set deinterlace mode, MODE can be: weave | blend |\n"
           "                                                top_field | bottom_field\n"
           "     --video-output-rect RECT DEVICE            set output rect of the video, RECT must be format with \n"
           "                                                [x,y,width,height] (eg. [0,0,400,500])\n"
           "     --video-output-aspect ASPECT-RATIO DEVICE  set video output aspect. ASPECT-RATIO is a ratio with\n"
           "                                                format: AspectX:AspectY (eg. 16:9). if option video-\n"
           "                                                output-aspect-convert is set to ignore, this option will\n"
           "                                                be invalid\n"
           "     --video-output-aspect-convert MODE DEVICE  set output aspect convert mode, MODE can be: ignore |\n"
           "                                                cropping | padding. if this option is set to ignore,\n"
           "                                                the option video-output-aspect will be invalid\n"
           "     --video-output-color-format COLOR-FORMAT DEVICE \n"
           "                                                set output color format. COLOR-FORMAT can be: auto | rgb\n"
           "                                                | bt601 | bt709 | bt2020 | bt2020c\n"
           "     --video-output-quantization QUANTIZATION DEVICE \n"
           "                                                set output quantization. QUANTIZATION can be: auto | full\n"
           "                                                | limited\n"
           "     --video-output-saturation SATURATION DEVICE  \n"
           "                                                set output saturation. SATURATION can be: auto | full |\n"
           "                                                limited | extended_gamut\n"
           "     --video-output-lowlatency MODE DEVICE      open or close lowlatency mode, MODE can be: on | off (on\n"
           "                                                -- open lowlatency mode, off -- close lowlatency mode)\n"
           "     --video-output-set-as-preset               set current stream settings as default\n\n"
           //"     --video-output-reload-preset               recover stream's settings to default\n"
           "EXAMPLE\n"
           "  1. set video input source\n"
           "      $ mwcap-control --video-input auto /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-input auto 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -v auto 0:0\n"
           "  2. set video input aspect\n"
           "      $ mwcap-control --video-input-aspect  16:9 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-input-aspect  16:9 0:0\n"
           "  3. set video input color format\n"
           "      $ mwcap-control --video-input-color-format auto /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-input-color-format auto 0:0\n"
           "  4. set video input quantization\n"
           "      $ mwcap-control --video-input-quantization auto /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-input-quantization auto 0:0\n"
           "  5. set audio input source\n"
           "      $ mwcap-control --audio-input auto /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --audio-input auto 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -a auto 0:0\n"
           "  6. save EDID info to a file\n"
           "      $ mwcap-control --edid-save ./edid.bin /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --edid-save ./edid.bin 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -e ./edid.bin 0:0\n"
           "  7. write EDID to a Pro Capture Card\n"
           "      $ mwcap-control --edid-load ./edid.bin /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --edid-load ./edid.bin 0:0\n"
           "  8. set LED mode\n"
           "      $ mwcap-control --led on /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --led on 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -m on 0:0\n"
           "  9. save signal timing\n"
           "      $ mwcap-control --timing-save ./timing.config /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --timing-save ./timing.config 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -t ./timing.config 0:0\n"
           "  10. set signal timing\n"
           "      $ mwcap-control --timing-set ./timing.config /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --timing-set ./timing.config 0:0\n"
           "  11. set brightness for v4l2 stream\n"
           "      $ mwcap-control --pixel-brightness 0 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --pixel-brightness 0 0:0\n"
           "      -- or --\n"
           "      $ mwcap-control -p 0 /dev/video0\n"
           "  12. set contrast for v4l2 stream\n"
           "      $ mwcap-control --pixel-contrast 100 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --pixel-contrast 100 0:0\n"
           "  13. set hue for v4l2 stream\n"
           "      $ mwcap-control --pixel-hue 0 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --pixel-hue 0 0:0\n"
           "  14. set saturation for v4l2 stream\n"
           "      $ mwcap-control --pixel-saturation 100 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --pixel-saturation 100 0:0\n"
           "  15. set deinterlace mode for v4l2 stream\n"
           "      $ mwcap-control --video-output-deinterlace blend /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-deinterlace blend 0:0\n"
           "  16. set output rect for v4l2 stream\n"
           "      $ mwcap-control --video-output-rect [0,0,400,500] /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-rect [0,0,400,500] 0:0\n"
           "  17. set output aspect for v4l2 stream\n"
           "      $ mwcap-control --video-output-aspect 16:9 /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-aspect 16:9 0:0\n"
           "  18. set output aspect convert mode for v4l2 stream\n"
           "      $ mwcap-control --video-output-aspect-convert ignore /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-aspect-convert ignore 0:0\n"
           "  19. set output color format for v4l2 stream\n"
           "      $ mwcap-control --video-output-color-format rgb /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-color-format rgb 0:0\n"
           "  20. set output quantization for v4l2 stream\n"
           "      $ mwcap-control --video-output-quantization full /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-quantization full 0:0\n"
           "  21. set output saturation for v4l2 stream\n"
           "      $ mwcap-control --video-output-saturation full /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-saturation full 0:0\n"
           "  22. set output low latency mode for v4l2 stream\n"
           "      $ mwcap-control --video-output-lowlatency on /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-lowlatency on 0:0\n"
           "  23. set as preset for v4l2 stream\n"
           "      $ mwcap-control --video-output-set-as-preset /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-control --video-output-set-as-preset 0:0\n"
           );
}


OPTION_RESULT ParseDeviceArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (strchr(szArg, ':') == NULL)
    {
        if (!MatchReg(szArg, "^/dev/video[0-9]{1,2}+$"))
            return OPTION_INVALID_PARAMS;

        if (access(szArg, F_OK) != 0)
            return OPTION_NOT_EXIST_DEVICE;

        SelectVideoDevicesByPath(szArg);
    }
    else
    {
        if (!MatchReg(szArg, "^([0-9]+):([0-9]+)$"))
            return OPTION_INVALID_PARAMS;

        char *p;
        char *temp = strdup(szArg);
        char *temp1 = temp;
        char *separator = ":";
        int nCount = 0;

        int nBoardIndex = -1;
        int nChannelIndex = -1;

        p = strtok(temp, separator);
        while(p != NULL)
        {
            switch(nCount)
            {
            case 0:
            {
                nBoardIndex = (BYTE)atoi(p);
                break;
            }
            case 1:
            {
                nChannelIndex = (BYTE)atoi(p);
                break;
            }
            default:
                break;
            }
            nCount++;

            p = strtok(NULL, separator);
        }

        free(temp1);
        temp1 = NULL;

        SelectVideoDevicesByID(nBoardIndex, nChannelIndex);
        if (GetSelectDeviceCount() == 0)
            return OPTION_ERROR_DEVICE_ID;
    }

    return OPTION_OK;
}

OPTION_RESULT ParseVideoSetArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|hdmi|sdi|vga|component|cvbs|yc)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 'a';
    else if (strcmp(szArg, "hdmi") == 0)
        c = 0;
    else if (strcmp(szArg, "vga") == 0)
        c = 1;
    else if (strcmp(szArg, "sdi") == 0)
        c = 2;
    else if (strcmp(szArg, "component") == 0)
        c = 3;
    else if (strcmp(szArg, "cvbs") == 0)
        c = 4;
    else if (strcmp(szArg, "yc") == 0)
        c = 5;

    g_nVideoInputSource = c;

    return OPTION_OK;
}

OPTION_RESULT ParseAudioSetArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^auto|hdmi|sdi|line_in|mic_in$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 'a';
    else if (strcmp(szArg, "hdmi") == 0)
        c = 0;
    else if (strcmp(szArg, "sdi") == 0)
        c = 1;
    else if (strcmp(szArg, "line_in") == 0)
        c = 2;
    else if (strcmp(szArg, "mic_in") == 0)
        c = 3;

    g_nAudioInputSource = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoAspectArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(([0-9]+):([0-9]+){1})|auto$"))
        return OPTION_INVALID_PARAMS;

    if (strcmp(szArg, "auto") == 0)
    {
        g_nAspectX = 0;
        g_nAspectY = 0;
        return OPTION_OK;
    }

    char *p;
    char *temp = strdup(szArg);
    char *separator = ":";
    int nCount = 0;

    p = strtok(temp, separator);
    while(p != NULL)
    {

        switch(nCount)
        {
        case 0:
        {
            g_nAspectX = (BYTE)atoi(p);
            break;
        }
        case 1:
        {
            g_nAspectY = (BYTE)atoi(p);
            break;
        }
        default:
            break;
        }
        nCount++;

        p = strtok(NULL, separator);
    }

    free(temp);

    return OPTION_OK;
}

OPTION_RESULT ParseVideoColorFormatArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|rgb|bt601|bt709|bt2020|bt2020c)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 'a';
    else if (strcmp(szArg, "rgb") == 0)
        c = 1;
    else if (strcmp(szArg, "bt601") == 0)
        c = 2;
    else if (strcmp(szArg, "bt709") == 0)
        c = 3;
    else if (strcmp(szArg, "bt2020") == 0)
        c = 4;
    else if (strcmp(szArg, "bt2020c") == 0)
        c = 5;
    else
        c = 'a';

    g_nVideoColorFormat = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoQuantizationArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|full|limited)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 'a';
    else if (strcmp(szArg, "full") == 0)
        c = 1;
    else if (strcmp(szArg, "limited") == 0)
        c = 2;
    else
        c = 'a';

    g_nVideoQuantization = c;

    return OPTION_OK;

}


OPTION_RESULT ParseBrightnessArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^-?([0-9]{1,2}|100)$"))
        return OPTION_INVALID_PARAMS;

    g_nBrightness = atoi(szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseContrastArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([5-9]{1}[0-9]{1}|1[0-9]{2}|200)$"))
        return OPTION_INVALID_PARAMS;

    g_nContrast = atoi(szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseHueArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^-?([0-8]{0,1}[0-9]{1}|90)$"))
        return OPTION_INVALID_PARAMS;

    g_nHue = atoi(szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseSaturationArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([0-9]{1,2}|1[0-9]{2}|200)$"))
        return OPTION_INVALID_PARAMS;

    g_nSaturation = atoi(szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseReadEDIDArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    strcpy(g_szReadEDIDPath, szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseWriteEDIDArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (access(szArg, F_OK | R_OK) != 0)
        return OPTION_INVALID_PARAMS;

    strcpy(g_szWriteEDIDPath, szArg);

    return OPTION_OK;
}


OPTION_RESULT ParseLEDModeArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|off|on|blink|dbl_blink|breath)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if(strcmp(szArg, "auto") == 0)
        c = 'a';
    else if(strcmp(szArg, "off") == 0)
        c = 0;
    else if(strcmp(szArg, "on") == 0)
        c = 1;
    else if(strcmp(szArg, "blink") == 0)
        c = 2;
    else if(strcmp(szArg, "dbl_blink") == 0)
        c = 3;
    else if(strcmp(szArg, "breath") == 0)
        c = 4;
    else
        c = 'a';

    g_nLEDMode = c;

    return OPTION_OK;
}


OPTION_RESULT ParseSetTimingArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (access(szArg, F_OK | R_OK) != 0)
        return OPTION_INVALID_PARAMS;

    strcpy(g_szSetTimingPath, szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseSaveTimingArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([.]{0,2}(/[^/]+)*)$"))
        return OPTION_INVALID_PARAMS;


    if (!CheckWriteAuthority(szArg))
    {
        char *pTemp = strdup(szArg);
        char *pStr = pTemp;
        while(*++pStr != '\0')
            ;
        if (*--pStr == '/')
            *pStr = '\0';

        char *pStr2 = strrchr(pTemp, '/');
        *pStr2 = '\0';

        if (!CheckWriteAuthority(pTemp))
        {
            free(pTemp);
            return OPTION_INVALID_PARAMS;
        }

        free(pTemp);
    }

    strcpy(g_szSaveTimingPath, szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseVideoOutputDeinterlaceArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(weave|blend|top_field|bottom_field)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "weave") == 0)
    {
        c = 0;
    }
    else if (strcmp(szArg, "blend") == 0)
    {
        c = 1;
    }
    else if (strcmp(szArg, "top_field") == 0)
    {
        c = 2;
    }
    else if (strcmp(szArg, "bottom_field") == 0)
    {
        c = 3;
    }

    g_nVideoOutputDeinterlace = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoOutputRectArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(\\[[0-9]{1,4},[0-9]{1,4},[0-9]{1,4},[0-9]{1,4}\\])$"))
        return OPTION_INVALID_PARAMS;

    char szTemp[64];
    char *p;
    strncpy(szTemp, szArg+1, strlen(szArg)-2);


    p = strtok(szTemp, ",");
    int nCount = 0;
    int x,y,width,height;
    while(p != NULL)
    {
        switch(nCount)
        {
        case 0:
        {
            x = atoi(p);
            break;
        }
        case 1:
        {
            y = atoi(p);
            break;
        }
        case 2:
        {
            width = atoi(p);
        }
            break;
        case 3:
        {
            height = atoi(p);
        }
            break;
        default:
            break;
        }
        nCount++;

        p = strtok(NULL, ",");
    }

    g_recVideoOutputRect.left = x;
    g_recVideoOutputRect.right = x + width;
    g_recVideoOutputRect.top = y;
    g_recVideoOutputRect.bottom = y +height;

    return OPTION_OK;
}


OPTION_RESULT ParseVideoOutputAspectArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(([0-9]+):([0-9]+){1})$"))
        return OPTION_INVALID_PARAMS;


    char *p;
    char *temp = strdup(szArg);
    char *separator = ":";
    int nCount = 0;

    p = strtok(temp, separator);
    while(p != NULL)
    {

        switch(nCount)
        {
        case 0:
        {
            g_nOutputAspectX = (BYTE)atoi(p);
            break;
        }
        case 1:
        {
            g_nOutputAspectY = (BYTE)atoi(p);
            break;
        }
        default:
            break;
        }
        nCount++;

        p = strtok(NULL, separator);
    }

    free(temp);

    return OPTION_OK;

}

OPTION_RESULT ParseVideoOutputLowlatencyArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(on|off)$"))
        return OPTION_INVALID_PARAMS;

    BOOLEAN b;

    if (strcmp(szArg, "on") == 0)
        b = 1;
    else
        b = 0;

    g_bLowlatency = b;

    return OPTION_OK;

}

OPTION_RESULT ParseVideoOutputColorFormatArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|rgb|bt601|bt709|bt2020|bt2020c)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 0;
    else if (strcmp(szArg, "rgb") == 0)
        c = 1;
    else if (strcmp(szArg, "bt601") == 0)
        c = 2;
    else if (strcmp(szArg, "bt709") == 0)
        c = 3;
    else if (strcmp(szArg, "bt2020") == 0)
        c = 4;
    else if (strcmp(szArg, "bt2020c") == 0)
        c = 5;
    else
        c = 0;

    g_nOutputColorFormat = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoOutputQuantizationArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|full|limited)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 0;
    else if (strcmp(szArg, "full") == 0)
        c = 1;
    else if (strcmp(szArg, "limited") == 0)
        c = 2;
    else
        c = 0;

    g_nOutputQuantization = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoOutputSaturationArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|full|limited|extended_gamut)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 0;
    else if (strcmp(szArg, "full") == 0)
        c = 1;
    else if (strcmp(szArg, "limited") == 0)
        c = 2;
    else if (strcmp(szArg, "extended_gamut") == 0)
        c = 3;
    else
        c = 0;

    g_nOutputSaturation = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoOutputAspectConvertArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(ignore|cropping|padding)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "ingore") == 0)
        c = 0;
    else if (strcmp(szArg, "cropping") == 0)
        c = 1;
    else if (strcmp(szArg, "padding") == 0)
        c = 2;
    else
        c = 0;

    g_nOutputAspectConvert = c;

    return OPTION_OK;

}


int getOptionInd(int argc, char *argv[])
{
    if (optind - 2 >= argc || optind - 3 < 0)
    {
        return 0;
    }

    if (argv[optind - 3][0] == '-')
        return optind - 3;
    else
        return optind - 2;
}

//add
OPTION_RESULT ParseVideoAutoScanArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(true|false)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "false") == 0)
        c = 0;
    else if (strcmp(szArg, "true") == 0)
        c = 1;

    g_nAutoScan = c;

    return OPTION_OK;

}

OPTION_RESULT ParseAudioLinkWithVideoArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(true|false)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "false") == 0)
        c = 0;
    else if (strcmp(szArg, "true") == 0)
        c = 1;

    g_nLinkWithVideo = c;

    return OPTION_OK;

}

OPTION_RESULT ParseVideoAutoSamplPhase(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(true|false)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "false") == 0)
        c = 0;
    else if (strcmp(szArg, "true") == 0)
        c = 1;

    g_nAutoSamplPhase = c;

    return OPTION_OK;

}

OPTION_RESULT ParseAudioAutoHAlign(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(true|false)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "false") == 0)
        c = 0;
    else if (strcmp(szArg, "true") == 0)
        c = 1;

    g_nAutoHAlign = c;

    return OPTION_OK;

}

OPTION_RESULT ParsePhaseArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([0-5]{0,1}[0-9]{1}|6[0-3]{1})$"))
        return OPTION_INVALID_PARAMS;

    g_nPhase = atoi(optarg);

    return OPTION_OK;

}

