#include "video-audio-set.h"
#define VIDEO_INPUT_TYPE_NUM 6
#define AUDIO_INPUT_TYPE_NUM 4


bool SetVideoInputSource(char *szDevicePath, int c)
{
    if (szDevicePath == NULL)
        return false;

    if (c != 'a' && (c < 0 || c > 5))
        return false;

    bool bRet;
    MW_RESULT mr;
    HCHANNEL hChannel = -1;
    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (c == 'a')
        {
            mr = MWSetInputSourceScan(hChannel, true);
            if(mr != MW_SUCCEEDED)
                break;

            bRet = true;
        }
        else
        {
            DWORD dwVideoInputSources[VIDEO_INPUT_TYPE_NUM];
            DWORD dwVideoInputSourceCount = VIDEO_INPUT_TYPE_NUM;
            mr = MWGetVideoInputSourceArray(hChannel, dwVideoInputSources, &dwVideoInputSourceCount);
            if(mr != MW_SUCCEEDED)
                break;

            int i;
            for (i = 0; i < dwVideoInputSourceCount; i++)
            {
                if (INPUT_TYPE(dwVideoInputSources[i]) == (0x01 << c))
                    break;
            }

            if (i == dwVideoInputSourceCount)
                return false;


            mr = MWSetInputSourceScan(hChannel, false);
            if(mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoInputSource(hChannel, dwVideoInputSources[i]);
            if(mr != MW_SUCCEEDED)
                break;

            bRet = true;
        }


    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;

}

bool SetAudioInputSource(char *szDevicePath, int c)
{
    if (szDevicePath == NULL)
        return false;

    if (c != 'a' && (c < 0 || c > 3))
        return false;

    bool bRet;
    MW_RESULT mr;
    HCHANNEL hChannel = -1;
    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (c == 'a')
        {
            mr = MWSetAVInputSourceLink(hChannel, true);
            if(mr != MW_SUCCEEDED)
                break;

            bRet = true;
        }
        else
        {
            DWORD dwAudioInputSources[AUDIO_INPUT_TYPE_NUM];
            DWORD dwAudioInputSourceCount = AUDIO_INPUT_TYPE_NUM;
            mr = MWGetAudioInputSourceArray(hChannel, dwAudioInputSources, &dwAudioInputSourceCount);
            if(mr != MW_SUCCEEDED)
                break;

            int i;
            for (i = 0; i < dwAudioInputSourceCount; i++)
            {
                if (INPUT_TYPE(dwAudioInputSources[i]) == (0x01 << c))
                    break;
            }

            if (i == dwAudioInputSourceCount)
                return false;


            mr = MWSetAVInputSourceLink(hChannel, false);
            if(mr != MW_SUCCEEDED)
                break;

            mr = MWSetAudioInputSource(hChannel, dwAudioInputSources[i]);
            if(mr != MW_SUCCEEDED)
                break;

            bRet = true;
        }

    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;


}


bool SetVideoAspect(char *szDevicePath, int nAspectX, int nAspectY)
{
    if (szDevicePath == NULL)
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;


    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWSetVideoInputAspectRatio(hChannel, nAspectX, nAspectY);
        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetVideoColorFormat(char *szDevicePath, int nColorFormat)
{
    if (szDevicePath == NULL)
        return false;

    if (nColorFormat != 'a' && (nColorFormat < 0 || nColorFormat > 5))
        return false;

    if (nColorFormat == 5)
    {
        printf("Error: Color format \"bt2020c\" is not supported\n");
        return false;
    }

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nColorFormat == 'a')
            mr = MWSetVideoInputColorFormat(hChannel, MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN);
        else
             mr = MWSetVideoInputColorFormat(hChannel, nColorFormat);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetVideoQuantization(char *szDevicePath, int nQuantization)
{
    if (szDevicePath == NULL)
        return false;

    if (nQuantization != 'a' && (nQuantization < 1 || nQuantization > 2))
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nQuantization == 'a')
            mr = MWSetVideoInputQuantizationRange(hChannel, MWCAP_VIDEO_QUANTIZATION_UNKNOWN);
        else
            mr = MWSetVideoInputQuantizationRange(hChannel, nQuantization);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}


bool ReadEDID(char *szDevice, char *szEDIDFile)
{
    if (szDevice == NULL || szEDIDFile == NULL)
        return false;

    MW_RESULT mr;
    FILE *fd = NULL;
    HCHANNEL hChannel = -1;
    bool bRet = false;

    do
    {
        fd = fopen(szEDIDFile, "w+");
        if (fd == NULL)
            break;

        hChannel= MWOpenChannelByPath(szDevice);
        if (hChannel == -1)
            break;

        BYTE pbData[512];
        ULONG cbData = 512;

        mr = MWGetEDID(hChannel, pbData, &cbData);
        if (mr != MW_SUCCEEDED)
            break;

        fwrite(pbData, cbData, 1, fd);

        bRet =true;

    }while(false);

    if (fd != NULL)
    {
        fclose(fd);
        fd = NULL;
    }

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;

}

bool WriteEDID(char *szDevice, char *szEDIDFile)
{
    if (szDevice == NULL || szEDIDFile == NULL)
        return false;

    MW_RESULT mr;
    FILE *fd = NULL;
    HCHANNEL hChannel = -1;
    bool bRet = false;


    do
    {
        hChannel= MWOpenChannelByPath(szDevice);
        if (hChannel == -1)
            break;

        fd = fopen(szEDIDFile, "rw+");
        if (fd == NULL)
            break;

        DWORD arrInputSources[16];
        DWORD nInputSourceCount = 16;
        mr = MWGetVideoInputSourceArray(hChannel, arrInputSources, &nInputSourceCount);
        int i;
        for (i = 0; i < nInputSourceCount; i++)
        {
            if(INPUT_TYPE(arrInputSources[i])== MWCAP_VIDEO_INPUT_TYPE_HDMI)
                break;
        }

        if(i == nInputSourceCount)
        {
            printf("Error: Device %s has no HDMI interface\n", szDevice);
            return false;
        }

        BYTE pbData[512];
        ULONG cbData = 512;


        fseek(fd, 0L, SEEK_END);
        cbData = ftell(fd);
        fseek(fd, 0L, SEEK_SET);
        if (fread(pbData, cbData, 1, fd) != 1)
            break;

        mr = MWSetEDID(hChannel, pbData, cbData);
        if (mr != MW_SUCCEEDED)
            break;

        bRet = true;

    }while(false);

    if (fd != NULL)
    {
        fclose(fd);
        fd = NULL;
    }

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetLEDMode(char *szDevice, int nMode)
{
    if (szDevice == NULL)
        return false;

    if (nMode != 'a' && (nMode < 0 || nMode > 4))
        return false;

    MWCAP_LED_MODE ledMode;

    switch(nMode)
    {
    case 'a':
    {
        ledMode = MWCAP_LED_AUTO;
    }
        break;
    case 0:
    {
        ledMode = MWCAP_LED_OFF;
    }
        break;
    case 1:
    {
        ledMode = MWCAP_LED_ON;
    }
        break;
    case 2:
    {
        ledMode = MWCAP_LED_BLINK;
    }
        break;
    case 3:
    {
        ledMode = MWCAP_LED_DBL_BLINK;
    }
        break;
    case 4:
    {
        ledMode = MWCAP_LED_BREATH;
    }
        break;
    default:
        break;
    }

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevice);
        if (hChannel == -1)
            break;

        mr = MWSetLEDMode(hChannel, ledMode);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetTiming(char *szDevicePath, char *szInFileName)
{
    if (szDevicePath == NULL || szInFileName == NULL)
        return false;

    MW_RESULT mr;
    HCHANNEL hChannel = -1;
    bool bRet = false;


    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        DWORD dwInputSource, dwInputType;
        mr = MWGetVideoInputSource(hChannel, &dwInputSource);
        if (mr != MW_SUCCEEDED)
            break;
        dwInputType = INPUT_TYPE(dwInputSource);
        if (dwInputType != MWCAP_VIDEO_INPUT_TYPE_VGA && dwInputType != MWCAP_VIDEO_INPUT_TYPE_COMPONENT)
        {
            printf("Error: Current input source is not \"VGA\" or \"COMPONENT\"\n");
            break;
        }

        char szConfigVal[256];
        MWCAP_VIDEO_CUSTOM_TIMING customTiming;

        ReadConfig(szInFileName, "SyncType", szConfigVal);
        customTiming.syncInfo.bySyncType = atoi(szConfigVal);

        ReadConfig(szInFileName, "HSPolarity", szConfigVal);
        customTiming.syncInfo.bHSPolarity = atoi(szConfigVal);

        ReadConfig(szInFileName, "VSPolarity", szConfigVal);
        customTiming.syncInfo.bVSPolarity = atoi(szConfigVal);

        ReadConfig(szInFileName, "Interlaced", szConfigVal);
        customTiming.syncInfo.bInterlaced = atoi(szConfigVal);

        ReadConfig(szInFileName, "FrameDuration", szConfigVal);
        customTiming.syncInfo.dwFrameDuration = atoi(szConfigVal);


        ReadConfig(szInFileName, "VSyncLineCount", szConfigVal);
        customTiming.syncInfo.wVSyncLineCount = atoi(szConfigVal);

        ReadConfig(szInFileName, "FrameLineCount", szConfigVal);
        customTiming.syncInfo.wFrameLineCount = atoi(szConfigVal);

        ReadConfig(szInFileName, "AspectX", szConfigVal);
        customTiming.videoTimingSettings.wAspectX = atoi(szConfigVal);

        ReadConfig(szInFileName, "AspectY", szConfigVal);
        customTiming.videoTimingSettings.wAspectY = atoi(szConfigVal);


        ReadConfig(szInFileName, "HValidPixels", szConfigVal);
        customTiming.videoTimingSettings.cx = atoi(szConfigVal);


        ReadConfig(szInFileName, "VValidPixels", szConfigVal);
        customTiming.videoTimingSettings.cy = atoi(szConfigVal);


        ReadConfig(szInFileName, "HOffset", szConfigVal);
        customTiming.videoTimingSettings.x = atoi(szConfigVal);

        ReadConfig(szInFileName, "VOffset", szConfigVal);
        customTiming.videoTimingSettings.y = atoi(szConfigVal);



        ReadConfig(szInFileName, "HTotalPixels", szConfigVal);
        customTiming.videoTimingSettings.cxTotal = atoi(szConfigVal);

        ReadConfig(szInFileName, "ClampPos", szConfigVal);
        customTiming.videoTimingSettings.byClampPos = atoi(szConfigVal);


        mr = MWSetCustomVideoTiming(hChannel, customTiming);
        if (mr != MW_SUCCEEDED)
            break;

        bRet = true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SaveTiming(char *szDevicePath, char *szInFileName)
{
    if (szDevicePath == NULL || szInFileName == NULL)
        return false;

    MW_RESULT mr;
    HCHANNEL hChannel = -1;
    bool bRet = false;


    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        DWORD dwVideoInputSource;
        mr = MWGetVideoInputSource(hChannel, &dwVideoInputSource);
        if(mr != MW_SUCCEEDED)
            break;

        if (INPUT_TYPE(dwVideoInputSource) != MWCAP_VIDEO_INPUT_TYPE_VGA &&
                INPUT_TYPE(dwVideoInputSource) != MWCAP_VIDEO_INPUT_TYPE_COMPONENT)
        {
            printf("Error: Current input source is not \"VGA\" or \"COMPONENT\"\n");
            break;
        }


        MWCAP_INPUT_SPECIFIC_STATUS specificStatus;
        mr = MWGetInputSpecificStatus(hChannel, &specificStatus);
        if(mr != MW_SUCCEEDED)
            break;

        char szConfigVal[256];

        sprintf(szConfigVal,"SyncType=%d", specificStatus.vgaComponentStatus.syncInfo.bySyncType);
        AddOrAltConfig(szInFileName, "SyncType", szConfigVal);


        sprintf(szConfigVal,"HSPolarity=%d", specificStatus.vgaComponentStatus.syncInfo.bHSPolarity);
        AddOrAltConfig(szInFileName, "HSPolarity", szConfigVal);


        sprintf(szConfigVal,"VSPolarity=%d", specificStatus.vgaComponentStatus.syncInfo.bVSPolarity);
        AddOrAltConfig(szInFileName, "VSPolarity", szConfigVal);


        sprintf(szConfigVal,"Interlaced=%d", specificStatus.vgaComponentStatus.syncInfo.bInterlaced);
        AddOrAltConfig(szInFileName, "Interlaced", szConfigVal);


        sprintf(szConfigVal,"FrameDuration=%d", specificStatus.vgaComponentStatus.syncInfo.dwFrameDuration);
        AddOrAltConfig(szInFileName, "FrameDuration", szConfigVal);


        sprintf(szConfigVal,"VSyncLineCount=%d", specificStatus.vgaComponentStatus.syncInfo.wVSyncLineCount);
        AddOrAltConfig(szInFileName, "VSyncLineCount", szConfigVal);


        sprintf(szConfigVal,"FrameLineCount=%d", specificStatus.vgaComponentStatus.syncInfo.wFrameLineCount);
        AddOrAltConfig(szInFileName, "FrameLineCount", szConfigVal);


        sprintf(szConfigVal,"AspectX=%d", specificStatus.vgaComponentStatus.videoTimingSettings.wAspectX);
        AddOrAltConfig(szInFileName, "AspectX", szConfigVal);


        sprintf(szConfigVal,"AspectY=%d", specificStatus.vgaComponentStatus.videoTimingSettings.wAspectY);
        AddOrAltConfig(szInFileName, "AspectY", szConfigVal);


        sprintf(szConfigVal,"HValidPixels=%d", specificStatus.vgaComponentStatus.videoTimingSettings.cx);
        AddOrAltConfig(szInFileName, "HValidPixels", szConfigVal);


        sprintf(szConfigVal,"VValidPixels=%d", specificStatus.vgaComponentStatus.videoTimingSettings.cy);
        AddOrAltConfig(szInFileName, "VValidPixels", szConfigVal);


        sprintf(szConfigVal,"HOffset=%d", specificStatus.vgaComponentStatus.videoTimingSettings.x);
        AddOrAltConfig(szInFileName, "HOffset", szConfigVal);


        sprintf(szConfigVal,"VOffset=%d", specificStatus.vgaComponentStatus.videoTimingSettings.y);
        AddOrAltConfig(szInFileName, "VOffset", szConfigVal);


        sprintf(szConfigVal,"HTotalPixels=%d", specificStatus.vgaComponentStatus.videoTimingSettings.cxTotal);
        AddOrAltConfig(szInFileName, "HTotalPixels", szConfigVal);


        sprintf(szConfigVal,"ClampPos=%d", specificStatus.vgaComponentStatus.videoTimingSettings.byClampPos);
        AddOrAltConfig(szInFileName, "ClampPos", szConfigVal);

        bRet = true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}


int SetPixelBrightness(char *szDevicePath, int nBrightness)
{
    if (szDevicePath == NULL)
        return -1;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoBrightness(hChannel, nBrightness);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

int SetPixelContrast(char *szDevicePath, int nContrast)
{
    if (szDevicePath == NULL)
        return -1;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoContrast(hChannel, nContrast);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

int SetPixelHue(char *szDevicePath, int nHue)
{
    if (szDevicePath == NULL)
        return -1;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoHue(hChannel, nHue);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}


int SetPixelSaturation(char *szDevicePath, int nSaturation)
{
    if (szDevicePath == NULL)
        return -1;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoSaturation(hChannel, nSaturation);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}


int SetVideoOutputDeinterlace(char *szDevicePath, int nDeinterlace)
{

    if (szDevicePath == NULL)
        return -1;

    if (nDeinterlace < 0 || nDeinterlace > 3)
        return -1;

    MWCAP_VIDEO_DEINTERLACE_MODE dm;
    switch (nDeinterlace)
    {
    case 0:
    {
        dm = MWCAP_VIDEO_DEINTERLACE_WEAVE;
    }
        break;
    case 1:
    {
        dm = MWCAP_VIDEO_DEINTERLACE_BLEND;
    }
        break;
    case 2:
    {
        dm = MWCAP_VIDEO_DEINTERLACE_TOP_FIELD;
    }
        break;
    case 3:
    {
        dm = MWCAP_VIDEO_DEINTERLACE_BOTTOM_FIELD;
    }
        break;
    default:
        dm = MWCAP_VIDEO_DEINTERLACE_WEAVE;
        break;
    }


    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.deinterlaceMode = dm;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;

}


int SetVideoOutputRect(char *szDevicePath, RECT rect)
{
    if (szDevicePath == NULL)
        return -1;


    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.rectSource = rect;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

int SetVideoOutputAspect(char *szDevicePath, int X, int Y)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.nAspectX = X;
            processSettings.nAspectY = Y;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

int SetVideoOutputLowlatency(char *szDevicePath, char bLowlatency)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.bLowLatency = bLowlatency;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;

}

int SetVideoOutputColorFormat(char *szDevicePath, int nColorFormat)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.colorFormat = nColorFormat;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;

}

int SetVideoOutputQuantization(char *szDevicePath, int nQuantization)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.quantRange = nQuantization;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;

}

int SetVideoOutputSaturation(char *szDevicePath, int nSaturation)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            processSettings.satRange = nSaturation;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

int SetVideoOutputAspectConvert(char *szDevicePath, int nAspectConvert)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;


            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            //printf("aspect convert: %d, %d\n", processSettings.aspectRatioConvertMode, nAspectConvert);

            processSettings.aspectRatioConvertMode = nAspectConvert;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
        }
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;

}

int SetAsPreset(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return -1;

    MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;


            mr = MWGetVideoProcessSettings(hChannel, &processSettings);
            if (mr != MW_SUCCEEDED)
                break;


            //printf("aspect convert: %d, %d\n", processSettings.aspectRatioConvertMode, nAspectConvert);
            break;
        }

        if (nV4l2Streams == 0)
            break;

        for(int i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming != 0)
                continue;

            mr = MWSetCtrlStreamID(hChannel, streamsInfo[i].stream_id);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSetVideoProcessSettings(hChannel, processSettings);
            if (mr != MW_SUCCEEDED)
                break;

            mr = MWSaveSettingsAsPreset(hChannel);
            if (mr != MW_SUCCEEDED)
                break;

            nRet++;
            break;
        }

    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}

/*int ReloadPreset(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return -1;

    //MWCAP_VIDEO_PROCESS_SETTINGS processSettings;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    int nRet = -1;

    MWCAP_STREAM_INFO streamsInfo[16];
    int nStreamsCount = 16;
    int nV4l2Streams = 0;

    do {
        hChannel = MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        mr = MWGetStreamInfos(hChannel, streamsInfo, &nStreamsCount);
        if (mr != MW_SUCCEEDED)
            break;

        int i;
        nRet = 0;
        for (i = 0; i < nStreamsCount; i++)
        {
            if (streamsInfo[i].streaming == 0)
                continue;

            nV4l2Streams++;
            break;
        }

        if (nV4l2Streams == 0)
            break;

        mr = MWReloadPreset(hChannel);
        if (mr != MW_SUCCEEDED)
            break;

        nRet++;
    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (nRet != nV4l2Streams)
        return -1;
    else
        return nRet;
}*/

//add
bool SetVideoAutoScan(char *szDevicePath, int nAutoScan)
{
    if (szDevicePath == NULL)
        return false;

    if (nAutoScan < 0 || nAutoScan > 1)
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nAutoScan == 0)
            mr = MWSetInputSourceScan(hChannel, 0);
        else
            mr = MWSetInputSourceScan(hChannel, 1);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}


bool SetAudioLinkWithVideo(char *szDevicePath, int nLinkWithVideo)
{
    if (szDevicePath == NULL)
        return false;

    if (nLinkWithVideo < 0 || nLinkWithVideo > 1)
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nLinkWithVideo == 0)
            mr = MWSetAVInputSourceLink(hChannel, 0);
        else
            mr = MWSetAVInputSourceLink(hChannel, 1);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetAutoSamplPhase(char *szDevicePath, int nAutoSamplPhase)
{
    if (szDevicePath == NULL)
        return false;

    if (nAutoSamplPhase < 0 || nAutoSamplPhase > 1)
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nAutoSamplPhase == 0)
            mr = MWSetVideoSamplingPhaseAutoAdjust(hChannel, 0);
        else
            mr = MWSetVideoSamplingPhaseAutoAdjust(hChannel, 1);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

bool SetAutoHAlign(char *szDevicePath, int nAutoHAlign)
{
    if (szDevicePath == NULL)
        return false;

    if (nAutoHAlign < 0 || nAutoHAlign > 1)
        return false;

    HCHANNEL hChannel = -1;
    MW_RESULT mr;
    bool bRet = false;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;

        if (nAutoHAlign == 0)
            mr = MWSetVideoAutoHAlign(hChannel, 0);
        else
            mr = MWSetVideoAutoHAlign(hChannel, 1);

        if (mr != MW_SUCCEEDED)
            break;

        bRet =true;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return bRet;
}

int SetPhase(char *szDevicePath, int nPhase)
{
    if (szDevicePath == NULL)
        return false;

    if (nPhase < 0 || nPhase > 63)
        return false;

    HCHANNEL hChannel = -1;
    int nRet = -1;

    do
    {
        hChannel= MWOpenChannelByPath(szDevicePath);
        if (hChannel == -1)
            break;
        nRet = MWSetVideoSamplingPhase(hChannel, nPhase);
        if (nRet != 0)
            break;
        nRet = 0;

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return nRet;
}




