#include "ccapturethread.h"
//#include "common.h"

static struct termios oldt;
CCaptureThread::CCaptureThread()
{
    m_hChannel = -1;
//    m_nDevice = 0;
    m_audioBuf = NULL;
    m_nThreadIDAudio = 0;
    m_nThreadIDVideo = 0;

    m_cx = 1920;
    m_cy = 1080;
    m_dwMinStride = 0;
    m_dwImageSize = 0;
    m_pbImage = 0;
    m_hVideoCapEvent = -1;
    m_hNotifyEvent = -1;
    m_hTimerEvent = -1;
    m_hTimer = 0;

    m_bAudioCap = false;
    m_nPresentChannel = 0;
    m_hAudioNotify = -1;
    m_hAudioNotifyEvent = -1;
    m_callback = NULL;

    m_bStopFlag = false;
}

BOOL CCaptureThread::InitCapture(char *szDevicePath, int nWidth, int nHeight)
{
    //init mwcapture
    bool bRet = MWCaptureInitInstance();
    if(bRet == false)
        return false;

    MW_RESULT nRet = MWRefreshDevice();
    if(nRet != MW_SUCCEEDED)
        return false;

    int nChannelCount = MWGetChannelCount();
    if(nChannelCount <= 0)
        return false;

    m_hChannel = MWOpenChannelByPath(szDevicePath);
    if(m_hChannel < 0)
        return false;

    MWCAP_VIDEO_CAPS videoCaps;
    nRet = MWGetVideoCaps(m_hChannel, &videoCaps);
    if(nRet != MW_SUCCEEDED)
        return false;

    if(nWidth > videoCaps.wMaxOutputWidth || nHeight > videoCaps.wMaxOutputHeight)
    {
        printf("relosution set :invalid arguments\n");
        return false;
    }

    return true;
}

void CCaptureThread::ExitCapture()
{
    if (m_hChannel != -1) {
        MWCloseChannel(m_hChannel);
        m_hChannel = -1;
    }

    MWCaptureExitInstance();
}

BOOL CCaptureThread::Create(int nWidth, int nHeight, int framerates, ICallback *pCallback)
{
    m_cx = nWidth;
    m_cy = nHeight;
    m_callback = pCallback;
    m_dwFrameDuration = framerates;
    int num = 0;
    bool bRet = StartVideoCapture();
    if(bRet == false)
    {
        num ++;
        StopVideoCapture();
    }

    /*bool*/ bRet = StartAudioCapture();
    if(bRet == false)
    {
        num++;
        StopAudioCapture();
    }

    if(num == 2)
        return false;

    return true;
}

void CCaptureThread::Destory()
{
    if(m_nThreadIDVideo > 0)
    {
        pthread_join(m_nThreadIDVideo, NULL);
        m_nThreadIDVideo = 0;
        StopVideoCapture();
    }

    if(m_nThreadIDAudio > 0)
    {
        pthread_join(m_nThreadIDAudio, NULL);
        m_nThreadIDAudio = 0;
        StopAudioCapture();
    }
}


BOOL CCaptureThread::StartVideoCapture()
{
    do
    {
        m_hVideoCapEvent = MWCreateEvent();
        if(m_hVideoCapEvent < 0)
            break;

        m_hNotifyEvent = MWCreateEvent();
        if(m_hNotifyEvent < 0)
            break;

        m_hTimerEvent = MWCreateEvent();
        if(m_hTimerEvent < 0)
            break;

        m_hTimer = MWRegisterTimer(m_hChannel, m_hTimerEvent);
        if(m_hTimer < 0)
            break;

        m_hNotifyBuffered = MWRegisterNotify(m_hChannel, m_hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED);
        if(m_hNotifyBuffered <= 0)
            break;

        m_dwMinStride = FOURCC_CalcMinStride(VIDEO_FORMAT, m_cx, 4);
        m_dwImageSize = FOURCC_CalcImageSize(VIDEO_FORMAT, m_cx, m_cy, m_dwMinStride);
        m_pbImage = (HANDLE64)(unsigned long)malloc(m_dwImageSize);

        MW_RESULT nRet;
        nRet = MWStartVideoCapture(m_hChannel, m_hVideoCapEvent);
        if(nRet != MW_SUCCEEDED)
            break;

        MWCAP_VIDEO_BUFFER_INFO bufferInfo;
        nRet = MWGetVideoBufferInfo(m_hChannel, &bufferInfo);
        if(nRet != MW_SUCCEEDED)
            break;

        MWCAP_VIDEO_FRAME_INFO frameInfo;
        nRet = MWGetVideoFrameInfo(m_hChannel, bufferInfo.iNewestBufferedFullFrame, &frameInfo);
        if(nRet != MW_SUCCEEDED)
            break;

        int rc = pthread_create(&m_nThreadIDVideo, NULL, VideoCaptureProc, this);
        if(rc != 0)
        {
            m_nThreadIDVideo = 0;
            break;
        }

        return true;

    }while(false);

    return false;
}

void CCaptureThread::StopVideoCapture()
{
    MWSetEvent(m_hVideoCapEvent);

    if(m_hChannel != -1)
    {
        MWStopVideoCapture(m_hChannel);
    }

    if(m_hNotifyBuffered > 0)
        MWUnregisterNotify(m_hChannel, m_hNotifyBuffered);

    if(m_hTimer > 0)
        MWUnregisterTimer(m_hChannel, m_hTimer);

    if(m_hTimerEvent != 0)
        MWCloseEvent(m_hTimerEvent);

    if(m_hNotifyEvent != 0)
        MWCloseEvent(m_hNotifyEvent);

    if(m_hVideoCapEvent != 0)
        MWCloseEvent(m_hVideoCapEvent);

    if(m_pbImage != 0)
    {
        free((uint8_t *)m_pbImage);
        m_pbImage = 0;
    }
}

BOOL CCaptureThread::StartAudioCapture()
{
    do
    {
        DWORD pdwInputSource[MAX_AUDIO_INPUT_SOURCE_NUM];
        DWORD dwInputCount = MAX_AUDIO_INPUT_SOURCE_NUM;

        MW_RESULT nRet = MWGetAudioInputSourceArray(m_hChannel, pdwInputSource, &dwInputCount);
        if(nRet != MW_SUCCEEDED || dwInputCount == 0)
            break;

        nRet = MWStartAudioCapture(m_hChannel);
        if(nRet != MW_SUCCEEDED)
            break;

        m_hAudioNotifyEvent = MWCreateEvent();
        if(m_hAudioNotifyEvent == 0)
            break;

        m_hAudioNotify = MWRegisterNotify(m_hChannel, m_hAudioNotifyEvent, MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED);
        if(m_hAudioNotify == 0)
            break;

        MWCAP_AUDIO_SIGNAL_STATUS mass;
        if(MW_SUCCEEDED != MWGetAudioSignalStatus(m_hChannel, &mass))
            break;

        if(!mass.bLPCM)
            break;

        int i = 0;
        for(i = 0; i < 4; i++)
        {
            bool bPresent  = (mass.wChannelValid & (0x01 << i)) ? true : false;
            m_nPresentChannel += bPresent ? 2 : 0;
        }

        if(m_nPresentChannel == 0)
            break;

        m_bAudioCap = true;
        m_audioBuf = (unsigned char *)malloc(AAC_SAMPLES * m_nPresentChannel * BIT_DEPTH_BYTE);
        memset(m_audioBuf, 0, AAC_SAMPLES * m_nPresentChannel * BIT_DEPTH_BYTE);

        int rc = pthread_create(&m_nThreadIDAudio, NULL, AudioCaptureProc, this);
        if(rc != 0)
        {
            m_nThreadIDAudio = 0;
            break;
        }

        return true;

    }while(false);

    return false;
}

void CCaptureThread::StopAudioCapture()
{
    if(m_hAudioNotify != 0)
    {
        MWUnregisterNotify(m_hChannel, m_hAudioNotify);
        m_hAudioNotify = 0;
    }

    if(m_hAudioNotifyEvent != 0)
    {
        MWCloseEvent(m_hAudioNotifyEvent);
        m_hAudioNotifyEvent =0;
    }

    if(m_hChannel != -1)
        MWStopAudioCapture(m_hChannel);
}

void CCaptureThread::VideoCaptureProcInternel()
{
    int mr = 0;
    MW_RESULT nRet;
    //bool bRet = true;

    LONGLONG llFrameCount = 0;
    LONGLONG llStartTime = 0;
    LONGLONG llTotaltime = 0;
    ULONGLONG status;
    nRet = MWGetDeviceTime(m_hChannel, &llStartTime);
    while(m_bStopFlag == false)
    {
        MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
        MWGetVideoSignalStatus(m_hChannel, &videoSignalStatus);
        if(videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
        {
            nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, -1, m_pbImage, m_dwImageSize, m_dwMinStride, 0, 0, VIDEO_FORMAT,
                                                         m_cx, m_cy, 0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND,
                                                         MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                         MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN,
                                                         MWCAP_VIDEO_SATURATION_UNKNOWN);
            if(nRet != MW_SUCCEEDED)
                break;

            mr = MWWaitEvent(m_hVideoCapEvent, -1);
            if(mr <= 0)
                break;

            MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
            nRet = MWGetVideoCaptureStatus(m_hChannel, &captureStatus);
            if(nRet != MW_SUCCEEDED)
                break;

            llFrameCount++;
            llTotaltime = llStartTime + llFrameCount * (LONGLONG)((double)10000000 / m_dwFrameDuration);
            nRet = MWScheduleTimer(m_hChannel, m_hTimer, llTotaltime);
            MWWaitEvent(m_hTimerEvent, -1);

            m_callback->VideoEncodeCallback((uint8_t *)m_pbImage, false);
        }
        else
        {
            MWWaitEvent(m_hNotifyEvent, -1);
            MWGetNotifyStatus(m_hChannel, m_hNotifyBuffered, &status);

            MWCAP_VIDEO_BUFFER_INFO bufferInfo;
            MWGetVideoBufferInfo(m_hChannel, &bufferInfo);
            if(status == MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED )
            {
                nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, bufferInfo.iNewestBufferedFullFrame, m_pbImage, m_dwImageSize, m_dwMinStride, 0, 0, VIDEO_FORMAT,
                                                             m_cx, m_cy, 0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND,
                                                             MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                             MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN,
                                                             MWCAP_VIDEO_SATURATION_UNKNOWN);
                if(nRet != MW_SUCCEEDED)
                    break;

                mr = MWWaitEvent(m_hVideoCapEvent, -1);
                if(mr <= 0)
                    break;

                MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
                nRet = MWGetVideoCaptureStatus(m_hChannel, &captureStatus);
                if(nRet != MW_SUCCEEDED)
                    break;

                llFrameCount++;
                llTotaltime = llStartTime + llFrameCount * (LONGLONG)((double)10000000 / m_dwFrameDuration);
                nRet = MWScheduleTimer(m_hChannel, m_hTimer, llTotaltime);
                MWWaitEvent(m_hTimerEvent, 1000);

                m_callback->VideoEncodeCallback((uint8_t *)m_pbImage, m_bAudioCap);
            }
        }
    }
}

void CCaptureThread::AudioCaptureProcInternel()
{
    int num = 0;
    int mr = 0;
    bool bRet = true;
    ULONGLONG ullStatusBit = 0LL;
    BYTE pbFrame[MWCAP_AUDIO_SAMPLES_PER_FRAME * m_nPresentChannel * BIT_DEPTH_BYTE];
    while(m_bStopFlag == false)
    {
        mr = MWWaitEvent(m_hAudioNotifyEvent, 1000);
        if(mr < 0)
            continue;

        if(MW_SUCCEEDED != MWGetNotifyStatus(m_hChannel, m_hAudioNotify, &ullStatusBit))
            continue;

        if(ullStatusBit & MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED)
        {
            MWCAP_AUDIO_CAPTURE_FRAME macf;
            if(MW_SUCCEEDED != MWCaptureAudioFrame(m_hChannel, &macf))
                continue;

            int j = 0;
            int i = 0;
            int nWritePos = 0;
            int nReadPosL = 0;
            int nReadPosR = 0;
            for(i = 0; i < MWCAP_AUDIO_SAMPLES_PER_FRAME; i++)
            {
                for(j = 0; j < (m_nPresentChannel / 2); j++)
                {
                    nWritePos = (i * m_nPresentChannel + j * 2) * BIT_DEPTH_BYTE;
                    nReadPosL = (i * MWCAP_AUDIO_MAX_NUM_CHANNELS + j);
                    nReadPosR = (i * MWCAP_AUDIO_MAX_NUM_CHANNELS + j + MWCAP_AUDIO_MAX_NUM_CHANNELS / 2);

                    DWORD dwLeft = macf.adwSamples[nReadPosL] >> (32 - BIT_DEPTH_BYTE * 8);
                    DWORD dwRight = macf.adwSamples[nReadPosR] >> (32 - BIT_DEPTH_BYTE * 8);

                    memcpy(&pbFrame[nWritePos], &dwLeft, BIT_DEPTH_BYTE);
                    memcpy(&pbFrame[nWritePos + BIT_DEPTH_BYTE], &dwRight, BIT_DEPTH_BYTE);
                }

                nWritePos = (i * m_nPresentChannel) * BIT_DEPTH_BYTE;
                memcpy(m_audioBuf + (num * m_nPresentChannel * BIT_DEPTH_BYTE), &pbFrame[nWritePos], m_nPresentChannel * BIT_DEPTH_BYTE);
                num++;
                if(num == AAC_SAMPLES)
                {
                    num = 0;
                    bRet = m_callback->AudioEncodeCallback(m_audioBuf);
                    if(bRet == false)
                        continue;
                }
            }
        }
    }
    m_callback->FlushAudioEncodeCallback();
}

static void restore_terminal_settings(void)
{
	tcsetattr(0, TCSANOW, &oldt);
}

BOOL CCaptureThread::CheckKey()
{
    struct termios newt;
    tcgetattr(0, &oldt);
    newt = oldt;
    newt.c_lflag &= ~(ICANON | ECHO);
    tcsetattr(0, TCSANOW, &newt);
    atexit(restore_terminal_settings);

    int key = 0;
    while(1)
    {
        key = getchar();
        if ((key == 'q') || (key == 'Q'))
        {
            m_bStopFlag = true;
            break;
        }
    }

    return true;
}
