#ifndef CFFMPEGENCODE_H
#define CFFMPEGENCODE_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "coptions.h"
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/pixfmt.h>
#include <libavutil/opt.h>
#include <libavutil/timestamp.h>
#include <libavutil/avassert.h>
#include <libavutil/imgutils.h>
#include <libswscale/swscale.h>
#include <libswresample/swresample.h>
#include <linux/input.h>
#include <pthread.h>

#ifdef __cplusplus
}
#endif

typedef struct OutputStream {

    int                 samples_count;
    int64_t             next_pts;
    AVStream            *st;
    AVFrame             *frame;
    AVFrame             *tmp_frame;
    struct SwrContext   *swr_ctx;

} OutputStream;

#define STREAM_PIX_FMT    AV_PIX_FMT_YUV420P

class ICallback
{
public:
    virtual bool VideoEncodeCallback(uint8_t *pImage, bool bAudioCapture) = 0;
    virtual bool AudioEncodeCallback(uint8_t *pAudioBuf) = 0;
    virtual void FlushAudioEncodeCallback() = 0;
};

class CFFmpegEncode: public ICallback
{
public:
    CFFmpegEncode();

public:
    bool InitFFmpeg(OPTIONS option);
    void CloseFFmpeg();

    virtual bool VideoEncodeCallback(uint8_t *pImage, bool bAudioCapture);
    virtual bool AudioEncodeCallback(uint8_t *pAudioBuf);
    virtual void FlushAudioEncodeCallback();

protected:
    int AddStream(OutputStream *ost, AVFormatContext *oc, AVCodec **codec, enum AVCodecID codec_id);
    void CloseStream(AVFormatContext *oc, OutputStream *ost);

    //write frame
    int WriteFrame(AVFormatContext *fmt_ctx, const AVRational *time_base, AVStream *st, AVPacket *pkt);

    //video
    bool OpenVideo(AVCodec *codec, OutputStream *ost, AVDictionary *opt_arg);
    int WriteVideoFrame(AVFormatContext *oc, OutputStream *ost, uint8_t *imageData);
    AVFrame *AllocPicture(enum AVPixelFormat pix_fmt, int width, int height);
    AVFrame *GetVideoFrame(OutputStream *ost, uint8_t *imageData);

    //audio
    bool OpenAudio(AVFormatContext *oc, AVCodec *codec, OutputStream *ost, AVDictionary *opt_arg);
    int WriteAudioFrame(AVFormatContext *oc, OutputStream *ost, uint8_t *audio_buf);
    AVFrame *AllocAudioFrame(enum AVSampleFormat sample_fmt, uint64_t channel_layout, int sample_rate, int nb_samples);
    AVFrame *GetAudioFrame(OutputStream *ost, uint8_t *pAudioBuf);

protected:
    //ffmpeg
    int     m_nHaveVideo;
    int     m_nHaveAudio;

    AVCodec             *m_videoCodec;
    AVCodec             *m_audioCodec;
    AVDictionary        *m_opt;
    AVOutputFormat      *m_fmt;
    AVFormatContext     *m_oc;
    OutputStream        m_videoSt;
    OutputStream        m_audioSt;

    OPTIONS             m_options;
    pthread_mutex_t     m_mutex;

};

#endif // CFFMPEGENCODE_H
