#include "coptions.h"
#include "common.h"
COptions::COptions()
{
    m_option.dwWidth = 1920;
    m_option.dwHeight = 1080;
    m_option.dwFPS = 25;
    m_option.dwBitrate = 4000;
    m_option.dwProfile = 0;
    m_option.dwIFrames = 12;
    m_option.dwBFrame = 0;
    m_option.dwMp4 = 0;
    memset(m_option.szOutputFileName, 0, 256);
    strcpy(m_option.szDevicePath, "/dev/video0");
}

void COptions::PrintUsage()
{
    printf("USAGE\n"
           "  mwcap-ffmpeg-muxer [OPTIONS]\n\n"
           "OPTIONS\n"
           " --help                  print usage\n"
           " --channel               set device name, default is /dev/video0\n"
           " --resolution            set width and height, default is 1920x1080\n"
           " --framerates            set frame duration, default is 25fps\n"
           " --bitrates              set video bit rates (kb/s), default is 4000kb/s\n"
           " --profile               set encode profile (hp | mp | bp), default is hp\n"
           " --iframe                set encode I frame duration, default is 12\n"
           " --bframes               set encode B frame number, default is 0\n"
           " --output                set output mp4 file path\n"
           );
}

OPTION_RESULT COptions::ParseDeviceNum(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        if (!MatchReg(szArg, "^/dev/video[0-9]{1,2}+$"))
            break;

        if (access(szArg, F_OK) != 0)
            break;

        strcpy(m_option.szDevicePath, szArg);
        return OPTION_OK;

    }while(false);

    printf("channel set :invalid argument\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseResolutionArgument(char *szArg)
{
    if(szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([0-9]{1,4})x([0-9]{1,4})$"))
    {
        printf("resolution set :invalid argument\n");
        return OPTION_INVALID_PARAMS;
    }

    char *p;
    char *temp = strdup(szArg);
    int nCount = 0;

    p = strtok(temp, "x");
    while(p != NULL)
    {
        switch (nCount) {
        case 0:
        {
            m_option.dwWidth = atoi(p);
            break;
        }
        case 1:
        {
            m_option.dwHeight = atoi(p);
            break;
        }
        default:
            break;
        }
        nCount++;

        p = strtok(NULL, "x");
    }

    if((m_option.dwWidth <= 0) || (m_option.dwHeight <= 0))
    {
        printf("resolution set :invalid argument\n");
        return OPTION_INVALID_PARAMS;
    }

    return OPTION_OK;
}

OPTION_RESULT COptions::ParseFrameRate(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        if (!MatchReg(szArg, "^([1-9]{1}|[1-2]{1}[0-9]{1}|3[0-5]{1})$"))
            break;

        m_option.dwFPS = atoi(szArg);
        return OPTION_OK;

    }while(false);

    printf("framerates set :invalid arguments\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseBitRate(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        if (!MatchReg(szArg, "^([0-9]{0,7})$"))
            break;

        m_option.dwBitrate = atoi(szArg);
        return OPTION_OK;

    }while(false);

    printf("bitrates set :invalid arguments\n");
    return OPTION_INVALID_PARAMS;

}

OPTION_RESULT COptions::ParseLevel(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        int c = -1;
        if (strncasecmp(szArg, "hp", strlen("hp")) == 0)
            c = 0;
        else if (strncasecmp(szArg, "mp", strlen("mp")) == 0)
            c = 1;
        else if(strncasecmp(szArg, "bp", strlen("bp")) == 0)
            c = 2;

        if(c != -1)
            m_option.dwProfile = c;
        else
            break;

        return OPTION_OK;

    }while(false);

    printf("profile set :invalid arguments\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseBFrame(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        if (!MatchReg(szArg, "^([0-9]{0,1}|1[0-1]{1})$"))
            break;

        m_option.dwBFrame = atoi(szArg);
        return OPTION_OK;

    }while(false);

    printf("bframes set :invalid arguments\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseIFrame(char *szArg)
{
    do
    {
        if (szArg == NULL)
            break;

        if (!MatchReg(szArg, "^([0-9]{0,3})$"))
            break;

        m_option.dwIFrames = atoi(szArg);
        return OPTION_OK;

    }while(false);

    printf("iframes set :invalid arguments\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseOutput(char *szArg)
{
    if(szArg == NULL)
        return OPTION_INVALID_PARAMS;

    strcpy(m_option.szOutputFileName, szArg);

    char *p;
    char *temp = strdup(szArg);
    char *fmt = (char *)malloc(10);

    p = strtok(temp, ".");
    while(p != NULL)
    {
        strcpy(fmt, p);
        p = strtok(NULL, ".");
    }

    free(temp);

    if(strlen(fmt) == 3)
    {
        if(strncasecmp(fmt, "mp4", 3) != 0)
        {
            m_option.dwMp4 = 0;
            free(fmt);
            printf("output format set :invalid argument\n");
            return OPTION_INVALID_PARAMS;
        }

        else
        {
            m_option.dwMp4 = 1;
            free(fmt);
            return OPTION_OK;
        }
    }

    free(fmt);
    printf("output format set :invalid argument\n");
    return OPTION_INVALID_PARAMS;
}

OPTION_RESULT COptions::ParseCommondLine(int argc, char **argv)
{
    static const char short_options[] = "";
    static const struct option long_options[] =
    {
        {"help",                        no_argument,        NULL,       0},
        {"channel",                     required_argument,  NULL,       0},
        {"resolution",                  required_argument,  NULL,       0},
        {"framerates",                  required_argument,  NULL,       0},
        {"bitrates",                    required_argument,  NULL,       0},
        {"profile",                     required_argument,  NULL,       0},
        {"iframe",                      required_argument,  NULL,       0},
        {"bframes",                     required_argument,  NULL,       0},
        {"output",                      required_argument,  NULL,       0},
        {NULL,                          no_argument,        NULL,       0},
    };

    OPTION_RESULT opr;
    int c;
    int nIndex;
    while((c = getopt_long(argc, argv, short_options, long_options, &nIndex)) != -1)
    {
        if(c == '?')
        {
            PrintUsage();
            return OPTION_INVALID_OPTION;
        }
        switch (c) {
        case 0:
        {
            if(strcmp(long_options[nIndex].name, "help") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_USAGE);
                PrintUsage();
                opr = OPTION_FAILED;
                break;
            }
            if(strcmp(long_options[nIndex].name, "channel") == 0) {
                m_option.dwCmd |= (0x01 << OPTION_CMD_CHANNEL);
                opr = ParseDeviceNum(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "resolution") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_RESOLUTION);
                opr = ParseResolutionArgument(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "framerates") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_FRAME_RATE);
                opr = ParseFrameRate(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "bitrates") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_BITRATE);
                opr = ParseBitRate(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "profile") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_LEVEL);
                opr = ParseLevel(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "iframe") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_IFRAME);
                opr = ParseIFrame(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "bframes") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_BFRAME);
                opr = ParseBFrame(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }

            if(strcmp(long_options[nIndex].name, "output") == 0)
            {
                m_option.dwCmd |= (0x01 << OPTION_CMD_OUTPUT);
                opr = ParseOutput(optarg);
                if(opr == OPTION_INVALID_PARAMS)
                    break;
            }
        }

        default:
            break;
        }
    }

    if(opr == OPTION_FAILED || opr == OPTION_INVALID_PARAMS)
        return OPTION_FAILED;

    if(strlen(m_option.szOutputFileName) == 0)
        PrintUsage();

    return OPTION_OK;
}

