#ifndef DEVICEPROPERTY_H
#define DEVICEPROPERTY_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdbool.h>


#include "win-types.h"
#include "mw-common.h"
#include "lib-mw-capture.h"

typedef enum _MWCAP_INFO_RESULT_
{
    RESULT_INVALID_PARAMS,
    RESULT_OK,
    RESULT_NOT_EXIST_DEVICE,
    RESULT_NOT_PRO_CAPTURE_DEVICE,
    RESULT_ERROR_ARGUMENT,
    RESULT_UNKNOW_ERROR,
    RESULT_ERROR_OPTION,
    RESULT_NO_MATCHING_DEVICE,

} MWCAP_INFO_RESULT;


//public
static MWCAP_INFO_RESULT GetDevicePath(char (*arrDevicePath)[MAX_CHANNEL_NAME_LEN] , int *nCount,int nBoardId, int nChannelId)
{
    MW_RESULT mr;
    int nChannelNum = MWGetChannelCount();

    int i;
    int nTemp = 0;
    MWCAP_CHANNEL_INFO channelInfo;
    for (i = 0; i < nChannelNum && nTemp < *nCount; i++)
    {
        mr = MWGetChannelInfoByIndex(i, &channelInfo);
        if (mr != MW_SUCCEEDED)
            continue;

        if (nBoardId != channelInfo.byBoardIndex || nChannelId != channelInfo.byChannelIndex)
            continue;

        mr = MWGetDevicePath(i, arrDevicePath[nTemp]);
        if (mr != MW_SUCCEEDED)
            continue;

        nTemp ++;
    }

    if (nTemp == 0)
        return RESULT_NO_MATCHING_DEVICE;

    *nCount = nTemp;

    return RESULT_OK;
}

static int GetDeviceIndexByPath(char * szDevicePath)
{
    MW_RESULT mr;
    int nChannelNum = MWGetChannelCount();

    int i;
    char szPath[MAX_CHANNEL_NAME_LEN];
    for (i = 0; i < nChannelNum; i++)
    {
        mr = MWGetDevicePath(i, szPath);
        if (mr != MW_SUCCEEDED)
            continue;

        if(strcmp(szDevicePath, szPath) == 0)
            return i;
    }

    return -1;
}

// channel info
static MW_RESULT PrintChannelInfo(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    MWCAP_CHANNEL_INFO channelInfo;
    mr = MWGetChannelInfo(hChannel, &channelInfo);
    if (mr != MW_SUCCEEDED)
        return MW_FAILED;

    char szChannelName[256] = {0};
    char szFirmwareVerCur[16] = {0};
    char szDriverVerCur[16] = {0};

    sprintf(szChannelName, "%02d:%02d %s",
                    channelInfo.byBoardIndex,
                    channelInfo.byChannelIndex,
                    channelInfo.szProductName);
    sprintf(szFirmwareVerCur, "%d.%d",
            (DWORD)(channelInfo.dwFirmwareVersion >> 16) & 0xFFFF,
            (DWORD)(channelInfo.dwFirmwareVersion) & 0xFFFF);

    sprintf(szDriverVerCur, "%d.%d.%d",
            (DWORD)(channelInfo.dwDriverVersion >> 24) & 0xFF,
            (DWORD)(channelInfo.dwDriverVersion >> 16) & 0xFF,
            (DWORD)(channelInfo.dwDriverVersion) & 0xFFFF);

    int nDeviceIndex = GetDeviceIndexByPath(szDevicePath);
    char szHwName[MAX_HW_NAME_LEN];
    if (nDeviceIndex != -1)
        GetAudioHwNameByindex(nDeviceIndex, szHwName);

    printf("%-15s %-15s %-15c %-15s %-15s %-30s\n",
                   szDevicePath, szFirmwareVerCur, channelInfo.chHardwareVersion, szDriverVerCur, szHwName, szChannelName);

    MWCloseChannel(hChannel);
    hChannel = -1;

    return MW_SUCCEEDED;
}

static void PrintAllChannelInfo()
{
    MW_RESULT mr;
    char szDevicePath[MAX_CHANNEL_NAME_LEN];

    int nChannelNum = MWGetChannelCount();
    printf("total: %d\n", nChannelNum);
    if (nChannelNum > 0)
        printf("%-15s %-15s %-15s %-15s %-15s %-30s\n",
               "device path", "firmware ver", "hardware ver", "driver ver", "alsa name","pro capture name");

    int i;
    for (i = 0; i < nChannelNum; i++)
    {
        mr = MWGetDevicePath(i, szDevicePath);
        if (mr != MW_SUCCEEDED)
            continue;

        PrintChannelInfo(szDevicePath);
    }

    return;
}

// input
static bool GetVideoInputTypeString(DWORD dwInputType, char *szInputType)
{
    if (szInputType == NULL)
        return false;

    switch(dwInputType)
    {
    case MWCAP_VIDEO_INPUT_TYPE_NONE:
    {
        strcpy(szInputType, "NONE");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_HDMI:
    {
        strcpy(szInputType, "HDMI");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_VGA:
    {
        strcpy(szInputType, "VGA");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_SDI:
    {
        strcpy(szInputType, "SDI");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_COMPONENT:
    {
        strcpy(szInputType, "COMPONENT");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_CVBS:
    {
        strcpy(szInputType, "CVBS");
        break;
    }
    case MWCAP_VIDEO_INPUT_TYPE_YC:
    {
        strcpy(szInputType, "YC");
        break;
    }
    default:
        break;
    }

    return true;
}



static bool GetAudioInputTypeString(DWORD dwInputType, char *szInputType)
{
    if (szInputType == NULL)
        return false;

    switch(dwInputType)
    {
    case MWCAP_AUDIO_INPUT_TYPE_NONE:
    {
        strcpy(szInputType, "NONE");
        break;
    }
    case MWCAP_AUDIO_INPUT_TYPE_HDMI:
    {
        strcpy(szInputType, "HDMI");
        break;
    }
    case MWCAP_AUDIO_INPUT_TYPE_SDI:
    {
        strcpy(szInputType, "SDI");
        break;
    }
    case MWCAP_AUDIO_INPUT_TYPE_LINE_IN:
    {
        strcpy(szInputType, "LINE_IN");
        break;
    }
    case MWCAP_AUDIO_INPUT_TYPE_MIC_IN:
    {
        strcpy(szInputType, "MIC_IN");
        break;
    }
    default:
        break;
    }

    return true;
}

static bool GetSignalStateString(DWORD dwSignalState, char *szSignalState)
{
    if (szSignalState == NULL)
        return false;

    switch(dwSignalState)
    {
    case MWCAP_VIDEO_SIGNAL_NONE:
    {
        strcpy(szSignalState, "None");
        break;
    }
    case MWCAP_VIDEO_SIGNAL_LOCKING:
    {
        strcpy(szSignalState, "Locking");
        break;
    }
    case MWCAP_VIDEO_SIGNAL_LOCKED:
    {
        strcpy(szSignalState, "Locked");
        break;
    }
    case MWCAP_VIDEO_SIGNAL_UNSUPPORTED:
    {
        strcpy(szSignalState, "Unsupported");
        break;
    }
    default:
        break;
    }

    return true;
}

static bool GetColorSpaceString(DWORD dwColorspace, char *szColorSpace)
{
    if (szColorSpace == NULL)
        return false;

    switch (dwColorspace) {
    case MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN:
    {
        strcpy(szColorSpace, "Unkmow");
        break;
    }
    case MWCAP_VIDEO_COLOR_FORMAT_RGB:
    {
        strcpy(szColorSpace, "RGB");
        break;
    }
    case MWCAP_VIDEO_COLOR_FORMAT_YUV601:
    {
        strcpy(szColorSpace, "YUV BT.601");
        break;
    }
    case MWCAP_VIDEO_COLOR_FORMAT_YUV709:
    {
        strcpy(szColorSpace, "YUV BT.709");
        break;
    }
    case MWCAP_VIDEO_COLOR_FORMAT_YUV2020:
    {
        strcpy(szColorSpace, "YUV BT.2020");
        break;
    }
    case MWCAP_VIDEO_COLOR_FORMAT_YUV2020C:
    {
        strcpy(szColorSpace, "YUV BT.2020c");
        break;
    }
    default:
        break;
    }

    return true;
}

static bool GetQuantizationString(DWORD dwQuantization, char *szQuantization)
{
    if (szQuantization == NULL)
        return false;

    switch (dwQuantization) {
    case MWCAP_VIDEO_QUANTIZATION_UNKNOWN:
    {
        strcpy(szQuantization, "Unkmow");
        break;
    }
    case MWCAP_VIDEO_QUANTIZATION_FULL:
    {
        strcpy(szQuantization, "Full");
        break;
    }
    case MWCAP_VIDEO_QUANTIZATION_LIMITED:
    {
        strcpy(szQuantization, "Limited");
        break;
    }
    default:
        break;
    }

    return true;
}

static bool GetSaturationString(DWORD dwSaturation, char *szSaturation)
{
    if (szSaturation == NULL)
        return false;

    switch (dwSaturation) {
    case MWCAP_VIDEO_SATURATION_UNKNOWN:
    {
        strcpy(szSaturation, "Unkmow");
        break;
    }
    case MWCAP_VIDEO_SATURATION_FULL:
    {
        strcpy(szSaturation, "Full");
        break;
    }
    case MWCAP_VIDEO_SATURATION_LIMITED:
    {
        strcpy(szSaturation, "Limited");
        break;
    }
    case MWCAP_VIDEO_SATURATION_EXTENDED_GAMUT:
    {
        strcpy(szSaturation, "Gamut");
        break;
    }
    default:
        break;
    }

    return true;
}

//Print common
static MW_RESULT PrintInputCommon(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    do
    {
        DWORD dwVInputSource, dwVInputType;
        char szVInputType[16];
        mr = MWGetVideoInputSource(hChannel, &dwVInputSource);
        if(mr != MW_SUCCEEDED)
            break;

        dwVInputType = INPUT_TYPE(dwVInputSource);
        GetVideoInputTypeString(dwVInputType, szVInputType);

        DWORD dwAInputSource, dwAInputType;
        char szAInputType[16];
        mr = MWGetAudioInputSource(hChannel, &dwAInputSource);
        if(mr != MW_SUCCEEDED)
            break;

        dwAInputType = INPUT_TYPE(dwAInputSource);
        GetAudioInputTypeString(dwAInputType, szAInputType);

        BOOLEAN bInputSoutceScan;
        mr = MWGetInputSourceScan(hChannel, &bInputSoutceScan);
        if(mr != MW_SUCCEEDED)
            break;

        BOOLEAN bAVLink;
        mr = MWGetAVInputSourceLink(hChannel, &bAVLink);
        if(mr != MW_SUCCEEDED)
            break;


        printf("Input common\n"
               "  Video input ............................ %s\n"
               "  Audio input ............................ %s\n"
               "  Auto scan .............................. %s\n"
               "  AV Link ................................ %s\n\n"
               ,
               szVInputType,szAInputType,
               bInputSoutceScan == true ? "Yes" : "No",
               bAVLink == true ? "Yes" : "No");

    }while (false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}

// print video

static MW_RESULT PrintInputVideo(char *szDevicePath)
{

    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;
    do
    {
        MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
        mr = MWGetVideoSignalStatus(hChannel, &signalStatus);
        if (mr != MW_SUCCEEDED)
            break;

        char szSignalState[16];
        char szColorSpace[16];
        char szQuantization[16];
        char szSaturation[16];
        GetSignalStateString(signalStatus.state, szSignalState);
        GetColorSpaceString(signalStatus.colorFormat, szColorSpace);
        GetQuantizationString(signalStatus.quantRange, szQuantization);
        GetSaturationString(signalStatus.satRange, szSaturation);

        char szScanningType[16];
        float fFrameRate;
        if (signalStatus.bSegmentedFrame)
        {
            strcpy(szScanningType, "sF");
            fFrameRate = 1.0 * 10000000 / signalStatus.dwFrameDuration;
        }
        else if(signalStatus.bInterlaced)
        {
            fFrameRate = 2.0 * 10000000 / signalStatus.dwFrameDuration;
            strcpy(szScanningType, "i");
        }
        else
        {
            fFrameRate = 1.0 * 10000000 / signalStatus.dwFrameDuration;
            strcpy(szScanningType, "p");
        }



        if(mr != MW_SUCCEEDED)
            break;
        if (signalStatus.state == MWCAP_VIDEO_SIGNAL_NONE)
            printf("Input video\n"
                   "  Signal ................................. None\n\n");
        else
            printf("Input video\n"
                   "  Signal state ........................... %s\n"
                   "  Resolution ............................. %dx%d%s %'.2f Hz\n"
                   "  Aspect ................................. %d:%d\n"
                   "  Total size ............................. %dx%d\n"
                   "  X offset ............................... %d\n"
                   "  Y offset ............................... %d\n"
                   "  Color space ............................ %s\n"
                   "  Quantization ........................... %s\n"
                   "  Saturation ............................. %s\n\n"
                   ,
                   szSignalState,
                   signalStatus.cx, signalStatus.cy, szScanningType,
                   fFrameRate,
                   signalStatus.nAspectX, signalStatus.nAspectY,
                   signalStatus.cxTotal, signalStatus.cyTotal,
                   signalStatus.x, signalStatus.y,
                   szColorSpace,
                   szQuantization,
                   szSaturation);

    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}


//print audio
static MW_RESULT PrintInputAudio(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    do
    {
        MWCAP_AUDIO_SIGNAL_STATUS aSignalStatus;
        mr = MWGetAudioSignalStatus(hChannel, & aSignalStatus);
        if (mr != MW_SUCCEEDED)
            break;

        if (aSignalStatus.wChannelValid == 0)
            printf("Input audio\n"
                   "  Signal status .......................... None\n\n");
        else
        {
            printf("Input audio\n"
                   "  Audio format ........................... %d Hz, %d bit%s\n"
                   "  Channel 1 & 2 .......................... %s\n"
                   "  Channel 3 & 4 .......................... %s\n"
                   "  Channel 5 & 6 .......................... %s\n"
                   "  Channel 7 & 8 .......................... %s\n"
                   "  Status data ............................"
                   ,
                   aSignalStatus.dwSampleRate, aSignalStatus.cBitsPerSample, aSignalStatus.bLPCM == true ? ", LPCM" : " ",
                   (aSignalStatus.wChannelValid & 0x01) == 0 ? "Invalid" : "Valid",
                   (aSignalStatus.wChannelValid & (0x01 << 1)) == 0 ? "Invalid" : "Valid",
                   (aSignalStatus.wChannelValid & (0x01 << 2)) == 0 ? "Invalid" : "Valid",
                   (aSignalStatus.wChannelValid & (0x01 << 3)) == 0 ? "Invalid" : "Valid");

            int i;
            for (i = 0; i < 24; i++)
            {
                if (i % 8 == 0 && i != 0)
                    printf("\n                                          ");

                printf(" %02x", aSignalStatus.channelStatus.abyData[i]);
            }

            printf("\n\n");
        }

    }while(false);


    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}


// print common
static void PrintInputSDI(MWCAP_INPUT_SPECIFIC_STATUS specificStatus)
{
    char *szSdiType;
    switch (specificStatus.sdiStatus.sdiType)
    {
    case SDI_TYPE_SD:
        szSdiType = "SD";
        //strcpy(szSdiType, "SD");
        break;
    case SDI_TYPE_HD:
        szSdiType = "HD";
        //strcpy(szSdiType, "HD");
        break;
    case SDI_TYPE_3GA:
        szSdiType = "3G-A";
        //strcpy(szSdiType, "3G-A");
        break;
    case SDI_TYPE_3GB_DL:
        szSdiType = "3G-B dual link";
        //strcpy(szSdiType, "3G-B dual link");
        break;
    case SDI_TYPE_3GB_DS:
        szSdiType = "3G-B dual stream";
        //strcpy(szSdiType, "3G-B dual stream");
        break;
    case SDI_TYPE_DL_CH1:
        szSdiType = "DL link CH1";
        //strcpy(szSdiType, "DL link CH1");
        break;
    case SDI_TYPE_DL_CH2:
        szSdiType = "DL link CH2";
        //strcpy(szSdiType, "DL link CH2");
        break;

    case SDI_TYPE_6G_MODE1:
        szSdiType = "6G Mode 1";
        //strcpy(szSdiType, "DL link CH2");
        break;
    case SDI_TYPE_6G_MODE2:
        szSdiType = "6G Mode 2";
        //strcpy(szSdiType, "DL link CH2");
        break;

    }

    char *szScanningFormat;
    switch (specificStatus.sdiStatus.sdiScanningFormat)
    {
    case SDI_SCANING_INTERLACED:
        szScanningFormat = "Interlaced";
        //strcpy(szScanningFormat, "Interlaced");
        break;
    case SDI_SCANING_SEGMENTED_FRAME:
        szScanningFormat = "Segmented frame";
        //strcpy(szScanningFormat, "Segmented frame");
        break;
    case SDI_SCANING_PROGRESSIVE:
        szScanningFormat = "Progressive";
        //strcpy(szScanningFormat, "Progressive");
        break;
    }


    char *szBitDepth;
    switch (specificStatus.sdiStatus.sdiBitDepth)
    {
    case SDI_BIT_DEPTH_8BIT:
        szBitDepth = "8 Bits";
        //strcpy(szBitDepth, "8 Bits");
        break;
    case SDI_BIT_DEPTH_10BIT:
        szBitDepth = "10 Bits";
        //strcpy(szBitDepth, "10 Bits");
        break;
    case SDI_BIT_DEPTH_12BIT:
        szBitDepth = "12 Bits";
        //strcpy(szBitDepth, "12 Bits");
        break;
    }

    char *szSamplingStruct;
    switch (specificStatus.sdiStatus.sdiSamplingStruct)
    {
    case SDI_SAMPLING_422_YCbCr:
        szSamplingStruct = "Y/Cb/Cr, 4:2:2";
        //strcpy(szSamplingStruct, "Y/Cb/Cr, 4:2:2");
        break;
    case SDI_SAMPLING_444_YCbCr:
        szSamplingStruct = "Y/Cb/Cr, 4:4:4";
        //strcpy(szSamplingStruct, "Y/Cb/Cr, 4:4:4");
        break;
    case SDI_SAMPLING_444_RGB:
        szSamplingStruct = "R/G/B, 4:4:4";
        //strcpy(szSamplingStruct, "R/G/B, 4:4:4");
        break;
    case SDI_SAMPLING_420_YCbCr:
        szSamplingStruct = "Y/Cb/Cr, 4:2:2";
        //strcpy(szSamplingStruct, "Y/Cb/Cr, 4:2:2");
        break;
    case SDI_SAMPLING_4224_YCbCrA:
        szSamplingStruct = "Y/Cb/Cr/A, 4:2:2:4";
        //strcpy(szSamplingStruct, "Y/Cb/Cr/A, 4:2:2:4");
        break;
    case SDI_SAMPLING_4444_YCbCrA:
        szSamplingStruct = "Y/Cb/Cr/A, 4:4:4:4";
        //strcpy(szSamplingStruct, "Y/Cb/Cr/A, 4:4:4:4");
        break;
    case SDI_SAMPLING_4444_RGBA:
        szSamplingStruct = "R/G/B/A, 4:4:4:4";
        //strcpy(szSamplingStruct, "R/G/B/A, 4:4:4:4");
        break;
    case SDI_SAMPLING_4224_YCbCrD:
        szSamplingStruct = "Y/Cb/Cr/D, 4:2:2:4";
        //strcpy(szSamplingStruct, "Y/Cb/Cr/D, 4:2:2:4");
        break;
    case SDI_SAMPLING_4444_YCbCrD:
        szSamplingStruct = "Y/Cb/Cr/A, 4:4:4:4";
        //strcpy(szSamplingStruct, "Y/Cb/Cr/A, 4:4:4:4");
        break;
    case SDI_SAMPLING_4444_RGBD:
        szSamplingStruct = "R/G/B/D, 4:4:4:4";
        //strcpy(szSamplingStruct, "R/G/B/D, 4:4:4:4");
        break;
    case SDI_SAMPLING_444_XYZ:
        szSamplingStruct = "X/Y/Z, 4:4:4";
        //strcpy(szSamplingStruct, "X/Y/Z, 4:4:4");
        break;
    }

    printf("Input specific\n"
           "  Signal status .......................... Valid\n"
           "  Type ................................... %s\n"
           "  Scanning type .......................... %s\n"
           "  Color depth ............................ %s\n"
           "  Sampling struct ........................ %s\n",
           szSdiType, szScanningFormat, szBitDepth, szSamplingStruct);

    if (specificStatus.sdiStatus.bST352DataValid)
        printf("  ST352 payloadID ........................ %d\n",
               specificStatus.sdiStatus.dwST352Data);

    printf("\n");
}

static void PrintInputHDMI(MWCAP_INPUT_SPECIFIC_STATUS specificStatus)
{
    char *szPixelEncoding;
    switch (specificStatus.hdmiStatus.pixelEncoding)
    {
    case HDMI_ENCODING_RGB_444:
        szPixelEncoding = "R/G/B 4:4:4";
        //strcpy(szPixelEncoding, "R/G/B 4:4:4");
        break;
    case HDMI_ENCODING_YUV_422:
        szPixelEncoding = "Y/U/V 4:2:2";
        //strcpy(szPixelEncoding, "Y/U/V 4:2:2");
        break;
    case HDMI_ENCODING_YUV_444:
        szPixelEncoding = "Y/U/V 4:4:4";
        //strcpy(szPixelEncoding, "Y/U/V 4:4:4");
        break;
    case HDMI_ENCODING_YUV_420:
        szPixelEncoding = "Y/U/V 4:2:0";
        //strcpy(szPixelEncoding, "Y/U/V 4:2:0");
        break;
    }

    printf("Input specific\n"
           "  Signal status .......................... Valid\n"
           "  Mode ................................... %s\n"
           "  HDCP ................................... %s\n"
           "  Color depth ............................ %d bits\n"
           "  Pixel encoding ......................... %s\n"
           "  VIC .................................... %d\n"
           "  IT content ............................. %s\n"
           "  Timing - Scanning format ............... %s\n"
           "  Timing - Frame rate .................... %.2f\n"
           "  Timing - H Total ....................... %d\n"
           "  Timing - H Active ...................... %d\n"
           "  Timing - H Front porch ................. %d\n"
           "  Timing - H Sync width .................. %d\n"
           "  Timing - H back porch .................. %d\n"
           "  Timing - H field 0 V total ............. %d\n"
           "  Timing - H field 0 V active ............ %d\n"
           "  Timing - H field 0 V front porch ....... %d\n"
           "  Timing - H field 0 V sync width ........ %d\n"
           "  Timing - H field 0 V back porch ........ %d\n"
           "  Timing - H field 1 V total ............. %d\n"
           "  Timing - H field 1 V active ............ %d\n"
           "  Timing - H field 1 V front porch ....... %d\n"
           "  Timing - H field 1 V sync width ........ %d\n"
           "  Timing - H field 1 V back porch ........ %d\n\n"
           ,
           specificStatus.hdmiStatus.bHDMIMode ? "HDMI" : "DVI",
           specificStatus.hdmiStatus.bHDCP ? "Yes" : "No",
           specificStatus.hdmiStatus.byBitDepth,
           szPixelEncoding,
           specificStatus.hdmiStatus.byVIC,
           specificStatus.hdmiStatus.bITContent ? "True" : "False",
           specificStatus.hdmiStatus.videoTiming.bInterlaced ? "Interlaced" : "Progressive",
           1.0 * 10000000 / specificStatus.hdmiStatus.videoTiming.dwFrameDuration,
           specificStatus.hdmiStatus.videoTiming.wHTotalWidth,
           specificStatus.hdmiStatus.videoTiming.wHActive,
           specificStatus.hdmiStatus.videoTiming.wHFrontPorch,
           specificStatus.hdmiStatus.videoTiming.wHSyncWidth,
           specificStatus.hdmiStatus.videoTiming.wHBackPorch,
           specificStatus.hdmiStatus.videoTiming.wField0VTotalHeight,
           specificStatus.hdmiStatus.videoTiming.wField0VActive,
           specificStatus.hdmiStatus.videoTiming.wField0VFrontPorch,
           specificStatus.hdmiStatus.videoTiming.wField0VSyncWidth,
           specificStatus.hdmiStatus.videoTiming.wField0VBackPorch,
           specificStatus.hdmiStatus.videoTiming.wField1VTotalHeight,
           specificStatus.hdmiStatus.videoTiming.wField1VActive,
           specificStatus.hdmiStatus.videoTiming.wField1VFrontPorch,
           specificStatus.hdmiStatus.videoTiming.wField1VSyncWidth,
           specificStatus.hdmiStatus.videoTiming.wField1VBackPorch
           );
}

static void PrintInputVgaComponent(MWCAP_INPUT_SPECIFIC_STATUS specificStatus)
{
    char *szSynctype;
    switch (specificStatus.vgaComponentStatus.syncInfo.bySyncType)
    {
    case VIDEO_SYNC_ALL:
        szSynctype = "All";
        //strcpy(szSynctype, "All");
        break;
    case VIDEO_SYNC_HS_VS:
        szSynctype = "+HS +VS";
        //strcpy(szSynctype, "+HS +VS");
        break;
    case VIDEO_SYNC_CS:
        szSynctype = "CS";
        //strcpy(szSynctype, "+CS");
        break;
    case VIDEO_SYNC_EMBEDDED:
        szSynctype = "Embedded Sync";
        //strcpy(szSynctype, "Embedded");
        break;
    }


    char *szTimingType;
    switch (specificStatus.vgaComponentStatus.videoTiming.dwType)
    {
    case MWCAP_VIDEO_TIMING_NONE:
        szTimingType = "None";
        //strcpy(szTimingType, "None");
        break;
    case MWCAP_VIDEO_TIMING_LEGACY:
        szTimingType = "Legacy";
        //strcpy(szTimingType, "Legacy");
        break;
    case MWCAP_VIDEO_TIMING_DMT:
        szTimingType = "DMT";
        //strcpy(szTimingType, "DMT");
        break;
    case MWCAP_VIDEO_TIMING_CEA:
        szTimingType = "CEA";
        //strcpy(szTimingType, "CEA");
        break;
    case MWCAP_VIDEO_TIMING_GTF:
        szTimingType = "GTF";
        //strcpy(szTimingType, "GTF");
        break;
    case MWCAP_VIDEO_TIMING_CVT:
        szTimingType = "CVT";
        //strcpy(szTimingType, "CVT");
        break;
    case MWCAP_VIDEO_TIMING_CVT_RB:
        szTimingType = "CVT_RB";
        //strcpy(szTimingType, "CVT_RB");
        break;
    case MWCAP_VIDEO_TIMING_FAILSAFE:
        szTimingType = "Failsafe";
        //strcpy(szTimingType, "Failsafe");
        break;
    }


    printf("Input specific\n"
           "  Signal status .......................... Valid\n"
           "  Sync type .............................. %s\n"
           "  Frame rate ............................. %.2f Hz\n"
           "  Scanning format ........................ %s\n"
           "  VS width ............................... %d\n"
           "  Total scan lines ....................... %d\n"
           ,
           szSynctype,
           1.0 * 10000000 / specificStatus.vgaComponentStatus.syncInfo.dwFrameDuration,
           specificStatus.vgaComponentStatus.syncInfo.bInterlaced ? "Interlaced" : "Progressive",
           specificStatus.vgaComponentStatus.syncInfo.wVSyncLineCount,
           specificStatus.vgaComponentStatus.syncInfo.wFrameLineCount
           );

    if (specificStatus.vgaComponentStatus.syncInfo.bySyncType == VIDEO_SYNC_EMBEDDED)
        printf("  Emb sync ............................... %s\n",
               specificStatus.vgaComponentStatus.bTriLevelSync ? "Tri-Levle" : "Bi-Level");


    printf("  Timing - Type .......................... %s\n"
           "  Timing - Pixel clock ................... %d MHz\n"
           "  Timing - H Active ...................... %d\n"
           "  Timing - H Front porch ................. %d\n"
           "  Timing - H Sync width .................. %d\n"
           "  Timing - H Back proch .................. %d\n"
           "  Timing - V Active ...................... %d\n"
           "  Timing - V Front porch ................. %d\n"
           "  Timing - V Sync width  ................. %d\n"
           "  Timing - V back proch .................. %d\n\n",
           szTimingType,
           specificStatus.vgaComponentStatus.videoTiming.dwPixelClock/1000/1000,
           specificStatus.vgaComponentStatus.videoTiming.wHActive,
           specificStatus.vgaComponentStatus.videoTiming.wHFrontPorch,
           specificStatus.vgaComponentStatus.videoTiming.wHSyncWidth,
           specificStatus.vgaComponentStatus.videoTiming.wHBackPorch,
           specificStatus.vgaComponentStatus.videoTiming.wVActive,
           specificStatus.vgaComponentStatus.videoTiming.wVFrontPorch,
           specificStatus.vgaComponentStatus.videoTiming.wVSyncWidth,
           specificStatus.vgaComponentStatus.videoTiming.wVBackPorch
           );


}

static void PrintInputCvbsYc(MWCAP_INPUT_SPECIFIC_STATUS specificStatus)
{
    char *szStandard;
    switch (specificStatus.cvbsYcStatus.standard)
    {
    case MWCAP_SD_VIDEO_NONE:
        szStandard = "None";
        //strcpy(szStandard, "None");
        break;
    case MWCAP_SD_VIDEO_NTSC_M:
        szStandard = "NTSC-M";
        //strcpy(szStandard, "NTSC-M");
        break;
    case MWCAP_SD_VIDEO_NTSC_433:
        szStandard = "NTSC-433";
        //strcpy(szStandard, "NTSC-433");
        break;
    case MWCAP_SD_VIDEO_PAL_M:
        szStandard = "PAL-m";
        //strcpy(szStandard, "PAL-m");
        break;
    case MWCAP_SD_VIDEO_PAL_60:
        szStandard = "PAL-60";
        //strcpy(szStandard, "PAL-60");
        break;
    case MWCAP_SD_VIDEO_PAL_COMBN:
        szStandard = "PAL-COMBN";
        //strcpy(szStandard, "PAL-COMBN");
        break;
    case MWCAP_SD_VIDEO_PAL_BGHID:
        szStandard = "PAL-BGHID";
        //strcpy(szStandard, "PAL-BGHID");
        break;
    case MWCAP_SD_VIDEO_SECAM:
        szStandard = "SECAM";
        //strcpy(szStandard, "SECAM");
        break;
    case MWCAP_SD_VIDEO_SECAM_60:
        szStandard = "SECAM-60";
        //strcpy(szStandard, "SECAM-60");
        break;
    }

    printf("Input specific\n"
           "  Signal status .......................... Valid\n"
           "  TV standard ............................ %s\n"
           ,
           szStandard
           );

    if (specificStatus.cvbsYcStatus.b50Hz)
        printf("  Field .................................. 50 Hz\n\n");
    else
        printf("\n");
}


static MW_RESULT PrintInputSpecific(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    do
    {
        MWCAP_INPUT_SPECIFIC_STATUS specificStatus;
        mr = MWGetInputSpecificStatus(hChannel, &specificStatus);
        if (mr != MW_SUCCEEDED)
            break;

        if (specificStatus.bValid == false)
            printf("Input specific\n"
                   "  Signal status .......................... None\n\n");
        else
        {
            switch (specificStatus.dwVideoInputType) {
            case MWCAP_VIDEO_INPUT_TYPE_SDI:
            {
                PrintInputSDI(specificStatus);
            }
                break;
            case MWCAP_VIDEO_INPUT_TYPE_HDMI:
            {
                PrintInputHDMI(specificStatus);
            }
                break;
            case MWCAP_VIDEO_INPUT_TYPE_VGA:
            {
                PrintInputVgaComponent(specificStatus);
            }
                break;
            case MWCAP_VIDEO_INPUT_TYPE_COMPONENT:
            {
                PrintInputVgaComponent(specificStatus);
            }
                break;
            case MWCAP_VIDEO_INPUT_TYPE_CVBS:
            {
                PrintInputCvbsYc(specificStatus);
            }
                break;
            case MWCAP_VIDEO_INPUT_TYPE_YC:
            {
                PrintInputCvbsYc(specificStatus);
            }

            default:
                break;
            }
        }

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}

//HDMI infoframe
static bool GetHdmiInfoframeNameByID(MWCAP_HDMI_INFOFRAME_ID infoframeID, char *szInfoframeID)
{
    if (szInfoframeID == NULL || infoframeID >= MWCAP_HDMI_INFOFRAME_COUNT)
        return false;

    switch(infoframeID)
    {
    case MWCAP_HDMI_INFOFRAME_ID_AVI:
    {
        strcpy(szInfoframeID, "AVI");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_AUDIO:
    {
        strcpy(szInfoframeID, "AUDIO");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_SPD:
    {
        strcpy(szInfoframeID, "SPD");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_MS:
    {
        strcpy(szInfoframeID, "MS");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_VS:
    {
        strcpy(szInfoframeID, "VS");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_ACP:
    {
        strcpy(szInfoframeID, "ACP");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_ISRC1:
    {
        strcpy(szInfoframeID, "ISRC1");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_ISRC2:
    {
        strcpy(szInfoframeID, "ISRC2");
    }
        break;
    case MWCAP_HDMI_INFOFRAME_ID_GAMUT:
    {
        strcpy(szInfoframeID, "GAMUT");
    }
        break;
    default:
        break;
    }

    return true;
}


static void PrintHdmiInfoframeByID(MWCAP_HDMI_INFOFRAME_ID infoframeID, HDMI_INFOFRAME_PACKET infoframePacket)
{
    char szinfoframeName[16];
    GetHdmiInfoframeNameByID(infoframeID, szinfoframeName);

    printf("HDMI information frame - %s\n"
           "  Type ................................... 0x%02x\n"
           "  Version ................................ 0x%02x\n"
           "  Length ................................. 0x%02x\n"
           "  Checksum ............................... 0x%02x (%s)\n"
           "  Data ..................................."
           ,
           szinfoframeName,
           infoframePacket.header.byPacketType,
           infoframePacket.header.byVersion,
           infoframePacket.header.byLength,
           infoframePacket.byChecksum,
           hdmi_info_IsValid(&infoframePacket) == 1 ? "Ok" : "Failed");

    for (int i = 0; i < infoframePacket.header.byLength; i++)
    {
        if (i % 8 == 0 && i != 0)
            printf("\n                                          ");

        printf(" %02x", infoframePacket.abyPayload[i]);
    }

    printf("\n\n");
}

static MW_RESULT PrintHdmiInfoframe(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    do
    {
        DWORD dwHdmiValidFlag = 0;
        mr = MWGetHDMIInfoFrameValidFlag(hChannel, &dwHdmiValidFlag);
        if (mr != MW_SUCCEEDED)
            break;

        for(int i = 0; i < MWCAP_HDMI_INFOFRAME_COUNT; i++)
        {
            char szinfoframeName[16];
            GetHdmiInfoframeNameByID(i, szinfoframeName);

            if (((0x01 << i) & dwHdmiValidFlag) == 0)
                printf("HDMI information frame -- %s\n"
                       "  N/A\n\n", szinfoframeName);
            else
            {
                HDMI_INFOFRAME_PACKET infoframePacket;
                mr = MWGetHDMIInfoFramePacket(hChannel, i, &infoframePacket);
                if (mr != MW_SUCCEEDED)
                    continue;

                PrintHdmiInfoframeByID(i, infoframePacket);
            }
        }
    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}

// device

static MW_RESULT PrintDeviceInfo(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return MW_FAILED;

    MW_RESULT mr;

    HCHANNEL hChannel = -1;
    hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return MW_FAILED;

    do
    {
        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfo(hChannel, &channelInfo);
        if (mr != MW_SUCCEEDED)
            break;

        MWCAP_PRO_CAPTURE_INFO proCaptureInfo;
        mr = MWGetFamilyInfo(hChannel, &proCaptureInfo, sizeof(MWCAP_PRO_CAPTURE_INFO));
        if (mr != MW_SUCCEEDED)
            break;

        MWCAP_VIDEO_CAPS videoCaps;
        mr = MWGetVideoCaps(hChannel, &videoCaps);
        if (mr != MW_SUCCEEDED)
            break;

        unsigned int nTemp;
        mr = MWGetTemperature(hChannel, &nTemp);
        if (mr != MW_SUCCEEDED)
            break;


        printf("Device\n"
               "  Family name ............................ %s\n"
               "  Product name ........................... %s\n"
               "  Firmware name .......................... %s\n"
               "  Serial number .......................... %s\n"
               "  Hardware version ....................... %c\n"
               "  Firmware version ....................... %d.%d\n"
               "  Driver version ......................... %d.%d.%d\n"
               "  Board ID ............................... %d\n"
               "  Channel ID ............................. %d\n"
               "  Bus address ............................ bus %d, device %d\n"
               "  PCIe speed ............................. gen %d\n"
               "  PCIe width ............................. x%d\n"
               "  Max playload size ...................... %d Bytes\n"
               "  Max read request szie .................. %d Bytes\n"
               "  Total memory size ...................... %d MB\n"
               "  Free memory size ....................... %d MB\n"
               "  Max input resolution ................... %dx%d\n"
               "  Max output resolution .................. %dx%d\n"
               "  Chipset temperature .................... %.1fºC\n\n"
               ,
               channelInfo.szFamilyName,
               channelInfo.szProductName,
               channelInfo.szFirmwareName,
               channelInfo.szBoardSerialNo,
               channelInfo.chHardwareVersion,
               (channelInfo.dwFirmwareVersion >> 16) & 0xFFFF, (channelInfo.dwFirmwareVersion) & 0xFFFF,
               (channelInfo.dwDriverVersion >> 24) & 0xFF, (channelInfo.dwDriverVersion >> 16) & 0xFF, (channelInfo.dwDriverVersion) & 0xFFFF,
               channelInfo.byBoardIndex,
               channelInfo.byChannelIndex,
               proCaptureInfo.byPCIBusID, proCaptureInfo.byPCIDevID,
               proCaptureInfo.byLinkType,
               proCaptureInfo.byLinkWidth,
               proCaptureInfo.wMaxPayloadSize,
               proCaptureInfo.wMaxReadRequestSize,
               proCaptureInfo.cbTotalMemorySize/1024/1024,
               proCaptureInfo.cbFreeMemorySize/1024/1024,
               videoCaps.wMaxInputWidth, videoCaps.wMaxInputHeight,
               videoCaps.wMaxOutputWidth, videoCaps.wMaxOutputHeight,
               1.0*nTemp/10);

    }while(false);

    if (hChannel != -1)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    return mr;
}



#endif // DEVICEPROPERTY_H
