#include "mwcapture.h"

MWCapture::MWCapture()
{
    m_hChannel = -1;
    m_hNotify = 0;

    m_hCaptureEvent = 0;
    m_hNotifyEvent = 0;

    m_cx = 1024;
    m_cy = 720;
    m_dwFourcc = MWFOURCC_YUY2;

    m_dwMinStride = 0;
    m_dwImageSize = 0;
    m_pbImage = 0;

    m_bLowLatency = FALSE;

    m_nChannelCount = MWGetChannelCount();
    m_nChannelIndex = -1;

    m_threadID = 0;
    m_hExitEvent = 0;
    m_bCapture = FALSE;

    m_nWinID = 0;

    m_llTotalLatency = 0;
    m_llCaptureLatency = 0;
    m_llFrameCount = 0;

    m_pCallback = NULL;
}

MWCapture::~MWCapture()
{
    CloseChannel();

}

BOOL MWCapture::OpenChannel(int nIndex, int cx, int cy, DWORD dwFourcc, char bLowLatency, int nWinID)
{
    Q_UNUSED(nWinID);
    if (nIndex >= m_nChannelCount)
        return false;

    if (m_hChannel != -1 && nIndex == m_nChannelIndex)
        return TRUE;

    if (m_hChannel != -1 && nIndex != m_nChannelIndex)
    {
        CloseChannel();
    }

    m_cx = cx;
    m_cy = cy;
    m_dwFourcc = dwFourcc;
    m_bLowLatency = bLowLatency;


    MWCAP_CHANNEL_INFO channelInfo;
    if (MW_SUCCEEDED != MWGetChannelInfoByIndex(nIndex, &channelInfo))
        return FALSE;

    m_hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
    if (m_hChannel == -1)
        return FALSE;

    if (StartCapture() != TRUE)
        return FALSE;

    m_nChannelIndex = nIndex;

    return TRUE;
}

void MWCapture::CloseChannel()
{
    StopCapture();

    if (m_hChannel != -1)
    {
        MWCloseChannel(m_hChannel);
        m_hChannel = -1;
        m_nChannelIndex = -1;
    }
}


void MWCapture::GetDeviceNameByIndex(int nIndex, char *pDeviceName)
{
    MW_RESULT mr;

    if (nIndex >= m_nChannelCount)
        return;

    MWCAP_CHANNEL_INFO channelInfo;
    mr = MWGetChannelInfoByIndex(nIndex, &channelInfo);
    if(mr != MW_SUCCEEDED)
        return;

    sprintf(pDeviceName, "%02d:%02d %s", channelInfo.byBoardIndex, channelInfo.byChannelIndex, channelInfo.szProductName);

    return;
}


int MWCapture::GetChannelCount()
{
    return m_nChannelCount;
}

void MWCapture::SetCallback(ICallback *pCallback)
{
    if (pCallback != NULL)
        m_pCallback = pCallback;

    return;
}

BOOL MWCapture::GetVideoSignalStatus(MWCAP_VIDEO_SIGNAL_STATUS *pStatus)
{
    if (m_hChannel == -1 || pStatus == NULL)
        return FALSE;

    MW_RESULT mr ;
    mr = MWGetVideoSignalStatus(m_hChannel, pStatus);

    return (mr == MW_SUCCEEDED);
}

void MWCapture::GetStatistics(long long *pllCaptureLatency, long long *pllTotalLatency, long long *pllFrameCount)
{
    *pllCaptureLatency = m_llCaptureLatency;
    *pllTotalLatency = m_llTotalLatency;
    *pllFrameCount = m_llFrameCount;

    return;
}

// private
BOOL MWCapture::StartCapture()
{
    if (m_hChannel == -1)
        return FALSE;

    m_hCaptureEvent = MWCreateEvent();
    if(m_hCaptureEvent == 0)
        return FALSE;

    if (MWStartVideoCapture(m_hChannel, m_hCaptureEvent) != MW_SUCCEEDED)
        return FALSE;

    m_hNotifyEvent = MWCreateEvent();
    if(m_hNotifyEvent == 0)
        return FALSE;

    if (!m_bLowLatency)
        m_hNotify = MWRegisterNotify(m_hChannel, m_hNotifyEvent,
                                    MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED | MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE);
    else
        m_hNotify = MWRegisterNotify(m_hChannel, m_hNotifyEvent,
                                    MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING | MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE);

    if (m_hNotify == 0)
        return FALSE;

    if (m_pbImage != 0)
    {
        free((void *)(unsigned long)m_pbImage);
        m_pbImage = 0;
    }

    m_dwMinStride = FOURCC_CalcMinStride(m_dwFourcc, m_cx, 4);
    m_dwImageSize = FOURCC_CalcImageSize(m_dwFourcc, m_cx, m_cy, m_dwMinStride);
    m_pbImage = (MWCAP_PTR)malloc(m_dwImageSize);


    m_hExitEvent = MWCreateEvent();
    if(m_hExitEvent == 0)
        return FALSE;

    if (pthread_create(&m_threadID, NULL, VideoCaptureProc, this) != 0)
        return FALSE;

    m_bCapture = TRUE;

    return TRUE;
}

void MWCapture::StopCapture()
{
    if (m_bCapture == FALSE || m_hExitEvent == 0 || m_hChannel == 0)
        return;

    MWSetEvent(m_hExitEvent);
    pthread_join(m_threadID, NULL);

    if(m_hExitEvent != 0)
    {
        MWCloseEvent(m_hExitEvent);
        m_hExitEvent = 0;
    }

    if (m_hNotify != 0)
    {
        MWUnregisterNotify(m_hChannel, m_hNotify);
        m_hNotify = 0;
    }

    if(m_hNotifyEvent != 0)
    {
        MWCloseEvent(m_hNotifyEvent);
        m_hNotifyEvent = 0;
    }

    MWStopVideoCapture(m_hChannel);

    if(m_hCaptureEvent != 0)
    {
        MWCloseEvent(m_hCaptureEvent);
        m_hCaptureEvent = 0;
    }

    if (m_pbImage != 0)
    {
        free((void *)(unsigned long)m_pbImage);
        m_pbImage = 0;
    }

    m_bCapture = FALSE;
}

void MWCapture::VideoCaptureProcInternel()
{
    MW_RESULT mr;
    DWORD dwRet;
    LONGLONG llStartTime = 0;
    LONGLONG llEndTime = 0;

    m_llTotalLatency = 0;
    m_llCaptureLatency = 0;
    m_llFrameCount = 0;


    while(true)
    {
        MWCAP_PTR hEvents[] = {m_hExitEvent, m_hNotifyEvent};
        dwRet = MWMultiWaitEvent(hEvents, 2, 500);

        if (dwRet == 0)
        {
            mr = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, m_pbImage, m_dwImageSize, m_dwMinStride,
                                                       FOURCC_IsRGB(m_dwFourcc),
                                                       0, m_dwFourcc, m_cx, m_cy,
                                                       0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                       MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
            if (mr != MW_SUCCEEDED)
                break;

            MWCAP_VIDEO_CAPTURE_STATUS capStatus;
            do
            {
                MWWaitEvent(m_hCaptureEvent, -1);
                mr = MWGetVideoCaptureStatus(m_hChannel, &capStatus);
            } while(mr == MW_SUCCEEDED && capStatus.bFrameCompleted == FALSE);

            if (m_pCallback != NULL)
                m_pCallback->VideoCaptureCallback(m_pbImage, m_dwImageSize, NULL);

        }

        if (dwRet & 0x01)
        {
            break;
        }

        if (dwRet & (0x01 << 1))
        {
            ULONGLONG llStatus;
            mr = MWGetNotifyStatus(m_hChannel, m_hNotify, &llStatus);
            if (mr != MW_SUCCEEDED)
                break;

            if (llStatus & MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED)
            {
                MWCAP_VIDEO_BUFFER_INFO bufferInfo;
                mr = MWGetVideoBufferInfo(m_hChannel, &bufferInfo);
                if (mr != MW_SUCCEEDED)
                    break;

                MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
                mr = MWGetVideoFrameInfo(m_hChannel, bufferInfo.iNewestBuffered,&videoFrameInfo);
                if (mr != MW_SUCCEEDED)
                    break;

                mr = MWGetDeviceTime(m_hChannel, &llStartTime);

                mr = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, bufferInfo.iNewestBuffered, m_pbImage, m_dwImageSize, m_dwMinStride,
                                                           FOURCC_IsRGB(m_dwFourcc),
                                                           0, m_dwFourcc, m_cx, m_cy,
                                                           0, 0, 0, 0, 0, 100, 0, 100, 0,
                                                           MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                           MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
                if (mr != MW_SUCCEEDED)
                    break;

                MWCAP_VIDEO_CAPTURE_STATUS capStatus;
                do
                {
                    MWWaitEvent(m_hCaptureEvent, -1);
                    mr = MWGetVideoCaptureStatus(m_hChannel, &capStatus);
                } while(mr == MW_SUCCEEDED && capStatus.bFrameCompleted == FALSE);

                mr = MWGetDeviceTime(m_hChannel, &llEndTime);
                m_llCaptureLatency += (llEndTime - llStartTime);
                m_llTotalLatency += (llEndTime - videoFrameInfo.allFieldStartTimes[0]);
                m_llFrameCount++;

                if (m_pCallback != NULL)
                    m_pCallback->VideoCaptureCallback(m_pbImage, m_dwImageSize, NULL);

            }
            else if (llStatus & MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING)
            {
                MWCAP_VIDEO_BUFFER_INFO bufferInfo;
                mr = MWGetVideoBufferInfo(m_hChannel, &bufferInfo);
                if (mr != MW_SUCCEEDED)
                    break;

                MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
                mr = MWGetVideoFrameInfo(m_hChannel, bufferInfo.iNewestBuffering,&videoFrameInfo);
                if (mr != MW_SUCCEEDED)
                    break;

                mr = MWGetDeviceTime(m_hChannel, &llStartTime);

                mr = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, bufferInfo.iNewestBuffering, m_pbImage, m_dwImageSize, m_dwMinStride,
                                                           FOURCC_IsRGB(m_dwFourcc),
                                                           0, m_dwFourcc, m_cx, m_cy,
                                                           0, NUM_PARTIAL_LINE, 0, 0, 0, 100, 0, 100, 0,
                                                           MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, 0, 0, 0, 0,
                                                           MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);

                if (mr != MW_SUCCEEDED)
                    break;

                MWCAP_VIDEO_CAPTURE_STATUS capStatus;
                do
                {
                    MWWaitEvent(m_hCaptureEvent, -1);
                    mr = MWGetVideoCaptureStatus(m_hChannel, &capStatus);
                } while(mr == MW_SUCCEEDED && capStatus.bFrameCompleted == FALSE);

                mr = MWGetDeviceTime(m_hChannel, &llEndTime);
                m_llCaptureLatency += (llEndTime - llStartTime);
                m_llTotalLatency += (llEndTime - videoFrameInfo.allFieldStartTimes[0]);
                m_llFrameCount++;

                if (m_pCallback != NULL)
                    m_pCallback->VideoCaptureCallback(m_pbImage, m_dwImageSize, NULL);

            }
            else if (llStatus & MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE)
            {
                if (m_pCallback != NULL)
                    m_pCallback->VideoSignalChangedCallback();

                m_llCaptureLatency = 0;
                m_llTotalLatency = 0;
                m_llFrameCount = 0;
            }

        }
    }
}



