#ifndef MWCAPTURE_H
#define MWCAPTURE_H

#define NUM_PARTIAL_LINE 128
#ifdef __cplusplus
extern "C" {
#endif

#include <pthread.h>
#include "lib-mw-capture.h"
#include "mw-fourcc.h"

#ifdef __cplusplus
}
#endif

#include <QtCore>

class ICallback
{
public:
    virtual void VideoCaptureCallback(MWCAP_PTR pbImage, DWORD cbImage, void *pvParam) = 0;
    virtual void VideoSignalChangedCallback() = 0;
};


class MWCapture
{
public:
    MWCapture();
    ~MWCapture();

    static BOOL Init()
    {
        return MWCaptureInitInstance();
    }

    static BOOL RefreshDevices()
    {
        return MWRefreshDevice() ==  MW_SUCCEEDED;
    }
    static void Exit()
    {
        MWCaptureExitInstance();
    }

    BOOL OpenChannel(int nIndex, int cx, int cy, DWORD dwFourcc, BOOL bLowLatency, int nWinID);
    void CloseChannel();

    int GetChannelCount();
    void GetDeviceNameByIndex(int nIndex, char *pDeviceName);

    void SetCallback(ICallback *pCallback);

    BOOL GetVideoSignalStatus(MWCAP_VIDEO_SIGNAL_STATUS *pStatus);
    void GetStatistics(LONGLONG *pllCaptureLatency, LONGLONG *pllTotalLatency, LONGLONG *pllFrameCount);
private:
    BOOL StartCapture();
    void StopCapture();

    static void *VideoCaptureProc(void *pvParam)
    {
        MWCapture *pThis = (MWCapture *)pvParam;
        pThis->VideoCaptureProcInternel();

        return NULL;
    }
    void VideoCaptureProcInternel();


private:
    HCHANNEL    m_hChannel;
    HNOTIFY     m_hNotify;

    MWCAP_PTR   m_hCaptureEvent;
    MWCAP_PTR   m_hNotifyEvent;

    int         m_cx;
    int         m_cy;
    DWORD       m_dwFourcc;

    DWORD       m_dwMinStride;
    DWORD       m_dwImageSize;
    MWCAP_PTR   m_pbImage;

    BOOL        m_bLowLatency;

    int         m_nChannelCount;
    int         m_nChannelIndex;

    pthread_t   m_threadID;
    MWCAP_PTR   m_hExitEvent;
    BOOL        m_bCapture;

    int         m_nWinID;

    LONGLONG    m_llTotalLatency;
    LONGLONG    m_llCaptureLatency;
    LONGLONG    m_llFrameCount;

    ICallback   *m_pCallback;

};

#endif // MWCAPTURE_H
