
#include <stdio.h>
#include <stdlib.h>
#include "ProductVer.h"
#include "lib-mw-capture.h"
#include "mw-fourcc.h"

#define NUM_PARTIAL_LINE 128
#define MAX_CAPTURE_FRAME_NUM 100

int main(void)
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    BYTE nMajor = 0;
    BYTE nMinor = 0;
    WORD nBuild = 0;
    MWGetVersion(&nMajor, &nMinor, &nBuild);
    printf("MWCapture SDK for Linux V%d.%d.%d - low-latency\n", nMajor, nMinor, nBuild);

    HCHANNEL hChannel = -1;
    MWCAP_PTR hCaptureEvent = 0;
    MWCAP_PTR hNotifyEvent = 0;
    HNOTIFY hNotify = 0;

    LONGLONG llTotalTime = 0;

    MWCAP_PTR pbImage = 0;

    int nRet;

    MW_RESULT mr;
    do
    {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("Error: No Pro Capture device is found\n");
            break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
            printf("Error: Get channel info error\n");
            break;
        }

        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1)
        {
            printf("Error: Open channel error\n");
            break;
        }

        printf("\nCapture channel info \n"
               "   %-13s : %d\n"
               "   %-13s : %d\n"
               "   %-13s : %s\n"
               "   %-13s : %s\n\n",
               "Board Index", channelInfo.byBoardIndex,
               "Channel Index", channelInfo.byChannelIndex,
               "Product Name", channelInfo.szProductName,
               "Serial Number", channelInfo.szBoardSerialNo);

		// capture image
        int cx = 1920;
        int cy = 1080;
        DWORD dwFourCC = MWFOURCC_BGR24;
        DWORD dwMinStride = FOURCC_CalcMinStride(dwFourCC, cx, 4);
        DWORD dwImageSize = FOURCC_CalcImageSize(dwFourCC, cx, cy, dwMinStride);
        pbImage = (MWCAP_PTR)(unsigned long)malloc(dwImageSize);

        hCaptureEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
        {
            printf("Error: Create capture event error\n");
            break;
        }

        hNotifyEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
        {
            printf("Error: Create notify event error\n");
            break;
        }
        
        mr = MWStartVideoCapture(hChannel, hCaptureEvent);
        if (mr != MW_SUCCEEDED) {
            printf("Error: Open Video Capture error!\n");
        }
        else {
            MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
            MWGetVideoBufferInfo(hChannel, &videoBufferInfo);

            MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
            MWGetVideoFrameInfo(hChannel, videoBufferInfo.iNewestBufferedFullFrame, &videoFrameInfo);

            MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
            MWGetVideoSignalStatus(hChannel, &videoSignalStatus);

            switch (videoSignalStatus.state) {
            case MWCAP_VIDEO_SIGNAL_NONE:
                printf("Input signal status: NONE\n");
                break;
            case MWCAP_VIDEO_SIGNAL_UNSUPPORTED:
                printf("Input signal status: Unsupported\n");
                break;
            case MWCAP_VIDEO_SIGNAL_LOCKING:
                printf("Input signal status: Locking\n");
                break;
            case MWCAP_VIDEO_SIGNAL_LOCKED:
                printf("Input signal status: Locked\n");
                break;
            }

            if (videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
				break;		
			
			if (videoSignalStatus.bInterlaced) {
                printf("Deinterlaced signal can't be captured by low latency mode!\n");
				break;
			}

            double fps = (double)10000000LL / videoSignalStatus.dwFrameDuration;
            printf("\nInput Signal Info\n"
                   "   %-10s : %d x %d\n"
                   "   %-10s : %.2f\n"
                   "   %-10s : %d\n\n",
                   "Resolution", videoSignalStatus.cx, videoSignalStatus.cy,
                   "Fps", fps,
                   "Interlaced",videoSignalStatus.bInterlaced);

            hNotify = MWRegisterNotify(hChannel, hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING);
            if (hNotify == 0) {
                printf("Error: Register Notify error\n");
			}
			else {
                printf("Start to capture frames by %.2f fps -- %d frames", fps, 0);
                fflush(stdout);

                for (int i = 0; i< MAX_CAPTURE_FRAME_NUM; i++)
                {
                    printf("\rStart to capture frames by %.2f fps -- %d frames", fps, i+1);
                    fflush(stdout);

                    nRet = MWWaitEvent(hNotifyEvent, 1000);
                    if (nRet <= 0)
                        continue;

                    ULONGLONG ullStatusBits = 0;
                    mr = MWGetNotifyStatus(hChannel, hNotify, &ullStatusBits);
                    if (mr != MW_SUCCEEDED)
                        continue;

                    mr = MWGetVideoBufferInfo(hChannel, &videoBufferInfo);
                    if (mr != MW_SUCCEEDED)
                        continue;

                    mr = MWGetVideoFrameInfo(hChannel, videoBufferInfo.iNewestBuffering, &videoFrameInfo);
                    if (mr != MW_SUCCEEDED)
                        continue;

                    if (ullStatusBits & MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING) {
                        mr = MWCaptureVideoFrameToVirtualAddressEx(hChannel, videoBufferInfo.iNewestBuffering, pbImage, dwImageSize, dwMinStride, 1, 0, dwFourCC, cx, cy,
                            0, NUM_PARTIAL_LINE, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, 0, 0, 0, 0,
                            MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
                        




                        MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
                        do {
                            MWWaitEvent(hCaptureEvent, -1);
                            mr = MWGetVideoCaptureStatus(hChannel, &captureStatus);
                        } while (mr == MW_SUCCEEDED && !captureStatus.bFrameCompleted);

                        LONGLONG llCurrent = 0LL;
                        mr = MWGetDeviceTime(hChannel, &llCurrent);
                        llTotalTime += (llCurrent - videoFrameInfo.allFieldStartTimes[0]);
                    }

                }
                MWUnregisterNotify(hChannel, hNotify);
                hNotify = 0;
            }
            MWStopVideoCapture(hChannel);
            printf("\nStop capture\n");
            printf("\nAverage capture latency time  is %lld ms.\n", (LONGLONG)(llTotalTime / (MAX_CAPTURE_FRAME_NUM * 10000)));
        }

    } while(false);

    if (hChannel != -1)
    {

        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (hNotifyEvent != 0)
    {
        MWCloseEvent(hNotifyEvent);
        hNotifyEvent = 0;
    }

    if (hCaptureEvent != 0)
    {
        MWCloseEvent(hCaptureEvent);
        hCaptureEvent = 0;
    }

    if (pbImage != 0)
    {
        free((void *)(unsigned long)pbImage);
        pbImage = 0;
    }

    MWCaptureExitInstance();

    return 0;
}
