#include "mainwindow.h"


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    setMinimumSize(1024,768);
    m_wndMain = new CMainView;
    setCentralWidget(m_wndMain);
    setWindowTitle(QString("MWCap-MultiView"));

    m_nRevChannelNum = 0;
    m_nRevFormat = 1;
    memset(&m_stRevSize,0,sizeof(struct st_size));
    m_dRevFPS = 59.94;

    m_dlgSettings = new CSettingsDlg;
    m_dlgAbout = new CAboutDlg;
    m_labFPS = new QLabel;

    createAction();
    createMenu();
    createStatus();
    //Default:Open channel 0
    CHANNEL_INDEX stChannelsInfo[2];
    MWCAP_CHANNEL_INFO channelInfo;
    MWGetChannelInfoByIndex(0,&channelInfo);
    stChannelsInfo[0].nBoard = channelInfo.byBoardIndex;
    stChannelsInfo[0].nChannel = channelInfo.byChannelIndex;
    m_wndMain->m_xcbRender->SetFrameFormat(DEFAULT_WIDTH,DEFAULT_HEIGHT,DEFAULT_FOURCC,false);
    m_wndMain->m_mwCapture->Create(stChannelsInfo,1,1,1,DEFAULT_WIDTH,DEFAULT_HEIGHT,DEFAULT_FOURCC,59.94,FALSE,(ICallback *)m_wndMain,0);
    m_wndMain->m_timerFPSInfo->start(1000);

    connect(m_dlgSettings,SIGNAL(infoSend(CHANNEL_INDEX*,int,int,st_size,double,bool)),this,SLOT(onRevFromSettingDlg(CHANNEL_INDEX*,int,int,st_size,double,bool)));
    connect(m_wndMain,SIGNAL(sigSendFPS(QString)),this,SLOT(onRevFPS(QString)));
}


MainWindow::~MainWindow()
{
    delete m_dlgSettings;
    m_dlgSettings = NULL;
    delete m_wndMain;
    m_wndMain = NULL;
}


void MainWindow::createAction()
{
    //Settings Action
    m_setAction = new QAction(QIcon("set.png"),tr("Settings"),this);
    m_setAction->setShortcut(tr("Ctrl+O"));
    m_setAction->setStatusTip(tr("Set display"));
    connect(m_setAction,SIGNAL(triggered()),this,SLOT(onSetClicked()));
    //Screen Shot Action
    m_shotAction = new QAction(QIcon("shot.png"),tr("Screen Shot"),this);
    m_shotAction->setShortcut(tr("Ctrl+P"));
    m_shotAction->setStatusTip(tr("Screen shot"));
    connect(m_shotAction,SIGNAL(triggered()),this,SLOT(onShotClicked()));
    //Close Action
    m_closeAction = new QAction(QIcon("close.png"),tr("Close"),this);
    m_closeAction->setShortcut(tr("Ctrl+W"));
    m_closeAction->setStatusTip(tr("Close"));
    connect(m_closeAction,SIGNAL(triggered()),this,SLOT(close()));
    //About Action
    m_aboutAction = new QAction(QIcon("about.png"),tr("About"),this);
    m_aboutAction->setShortcut(tr("Ctrl+B"));
    m_aboutAction->setStatusTip(tr("About info"));
    connect(m_aboutAction,SIGNAL(triggered()),this,SLOT(onAboutClicked()));
}


void MainWindow::createMenu()
{
    //Settings Menu
    m_fileMenu = menuBar()->addMenu(tr("File"));
    m_fileMenu->addAction(m_setAction);
    m_fileMenu->addAction(m_shotAction);
    m_fileMenu->addSeparator();
    m_fileMenu->addAction(m_closeAction);
    //Help Menu
    m_helpMenu = menuBar()->addMenu(tr("Help"));
    m_helpMenu->addAction(m_aboutAction);
}


void MainWindow::createStatus()
{
    m_labFPS->setText(tr("FPS:----"));
    m_labFPS->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    statusBar()->addWidget(m_labFPS);
}


void MainWindow::onSetClicked()
{
    m_dlgSettings->initStatus();
    m_dlgSettings->show();
}


void MainWindow::onShotClicked()
{
    m_wndMain->m_mwCapture->SetShot(true);
}


void MainWindow::onAboutClicked()
{
    m_dlgAbout->show();
}


void MainWindow::onRevFromSettingDlg(CHANNEL_INDEX* stChannelsInfo,int nChannels,int nFormat,st_size stSize,double dFPS,bool bSync)
{
    int nLayoutx = 1;
    int nLayouty = 1;

    m_nRevChannelNum = nChannels;
    m_nRevFormat = nFormat;
    m_stRevSize = stSize;
    m_dRevFPS = dFPS;

    m_dlgSettings->hide();

    switch(m_nRevFormat)
    {
    case 1://1x1
        nLayoutx = 1;
        nLayouty = 1;
        break;
    case 2://2x1
        nLayoutx = 2;
        nLayouty = 1;
        break;
    case 4://2x2
        nLayoutx = 2;
        nLayouty = 2;
        break;
    default:
        break;
    }

    m_wndMain->m_timerFPSInfo->stop();
    m_wndMain->m_mwCapture->Destroy();
    m_wndMain->m_xcbRender->SetFrameFormat(nLayoutx*m_stRevSize.width,nLayouty*m_stRevSize.height,DEFAULT_FOURCC,false);
    m_wndMain->m_mwCapture->Create(stChannelsInfo,m_nRevChannelNum,nLayoutx,nLayouty,m_stRevSize.width,m_stRevSize.height,DEFAULT_FOURCC,m_dRevFPS,bSync,(ICallback *)m_wndMain,0);
    m_wndMain->m_timerFPSInfo->start(1000);
}

void MainWindow::onRevFPS(QString fps)
{
   m_labFPS->setText("FPS:" + fps);
}

