#include "mwcapture.h"


/*****************************************************
* YUV2RGB
* pYUV         point to the YUV data
* pRGB         point to the RGB data
* width        width of the picture
* height       height of the picture
******************************************************/
int YUV2RGB(void* pYUV, void* pRGB, int width, int height)
{
    int Y1, U1, V1, Y2, R1, G1, B1, R2, G2, B2;
    int C1, D1, E1, C2;
    unsigned char* pYUVData = (unsigned char *)pYUV;
    unsigned char* pRGBData = (unsigned char *)pRGB;

    if (pYUV == NULL)
        return -1;
    if (pRGBData == NULL)
        pRGBData = new unsigned char[width*height*3];

    for(int i=0; i<height; ++i)
    {
        for (int j=0; j<width/2; ++j)
        {
            Y1 = *(pYUVData+i*width*2+j*4);
            U1 = *(pYUVData+i*width*2+j*4+1);
            Y2 = *(pYUVData+i*width*2+j*4+2);
            V1 = *(pYUVData+i*width*2+j*4+3);
            C1 = Y1-16;
            C2 = Y2-16;
            D1 = U1-128;
            E1 = V1-128;
            R1 = ((298*C1 + 409*E1 + 128)>>8>255 ? 255 : (298*C1 + 409*E1 + 128)>>8);
            G1 = ((298*C1 - 100*D1 - 208*E1 + 128)>>8>255 ? 255 : (298*C1 - 100*D1 - 208*E1 + 128)>>8);
            B1 = ((298*C1+516*D1 +128)>>8>255 ? 255 : (298*C1+516*D1 +128)>>8);
            R2 = ((298*C2 + 409*E1 + 128)>>8>255 ? 255 : (298*C2 + 409*E1 + 128)>>8);
            G2 = ((298*C2 - 100*D1 - 208*E1 + 128)>>8>255 ? 255 : (298*C2 - 100*D1 - 208*E1 + 128)>>8);
            B2 = ((298*C2 + 516*D1 +128)>>8>255 ? 255 : (298*C2 + 516*D1 +128)>>8);
            *(pRGBData+(height-i-1)*width*3+j*6+2) = R1<0 ? 0 : R1;
            *(pRGBData+(height-i-1)*width*3+j*6+1) = G1<0 ? 0 : G1;
            *(pRGBData+(height-i-1)*width*3+j*6) = B1<0 ? 0 : B1;
            *(pRGBData+(height-i-1)*width*3+j*6+5) = R2<0 ? 0 : R2;
            *(pRGBData+(height-i-1)*width*3+j*6+4) = G2<0 ? 0 : G2;
            *(pRGBData+(height-i-1)*width*3+j*6+3) = B2<0 ? 0 : B2;
        }
    }

    return 0;
}



MWCapture::MWCapture()
{
    m_nChannelCount = MWGetChannelCount();;
    m_nNumChannel = 0;
    m_nValidChannel = 0;
    memset(m_arrChannel,0,sizeof(CHANNEL_ITEM)*MAX_CHANNEL_COUNT);
    for(int i=0;i<MAX_CHANNEL_COUNT;i++)
        m_arrChannel[i].hChannel = -1;

    m_pbImage = 0;
    m_dwImageSize = 0;

    m_nLayoutX = 1;
    m_nLayoutY = 1;
    m_cx = 1920;
    m_cy = 1080;
    m_dwFourcc = MWFOURCC_YUY2;
    m_dwFrameDuration = 59.94;
    m_dCurrentFps = 0.0;//return fps
    m_bSyncMode = false;
    m_pCallback = NULL;
    m_dwParam = 0;

    m_hExitCap = 0;
    m_hExitSignal = 0;//sync mode
    m_threadCap = -1;
    m_threadSig = -1;

    m_bShot = false;
    m_threadShot = -1;
    m_bShotExit = true;
    m_copyImage = 0;
}

MWCapture::~MWCapture()
{
    Destroy();
}


BOOL MWCapture::Create(const CHANNEL_INDEX *pItems, int nNumChannel, int nLayoutX, int nLayoutY, int cx, int cy, unsigned int dwFourcc, double dFrameDuration,bool bSyncMode,ICallback *pCallback, unsigned int dwParam)
{
    MW_RESULT mr;
    BOOL bRet = FALSE;

    m_nNumChannel = nNumChannel;

    m_nLayoutX = nLayoutX;
    m_nLayoutY = nLayoutY;
    m_cx = cx;
    m_cy = cy;

    m_dwFourcc	= dwFourcc;
    m_dwFrameDuration = dFrameDuration;
    m_bSyncMode = bSyncMode;

    m_pCallback = pCallback;
    m_dwParam	= dwParam;

    do{
        //Open Channel/Event/StartCapture
        for(int i=0;i<nLayoutX*nLayoutY;i++)
        {
            if(i >= nNumChannel)
                continue;

            CHANNEL_ITEM& item = m_arrChannel[i];
            item.nBoard = pItems[i].nBoard;
            item.nChannel = pItems[i].nChannel;
            item.hChannel = MWOpenChannel(item.nBoard, item.nChannel);//error return -1
            if(item.hChannel == -1)
                continue;
            bRet = FALSE;

            do{
                item.hCaptureEvent = MWCreateEvent();
                if(item.hCaptureEvent == 0)
                    break;
                item.hNotifyEvent = MWCreateEvent();
                if(item.hNotifyEvent == 0)
                    break;

                mr = MWStartVideoCapture(item.hChannel,item.hCaptureEvent);
                if (mr != MW_SUCCEEDED)
                    break;

                item.hNotify = MWRegisterNotify(item.hChannel,item.hNotifyEvent,MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED | MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE);
                if(item.hNotify == 0)
                    break;

                m_nValidChannel++;
                bRet = TRUE;
            }while(FALSE);

            if(!bRet)
            {
                MWStopVideoCapture(item.hChannel);
                MWCloseChannel(item.hChannel);
                item.hChannel = -1;

                if(item.hCaptureEvent != 0)
                {
                    MWCloseEvent(item.hCaptureEvent);
                    item.hCaptureEvent = 0;
                }
                if(item.hNotifyEvent != 0)
                {
                    MWCloseEvent(item.hNotifyEvent);
                    item.hNotifyEvent = 0;
                }
            }

            //position
            int x = i % nLayoutX;
            int y = i / nLayoutX;
            int w = cx;
            int h = cy;

            item.rcPos.left		= x * w;
            item.rcPos.right	= (x + 1) * w;
            item.rcPos.top		= y * h;
            item.rcPos.bottom	= (y + 1) * h;
        }//End for

        DWORD dwStride = FOURCC_CalcMinStride(dwFourcc, cx * nLayoutX, 4);
        m_dwImageSize = FOURCC_CalcImageSize(dwFourcc, cx * nLayoutX, cy * nLayoutY, dwStride);
        if(m_pbImage != 0)
        {
            free((void *)(unsigned long)m_pbImage);
            m_pbImage = 0;
        }
        m_pbImage = (MWCAP_PTR)(unsigned long)malloc(m_dwImageSize);
        if(m_pbImage != 0)
            memset((void *)(unsigned long)m_pbImage,0,m_dwImageSize);


        /*****************SyncMode*******************************/
        if(m_bSyncMode)
        {
            m_hExitSignal = MWCreateEvent();
            if (m_hExitSignal == 0)
                break;

            if(pthread_create(&m_threadSig,NULL,SignalThreadProc,this) != 0)
                break;
        }
        /*****************SyncMode*******************************/

        m_hExitCap = MWCreateEvent();
        if(m_hExitCap == 0)
            break;
        if(pthread_create(&m_threadCap,NULL,VideoCaptureProc,this) != 0)
            break;

        bRet = TRUE;
    }while(FALSE);

    if(!bRet)
        Destroy();

    return TRUE;
}


void MWCapture::Destroy()
{
    MWSetEvent(m_hExitCap);
    if(m_threadCap != -1)
    {
        pthread_join(m_threadCap, NULL);
        m_threadCap = -1;
        if(m_hExitCap != 0)
        {
            MWCloseEvent(m_hExitCap);
            m_hExitCap = 0;
        }
    }

    if(m_threadSig != -1)
    {
        pthread_kill(m_threadSig,0);
        m_threadSig = -1;
        MWSetEvent(m_hExitSignal);
        if(m_hExitSignal != 0)
        {
            MWCloseEvent(m_hExitSignal);
            m_hExitSignal = 0;
        }
    }

    for(int i = 0; i < m_nNumChannel; i++)
    {
        CHANNEL_ITEM& item = m_arrChannel[i];
        if(item.hNotify != 0)
        {
            MWUnregisterNotify(item.hChannel,item.hNotify);
            item.hNotify = 0;
        }
        if(item.hChannel != -1)
        {
            MWStopVideoCapture(item.hChannel);
            MWCloseChannel(item.hChannel);
            item.hChannel = -1;
        }
        if(item.hCaptureEvent != 0)
        {
            MWCloseEvent(item.hCaptureEvent);
            item.hCaptureEvent = 0;
        }
        if(item.hNotifyEvent != 0)
        {
            MWCloseEvent(item.hNotifyEvent);
            item.hNotifyEvent = 0;
        }
    }

    if (m_pbImage != 0)
    {
        free((void *)(unsigned long)m_pbImage);
        m_pbImage = 0;
    }

    m_nValidChannel = 0;
}


void MWCapture::VideoCaptureProcInternel()
{
    int nWidth = m_cx * m_nLayoutX;
    int nHeight = m_cy * m_nLayoutY;
    int nStride = FOURCC_CalcMinStride(m_dwFourcc, nWidth, 4);

    int nFrameCount = 0;
    int nFPSCount = 0;

    HCHANNEL hChannel = -1;
    for(int i = 0; i < m_nNumChannel; i++)
    {
        CHANNEL_ITEM& item = m_arrChannel[i];
        if(item.hChannel != -1)
        {
            hChannel = item.hChannel;
                break;
        }
    }
    if(hChannel == -1)
        return;

    //FPS
    MWCAP_PTR hTimerEvent = MWCreateEvent();
    HTIMER hTimerNotify = MWRegisterTimer(hChannel, hTimerEvent);

    MW_RESULT mr;
    LONGLONG llBegin = 0LL;
    mr = MWGetDeviceTime(hChannel, &llBegin);

    LONGLONG llExpireTime = llBegin;
    LONGLONG llLast = llBegin;

    //Shot
    if(m_copyImage != 0)
    {
        free((void *)(unsigned long)m_copyImage);
        m_copyImage = 0;
    }
    m_copyImage = (MWCAP_PTR)(unsigned long)malloc(m_dwImageSize);

    while (TRUE)
    {
        llExpireTime += (long long)(10000000/m_dwFrameDuration);

        mr = MWScheduleTimer(hChannel,hTimerNotify,llExpireTime);
        if(mr != MW_SUCCEEDED)
            continue;

        MWCAP_PTR aEventNotify[] = {m_hExitCap,hTimerEvent};
        DWORD dwRet = MWMultiWaitEvent(aEventNotify,2,500);

        if(dwRet == 0)//Wait error
        {
            break;
        }

        if(dwRet & 0x01)//Exit Event
        {
            break;
        }

        if(dwRet & (0x01 << 1))//Timer Event
        {
            int nNumEvent = 0;
            MWCAP_PTR arrEvent[MAX_CHANNEL_COUNT];

            /*****************SyncMode*******************************/
            if(m_bSyncMode)
            {
                QMutexLocker lock(&m_lock);
                for(int i = 0; i < m_nNumChannel; i++)
                {
                    DWORD dwMask = (1 << i);
                    if (!(m_dwCapMask & dwMask))
                        continue;

                    CHANNEL_ITEM& item = m_arrChannel[i];

                    mr = MWCaptureVideoFrameToVirtualAddressEx(item.hChannel, item.byCapIndex, m_pbImage, m_dwImageSize, nStride,FALSE, NULL, m_dwFourcc,
                        nWidth, nHeight, 0, 0, NULL, NULL, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE,
                        NULL, &(item.rcPos), 0, 0, MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
                    if(mr != MW_SUCCEEDED)
                        continue;

                    arrEvent[nNumEvent++] = item.hCaptureEvent;
                }
            }
            else
            {
                QMutexLocker lock(&m_lock);
                for (int i = 0; i < m_nNumChannel; i ++)
                {

                    CHANNEL_ITEM& item = m_arrChannel[i];
                    if(item.hChannel == -1)
                        continue;

                    mr = MWCaptureVideoFrameToVirtualAddressEx(item.hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, m_pbImage, m_dwImageSize, nStride,FALSE, NULL, m_dwFourcc,
                        nWidth, nHeight, 0, 0, NULL, NULL, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE,
                        NULL, &(item.rcPos), 0, 0, MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
                    if(mr != MW_SUCCEEDED)
                        continue;

                    arrEvent[nNumEvent++] = item.hCaptureEvent;
                }
            }
            /*****************SyncMode*******************************/

            MWMultiWaitEvent(arrEvent,nNumEvent,-1);
            for(int i = 0; i < m_nNumChannel;i++)
            {
                CHANNEL_ITEM& item = m_arrChannel[i];
                if(item.hChannel != -1)
                {
                    MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
                    MWGetVideoCaptureStatus(item.hChannel, &captureStatus);
                }
            }

            if(m_bShot & m_bShotExit)
            {
                m_lock.lock();
                memset((void *)(unsigned long)m_copyImage,0,m_dwImageSize);
                memcpy((void *)m_copyImage,(void *)m_pbImage,m_dwImageSize);
                m_lock.unlock();
                if(pthread_create(&m_threadShot,NULL,ShotThreadProc,this) != 0)
                    break;

                m_bShot = false;
            }

            if (m_pCallback != NULL)
            {
                m_pCallback->VideoCaptureCallback(m_pbImage,m_dwImageSize,(void*)m_dwParam);
            }

            nFrameCount++;

            /*****************SyncMode*******************************/
            if(m_bSyncMode && (nFrameCount % 10 == 0))
            {
                LONGLONG llStandardTime = 0LL;
                for (int i = 0; i < m_nNumChannel; i ++)
                {
                    CHANNEL_ITEM& item = m_arrChannel[i];
                    if (item.hChannel == -1)
                        continue;

                    if (llStandardTime == 0LL)
                        mr = MWGetDeviceTime(item.hChannel, &llStandardTime);
                    else
                        mr = MWRegulateDeviceTime(item.hChannel, llStandardTime);
                }
            }
            /*****************SyncMode*******************************/


            LONGLONG llCurrent = 0LL;
            MWGetDeviceTime(hChannel, &llCurrent);

            nFPSCount++;
            if (nFPSCount % 10 == 0)
            {
                m_dCurrentFps = (double)nFPSCount * 10000000LL / (llCurrent - llLast);
                if (llCurrent - llLast > 30000000LL) {
                    llLast = llCurrent;
                    nFPSCount = 0;
                }
            }
        }
    }//End While

    if(m_copyImage != 0)
    {
        free((void *)(unsigned long)m_copyImage);
        m_copyImage = 0;
    }

    if(hTimerNotify != 0)
    {
        mr = MWUnregisterTimer(hChannel,hTimerNotify);
    }
    if(hTimerEvent != 0)
    {
        MWCloseEvent(hTimerEvent);
    }

    return;
}



void MWCapture::SignalThreadProcInternel()
{
    //Set Priority
    setpriority(PRIO_PROCESS,m_threadSig,-1);//-20~19 0:normal

    MWCAP_PTR	arrEvent[MAX_CHANNEL_COUNT + 1];
    int         arrIndex[MAX_CHANNEL_COUNT];
    int         nCount = 0;
    MW_RESULT   mr;

    m_dwStdMask = 0;//Mask:Opened locked channels
    m_dwCurMask = 0;//Mask:Current opened channels
    m_dwCapMask = 0;//Mask:Capture channels

    LONGLONG llStandardTime = 0LL;
    for (int i = 0; i < m_nNumChannel; i ++)
    {
        CHANNEL_ITEM& item = m_arrChannel[i];
        if (item.hChannel == -1 || item.hCaptureEvent == 0 || item.hNotifyEvent == 0 || item.hNotify == 0)
            continue;

        if (llStandardTime == 0LL)
            mr = MWGetDeviceTime(item.hChannel,&llStandardTime);
        else
            mr = MWSetDeviceTime(item.hChannel, llStandardTime);
        if (mr != MW_SUCCEEDED)
            continue;

        arrEvent[nCount] = item.hNotifyEvent;
        arrIndex[nCount] = i;
        nCount++;

        MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
        mr = MWGetVideoSignalStatus(item.hChannel, &signalStatus);
        if(mr != MW_SUCCEEDED)
            continue;

        if (signalStatus.state == MWCAP_VIDEO_SIGNAL_LOCKED)
        {
            m_dwStdMask |= (1 << i);
        }
    }

    arrEvent[nCount] = m_hExitSignal;

    // less than 2 channels -> change to NoSync Mode
    if(m_dwStdMask < 2)
    {
        m_bSyncMode = false;
        return;
    }

    LONGLONG	llStdTime = 0LL;
    DWORD		dwStdFrameDuration = 0;

    int nPos = -1;
    while(TRUE)
    {
        DWORD dwRet = MWMultiWaitEvent(arrEvent,nCount+1,-1);

        for(int i=0;i<(nCount+1);i++)
        {
            nPos = -1;
            if(dwRet & (0x01<<i))
            {
                nPos = i;
            }else
                continue;

            if(nPos == nCount)//m_hExitSignal
                return;

            int nChannelIndex = arrIndex[nPos];
            CHANNEL_ITEM& item = m_arrChannel[nChannelIndex];
            ULONGLONG ullStatusBits = 0;
            mr = MWGetNotifyStatus(item.hChannel,item.hNotify, &ullStatusBits);
            if(mr != MW_SUCCEEDED)
                continue;

            MWCAP_VIDEO_SIGNAL_STATUS signalStatus;
            mr = MWGetVideoSignalStatus(item.hChannel, &signalStatus);
            if(mr != MW_SUCCEEDED)
                continue;

            if(ullStatusBits & MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE)
            {
                if (signalStatus.state == MWCAP_VIDEO_SIGNAL_LOCKED)
                {
                    m_dwStdMask |= (1 << nChannelIndex);
                }
                else
                {
                    m_dwStdMask &= ~(1 << nChannelIndex);
                }

                m_dwCurMask = 0;
                llStdTime = 0LL;
            }

            if(ullStatusBits & MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED)
            {
                MWCAP_VIDEO_BUFFER_INFO buffInfo;
                mr = MWGetVideoBufferInfo(item.hChannel, &buffInfo);
                if (mr != MW_SUCCEEDED)
                    continue;

                MWCAP_VIDEO_FRAME_INFO frameInfo;
                mr = MWGetVideoFrameInfo(item.hChannel, buffInfo.iNewestBufferedFullFrame, &frameInfo);
                if (mr != MW_SUCCEEDED)
                    continue;

                if (m_dwCurMask == m_dwStdMask)
                {
                    mr = MWGetDeviceTime(item.hChannel, &llStdTime);
                    if (mr != MW_SUCCEEDED)
                        continue;

                    dwStdFrameDuration = signalStatus.dwFrameDuration;
                    llStdTime = frameInfo.allFieldBufferedTimes[0];
                    m_dwCurMask = (1 << nChannelIndex);
                }
                else
                {
                    if(MWCheckTime(llStdTime, frameInfo.allFieldBufferedTimes[0], dwStdFrameDuration, 0.2f))
                    {
                        m_dwCurMask |= (1 << nChannelIndex);
                    }
                    else
                    {
                        dwStdFrameDuration = signalStatus.dwFrameDuration;
                        llStdTime = frameInfo.allFieldBufferedTimes[0];
                        m_dwCurMask = (1 << nChannelIndex);
                    }
                }
                item.byCurIndex = buffInfo.iNewestBufferedFullFrame;
                item.byCurFieldStartTimes = frameInfo.allFieldStartTimes[0];//check

                if(m_dwCurMask == m_dwStdMask)
                {
                    QMutexLocker lock(&m_lock);
                    for (int i = 0; i < m_nNumChannel; i ++)
                    {
                        CHANNEL_ITEM& item = m_arrChannel[i];
                        if(item.hChannel == -1 || item.hCaptureEvent == 0)
                            continue;

                        DWORD dwMask = (1 << i);
                        if(!(m_dwCurMask & dwMask))
                            continue;

                        item.byCapIndex = item.byCurIndex;
                        item.byCapFieldStartTimes = item.byCurFieldStartTimes;//check
                    }
                    m_dwCapMask = m_dwCurMask;
                }
            }
        }//end for
    }

    return;
}


void MWCapture::ShotThreadProcInternel(MWCAP_PTR llImage)
{
    m_bShotExit =false;

    DWORD rgbStride = FOURCC_CalcMinStride(MWFOURCC_BGR24,m_cx * m_nLayoutX, 4);
    DWORD rgbImageSize = FOURCC_CalcImageSize(MWFOURCC_BGR24, m_cx * m_nLayoutX, m_cy * m_nLayoutY, rgbStride);
    MWCAP_PTR rgbBuffer = (MWCAP_PTR)(unsigned long)malloc(rgbImageSize);
    memset((void *)(unsigned long)rgbBuffer,0,rgbImageSize);

    char szBitmapName[64];
    bool bFlag = true;
    int i = 1;

    do{
        memset(szBitmapName,0,sizeof(szBitmapName));
        sprintf(szBitmapName, "./MultiVideo_%d.BMP",i);
        if(access(szBitmapName,F_OK) == 0)
            i++;
        else
            bFlag = false;
    }while(bFlag);

    YUV2RGB((void *)llImage,(void *)rgbBuffer,m_nLayoutX*m_cx,m_nLayoutY*m_cy);
    create_bitmap(szBitmapName,rgbBuffer,rgbImageSize,m_nLayoutX*m_cx,m_nLayoutY*m_cy);
    free((void *)(unsigned long)rgbBuffer);

    m_bShotExit = true;
}


void MWCapture::GetDeviceNameByIndex(int nIndex, char *pDeviceName)
{
    MW_RESULT mr;

    if (nIndex >= m_nChannelCount)
        return;

    MWCAP_CHANNEL_INFO channelInfo;
    mr = MWGetChannelInfoByIndex(nIndex, &channelInfo);
    if(mr != MW_SUCCEEDED)
        return;

    sprintf(pDeviceName, "%02d:%02d %s", channelInfo.byBoardIndex, channelInfo.byChannelIndex, channelInfo.szProductName);

    return;
}


int MWCapture::GetChannelCount()
{
    return m_nChannelCount;
}


double MWCapture::GetCurrentFPS()
{
    return m_dCurrentFps;
}

void MWCapture::SetShot(bool bShot)
{
    m_bShot = bShot;
}
