#ifndef MWCAPTURE_H
#define MWCAPTURE_H

#define NUM_PARTIAL_LINE 128
#ifdef __cplusplus
extern "C" {
#endif

#include <pthread.h>
#include <signal.h>
#include <unistd.h>
#include<sys/time.h>
#include<sys/resource.h>
#include "lib-mw-capture.h"
#include "mw-fourcc.h"
#include "bitmap.h"

#ifdef __cplusplus
}
#endif

#include <QtCore>


//Open used
typedef struct tagCHANNEL_INDEX {
    int			nBoard;
    int			nChannel;
} CHANNEL_INDEX, *PCHANNEL_INDEX;


//Sync used
inline BOOL MWCheckTime(LONGLONG llStdTime, LONGLONG llTime, DWORD dwFrameDuration, float fRange)
{
    LONGLONG llOffset = (LONGLONG)(dwFrameDuration * fRange);
    return (llTime >= (llStdTime - llOffset) && llTime <= (llStdTime + llOffset));
}


//Callback
class ICallback
{
public:
    virtual void VideoCaptureCallback(MWCAP_PTR pbImage, DWORD cbImage, void *pvParam) = 0;
    virtual void VideoSignalChangedCallback() = 0;
};


class MWCapture
{
public:
    MWCapture();
    ~MWCapture();

    static BOOL Init(){return MWCaptureInitInstance();}
    static BOOL RefreshDevices(){return MWRefreshDevice() ==  MW_SUCCEEDED;}
    static void Exit(){MWCaptureExitInstance();}

    BOOL Create(const CHANNEL_INDEX* pItems, int nNumChannel, int nLayoutX, int nLayoutY, int cx, int cy, DWORD dwFourcc,double dFrameDuration,bool bSyncMode,ICallback* pCallback, DWORD dwParam = 0);
    void Destroy();
    void GetDeviceNameByIndex(int nIndex, char *pDeviceName);
    int GetChannelCount();
    double GetCurrentFPS();
    void SetShot(bool bShot);

    //Cap Thread
    static void *VideoCaptureProc(void *pvParam)
    {
        MWCapture *pThis = (MWCapture *)pvParam;
        pThis->VideoCaptureProcInternel();
        return NULL;
    }
    void VideoCaptureProcInternel();

    //Sig-sync Thread
    static void *SignalThreadProc(void *pvParam)
    {
        MWCapture* pThis = (MWCapture *)pvParam;
        pThis->SignalThreadProcInternel();
        return NULL;
    }
    void SignalThreadProcInternel();

    static void *ShotThreadProc(void *pvParam)
    {
        MWCapture* pThis = (MWCapture *)pvParam;
        pThis->ShotThreadProcInternel(pThis->m_copyImage);
        return NULL;
    }
    void ShotThreadProcInternel(MWCAP_PTR llImage);

protected:
    typedef struct tagCHANNEL_ITEM {
            int			nBoard;
            int			nChannel;

            HCHANNEL	hChannel;//-1
            MWCAP_PTR   hCaptureEvent;
            MWCAP_PTR 	hNotifyEvent;
            HNOTIFY		hNotify;

            BYTE		byCapIndex;//capBufferInfo
            BYTE		byCurIndex;//curBufferInfo
            long long   byCurFieldStartTimes;
            long long   byCapFieldStartTimes;
            RECT		rcPos;
    } CHANNEL_ITEM, *PCHANNEL_ITEM;

    //Create
    int                 m_nChannelCount;//Total channel number
    int                 m_nNumChannel;//Channel number to open
    int					m_nValidChannel;//Opened channel number

    int					m_nLayoutX;
    int					m_nLayoutY;
    int					m_cx;
    int					m_cy;
    DWORD               m_dwFourcc;
    double				m_dwFrameDuration;
    double              m_dCurrentFps;//Current FPS to return
    bool                m_bSyncMode;
    ICallback           *m_pCallback;
    DWORD				m_dwParam;

    //Image
    DWORD				m_dwImageSize;
    MWCAP_PTR           m_pbImage;

    //Capture thred
    MWCAP_PTR           m_hExitCap;
    pthread_t           m_threadCap;

    //Sig-sync thread
    MWCAP_PTR           m_hExitSignal;
    pthread_t           m_threadSig;
    DWORD               m_dwStdMask;
    DWORD               m_dwCurMask;
    DWORD               m_dwCapMask;
    QMutex              m_lock;

    //Shot
    bool                m_bShot;
    pthread_t           m_threadShot;
    bool                m_bShotExit;
    MWCAP_PTR           m_copyImage;
public:
    CHANNEL_ITEM		m_arrChannel[MAX_CHANNEL_COUNT];

};

#endif // MWCAPTURE_H
