#include <dirent.h>
#include <fnmatch.h>
#include "options.h"
#include "lib-mw-capture.h"
#include "mw-fourcc.h"
#include "video-device-info.h"
#include "lodepng.h"
#include "bitmap.h"


static int CustomFilter(const struct dirent *pDir)
{
    if (pDir->d_type == DT_REG && fnmatch("*.png", pDir->d_name, FNM_PATHNAME) == 0)
        return 1;

    return 0;
}

int CaptureFrameWithOSD_PNG()
{
    int nReturnRet = -1;

    int nPNGFileCount = 0;
    struct dirent **namelist = NULL;

    unsigned uRet;
    unsigned char *pPngImage = NULL;
    unsigned uPngWidth, uPngHeight;
    unsigned char *pPngBuffer = NULL;
    size_t tPngSize;

    MW_RESULT mr;
    char szDevicePath[32];
    HCHANNEL hChannel = -1;
    MWCAP_PTR hCaptureEvent = 0;

    MWCAP_PTR hOSDImage = 0;

    DWORD dwFourCC;
    DWORD dwMinStride;
    DWORD dwImageSize;
    HANDLE64 pbImage = 0;


    do
    {
        nPNGFileCount = scandir(".", &namelist, CustomFilter, alphasort);
        if (nPNGFileCount <= 0)
        {
            printf("Error: No PNG file is found on current dirent\n");
            break;
        }

        uRet = lodepng_load_file(&pPngBuffer, &tPngSize, namelist[0]->d_name);
        if (uRet != 0) {
            printf("Error: Load PNG file \"%s\" error\n", namelist[0]->d_name);
            break;
        }

        uRet = lodepng_decode32(&pPngImage, &uPngWidth, &uPngHeight, pPngBuffer, tPngSize);
        if (uRet != 0) {
            printf("Error: Decode PNG buffer error -- %s\n", lodepng_error_text(uRet));
            break;
        }

        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("Error: No Pro Capture device is found\n");
            break;
        }

        mr = MWGetDevicePath(0, szDevicePath);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get device path error\n");
           break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get device %s' info error\n", szDevicePath);
           break;
        }

        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1)
        {
            printf("Error: Open device %s error\n", szDevicePath);
            break;
        }

        printf("Open channel info :\n"
               "   Board Index = %d\n"
               "   Channel Index = %d\n"
               "   Product Name = %s\n"
               "   Serial Number = %s\n\n"
               ,
               channelInfo.byBoardIndex,
               channelInfo.byChannelIndex,
               channelInfo.szProductName,
               channelInfo.szBoardSerialNo);

        MWCAP_VIDEO_SIGNAL_STATUS videoStatus;
        mr = MWGetVideoSignalStatus(hChannel, &videoStatus);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get video signal status error\n");
           break;
        }

        if (videoStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
        {
            printf("Error: No video signal is locked\n");
            break;
        }


        //allocate image buffer
        int cx = videoStatus.cx;
        int cy = videoStatus.cy;
        dwFourCC = MWFOURCC_BGRA;
        dwMinStride = FOURCC_CalcMinStride(dwFourCC, cx, 4);
        dwImageSize = FOURCC_CalcImageSize(dwFourCC, cx, cy, dwMinStride);
        pbImage = (HANDLE64)(unsigned long)malloc(dwImageSize);


        hCaptureEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
        {
            printf("Error: Create capture event error\n");
            break;
        }

        mr = MWStartVideoCapture(hChannel, hCaptureEvent);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Start video capture error\n");
           break;
        }

        hOSDImage = MWCreateImage(hChannel, cx, cy);
        if (hOSDImage == 0)
        {
            printf("Error: Create OSD image error\n");
            break;
        }

        mr = MWUploadImageFromVirtualAddress(hChannel, hOSDImage, MWCAP_VIDEO_COLOR_FORMAT_RGB, MWCAP_VIDEO_QUANTIZATION_UNKNOWN,
                                             MWCAP_VIDEO_SATURATION_UNKNOWN, 0, 0, cx/8, cy/8, (MWCAP_PTR)(unsigned long)pPngImage, uPngWidth * uPngHeight * 4,
                                             uPngWidth * 4, uPngWidth, uPngHeight, FALSE, TRUE, FALSE);

        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Upload OSD image error\n");
           fprintf(stderr, "Error: %s\n", strerror(errno));
           break;
        }

        const RECT aOSDRects[] = {{0,0,cx/8,cy/8}};
        mr = MWCaptureVideoFrameToVirtualAddressEx(hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, pbImage, dwImageSize, dwMinStride, TRUE, 0, dwFourCC, cx, cy,
                                    0, 0, hOSDImage, aOSDRects, 1, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, 0, 0, 0, 0,
                                    MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Capture image with OSD error\n");
           break;
        }

        MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
        do
        {
            MWWaitEvent(hCaptureEvent, 1000);
            mr = MWGetVideoCaptureStatus(hChannel, &captureStatus);
        }while(mr == MW_SUCCEEDED && !captureStatus.bFrameCompleted);


        create_bitmap("./osd_png.bmp", (BYTE *)(unsigned long)pbImage, dwImageSize, cx, cy);

        nReturnRet = 1;
    }while(false);

    if (nPNGFileCount != 0)
    {
        for (int i = 0; i < nPNGFileCount; i++)
        {
            free(namelist[i]);
            namelist[i] = NULL;
        }

        free(namelist);
        namelist = NULL;
    }

    if (pPngBuffer != NULL)
    {
        free(pPngBuffer);
        pPngBuffer = NULL;
    }

    if (pPngImage != NULL)
    {
        free(pPngImage);
        pPngImage = NULL;
    }

    if (pbImage != 0)
    {
        free((void *)(unsigned long) pbImage);
        pbImage = 0;
    }

    if (hOSDImage != 0 && hChannel != -1)
    {
        long lRet = 0;
        MWCloseImage(hChannel, hOSDImage, &lRet);
        hOSDImage = 0;
    }

    if (hChannel != -1)
    {
        MWStopVideoCapture(hChannel);
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (hCaptureEvent != 0)
    {
        MWCloseEvent(hCaptureEvent);
        hCaptureEvent = 0;
    }

    return nReturnRet;
}


int CaptureFrameWithOSD_Video()
{
    int nReturnRet = -1;


    MW_RESULT mr;
    char szDevicePath[32];
    HCHANNEL hChannel = -1;
    MWCAP_PTR hCaptureEvent = 0;

    MWCAP_PTR hOSDImage = 0;

    DWORD dwFourCC0, dwFourCC1;
    DWORD dwMinStride0, dwMinStride1;
    DWORD dwImageSize0, dwImageSize1;
    HANDLE64 pbImage0 = 0;
    HANDLE64 pbImage1 = 0;


    do
    {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0)
        {
            printf("Error: No Pro Capture device is found\n");
            break;
        }

        mr = MWGetDevicePath(0, szDevicePath);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get device path error\n");
           break;
        }

        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(0, &channelInfo);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get device %s' info error\n", szDevicePath);
           break;
        }

        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1)
        {
            printf("Error: Open device %s error\n", szDevicePath);
            break;
        }

        printf("Open channel info :\n"
               "   Board Index = %d\n"
               "   Channel Index = %d\n"
               "   Product Name = %s\n"
               "   Serial Number = %s\n\n"
               ,
               channelInfo.byBoardIndex,
               channelInfo.byChannelIndex,
               channelInfo.szProductName,
               channelInfo.szBoardSerialNo);

        MWCAP_VIDEO_SIGNAL_STATUS videoStatus;
        mr = MWGetVideoSignalStatus(hChannel, &videoStatus);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Get video signal status error\n");
           break;
        }

        if (videoStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
        {
            printf("Error: No video signal is locked\n");
            break;
        }

        hCaptureEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
        {
            printf("Error: Create capture event error\n");
            break;
        }

        mr = MWStartVideoCapture(hChannel, hCaptureEvent);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Start video capture error\n");
           break;
        }

        //capture frame0 as OSD image
        int cx0 = videoStatus.cx;
        int cy0 = videoStatus.cy;
        dwFourCC0 = MWFOURCC_BGRA;
        dwMinStride0 = FOURCC_CalcMinStride(dwFourCC0, cx0, 4);
        dwImageSize0 = FOURCC_CalcImageSize(dwFourCC0, cx0, cy0, dwMinStride0);
        pbImage0 = (HANDLE64)(unsigned long)malloc(dwImageSize0);

        mr = MWCaptureVideoFrameToVirtualAddressEx(hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, pbImage0, dwImageSize0, dwMinStride0, TRUE, 0, dwFourCC0, cx0, cy0,
                                    0, 0, 0, NULL, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, 0, 0, 0, 0,
                                    MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Capture iamge0 error\n");
           break;
        }

        MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
        do
        {
            MWWaitEvent(hCaptureEvent, 1000);
            mr = MWGetVideoCaptureStatus(hChannel, &captureStatus);
        }while(mr == MW_SUCCEEDED && !captureStatus.bFrameCompleted);


        //Capture frame1 with OSD iamge
        int cx1 = videoStatus.cx;
        int cy1 = videoStatus.cy;
        dwFourCC1 = MWFOURCC_BGRA;
        dwMinStride1 = FOURCC_CalcMinStride(dwFourCC1, cx1, 4);
        dwImageSize1 = FOURCC_CalcImageSize(dwFourCC1, cx1, cy1, dwMinStride1);
        pbImage1 = (HANDLE64)(unsigned long)malloc(dwImageSize1);

        hOSDImage = MWCreateImage(hChannel, cx1, cy1);
        if (hOSDImage == 0)
        {
            printf("Error: Create OSD image error\n");
            break;
        }

        mr = MWUploadImageFromVirtualAddress(hChannel, hOSDImage, MWCAP_VIDEO_COLOR_FORMAT_RGB, MWCAP_VIDEO_QUANTIZATION_UNKNOWN,
                                             MWCAP_VIDEO_SATURATION_UNKNOWN, 0, 0, cx1/8, cy1/8, (MWCAP_PTR)pbImage0, dwImageSize0,
                                             dwMinStride0, cx0, cy0, TRUE, TRUE, TRUE);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Upload OSD iamge error\n");
           break;
        }

        const RECT aOSDRects[] = {{0,0,cx1/8,cy1/8}};
        mr = MWCaptureVideoFrameToVirtualAddressEx(hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, pbImage1, dwImageSize1, dwMinStride1, TRUE, 0, dwFourCC1, cx1, cy1,
                                    0, 0, hOSDImage, aOSDRects, 1, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, 0, 0, 0, 0,
                                    MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
        if(mr != MW_SUCCEEDED)
        {
           printf("Error: Capture iamge with OSD error\n");
           break;
        }

        do
        {
            MWWaitEvent(hCaptureEvent, 1000);
            mr = MWGetVideoCaptureStatus(hChannel, &captureStatus);
        }while(mr == MW_SUCCEEDED && !captureStatus.bFrameCompleted);


        create_bitmap("./osd_video.bmp", (BYTE *)(unsigned long)pbImage1, dwImageSize1, cx1, cy1);
        nReturnRet = 1;

    }while(false);


    if (pbImage0 != 0)
    {
        free((void *)(unsigned long) pbImage0);
        pbImage0 = 0;
    }

    if (pbImage1 != 0)
    {
        free((void *)(unsigned long) pbImage1);
        pbImage1 = 0;
    }

    if (hOSDImage != 0 && hChannel != -1)
    {
        long lRet = 0;
        MWCloseImage(hChannel, hOSDImage, &lRet);
        hOSDImage = 0;
    }

    if (hChannel != -1)
    {
        MWStopVideoCapture(hChannel);
        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (hCaptureEvent != 0)
    {
        MWCloseEvent(hCaptureEvent);
        hCaptureEvent = 0;
    }

    return nReturnRet;
}


int main(int argc, char *argv[])
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    int nRet;


    do
    {
        OPTION_RESULT opr =  ParseCommandLine(argc, argv);

        switch (opr) {
        case OPTION_INVALID_OPTION:
        {
            return -1;
        }
            break;
        default:
            break;
        }

        if ((g_dwCMD & (0x01 << OPTION_CMD_USAGE)) || g_dwCMD == 0)
        {
            PrintUsage();
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_LIST))
        {
            PrintSelectedVideoDevicesInfo();
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_OSD_PNG))
        {
            nRet = CaptureFrameWithOSD_PNG();
            if (nRet == -1)
                printf("Error: Capture frame with OSD form PNG file error\n");
            else
                printf("Capture video frame with OSD from PNG file successfully\n"
                       "Image is saved in \"osd_png.bmp\"\n");
        }
        else if (g_dwCMD & (0x01 << OPTION_CMD_OSD_VIDEO))
        {
            nRet = CaptureFrameWithOSD_Video();
            if (nRet == -1)
                printf("Error: Capture frame with OSD form Video error\n");
            else
                printf("Capture video frame with OSD from video successfully\n"
                       "Image is saved in \"osd_video.bmp\"\n");
        }

    }while(false);



    MWCaptureExitInstance();

    return nRet;
}
