#include <stdio.h>
#include <stdlib.h>
//#include <curses.h>
#include <termios.h>
#include <unistd.h>
#include <dirent.h>
#include <getopt.h>
#include <signal.h>
#include "settings.h"
#include "ProductVer.h"


//#define DEFAULT_MWF_FILE_PATH       "upgrade/pro_capture_hdmi_rev_a_1_15.mwf"
typedef enum _MWCAP_UPGRADE_CMD_
{
    MWCAP_UPGRADE_USAGE             = 0x01,
    MWCAP_UPGRADE_LIST_CHANNEL_INFO,
    MWCAP_UPGRADE_FOURCE,
    MWCAP_UPGRADE_YES,
    MWCAP_UPGRADE_ALL,
    MWCAP_UPGRADE_BOARD,
    MWCAP_UPGRADE_DEVICE,

}MWCAP_UPGRADE_CMD;

typedef enum _MWCAP_PRAISE_RESULT_
{
    PRAISE_RESULT_OK,
    PRAISE_RESULT_NOT_PRO_CAPTURE_DEVICE,
    PRAISE_RESULT_NOT_EXIT_DEVICE,
    PRAISE_RESULT_ERROR_ARGUMENT,
    PRAISE_RESULT_ERROR_OPTION,
    PRAISE_RESULT_UNKNOW_ERROR,
} MWCAP_PRAISE_RESULT;

typedef struct _MWCAP_UPGRADE_LIST_INFO_
{
    char szChannelName[MAX_FILE_NAME_NUM];
    char szDeviceName[MAX_CHANNEL_NAME_LEN];
    char szFirmwareVerCur[16];
    char szFirmwareVerNew[16];

    bool bNeedUpgrade;

    bool bSafeUpgrade;

    CHAR chHardwareVersion;

} MWCAP_UPGRADE_LIST_INFO;

static DWORD g_dwCMD = 0;

static int g_nBoardIndex = -1;
static int g_nChannelIndex = -1;

static MWCAP_UPGRADE_LIST_INFO g_arrChannsInfo[MAX_CHANNEL_COUNT] = {{{0},{0},{0},{0},false,false,0}};
static int g_nDevicesNum = 0;
static int g_nDevicesNeedUpgrade = 0;

static char (*g_szMWFFileList)[MAX_FILE_NAME_NUM] = NULL;
static int g_nMWFFileNum = 0;

static const char short_options[] = "ab:d:fylh";

static const struct option long_options[] = {
    { "all",                no_argument,       NULL, 'a' },
    { "board",              required_argument, NULL, 'b' },
    { "device",             required_argument, NULL, 'd' },
    { "fource",             no_argument,       NULL, 'f' },
    { "yes",                no_argument,       NULL, 'y' },
    { "list",               no_argument,       NULL, 'l' },
    { "help",               no_argument,       NULL, 'h' },


    { 0, 0, 0, 0 }
};



void PrintUsage()
{
    printf("USAGE\n"
           "  mwcap-upgrade [OPTIONS]\n\n"
           "DESCRIPTION\n"
           "  Upgrade firmware for Pro Capture cards.\n\n"
           "OPTIONS\n"
           "  -h                    print usage (Default option)\n"
           "  -l                    list all capture device information\n\n"
           "  -a                    upgrade all channels\n"
           "  -b BOARD_ID           upgrade devices with specifed board ID\n"
           "  -d CAPTURE_DEVICE     upgrade specifed devices. CAPTURE_DEVICE can be\n"
           "                        V4L2 device path (eg. /dev/video0) or capture\n"
           "                        device ID (Board ID:Channel ID, eg. 0:0)\n\n"
           "  -f                    fource to upgrade firmware to target version\n"
           "  -y                    upgrade without interacting\n\n"
           "EXAMPLES\n"
           "  1. upgrade all devices"
           "      $ mwcap-upgrade -a\n"
           "  2. upgrade devices with specifed board ID\n"
           "      $ mwcap-upgrade -b 0\n"
           "  3. upgrade specifed devices"
           "      $ mwcap-upgrade -d /dev/video0\n"
           "      -- or --\n"
           "      $ mwcap-upgrade -d 0:0\n"
           "  4. fource to upgrade device, even if target version is older than current\n"
           "      $ mwcap-upgrade -d 0:0 -f\n"
           "  5. upgrade device without interacting\n"
           "      $ mwcap-upgrade -d 0:0 -y\n"
           );
}

static bool IsProCaptureDevice(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return false;
    int nCount = MWGetChannelCount();

    for (int i = 0; i < nCount; i++)
    {
        char szTemp[MAX_CHANNEL_NAME_LEN];
        MWGetDevicePath(i, szTemp);

        if (strcmp(szTemp, szDevicePath) == 0)
            return true;
    }

    return false;
}

static MWCAP_PRAISE_RESULT GetDeviceInfo(char *szDevicePath, MWCAP_UPGRADE_LIST_INFO *pChannelInfo)
{
    if (szDevicePath == NULL || pChannelInfo == NULL)
        return PRAISE_RESULT_ERROR_ARGUMENT;

    if (access(szDevicePath, F_OK) != 0)
        return PRAISE_RESULT_NOT_EXIT_DEVICE;

    if (!IsProCaptureDevice(szDevicePath))
        return PRAISE_RESULT_NOT_PRO_CAPTURE_DEVICE;

    MW_RESULT mr;

    strcpy(pChannelInfo->szDeviceName, szDevicePath);
    HCHANNEL hChannel = MWOpenChannelByPath(szDevicePath);
    if (hChannel == -1)
        return PRAISE_RESULT_UNKNOW_ERROR;

    MWCAP_CHANNEL_INFO channelInfo;
    mr = MWGetChannelInfo(hChannel, &channelInfo);
    if (mr != MW_SUCCEEDED)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
        return PRAISE_RESULT_UNKNOW_ERROR;
    }

    sprintf(pChannelInfo->szChannelName, "%02d:%02d %s",
            channelInfo.byBoardIndex,
            channelInfo.byChannelIndex,
            channelInfo.szProductName);
    sprintf(pChannelInfo->szFirmwareVerCur, "%d.%d",
            (DWORD)(channelInfo.dwFirmwareVersion >> 16) & 0xFFFF,
            (DWORD)(channelInfo.dwFirmwareVersion) & 0xFFFF);

    pChannelInfo->chHardwareVersion = channelInfo.chHardwareVersion;

    MW_FIRMWARE_HEADER firmwareHeader;
    BYTE *apbySectionData[MW_MAX_NUM_FIRMWARE_SECTIONS] = {NULL};

    mr = GetUpgradeDataFromMWFFile(hChannel,g_szMWFFileList, g_nMWFFileNum, &firmwareHeader, apbySectionData);
    if (mr == MW_SUCCEEDED)
        sprintf(pChannelInfo->szFirmwareVerNew, "%d.%d",
                (DWORD)(firmwareHeader.infoHeader.dwFirmwareVersion >> 16) & 0xFFFF,
                (DWORD)(firmwareHeader.infoHeader.dwFirmwareVersion) & 0xFFFF);


    if (channelInfo.dwFirmwareVersion >= firmwareHeader.infoHeader.dwFirmwareVersion )
        pChannelInfo->bNeedUpgrade = false;
    else
    {
        g_nDevicesNeedUpgrade++;
        pChannelInfo->bNeedUpgrade = true;
    }

    MWCAP_FIRMWARE_STORAGE storageInfo;
    mr = MWGetFirmwareStorageInfo(hChannel, &storageInfo);
    if (mr != MW_SUCCEEDED)
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
        return PRAISE_RESULT_UNKNOW_ERROR;
    }

    MW_FIRMWARE_INFO_HEADER firmwareInfoHeader;
    DWORD dwRead;

    mr = MWReadFirmwareData(hChannel, storageInfo.cbHeaderOffset, (BYTE *)&firmwareInfoHeader, sizeof(MW_FIRMWARE_INFO_HEADER), &dwRead);
    if ((mr != MW_SUCCEEDED) || (dwRead != sizeof(MW_FIRMWARE_INFO_HEADER)))
    {
        MWCloseChannel(hChannel);
        hChannel = -1;
        return PRAISE_RESULT_UNKNOW_ERROR;
    }

    WORD wLayoutVersion;
    if (firmwareInfoHeader.dwFirmwareVersion != channelInfo.dwFirmwareVersion)
    {
        wLayoutVersion = 0;
    } else {
        wLayoutVersion = firmwareInfoHeader.wVersion;
    }

    pChannelInfo->bSafeUpgrade = ((firmwareHeader.infoHeader.wVersion == wLayoutVersion) && (firmwareHeader.infoHeader.wVersion != 1));

    MWCloseChannel(hChannel);
    hChannel = -1;

    FreeSectionData(apbySectionData, firmwareHeader.infoHeader.cSections);

    return PRAISE_RESULT_OK;

}

static MWCAP_PRAISE_RESULT GetDevicesInfo( MWCAP_UPGRADE_LIST_INFO *arrChannelInfos, int *nCount,int nBoardId, int nChannelId)
{
    MW_RESULT mr;
    MWCAP_PRAISE_RESULT pr;
    int nChannelNum = MWGetChannelCount();

    int i;
    int nTemp = 0;

    for (i = 0; i < nChannelNum && nTemp < *nCount; i++)
    {
        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(i, &channelInfo);
        if (mr != MW_SUCCEEDED)
            continue;

        if (nBoardId != channelInfo.byBoardIndex && nBoardId != -1)
            continue;

        if (nChannelId != channelInfo.byChannelIndex && nChannelId != -1)
            continue;


        char szDevicePath[MAX_CHANNEL_NAME_LEN];
        mr = MWGetDevicePath(i, szDevicePath);
        if (mr != MW_SUCCEEDED)
            continue;

        pr = GetDeviceInfo(szDevicePath, &arrChannelInfos[nTemp]);
        if ( pr != PRAISE_RESULT_OK)
            return pr;

        nTemp ++;
    }

    *nCount = nTemp;

    return PRAISE_RESULT_OK;
}

static MWCAP_PRAISE_RESULT PraiseDeviceArg(char *szArg, MWCAP_UPGRADE_LIST_INFO *arrChannelsInfo, int *nDevicesNum)
{
    MWCAP_PRAISE_RESULT pr;

    if (szArg == NULL)
        return PRAISE_RESULT_ERROR_ARGUMENT;

    int nTemp = 0;

    if (strchr(szArg, ':') == NULL)
    {

        if (nTemp < *nDevicesNum)
            pr = GetDeviceInfo(optarg, &arrChannelsInfo[nTemp]);

        if (pr != PRAISE_RESULT_OK)
            return pr;

        nTemp++;
        *nDevicesNum = nTemp;
    }
    else
    {
        char *p;
        char *temp = strdup(szArg);
        char *separator = ":";
        int nCount = 0;

        p = strtok(temp, separator);
        while(p != NULL)
        {
            switch(nCount)
            {
            case 0:
            {
                g_nBoardIndex = (BYTE)atoi(p);
                break;
            }
            case 1:
            {
                g_nChannelIndex = (BYTE)atoi(p);
                break;
            }
            default:
                break;
            }
            nCount++;

            p = strtok(NULL, separator);
        }

        if (nCount != 2)
            return PRAISE_RESULT_ERROR_ARGUMENT;

        free(temp);
        temp = NULL;


        pr = GetDevicesInfo(arrChannelsInfo, nDevicesNum, g_nBoardIndex, g_nChannelIndex);
        if (pr != PRAISE_RESULT_OK)
            return pr;
    }

    return PRAISE_RESULT_OK;
}


static MWCAP_PRAISE_RESULT PraseCommandLine (int argc, char *argv[])
{
    MWCAP_PRAISE_RESULT pr;
    int nIdx;
    int c;
    while ((c = getopt_long(argc, argv, short_options, long_options, &nIdx)) != -1) {
        if (c == '?')
            return false;

        switch(c)
        {
        case 'a':
        {     
            int nTemp = 64;
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_ALL);

            pr = GetDevicesInfo(g_arrChannsInfo, &nTemp, g_nBoardIndex, g_nChannelIndex);
            if ( pr != PRAISE_RESULT_OK )
                return pr;

           g_nDevicesNum = nTemp;

        }
            break;
        case 'b':
        {
            int nTemp = 64;
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_BOARD);
            g_nBoardIndex = atoi(optarg);

            pr = GetDevicesInfo(g_arrChannsInfo, &nTemp, g_nBoardIndex, g_nChannelIndex);
            if ( pr != PRAISE_RESULT_OK )
                return pr;

            g_nDevicesNum = nTemp;
        }
            break;
        case 'd':
        {
            int nTemp = 64;
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_DEVICE);

            pr = PraiseDeviceArg(optarg, g_arrChannsInfo, &nTemp);
            if (pr != PRAISE_RESULT_OK)
                return pr;

            g_nDevicesNum = nTemp;
        }
        break;
        case 'f':
        {
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_FOURCE);
        }
            break;
        case 'y':
        {
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_YES);
        }
            break;
        case 'l':
        {
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_LIST_CHANNEL_INFO);
            int nTemp = 64;
            pr = GetDevicesInfo(g_arrChannsInfo, &nTemp, g_nBoardIndex, g_nChannelIndex);
            if ( pr != PRAISE_RESULT_OK )
                return pr;

            g_nDevicesNum = nTemp;

        }
            break;
        case 'h':
        {
            g_dwCMD |= (0x01 << MWCAP_UPGRADE_USAGE);
        }
            break;
        default:
            break;

        }
    }

    if(((g_dwCMD & 0x01 << MWCAP_UPGRADE_ALL) && (g_dwCMD & 0x01 << MWCAP_UPGRADE_BOARD)) ||
            ((g_dwCMD & 0x01 << MWCAP_UPGRADE_ALL) && (g_dwCMD & 0x01 << MWCAP_UPGRADE_DEVICE)) ||
            ((g_dwCMD & 0x01 << MWCAP_UPGRADE_DEVICE) && (g_dwCMD & 0x01 << MWCAP_UPGRADE_BOARD)))
        return PRAISE_RESULT_ERROR_OPTION;

    return PRAISE_RESULT_OK;
}

static void ExitMWFList()
{
    if (g_szMWFFileList != NULL)
    {
        free(g_szMWFFileList);
        g_szMWFFileList = NULL;
    }

    return;
}

static MW_RESULT InitialMWFList()
{
    if (g_szMWFFileList != NULL)
    {
        free(g_szMWFFileList);
        g_szMWFFileList = NULL;
    }

    MW_RESULT xr;

    char arrCurPath[MAX_FILE_NAME_NUM];
    getcwd(arrCurPath, sizeof(arrCurPath));


    g_szMWFFileList = (char (*)[MAX_FILE_NAME_NUM]) malloc (sizeof(char) * MAX_FILE_COUNT * MAX_FILE_NAME_NUM);
    g_nMWFFileNum = MAX_FILE_COUNT;
    xr = GetMWFFileList(arrCurPath, g_szMWFFileList, &g_nMWFFileNum);
    if (xr != MW_SUCCEEDED)
    {
        ExitMWFList();
        return xr;
    }

    return MW_SUCCEEDED;
}

static void ListAllChannelInfo()
{
    printf("total %d  need upgrade %d\n", g_nDevicesNum, g_nDevicesNeedUpgrade);

    if (g_nDevicesNum > 0)
        printf("%-30s %-15s %-15s %-15s %-15s\n",
              "pro capture name", "current ver", "hardware ver", "new ver", "status");

    for (int i = 0; i < g_nDevicesNum; i++)
    {
        printf("%-30s %-15s %-15c %-15s %-15s\n",
               g_arrChannsInfo[i].szChannelName, g_arrChannsInfo[i].szFirmwareVerCur,
               g_arrChannsInfo[i].chHardwareVersion, g_arrChannsInfo[i].szFirmwareVerNew,
               g_arrChannsInfo[i].bNeedUpgrade ? "Need upgrade" : "Need not upgrade");
    }
}

static int mygetch(void)
{
    struct termios oldt,
    newt;
    int ch;
    tcgetattr( STDIN_FILENO, &oldt );
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
    return ch;
}


static MWCAP_UPGRADE_RESULT UpgradeInternel(HCHANNEL hChannel, int nCurIndex, int nTotalNum, BOOL bFullUpgrade, DWORD *pdwPercentages)
{
    if (hChannel == -1)
        return 0;

    MW_RESULT xr;

    MWCAP_CHANNEL_INFO channelInfo;
    xr = MWGetChannelInfo(hChannel, &channelInfo);
    if (xr != MW_SUCCEEDED)
        return RESULT_NORMAL;

    char szChannelName[MAX_FILE_NAME_NUM];
    sprintf(szChannelName, "%02d:%02d %s", channelInfo.byBoardIndex, channelInfo.byChannelIndex, channelInfo.szProductName);

    MW_FIRMWARE_HEADER firmwareHeader;
    BYTE *apbySectionData[MW_MAX_NUM_FIRMWARE_SECTIONS] = {NULL};

    xr = GetUpgradeDataFromMWFFile(hChannel,g_szMWFFileList, g_nMWFFileNum, &firmwareHeader, apbySectionData);
    if (xr != MW_SUCCEEDED)
    {
        FreeSectionData(apbySectionData, firmwareHeader.infoHeader.cSections);
        return RESULT_NO_FMDATA;
    }

    MWCAP_FIRMWARE_STORAGE storageInfo;
    xr = MWGetFirmwareStorageInfo(hChannel, &storageInfo);
    if (xr != MW_SUCCEEDED)
    {
        FreeSectionData(apbySectionData, firmwareHeader.infoHeader.cSections);
        return RESULT_NORMAL;
    }

    char szFirmwareVerCur[16];
    char szFirmwareVerNew[16];
    sprintf(szFirmwareVerCur, "%d.%d", (DWORD)(channelInfo.dwFirmwareVersion >> 16) & 0xFFFF, (DWORD)(channelInfo.dwFirmwareVersion) & 0xFFFF);
    sprintf(szFirmwareVerNew, "%d.%d", (DWORD)(firmwareHeader.infoHeader.dwFirmwareVersion >> 16) & 0xFFFF, (DWORD)(firmwareHeader.infoHeader.dwFirmwareVersion) & 0xFFFF);


    if (((g_dwCMD & (0x01 << MWCAP_UPGRADE_FOURCE)) == 0) && channelInfo.dwFirmwareVersion >= firmwareHeader.infoHeader.dwFirmwareVersion)
        return RESULT_NEEDNOT_UPGRADE;



    if (g_dwCMD & (0x01 << MWCAP_UPGRADE_YES))
    {
        printf ("[%d/%d] [%s]  %s from %s to %s   ",
                nCurIndex, nTotalNum, szChannelName,
                !bFullUpgrade ? "Safe upgrade" : "Upgrade",
                szFirmwareVerCur, szFirmwareVerNew);
        fflush(stdout);
    }
    else
    {
        printf ("[%d/%d] [%s]  Do you want to %s from %s to %s? -- (y)-YES or (n)-NO:", nCurIndex,
                nTotalNum, szChannelName,
                !bFullUpgrade ? "safe upgrade" : "upgrade",
                szFirmwareVerCur, szFirmwareVerNew);
        fflush(stdout);
    }

    char c = 'Y';

    if ((g_dwCMD & (0x01 << MWCAP_UPGRADE_YES)) == 0)
    {
        while (true)
        {
            c = mygetch();
            if (c == 'y' || c == 'Y' || c == 'n' || c == 'N')
                break;
        }

        if ( c == 'Y' || c == 'y')
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s   ",
                    nCurIndex, nTotalNum, szChannelName,
                    !bFullUpgrade ? "Safe upgrade" : "Upgrade",
                    szFirmwareVerCur, szFirmwareVerNew);
            fflush(stdout);
        }
    }


    if (c == 'Y' || c == 'y')
    {
        MWCAP_UPGRADE_RESULT bRet;
        bRet = UpgradeFirmware(hChannel, &firmwareHeader, apbySectionData, &storageInfo, bFullUpgrade, pdwPercentages);

        return bRet;
    }

    FreeSectionData(apbySectionData, firmwareHeader.infoHeader.cSections);


    return RESULT_INTERRUPT;
}


int main(int argc, char *argv[])
{

    MW_RESULT xr;

    MWCaptureInitInstance();
    MWRefreshDevice();
    xr = InitialMWFList();
    if (xr != MW_SUCCEEDED)
    {
        printf("Initial MWF file list failed\n");
        MWCaptureExitInstance();
        return 0;
    }

    if (g_nMWFFileNum == 0)
    {
        printf("No MWF file found in the current dirtionary\n");
        MWCaptureExitInstance();
        return 0;
    }

    MWCAP_PRAISE_RESULT pr;
    pr = PraseCommandLine (argc, argv);

    switch (pr)
    {
    case PRAISE_RESULT_ERROR_ARGUMENT:
    {
        printf("error argument, please use \"-h\" option to see usage\n");
        MWCaptureExitInstance();
        return -1;
    }
    case PRAISE_RESULT_ERROR_OPTION:
    {
        printf("option \"-a\" \"-b\" and \"-d\" can't not be used at the same time\n");
        MWCaptureExitInstance();
        return -1;
    }
        break;
    case PRAISE_RESULT_NOT_EXIT_DEVICE:
    {
        printf("device %s doesn't exit\n", optarg);
        MWCaptureExitInstance();
        return -1;
    }
    case PRAISE_RESULT_NOT_PRO_CAPTURE_DEVICE:
    {
        printf("device %s isn't a pro capture device\n", optarg);
        MWCaptureExitInstance();
        return -1;
    }
    case PRAISE_RESULT_OK:
        break;
    default:
        break;
    }


    if (g_dwCMD & (0x01 << MWCAP_UPGRADE_USAGE) || g_dwCMD == 0)
    {
        PrintUsage();
        MWCaptureExitInstance();
        return 0;
    }
    else if (g_dwCMD & (0x01 << MWCAP_UPGRADE_LIST_CHANNEL_INFO))
    {
        ListAllChannelInfo();
        MWCaptureExitInstance();
        return 0;
    }

    if (g_nDevicesNum <= 0)
    {
        printf("total %d, need upgrade %d\n", g_nDevicesNum, g_nDevicesNeedUpgrade);
        MWCaptureExitInstance();
        return 0;
    }

    printf("\e[1;31m"
           "/////////////////////////////////////////////////////////////////////////////////\n"
           "//                              !!! Waring !!!                                 //\n"
           "//  Upgrad firmware is an dangerous operation,so please close all applications //\n"
           "//before this operation and don't turn off the computer when upgrading firmware//\n"
           "/////////////////////////////////////////////////////////////////////////////////\n\n"
           "\e[0m");

    MWCAP_UPGRADE_RESULT bRet;
    HCHANNEL hChannel = -1;

    int nChannelCount = MWGetChannelCount();
    if (nChannelCount <= 0)
    {
        printf("No video channel is found\n");
        MWCaptureExitInstance();
        return 0;
    }

    //printf("total %d, need upgrade %d\n\n",
           //g_nDevicesNum, g_nDevicesNeedUpgrade);
    ListAllChannelInfo();
    printf("\n");

    int nUpgradeSuccessfully = 0;
    int nUpgradeFailed = 0;
    int nUpgradeCancle = 0;
    int nUpgradeSkip = 0;

    for(int i = 0; i < g_nDevicesNum; i++)
    {
        hChannel = MWOpenChannelByPath(g_arrChannsInfo[i].szDeviceName);
        if (hChannel == -1)
            continue;

        if (signal(SIGINT, SIG_IGN) == SIG_ERR)
        {
            MWCloseChannel(hChannel);
            hChannel = -1;
            continue;
        }

        DWORD dwPercentage = 0;
        bRet = UpgradeInternel(hChannel, i + 1, g_nDevicesNum, !g_arrChannsInfo[i].bSafeUpgrade, &dwPercentage);

        if (signal(SIGINT, SIG_DFL) == SIG_ERR)
        {
            MWCloseChannel(hChannel);
            hChannel = -1;
            continue;
        }


        switch (bRet)
        {
        case RESULT_NORMAL:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s error (unknow reason).",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                    g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade",
                    g_arrChannsInfo[i].szFirmwareVerCur, g_arrChannsInfo[i].szFirmwareVerNew);
            fflush(stdout);
            nUpgradeFailed++;
        }
            break;
        case RESULT_ERASE_ERROR:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s error (erasing at %d%%).",
                   i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                   g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade",
                   g_arrChannsInfo[i].szFirmwareVerCur, g_arrChannsInfo[i].szFirmwareVerNew, dwPercentage);
            fflush(stdout);
            nUpgradeFailed++;
        }
            break;
        case RESULT_WRITE_ERROR:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s error (writting at %d%%).",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                    g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade",
                    g_arrChannsInfo[i].szFirmwareVerCur, g_arrChannsInfo[i].szFirmwareVerNew, dwPercentage);
            fflush(stdout);
            nUpgradeFailed++;
        }
            break;
        case RESULT_VERIFY_ERROR:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s error (verifying at %d%%).",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                    g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade",
                    g_arrChannsInfo[i].szFirmwareVerCur, g_arrChannsInfo[i].szFirmwareVerNew, dwPercentage);
            fflush(stdout);
            nUpgradeFailed++;
        }
            break;
        case RESULT_NEEDNOT_UPGRADE:
        {
            printf ("\r[%d/%d] [%s]  Needn't upgrade",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName);
            fflush(stdout);
            nUpgradeSkip++;
        }
            break;
        case RESULT_INTERRUPT:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s cancled.",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                    g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade");
            fflush(stdout);
            nUpgradeCancle++;
        }
            break;
        case RESULT_UPGRADE_OK:
        {
            PrintClear(80);
            printf ("\r[%d/%d] [%s]  %s from %s to %s successfully.",
                    i+1, g_nDevicesNum, g_arrChannsInfo[i].szChannelName,
                    g_arrChannsInfo[i].bSafeUpgrade ? "Safe upgrade" : "Upgrade",
                    g_arrChannsInfo[i].szFirmwareVerCur, g_arrChannsInfo[i].szFirmwareVerNew);
            fflush(stdout);
            nUpgradeSuccessfully++;
        }
            break;
        default:
            break;

        }

        MWCloseChannel(hChannel);
        hChannel = -1;

        printf("\n");
    }

    printf("\n");
    printf("total %d,  success %d,  fail %d,  skip %d,  cancled %d\n", g_nDevicesNum,
           nUpgradeSuccessfully, nUpgradeFailed, nUpgradeSkip, nUpgradeCancle);


    ExitMWFList();
    g_nMWFFileNum = 0;

    MWCaptureExitInstance();

    if (nUpgradeFailed == 0 && nUpgradeSuccessfully > 0)
    {
        printf ("======================================================================\n"
                "\e[7m"
                "Need turn off and then turn on to reload the upgraded firmware!\n"
                "Do you want to turn off your computer now? -- (y)-YES or (n)-NO:"
                "\e[0m");

        char c;
        while (true)
        {
            c = mygetch();
            if (c == 'y' || c == 'Y' || c == 'n' || c == 'N')
                break;
        }
        printf("\n");

        if (c == 'y' || c == 'Y')
        {
            system("sync; if [ `id -u` -ne 0 ]; then  sudo poweroff; echo; else poweroff; echo; fi");
        }
    }

    return 0;
}


