#ifndef XIDSETTINGS_H
#define XIDSETTINGS_H
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "win-types.h"
#include "mw-common.h"
#include "lib-mw-capture.h"


#define LEN_BLOCK (1024 * 64)

#define CLEAR(x) memset(&(x), 0, sizeof(x))

#define MAX_FILE_NAME_NUM           256
#define MAX_FILE_COUNT              32

#ifndef LPBYTE
#define LPBYTE BYTE *
#endif

typedef enum _MWCAP_UPGRADE_RESULT_{
    RESULT_NORMAL = 0,
    RESULT_NO_FMDATA,
    RESULT_NEEDNOT_UPGRADE,
    RESULT_ERASING,
    RESULT_ERASE_ERROR,
    RESULT_WRITTING,
    RESULT_WRITE_ERROR,
    RESULT_VERIFYING,
    RESULT_VERIFY_END,
    RESULT_VERIFY_ERROR,
    RESULT_RECONFIG_ERROR,
    RESULT_DISABLE,
    RESULT_ENABLE,
    RESULT_DISABLE_REBOOT,
    RESULT_ENABLE_REBOOT,
    RESULT_INTERRUPT,
    RESULT_UPGRADE_OK,
    RESULT_UPGRADE_REBOOT,
} MWCAP_UPGRADE_RESULT;

static void PrintClear(int nCount)
{
    if (nCount > 256)
        return;

    char szBackSpace[256] = {0};
    char szBlankSpace[256] = {0};

    memset(szBackSpace, '\b', nCount);
    memset(szBlankSpace, ' ', nCount);
    printf("%s", szBackSpace);
    printf("%s", szBlankSpace);
    printf("%s", szBackSpace);
}

static VOID FreeSectionData(LPBYTE apbySectionData[], int cSections) {
    for (int i = 0; i < cSections; i++) {
        if (apbySectionData[i])
            free(apbySectionData[i]);

        apbySectionData[i] = NULL;
    }
}

static MW_RESULT GetMWFFileList(char *szDirPath, char (*szMWFFile)[MAX_FILE_NAME_NUM], int *pnCount)
{
    if (szDirPath == NULL || szMWFFile == NULL || *pnCount <= 0)
        return MW_FAILED;

    DIR *dir;

    struct dirent *ptr;
    dir = opendir(szDirPath);
    if (dir == NULL)
    {
        printf("Open dir error\n");
        *pnCount = 0;
        return MW_FAILED;
    }

    int nTemp = 0;
    while ((ptr = readdir(dir)) != NULL && *pnCount > 0)
    {
	struct stat sb;
	if (stat(ptr -> d_name, &sb) == 0) 
	{
		if (S_ISREG(sb.st_mode)) 
		{
			if (strstr(ptr->d_name, ".mwf") == NULL)
                		continue;

            		strcpy(*(szMWFFile + nTemp), ptr->d_name);
            		nTemp ++;
            		(*pnCount)--;

		}
	}		
    }

    *pnCount = nTemp;
    return MW_SUCCEEDED;
}



static BOOL READMWF(const char * szPath, MW_FIRMWARE_HEADER *pHeader, BYTE *apbySectionData[])
{
    FILE *fp = fopen(szPath, "rb");

    BOOL bRet = (fp != NULL);

    if (bRet)
        bRet = (sizeof(pHeader->infoHeader) == fread(&pHeader->infoHeader, 1, sizeof(pHeader->infoHeader), fp));

    if (bRet)
        bRet = (pHeader->infoHeader.dwMagic == MW_FIRMWARE_HEADER_MAGIC);

    int cbSectionHeaders;

    if (bRet) {
        cbSectionHeaders = pHeader->infoHeader.cSections * sizeof(MW_FIRMWARE_SECTION_HEADER);
        bRet = (pHeader->infoHeader.cSections <= MW_MAX_NUM_FIRMWARE_SECTIONS
            && pHeader->infoHeader.cbHeader == (sizeof(pHeader->infoHeader) + cbSectionHeaders));
    }

    if (bRet)
        bRet = (cbSectionHeaders == fread(pHeader->aSectionHeaders, 1, cbSectionHeaders, fp));

    for (int i = 0; bRet && i < pHeader->infoHeader.cSections; i++) {
        apbySectionData[i] = (LPBYTE)malloc(pHeader->aSectionHeaders[i].cbSection);

        if (NULL == apbySectionData[i])
            bRet = FALSE;
        else
            bRet = (pHeader->aSectionHeaders[i].cbSection == fread(apbySectionData[i], 1, pHeader->aSectionHeaders[i].cbSection, fp));
    }

    if (!bRet)
        FreeSectionData(apbySectionData, pHeader->infoHeader.cSections);

    if (fp)
        fclose(fp);

    return bRet;
}

static MW_RESULT GetUpgradeDataFromMWFFile(HCHANNEL hChannel, char (*szMWFFile)[MAX_FILE_NAME_NUM], int nCount, MW_FIRMWARE_HEADER *firmwareHeader, BYTE *apbySectionData[])
{
    if (hChannel == -1 || szMWFFile == NULL || nCount == 0)
        return MW_FAILED;

    MWCAP_CHANNEL_INFO channelInfo;
    CLEAR(channelInfo);
    MWGetChannelInfo(hChannel, &channelInfo);

    for (int i = 0; i < nCount; i++)
    {

        BOOL bRet;

        bRet = READMWF(*(szMWFFile + i), firmwareHeader, apbySectionData);
        if (bRet == FALSE)
           printf("Read %s file failed\n", *(szMWFFile + i));
        else
        {
            if (firmwareHeader->infoHeader.chHardwareVersion == channelInfo.chHardwareVersion &&
                    firmwareHeader->infoHeader.wProductID == channelInfo.wProductID)
            {
                return MW_SUCCEEDED;
            }

            FreeSectionData(apbySectionData, firmwareHeader->infoHeader.cSections);

        }
    }

    return MW_FAILED;
}

BOOL IsMatchedSection(BOOL bFullUpgrade, const MW_FIRMWARE_SECTION_HEADER * pSectionHeader) {
        BOOL bUpgradeSection = (strncmp(pSectionHeader->szName, "Upgrade.", 8) == 0);
        return !((bFullUpgrade && bUpgradeSection) || (!bFullUpgrade && !bUpgradeSection));
    }


BOOL IsCLRSection(const char* szName) {
        return (strstr(szName, "Factory.CLR.") != NULL || strstr(szName, "Upgrade.CLR") != NULL);
    }


static MWCAP_UPGRADE_RESULT UpgradeFirmware(HCHANNEL hChannel, MW_FIRMWARE_HEADER * pHeader, BYTE *apbySectionData[], MWCAP_FIRMWARE_STORAGE *pStorageInfo, BOOL bFullUpgrade, DWORD *dwPercentage)
{

    DWORD cbHeaderOffset = pStorageInfo->cbHeaderOffset;
    if(!bFullUpgrade)
        cbHeaderOffset += 0x1000;

    DWORD dwTotal = pHeader->infoHeader.cbHeader;
    DWORD dwCurrent = 0;
    for (int i = 0; i < pHeader->infoHeader.cSections; i++) {
        if (!IsMatchedSection(bFullUpgrade, &pHeader->aSectionHeaders[i]))
                continue;

        dwTotal += pHeader->aSectionHeaders[i].cbSection;
    }



    MW_RESULT xr;
    MWCAP_UPGRADE_RESULT bRet = RESULT_NORMAL;
    char szTemp[256];
    do
    {
        // Erase firmwre
        bRet = RESULT_ERASING;
        printf("(Erasing ------------- %2d%%)", 0);
        sprintf(szTemp,"(Erasing ------------- %2d%%)", 10);
        fflush(stdout);
        xr = MWEraseFirmwareData(hChannel, cbHeaderOffset, pHeader->infoHeader.cbHeader);
        if (xr != MW_SUCCEEDED)
        {
            bRet = RESULT_ERASE_ERROR;
            *dwPercentage = (dwCurrent * 100)/dwTotal;
            PrintClear(strlen(szTemp) + 1);
            fflush(stdout);
            break;
        }

        dwCurrent += pHeader->infoHeader.cbHeader;
        PrintClear(strlen(szTemp) + 1);
        printf("(Erasing ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
        fflush(stdout);

        for (int i = 0; i < pHeader->infoHeader.cSections; i++)
        {
            if (!IsMatchedSection(bFullUpgrade, &pHeader->aSectionHeaders[i]))
                continue;

            DWORD dwErased = 0;

            while (dwErased < pHeader->aSectionHeaders[i].cbSection)
            {
                DWORD cbCurrent = (pHeader->aSectionHeaders[i].cbSection - dwErased >= LEN_BLOCK) ? LEN_BLOCK : (pHeader->aSectionHeaders[i].cbSection - dwErased);
                xr = MWEraseFirmwareData(hChannel, pHeader->aSectionHeaders[i].cbOffset + dwErased, cbCurrent);
                if (xr != MW_SUCCEEDED)
                {
                    bRet = RESULT_ERASE_ERROR;
                    *dwPercentage = (dwCurrent * 100)/dwTotal;
                    PrintClear(strlen(szTemp) + 1);
                    fflush(stdout);
                    break;
                }

                dwErased	+= cbCurrent;
                dwCurrent	+= cbCurrent;

                PrintClear(strlen(szTemp));
                printf("(Erasing ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
                fflush(stdout);
            }

            if (bRet == RESULT_ERASE_ERROR)
                break;

            //  clr area check
            if (IsCLRSection(pHeader->aSectionHeaders[i].szName))
            {
                BYTE byVerifyData[LEN_BLOCK];
                memset(byVerifyData, 0xFF, LEN_BLOCK);

                DWORD cbReadData = 0;
                BYTE byReadData[LEN_BLOCK];

                DWORD dwRead = 0;
                while (dwRead < pHeader->aSectionHeaders[i].cbSection)
                {
                    DWORD cbCurrent = (pHeader->aSectionHeaders[i].cbSection - dwRead >= LEN_BLOCK) ? LEN_BLOCK : (pHeader->aSectionHeaders[i].cbSection - dwRead);
                    xr = MWReadFirmwareData(hChannel, pHeader->aSectionHeaders[i].cbOffset + dwRead, byReadData, cbCurrent, &cbReadData);
                    if (xr != MW_SUCCEEDED || cbReadData != cbCurrent || memcmp(byReadData, byVerifyData, cbCurrent) != 0)
                    {
                        bRet = RESULT_ERASE_ERROR;
                        *dwPercentage = (dwCurrent * 100)/dwTotal;
                        PrintClear(strlen(szTemp) + 1);
                        fflush(stdout);
                        break;
                    }

                    dwRead += cbCurrent;
                }

                if (bRet == RESULT_ERASE_ERROR)
                    break;
            }
        }

        PrintClear(strlen(szTemp) + 1);
        fflush(stdout);

        if (bRet == RESULT_ERASE_ERROR)
            break;

        //Write
        bRet = RESULT_WRITTING;
        printf("(Writting ------------- %2d%%)", 0);
        fflush(stdout);

        sprintf(szTemp,"(Writting ------------- %2d%%)", 10);
        dwCurrent = 0;
        xr = MWWriteFirmwareData(hChannel, cbHeaderOffset, (BYTE *)pHeader, pHeader->infoHeader.cbHeader);
        if (xr != MW_SUCCEEDED)
        {
            bRet = RESULT_WRITE_ERROR;
            *dwPercentage = (dwCurrent * 100)/dwTotal;
            PrintClear(strlen(szTemp));
            fflush(stdout);
            break;
        }

        dwCurrent += pHeader->infoHeader.cbHeader;
        PrintClear(strlen(szTemp));
        printf("(Writting ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
        fflush(stdout);

        for (int i = 0; i < pHeader->infoHeader.cSections; i++)
        {
            if (!IsMatchedSection(bFullUpgrade, &pHeader->aSectionHeaders[i]))
                continue;

            if (IsCLRSection(pHeader->aSectionHeaders[i].szName))
                continue;

            DWORD dwWritten	= 0;
            while (dwWritten < pHeader->aSectionHeaders[i].cbSection)
            {
                DWORD cbCurrent = (pHeader->aSectionHeaders[i].cbSection - dwWritten >= LEN_BLOCK) ? LEN_BLOCK : (pHeader->aSectionHeaders[i].cbSection - dwWritten);
                xr = MWWriteFirmwareData(hChannel, pHeader->aSectionHeaders[i].cbOffset + dwWritten, apbySectionData[i] + dwWritten, cbCurrent);

                if (xr != MW_SUCCEEDED)
                {
                    bRet = RESULT_WRITE_ERROR;
                    *dwPercentage = (dwCurrent * 100)/dwTotal;
                    PrintClear(strlen(szTemp));
                    fflush(stdout);
                    break;
                }

                //printf("cbReadData = %d, cbCurrent = %d, ret = %d\n", cbReadData, cbCurrent, ret);
                dwWritten	+= cbCurrent;
                dwCurrent	+= cbCurrent;
                PrintClear(strlen(szTemp));
                printf("(Writting ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
                fflush(stdout);
            }

            if (bRet == RESULT_WRITE_ERROR)
                break;
        }

        PrintClear(strlen(szTemp));
        fflush(stdout);

        if (bRet == RESULT_WRITE_ERROR)
            break;

        //verify
        bRet = RESULT_VERIFYING;
        printf("(Verifying ------------- %2d%%)", 0);
        fflush(stdout);
        sprintf(szTemp,"(Verifying ------------- %2d%%)", 10);
        DWORD cbReadData = 0;
        BYTE byReadData[LEN_BLOCK];
        dwCurrent = 0;

        //hr = filter.ReadFirmwareData(m_arrDevice[nIndex].storageInfo.cbHeaderOffset, byReadData, pHeader->infoHeader.cbHeader, &cbReadData);
        xr = MWReadFirmwareData(hChannel, cbHeaderOffset, byReadData, pHeader->infoHeader.cbHeader,  &cbReadData);
        if (xr!= MW_SUCCEEDED || cbReadData != pHeader->infoHeader.cbHeader || memcmp(byReadData, pHeader, pHeader->infoHeader.cbHeader) != 0) {
            bRet = RESULT_VERIFY_ERROR;
            *dwPercentage = (dwCurrent * 100)/dwTotal;
            PrintClear(strlen(szTemp) + 1);
            fflush(stdout);
            break;
        }

        dwCurrent += pHeader->infoHeader.cbHeader;
        PrintClear(strlen(szTemp) + 1);
        printf("(Verifying ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
        fflush(stdout);


        for (int i = 0; i < pHeader->infoHeader.cSections; i++)
        {
            if (!IsMatchedSection(bFullUpgrade, &pHeader->aSectionHeaders[i]))
                continue;

            if (IsCLRSection(pHeader->aSectionHeaders[i].szName))
                continue;

            DWORD dwRead = 0;

            while (dwRead < pHeader->aSectionHeaders[i].cbSection)
            {
                DWORD cbCurrent = (pHeader->aSectionHeaders[i].cbSection - dwRead >= LEN_BLOCK) ? LEN_BLOCK : (pHeader->aSectionHeaders[i].cbSection - dwRead);
                //hr = filter.ReadFirmwareData(pHeader->aSectionHeaders[i].cbOffset + dwRead, byReadData, cbCurrent, &cbReadData);
                xr = MWReadFirmwareData(hChannel, pHeader->aSectionHeaders[i].cbOffset + dwRead, byReadData, cbCurrent, &cbReadData);
                if (xr != MW_SUCCEEDED || cbReadData != cbCurrent || memcmp(byReadData, apbySectionData[i] + dwRead, cbCurrent) != 0)
                {
                    printf("read error %d\n", dwRead);
                    bRet = RESULT_VERIFY_ERROR;
                    *dwPercentage = (dwCurrent * 100)/dwTotal;
                    PrintClear(strlen(szTemp) + 1);
                    fflush(stdout);
                    break;
                }

                if (bRet != RESULT_VERIFYING)
                    return bRet;

                dwRead		+= cbCurrent;
                dwCurrent	+= cbCurrent;
                PrintClear(strlen(szTemp));
                printf("(Verifying ------------- %2d%%)", (dwCurrent * 100)/dwTotal);
                fflush(stdout);
            }

            if (bRet == RESULT_VERIFY_ERROR)
                break;
        }

        if (bRet == RESULT_VERIFY_ERROR)
            break;

        PrintClear(strlen(szTemp) + 1);
        fflush(stdout);
        bRet = RESULT_VERIFY_END;

        //reconfig
        xr = MWSetPostReconfig(hChannel, 200);
        if (xr != MW_SUCCEEDED)
        {
            bRet = RESULT_RECONFIG_ERROR;
            printf("Set post reconfig failed");
            fflush(stdout);
            break;
        }

        bRet = RESULT_UPGRADE_OK;
    }while(false);


    return bRet;
}

#endif // XIDSETTINGS_H
