#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>

#include <sys/select.h>
#include "bitmap.h"

#include "lib-mw-capture.h"
#include "mw-fourcc.h"
#include "ProductVer.h"


#define NUM_CAPTURE 100

int main(void)
{ 
    MWCaptureInitInstance();
    MWRefreshDevice();

    BYTE nMajor = 0;
    BYTE nMinor = 0;
    WORD nBuild = 0;
    MWGetVersion(&nMajor, &nMinor, &nBuild);
    printf("MWCapture SDK for Linux V%d.%d.%d - video-capture\n", nMajor, nMinor, nBuild);
    // Enum Device

    HCHANNEL hChannel = -1;
    HNOTIFY hNotify = 0;
    MWCAP_PTR hNotifyEvent = 0;
    MWCAP_PTR hCaptureEvent = 0;

    HANDLE64 pbImage = 0;

    LONGLONG llTotalTime = 0LL;

    int nRet;

    do {
        int nChannelCount = MWGetChannelCount();
        if (nChannelCount == 0) {
            printf("ERROR: Can't find channels!\n");
            return -1;
        }
        printf("Pro Capture channel counts %d\n", nChannelCount);

        MWCAP_CHANNEL_INFO channelInfo = {0};
        if (MW_SUCCEEDED != MWGetChannelInfoByIndex(0, &channelInfo)) {
            printf("ERROR: Can't get channel info!\n");
            break;
        }


        // Open channel
        hChannel = MWOpenChannel(channelInfo.byBoardIndex, channelInfo.byChannelIndex);
        if (hChannel == -1) {
            printf("ERROR: Open channel error!\n");
            break;
        }

        printf("\nCapture channel info \n"
               "   %-13s : %d\n"
               "   %-13s : %d\n"
               "   %-13s : %s\n"
               "   %-13s : %s\n\n",
               "Board Index", channelInfo.byBoardIndex,
               "Channel Index", channelInfo.byChannelIndex,
               "Product Name", channelInfo.szProductName,
               "Serial Number", channelInfo.szBoardSerialNo);

        //create event
        hCaptureEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
        {
            printf("Create timer event error\n");
            break;
        }

        hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent == 0)
        {
            printf("Create notify event error\n");
            break;
        }

        // Allocate capture buffer
        int cx = 1920;
        int cy = 1080;
        DWORD dwFourCC = MWFOURCC_BGR24;
        //int nBpp = FOURCC_GetBpp(dwFourCC);
        DWORD dwMinStride = FOURCC_CalcMinStride(dwFourCC, cx, 4);
        DWORD dwImageSize = FOURCC_CalcImageSize(dwFourCC, cx, cy, dwMinStride);
        pbImage = (HANDLE64)(unsigned long)malloc(dwImageSize);


        MW_RESULT mr;
        mr = MWStartVideoCapture(hChannel, hCaptureEvent);
        if (mr != MW_SUCCEEDED) {
            printf("ERROR: Open Video Capture error!\n");
        }
        else {
            MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
            MWGetVideoBufferInfo(hChannel, &videoBufferInfo);

            MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
            MWGetVideoFrameInfo(hChannel, videoBufferInfo.iNewestBufferedFullFrame, &videoFrameInfo);

            MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
            MWGetVideoSignalStatus(hChannel, &videoSignalStatus);

            switch (videoSignalStatus.state) {
            case MWCAP_VIDEO_SIGNAL_NONE:
                printf("Input signal status: None\n");
                break;
            case MWCAP_VIDEO_SIGNAL_UNSUPPORTED:
                printf("Input signal status: Unsupported\n");
                break;
            case MWCAP_VIDEO_SIGNAL_LOCKING:
                printf("Input signal status: Locking\n");
                break;
            case MWCAP_VIDEO_SIGNAL_LOCKED:
                printf("Input signal status: Locked\n");
                break;
            }

            if (videoSignalStatus.state == MWCAP_VIDEO_SIGNAL_LOCKED) {
                double fps = (double)10000000LL / videoSignalStatus.dwFrameDuration;
                printf("\nInput Signal Info\n"
                       "   %-10s : %d x %d\n"
                       "   %-10s : %.2f\n"
                       "   %-10s : %d\n\n",
                       "Resolution", videoSignalStatus.cx, videoSignalStatus.cy,
                       "Fps", fps,
                       "Interlaced",videoSignalStatus.bInterlaced);

                hNotify = MWRegisterNotify(hChannel, hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED);

                if (hNotify == 0) {
                    printf("ERROR: Register Notify  error.\n");
                }
                else {
                    printf("Start to capture frames by %.2f fps -- %d frames", fps, 0);
                    fflush(stdout);

                    for (int i = 0; i < NUM_CAPTURE; i++) {
                        printf("\rStart to capture frames by %.2f fps -- %d frames", fps, i+1);
                        fflush(stdout);

                        nRet = MWWaitEvent(hNotifyEvent, 1000);
                        if (nRet <= 0)
                        {
                            printf("Error: wait notify error or timeout\n");
                            break;
                        }

                        ULONGLONG ullStatusBits = 0;
                        mr = MWGetNotifyStatus(hChannel, hNotify, &ullStatusBits);
                        if (mr != MW_SUCCEEDED)
                            continue;

                        mr = MWGetVideoBufferInfo(hChannel, &videoBufferInfo);
                        if (mr != MW_SUCCEEDED)
                            continue;

                        mr = MWGetVideoFrameInfo(hChannel, videoBufferInfo.iNewestBufferedFullFrame, &videoFrameInfo);
                        if (mr != MW_SUCCEEDED)
                            continue;

                        if (ullStatusBits & MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED) {
                            mr = MWCaptureVideoFrameToVirtualAddressEx(hChannel, videoBufferInfo.iNewestBufferedFullFrame, pbImage, dwImageSize, dwMinStride, 1, 0, dwFourCC, cx, cy,
                                0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
                            if (mr != MW_SUCCEEDED)
                                continue;

                            nRet = MWWaitEvent(hCaptureEvent, 1000);
                            if (nRet <= 0)
                            {
                                printf("Error: wait captur event error or timeout\n");
                                break;
                            }

                            MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
                            mr = MWGetVideoCaptureStatus(hChannel, &captureStatus);

                            LONGLONG llCurrent = 0LL;
                            mr = MWGetDeviceTime(hChannel, &llCurrent);

                            llTotalTime += (llCurrent - videoFrameInfo.allFieldBufferedTimes[0]);
                            char szBitmapName[64];
                            sprintf(szBitmapName, "./Video_%d.BMP", NUM_CAPTURE);
                            if (i == NUM_CAPTURE -1)
                                create_bitmap(szBitmapName, pbImage, dwImageSize, cx, cy);
                        }
                    }
                }
                MWUnregisterNotify(hChannel, hNotify);
                hNotify = 0;
            }
            MWStopVideoCapture(hChannel);
            printf("\nStop capture\n");
            printf("\nAverage frame duration is %lld ms.\n", (LONGLONG)(llTotalTime / (NUM_CAPTURE * 10000)));
            printf("Write 100th frame to file Video_%d\n", NUM_CAPTURE);
        }

    } while (FALSE);

    if (hChannel != -1)
    {

        MWCloseChannel(hChannel);
        hChannel = -1;
    }

    if (hNotifyEvent != 0)
    {
        MWCloseEvent(hNotifyEvent);
        hNotifyEvent = 0;
    }

    if (hCaptureEvent != 0)
    {
        MWCloseEvent(hCaptureEvent);
        hCaptureEvent = 0;
    }

    if (pbImage != 0)
    {
        free((void *)(unsigned long)pbImage);
        pbImage = 0;
    }

    MWCaptureExitInstance();

    //printf("\nPress Enter to exit!");
    //fflush(stdout);
    //getchar();

    return 0;
}

