#ifndef _SETTING_H
#define _SETTING_H

#include "lib-mw-capture.h"
#include "mw-fourcc.h"
#include "lodepng.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#pragma pack(push)
#pragma pack(1)

int i = 0;
int g_hChannel[MAX_CHANNEL_COUNT];
int g_nChannelCount;

void printResult(char * pRes, char *pCmdName)
{
    printf("%-35s........................\t[%s]\n",pCmdName, pRes);
}


int Init()
{
    int nRet = MWCaptureInitInstance();
    if (nRet != TRUE)
    {
        printResult("FALSE", "MWCaptureInitInstance");
        return -1;
    }
    printResult("TRUE","MWCaptureInitInstance");

    nRet = MWRefreshDevice();
    if (nRet != 0)
    {
        printResult("FALSE", "MWRefreshDevice");
        return -1;
    }
    printResult("TRUE","MWRefreshDevice");

    int nCount = MWGetChannelCount();
    if (nCount <= 0)
    {
        printResult("FALSE", "MWGetChannelCount");
        return -1;
    }
    printResult("TRUE","MWGetChannelCount");

    g_nChannelCount = nCount;
    int i = 0;
    for(i = 0; i < nCount; i++)
    {

        MWCAP_CHANNEL_INFO *pChannelInfo = (MWCAP_CHANNEL_INFO *)malloc(sizeof(MWCAP_CHANNEL_INFO));
        nRet = MWGetChannelInfoByIndex(i, pChannelInfo);
        if(nRet != 0)
        {
            printResult("FALSE", "MWGetChannelInfoByIndex");
            return -1;
        }
        g_hChannel[i] = MWOpenChannel(pChannelInfo->byBoardIndex, pChannelInfo->byChannelIndex);
        if(g_hChannel[i] == -1)
        {
            printResult("FALSE", "MWOpenChannel");
            return -1 ;
        }
    }
    printResult("TRUE","MWOpenChannel");

    return 0;

}

int closeChannel()
{
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCloseChannel(g_hChannel[i]);
    }
    printResult("TRUE", "MWCloseChannel");

    MWCaptureExitInstance();
    printResult("TRUE", "MWCaptureExitInstance");

    return 0;
}



void testMWGetVersion()
{
    char * strName = "MWGetVersion";
    unsigned char *pbyMaj = (unsigned char *)malloc(sizeof(unsigned char));
    unsigned char *pbyMin = (unsigned char *)malloc(sizeof(unsigned char));
    unsigned short *pwBuild = (unsigned short *)malloc(sizeof(unsigned short));

    int nRet = MWRefreshDevice();
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }

    nRet = MWGetVersion(pbyMaj, pbyMin, pwBuild);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWCaptureInitInstance()
{
    return;
}

void testMWCaptureExitInstance()
{
    return;
}

void testMWRefreshDevice()
{
    return;
}

void testMWGetChannelCount()
{
    return;
}

void testMWGetChannelInfoByIndex()
{
    return;
}

void testMWGetFamilyInfoByIndex()
{
    char * strName = "MWGetFamilyInfoByIndex";

    int nRet = 0;
    for(i = 0; i < g_nChannelCount; i++)
    {
        LPVOID pFamilyInfo = (LPVOID)malloc(sizeof(MWCAP_PRO_CAPTURE_INFO));
        nRet = MWGetFamilyInfoByIndex(i, pFamilyInfo, sizeof(MWCAP_PRO_CAPTURE_INFO));
        if(nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        free(pFamilyInfo);
        pFamilyInfo = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetDevicePath()
{
    char * strName = "MWGetDevicePath";
    int nRet = 0;
    for(i = 0; i < g_nChannelCount; i++)
    {
        char *pDevicePath = (char *)malloc(MAX_CHANNEL_NAME_LEN);
        nRet = MWGetDevicePath(i, pDevicePath);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        free(pDevicePath);
        pDevicePath = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void testMWOpenChannel()
{
    return;
}

void testMWOpenChannelByPath()
{
    char * strName = "MWOpenChannelByPath";

    char *pDevicePath = (char *)malloc(MAX_CHANNEL_NAME_LEN);
    for(int i = 0; i < g_nChannelCount; i++)
    {
        if(MWGetDevicePath(i, pDevicePath) != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        int hChannel = MWOpenChannelByPath(pDevicePath);
        if(hChannel == -1)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWCloseChannel()
{
    return;
}

void testMWGetChannelInfo()
{
    char * strName = "MWGetChannelInfo";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_CHANNEL_INFO channelInfo;
        int nRet = MWGetChannelInfo(g_hChannel[i], &channelInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}


void testMWGetFamilyInfo()
{
    char * strName = "MWGetFamilyInfo";

    for(i = 0; i < g_nChannelCount; i++)
    {
        LPVOID pFamilyInfo = (LPVOID)malloc(sizeof(MWCAP_PRO_CAPTURE_INFO));
        int nRet = MWGetFamilyInfo(g_hChannel[i], pFamilyInfo, sizeof(MWCAP_PRO_CAPTURE_INFO));
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        free(pFamilyInfo);
        pFamilyInfo = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoCaps()
{
    char * strName = "MWGetVideoCaps";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_CAPS  *pVideoCaps = (MWCAP_VIDEO_CAPS *)malloc(sizeof(MWCAP_VIDEO_CAPS));
        int nRet = MWGetVideoCaps(g_hChannel[i], pVideoCaps);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        free(pVideoCaps);
        pVideoCaps = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void  testMWGetAudioCaps()
{
    char * strName = "MWGetAudioCaps";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_AUDIO_CAPS  *pAudioCaps = (MWCAP_AUDIO_CAPS *)malloc(sizeof(MWCAP_AUDIO_CAPS));
        int nRet = MWGetAudioCaps(g_hChannel[i], pAudioCaps);
        if (nRet != 0)
        {
            printResult("FALSE", strName);

            return;
        }
        free(pAudioCaps);
        pAudioCaps = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoInputSourceArray()
{
    char * strName = "MWGetVideoInputSourceArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwVideoInputSource[7];
        DWORD dwVideoInputSourceCount = 7;
        int nRet = MWGetVideoInputSourceArray(g_hChannel[i], dwVideoInputSource, &dwVideoInputSourceCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);

    return;
}

void testMWGetAudioInputSourceArray()
{
    char * strName = "MWGetAudioInputSourceArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwAudioInputSources[5];
        DWORD dwAudioInputSourceCount = 5;
        int nRet = MWGetAudioInputSourceArray(g_hChannel[i], dwAudioInputSources, &dwAudioInputSourceCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);

            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetInputSourceScan()
{
    char * strName = "MWGetInputSourceScan";

    for(i = 0; i < g_nChannelCount; i++)
    {
       BOOLEAN bScan = FALSE;
        int nRet = MWGetInputSourceScan(g_hChannel[i], &bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetInputSourceScan()
{
    char * strName = "MWSetInputSourceScan";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan = TRUE;
        int nRet = MWSetInputSourceScan(g_hChannel[i], bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetAVInputSourceLink()
{
    char * strName = "MWGetAVInputSourceLink";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan;
        int nRet = MWGetAVInputSourceLink(g_hChannel[i], &bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetAVInputSourceLink()
{
    char * strName = "MWSetAVInputSourceLink";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan = TRUE;
        int nRet = MWSetAVInputSourceLink(g_hChannel[i], bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoInputSource()
{
    char * strName = "MWGetVideoInputSource";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCurSource;
        int nRet = MWGetVideoInputSource(g_hChannel[i], &dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}


void testMWSetVideoInputSource()
{
    char * strName = "MWSetVideoInputSource";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCurSource;
        int nRet = MWGetVideoInputSource(g_hChannel[i], &dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        nRet = MWSetVideoInputSource(g_hChannel[i], dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetAudioInputSource()
{
    char * strName = "MWGetAudioInputSource";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCurSource;
        int nRet = MWGetAudioInputSource(g_hChannel[i], &dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetAudioInputSource()
{
    char * strName = "MWSetAudioInputSource";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCurSource;
        int nRet = MWGetAudioInputSource(g_hChannel[i], &dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetAudioInputSource(g_hChannel[i], dwCurSource);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

//edid
void testMWGetEDID()
{
    char * strName = "MWGetEDID";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BYTE pbData[256];
        ULONG cbSize = 256;
        int nRet = MWGetEDID(g_hChannel[i], pbData,&cbSize);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetEDID()
{
    char * strName = "MWSetEDID";

    for(i = 0; i < g_nChannelCount; i++)
    {

        BYTE pbData[256];
        ULONG cbSize = 256;
        int nRet = MWGetEDID(g_hChannel[i], pbData, &cbSize);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }


        /*int*/ nRet = MWSetEDID(g_hChannel[i], pbData, cbSize);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

//signal status
void testMWGetInputSpecificStatus()
{
    char * strName = "MWGetInputSpecificStatus";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_INPUT_SPECIFIC_STATUS inputStatus;
        memset(&inputStatus, 0, sizeof(MWCAP_INPUT_SPECIFIC_STATUS));
        int nRet = MWGetInputSpecificStatus(g_hChannel[i],&inputStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoSignalStatus()
{
    char * strName = "MWGetVideoSignalStatus";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_SIGNAL_STATUS vSignalStatus;
        memset(&vSignalStatus, 0, sizeof(MWCAP_VIDEO_SIGNAL_STATUS));
        int nRet = MWGetVideoSignalStatus(g_hChannel[i],&vSignalStatus);
        if (nRet == MW_FAILED)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetAudioSignalStatus()
{
    char * strName = "MWGetAudioSignalStatus";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_AUDIO_SIGNAL_STATUS aSignalStatus;
        memset(&aSignalStatus, 0, sizeof(MWCAP_AUDIO_SIGNAL_STATUS));
        int nRet = MWGetAudioSignalStatus(g_hChannel[i],&aSignalStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetHDMIInfoFrameValidFlag()
{
    char * strName = "MWGetHDMIInfoFrameValidFlag";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwValidFlag;
        int nRet = MWGetHDMIInfoFrameValidFlag(g_hChannel[i],&dwValidFlag);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetHDMIInfoFramePacket()
{
    char * strName = "MWGetHDMIInfoFramePacket";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwValidFlag;
        int nRet = MWGetHDMIInfoFrameValidFlag(g_hChannel[i],&dwValidFlag);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        if (dwValidFlag == 0)
        {
           printResult("NO SIGNAL", strName);
           return;
        }
        DWORD arrInfoFrameIDs[10] = {0};
        DWORD nInfoFrameIDNum = 0;
        int j = 0;
        for (j = 0; j < 10 ; j++)
        {
            if ((dwValidFlag >> j) & 0x01)
            {
                arrInfoFrameIDs[nInfoFrameIDNum] = j;
                nInfoFrameIDNum++;
            }
        }
        j = 0;
        for (j = 0; j < nInfoFrameIDNum; j++)
        {
            HDMI_INFOFRAME_PACKET infoFrame;
            nRet = MWGetHDMIInfoFramePacket(g_hChannel[i], (MWCAP_HDMI_INFOFRAME_ID)arrInfoFrameIDs[j]/*MWCAP_HDMI_INFOFRAME_ID_AVI*/, &infoFrame);
            if(nRet != 0)
            {
                printResult("FALSE", strName);
                return;
            }
        }
    }
    printResult("TRUE", strName);

    return;
}

//device time
void testMWGetDeviceTime()
{
    char * strName = "MWGetDeviceTime";

    for(i = 0; i < g_nChannelCount; i++)
    {
        LONGLONG llTime = 0;
        int nRet = MWGetDeviceTime(g_hChannel[i], &llTime);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetDeviceTime()
{
    char * strName = "MWSetDeviceTime";

    for(i = 0; i < g_nChannelCount; i++)
    {
        LONGLONG llTime = 0;
        int nRet = MWGetDeviceTime(g_hChannel[i], &llTime);
        if(nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        nRet = MWSetDeviceTime(g_hChannel[i], llTime);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWRegulateDeviceTime()
{
    char * strName = "MWRegulateDeviceTime";

    for(i = 0; i < g_nChannelCount; i++)
    {
        LONGLONG llTime = 0;
	MW_RESULT nRet = MWGetDeviceTime(g_hChannel[i], &llTime);
	if(nRet != MW_SUCCEEDED)
	{
	    printResult("FALSE", strName);		
	    return;
	}
        nRet = MWRegulateDeviceTime(g_hChannel[i], llTime);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);

    return;
}
//timer event
void testMWRegisterTimer()
{
    char * strName = "MWRegisterTimer";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hEvent = MWCreateEvent();
        if (hEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }
        HTIMER hTimer = MWRegisterTimer(g_hChannel[i], hEvent);
        if (hTimer <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }
        int nRet = MWUnregisterTimer(g_hChannel[i], hTimer);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }
        MWCloseEvent(hEvent);
    }
    printResult("TRUE", strName);

    return;
}

void testMWUnregisterTimer()
{
    char * strName = "MWUnregisterTimer";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hEvent = MWCreateEvent();
        if (hEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }
        HTIMER hTimer = MWRegisterTimer(g_hChannel[i], hEvent);
        if (hTimer <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }
        int nRet = MWUnregisterTimer(g_hChannel[i], hTimer);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }
        MWCloseEvent(hEvent);
    }
    printResult("TRUE", strName);

    return;
}

void testMWScheduleTimer()
{
    char * strName = "MWScheduleTimer";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hEvent = MWCreateEvent();
        if (hEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }
        HTIMER hTimer = MWRegisterTimer(g_hChannel[i], hEvent);
        if (hTimer <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }

        LONGLONG llExpireTime = 10;
        int nRet = MWScheduleTimer(g_hChannel[i], hTimer, llExpireTime);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWUnregisterTimer(g_hChannel[i], hTimer);
            MWCloseEvent(hEvent);
            return;
        }
        nRet = MWUnregisterTimer(g_hChannel[i], hTimer);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hEvent);
            return;
        }
        MWCloseEvent(hEvent);
    }
    printResult("TRUE", strName);

    return;
}

//notify event
void testMWRegisterNotify()
{
    char * strName = "MWRegisterNotify";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }

        HNOTIFY hNotify = MWRegisterNotify(g_hChannel[i], hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING);
        if (hNotify <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }
        int nRet = MWUnregisterNotify(g_hChannel[i], hNotify);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }
        MWCloseEvent(hNotifyEvent);
    }
    printResult("TRUE", strName);

    return;
}

void testMWUnregisterNotify()
{
    char * strName = "MWUnregisterNotify";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }

        HNOTIFY hNotify = MWRegisterNotify(g_hChannel[i], hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING);
        if (hNotify <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }
        int nRet = MWUnregisterNotify(g_hChannel[i], hNotify);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }
        MWCloseEvent(hNotifyEvent);
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetNotifyStatus()
{
    char * strName = "MWGetNotifyStatus";

    for(i = 0; i < g_nChannelCount; i++)
    {
        HANDLE64 hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent <= 0)
        {
            printResult("FALSE", strName);
            return;
        }

        HNOTIFY hNotify = MWRegisterNotify(g_hChannel[i], hNotifyEvent, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING);
        if (hNotify <= 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }

        ULONGLONG ullStatus = 0;
        int nRet = MWGetNotifyStatus(g_hChannel[i], hNotify, &ullStatus);

        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWUnregisterNotify(g_hChannel[i], hNotify);
            MWCloseEvent(hNotifyEvent);
            return;
        }

        nRet = MWUnregisterNotify(g_hChannel[i], hNotify);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            return;
        }
        MWCloseEvent(hNotifyEvent);
    }
    printResult("TRUE", strName);

    return;
}

//video capture
void testMWStartVideoCapture()
{
    char * strName = "MWStartVideoCapture";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_PTR hCaptureEvent = MWCreateEvent();
        if ((hCaptureEvent <= 0) || (hCaptureEvent == 2))
        {
            printResult("FALSE", strName);
            return;
        }
        int nRet = MWStartVideoCapture(g_hChannel[i],hCaptureEvent);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWStopVideoCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        MWCloseEvent(hCaptureEvent);
    }
    printResult("TRUE", strName);

    return;
}

void testMWStopVideoCapture()
{
    char * strName = "MWStopVideoCapture";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_PTR hCaptureEvent = MWCreateEvent();
        if ((hCaptureEvent <= 0) || (hCaptureEvent == 2))
        {
            printResult("FALSE", strName);
            return;
        }
        int nRet = MWStartVideoCapture(g_hChannel[i],hCaptureEvent);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWStopVideoCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        nRet = MWCloseEvent(hCaptureEvent);
        if(nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);

    return;
}

void testMWPinVideoBuffer()
{
    char * strName = "MWPinVideoBuffer";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_PTR *pbFrame = (MWCAP_PTR *)malloc(10);
        memset(pbFrame, 0, 10);
        DWORD cbFrame = 10;
        int nRet = MWPinVideoBuffer(g_hChannel[i], (MWCAP_PTR)(unsigned long)pbFrame, cbFrame);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWUnpinVideoBuffer(g_hChannel[i], (MWCAP_PTR)(unsigned long)pbFrame);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        free(pbFrame);
        pbFrame = NULL;
    }
    printResult("TRUE", strName);

    return;
}

void testMWUnpinVideoBuffer()
{
    char * strName = "MWUnpinVideoBuffer";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_PTR *pbFrame = (MWCAP_PTR *)malloc(10);
        memset(pbFrame, 0, 10);
        DWORD cbFrame = 10;
        int nRet = MWPinVideoBuffer(g_hChannel[i], (MWCAP_PTR)(unsigned long)pbFrame, cbFrame);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopVideoCapture(g_hChannel[i]);
            return;
        }

        nRet = MWUnpinVideoBuffer(g_hChannel[i], (MWCAP_PTR)(unsigned long)pbFrame);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopVideoCapture(g_hChannel[i]);
            return;
        }

        free(pbFrame);
        pbFrame = NULL;
    }
    printResult("TRUE", strName);
    return;
}

void testMWCaptureVideoFrameToVirtualAddress()
{
    return;
}
void testMWCaptureVideoFrameToPhysicalAddress()
{
    return;
}
void testMWCaptureVideoFrameWithOSDToVirtualAddress()
{
    return;
}
void testMWCaptureVideoFrameWithOSDToPhysicalAddress()
{
    return;
}
void testMWCaptureVideoFrameToVirtualAddressEx()
{
    return;
}

void testMWCaptureVideoFrameToPhysicalAddressEx()
{
    return;
}

void testMWGetVideoBufferInfo()
{
    char * strName = "MWGetVideoBufferInfo";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
        int nRet = MWGetVideoBufferInfo(g_hChannel[i], &videoBufferInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWGetVideoFrameInfo()
{
    char * strName = "MWGetVideoFrameInfo";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
        int nRet = MWGetVideoBufferInfo(g_hChannel[i], &videoBufferInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
        nRet = MWGetVideoFrameInfo(g_hChannel[i], videoBufferInfo.iNewestBufferedFullFrame, &videoFrameInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWGetVideoCaptureStatus()
{
    return;
}

void testMWStartAudioCapture()
{
    char * strName = "MWStartAudioCapture";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nRet = MWStartAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }
        nRet = MWStopAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);
    return;
}

void testMWStopAudioCapture()
{
    char * strName = "MWStopAudioCapture";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nRet = MWStartAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }
        nRet = MWStopAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

    }
    printResult("TRUE", strName);
    return;
}

void testMWCaptureAudioFrame()
{
    char * strName = "MWCaptureAudioFrame";
    for(i = 0; i < g_nChannelCount; i++)
    {


        MWCAP_AUDIO_SIGNAL_STATUS audioStatus;
        int nRet = MWGetAudioSignalStatus(g_hChannel[i], &audioStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        if (audioStatus.bChannelStatusValid == 0)
        {
            printResult("NO SIGNAL", strName);
            return;
        }

        nRet = MWStartAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }

        MWCAP_PTR hNotifyEvent = MWCreateEvent();
        if (hNotifyEvent <= 0)
        {
            printResult("FALSE", strName);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }

        MWCAP_PTR hNotify = MWRegisterNotify(g_hChannel[i], hNotifyEvent, MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED);
        if (hNotify == 0)
        {
            MWCloseEvent(hNotifyEvent);
            MWStopAudioCapture(g_hChannel[i]);
            printResult("FALSE", strName);
            return;
        }

        nRet = MWWaitEvent(hNotifyEvent, 1000);
        if (nRet <= 0)
        {
            MWUnregisterNotify(g_hChannel[i], hNotify);
            MWCloseEvent(hNotifyEvent);
            MWStopAudioCapture(g_hChannel[i]);
            printResult("FALSE", strName);
            return;
        }

        ULONGLONG ullStatusBit;
        nRet = MWGetNotifyStatus(g_hChannel[i], hNotify, &ullStatusBit);
        if (nRet != 0)
        {
            MWUnregisterNotify(g_hChannel[i], hNotify);
            MWCloseEvent(hNotifyEvent);
            MWStopAudioCapture(g_hChannel[i]);
            printResult("FALSE", strName);
            return;
        }

        if (ullStatusBit & MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED)
        {
            MWCAP_AUDIO_CAPTURE_FRAME audioFrame;
            memset(&audioFrame, 0, sizeof(MWCAP_AUDIO_CAPTURE_FRAME));
            nRet = MWCaptureAudioFrame(g_hChannel[i], &audioFrame);
            if (nRet != 0)
            {
                printResult("FALSE", strName);
                MWUnregisterNotify(g_hChannel[i], hNotify);
                MWCloseEvent(hNotifyEvent);
                MWStopAudioCapture(g_hChannel[i]);
                return;
            }
        }

        nRet = MWUnregisterNotify(g_hChannel[i], hNotify);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWCloseEvent(hNotifyEvent);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }

        nRet = MWCloseEvent(hNotifyEvent);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            MWStopAudioCapture(g_hChannel[i]);
            return;
        }

        nRet = MWStopAudioCapture(g_hChannel[i]);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

//video processing
void testMWSetVideoInputAspectRatio()//NULL
{
    char * strName = "MWSetVideoInputAspectRatio";
    for(i = 0; i < g_nChannelCount; i++)
    {
//       int nAspectX = 4;
//       int nAspectY = 3;
        int nAspectX = 0;
        int nAspectY = 0;
        int nRet = MWGetVideoInputAspectRatio(g_hChannel[i], &nAspectX, &nAspectY);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
       nRet = MWSetVideoInputAspectRatio(g_hChannel[i], nAspectX, nAspectY);
       if(nRet != 0)
       {
            printResult("FALSE", strName);
           return;
       }
    }
    printResult("TRUE", strName);
    return;
}

void testMWGetVideoInputAspectRatio()
{
    char * strName = "MWGetVideoInputAspectRatio";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nAspectX = 0;
        int nAspectY = 0;
        int nRet = MWGetVideoInputAspectRatio(g_hChannel[i], &nAspectX, &nAspectY);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWSetVideoInputColorFormat()
{
    char * strName = "MWSetVideoInputColorFormat";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_COLOR_FORMAT colorFormat = MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN;
        int nRet = MWGetVideoInputColorFormat(g_hChannel[i], &colorFormat);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoInputColorFormat(g_hChannel[i], colorFormat);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWGetVideoInputColorFormat()
{
    char * strName = "MWGetVideoInputColorFormat";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_COLOR_FORMAT colorFormat = MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN;
        int nRet = MWGetVideoInputColorFormat(g_hChannel[i], &colorFormat);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWSetVideoInputQuantizationRange()
{
    char * strName = "MWSetVideoInputQuantizationRange";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_QUANTIZATION_RANGE quantization = MWCAP_VIDEO_QUANTIZATION_UNKNOWN;
        int nRet = MWGetVideoInputQuantizationRange(g_hChannel[i], &quantization);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoInputQuantizationRange(g_hChannel[i], quantization);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWGetVideoInputQuantizationRange()
{
    char * strName = "MWGetVideoInputQuantizationRange";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_QUANTIZATION_RANGE quantization = MWCAP_VIDEO_QUANTIZATION_UNKNOWN;
        int nRet = MWGetVideoInputQuantizationRange(g_hChannel[i], &quantization);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWSetLEDMode()
{
    char * strName = "MWSetLEDMode";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nRet = MWSetLEDMode(g_hChannel[i], MWCAP_LED_AUTO);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

//upgrade firmware
void testMWGetFirmwareStorageInfo()
{
    char * strName = "MWGetFirmwareStorageInfo";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_FIRMWARE_STORAGE storageInfo;
        int nRet = MWGetFirmwareStorageInfo(g_hChannel[i], &storageInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWEraseFirmwareData()
{
    return;
}

void testMWReadFirmwareData()
{
    char * strName = "MWReadFirmwareData";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_FIRMWARE_STORAGE storageInfo;
        int nRet = MWGetFirmwareStorageInfo(g_hChannel[i], &storageInfo);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MW_FIRMWARE_INFO_HEADER firmwareInfoHeader;
        DWORD dwRead;
        nRet = MWReadFirmwareData(g_hChannel[i], storageInfo.cbHeaderOffset, (BYTE *)&firmwareInfoHeader, sizeof(MW_FIRMWARE_INFO_HEADER), &dwRead);
        if ((nRet !=0) || (dwRead != sizeof(MW_FIRMWARE_INFO_HEADER)))
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWWriteFirmwareData()
{
    return;
}

void testMWSetPostReconfig()
{
    char * strName = "MWSetPostReconfig";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nRet = MWSetPostReconfig(g_hChannel[i], 200);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

//osd
void testMWCreateImage()
{
    char * strName = "MWCreateImage";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int cx = 1920;
        int cy = 1080;
        MWCAP_PTR pvImage = MWCreateImage(g_hChannel[i], cx, cy);
        if (pvImage == 0)
        {
            printResult("FALSE", strName);
            return;
        }

        LONG lCount = 0;
        int nRet = MWCloseImage(g_hChannel[i], pvImage, &lCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}

void testMWOpenImage()
{

    char * strName = "MWOpenImage";
    HOSD pvImage[MAX_CHANNEL_COUNT];
    for(i = 0; i < g_nChannelCount; i++)
    {
        int cx = 1920;
        int cy = 1080;
        /*HOSD*/ pvImage[i] = MWCreateImage(g_hChannel[i], cx, cy);
        if (pvImage[i] == 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    pid_t pid = 0;
    pid = fork();
    if (pid == 0)
    {
        int j = 0;
        int nFlag = 0;
        int hChannel[MAX_CHANNEL_COUNT];
        for (j = 0; j < g_nChannelCount; j++)
        {

            MWCAP_CHANNEL_INFO pChannelInfo;
            int nRet = MWGetChannelInfoByIndex(j, &pChannelInfo);
            if(nRet != 0)
            {
                nFlag = 1;
                printResult("FALSE", strName);
                break;
            }

            hChannel[j] = MWOpenChannel(pChannelInfo.byBoardIndex, pChannelInfo.byChannelIndex);
            if(hChannel[j] == -1)
            {
                nFlag = 1;
                printResult("FALSE", strName);
                hChannel[j] = 0;
                break;
            }
            long lCount = 0;
            nRet = MWOpenImage(hChannel[j], pvImage[j], &lCount);
            if (nRet != 0)
            {
                nFlag = 1;
                MWCloseImage(hChannel[j], pvImage[j], &lCount);
                MWCloseChannel(hChannel[j]);
                break;
            }
            nRet = MWCloseImage(hChannel[j],pvImage[j], &lCount);
            MWCloseChannel(hChannel[j]);
        }
        if (nFlag ==0)
            printResult("TRUE",strName);
        exit(0);
    }
    else if (pid > 0)
    {
        int status;
        waitpid(pid, &status, 0);
        for(i = 0; i < g_nChannelCount; i++)
        {
            long lCount = 0;
            int nRet = MWCloseImage(g_hChannel[i], pvImage[i], &lCount);
            if (nRet != 0)
            {
                printResult("FALSE", strName);
                return;
            }
        }
    }

    return;
}

void testMWCloseImage()
{
    char * strName = "MWCloseImage";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int cx = 1920;
        int cy = 1080;
        MWCAP_PTR pvImage = MWCreateImage(g_hChannel[i], cx, cy);
        if (pvImage == 0)
        {
            printResult("FALSE", strName);
            return;
        }

        long lCount;
        int nRet = MWCloseImage(g_hChannel[i], pvImage, &lCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);
    return;
}
void testMWUploadImageFromVirtualAddress()
{
    return;
}

void testMWUploadImageFromPhysicalAddress()
{
    return;
}

//temperature
void testMWGetTemperature()
{
    char * strName = "MWGetTemperature";
    for(i = 0; i < g_nChannelCount; i++)
    {
        unsigned int dwTemp = 0;
        int nRet = MWGetTemperature(g_hChannel[i], &dwTemp);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

//v4l2
void testMWGetStreamCount()
{
    char * strName = "MWGetStreamCount";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nStreamCount = 0;
        int nRet = MWGetStreamCount(g_hChannel[i], &nStreamCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetStreamInfos()
{

    char * strName = "MWGetStreamInfos";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nStreamCount = 0;
        int nRet = MWGetStreamCount(g_hChannel[i], &nStreamCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        MWCAP_STREAM_INFO streamInfo[32];
        nRet = MWGetStreamInfos(g_hChannel[i], streamInfo, &nStreamCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetCtrlStreamID()
{
    char * strName = "MWSetCtrlStreamID";
    for(i = 0; i < g_nChannelCount; i++)
    {
        int nStreamCount = 0;
        int nRet = MWGetStreamCount(g_hChannel[i], &nStreamCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        MWCAP_STREAM_INFO streamInfo[32];
        nRet = MWGetStreamInfos(g_hChannel[i], streamInfo, &nStreamCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        int nStreamId = streamInfo[0].stream_id;
        nRet = MWSetCtrlStreamID(g_hChannel[i], nStreamId);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoConnectFormat()
{
    char * strName = "MWGetVideoConnectFormat";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_CONNECTION_FORMAT connectFormat;
        int nRet = MWGetVideoConnectFormat(g_hChannel[i], &connectFormat);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoProcessSettings()
{
    char * strName = "MWGetVideoProcessSettings";
    for(i = 0; i < g_nChannelCount; i++)
    {

        MWCAP_VIDEO_PROCESS_SETTINGS processSettings;
        int nRet = MWGetVideoProcessSettings(g_hChannel[i], &processSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoProcessSettings()//null
{
    char * strName = "MWSetVideoProcessSettings";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_PROCESS_SETTINGS processSettings;
        int nRet = MWGetVideoProcessSettings(g_hChannel[i], &processSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoProcessSettings(g_hChannel[i], processSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoOSDSettings()
{
    char * strName = "MWGetVideoOSDSettings";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_OSD_SETTINGS osdSettings;
        int nRet = MWGetVideoOSDSettings(g_hChannel[i], &osdSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWSetVideoOSDSettings()
{
    char * strName = "MWSetVideoOSDSettings";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_OSD_SETTINGS osdSettings;
        int nRet = MWGetVideoOSDSettings(g_hChannel[i], &osdSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        /*int*/ nRet = MWSetVideoOSDSettings(g_hChannel[i], osdSettings);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWGetVideoOSDImage()
{
    char * strName = "MWGetVideoOSDImage";
    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_OSD_IMAGE osdImage;
        int nRet = MWGetVideoOSDImage(g_hChannel[i], &osdImage);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoOSDImage()
{
    char * strName = "MWSetVideoOSDImage";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_OSD_IMAGE osdImage;
        int nRet = MWGetVideoOSDImage(g_hChannel[i], &osdImage);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        /*int*/ nRet = MWSetVideoOSDImage(g_hChannel[i], osdImage);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoBrightness()
{
    char * strName = "MWGetVideoBrightness";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nBrightness;
        int nRet = MWGetVideoBrightness(g_hChannel[i], &nBrightness);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWSetVideoBrightness()
{
    char * strName = "MWSetVideoBrightness";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nBrightness = 0;
        int nRet = MWGetVideoBrightness(g_hChannel[i], &nBrightness);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        /*int*/ nRet = MWSetVideoBrightness(g_hChannel[i], nBrightness);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWGetVideoContrast()
{
    char * strName = "MWGetVideoContrast";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nContrast = 0;
        int nRet = MWGetVideoContrast(g_hChannel[i], &nContrast);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWSetVideoContrast()
{
    char * strName = "MWSetVideoContrast";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nContrast = 0;
        int nRet = MWGetVideoContrast(g_hChannel[i], &nContrast);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        /*int*/ nRet = MWSetVideoContrast(g_hChannel[i], nContrast);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    printResult("TRUE", strName);

    return;
}

void testMWGetVideoHue()
{
    char * strName = "MWGetVideoHue";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nHue = 0;
        int nRet = MWGetVideoHue(g_hChannel[i], &nHue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoHue()
{
    char * strName = "MWSetVideoHue";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nHue = 0;
        int nRet = MWGetVideoHue(g_hChannel[i], &nHue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        /*int*/ nRet = MWSetVideoHue(g_hChannel[i], nHue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoSaturation()
{
    char * strName = "MWGetVideoSaturation";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nSaturation = 0;
        int nRet = MWGetVideoSaturation(g_hChannel[i], &nSaturation);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoSaturation()
{
    char * strName = "MWSetVideoSaturation";

    for(i = 0; i < g_nChannelCount; i++)
    {
        int nSaturation = 0;
        int nRet = MWGetVideoSaturation(g_hChannel[i], &nSaturation);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
        /*int*/ nRet = MWSetVideoSaturation(g_hChannel[i], nSaturation);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

 // VGA/Component timings
void testMWGetVideoAutoHAlign()
{
    char * strName = "MWGetVideoAutoHAlign";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bAuto = TRUE;
        int nRet = MWGetVideoAutoHAlign(g_hChannel[i], &bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoAutoHAlign()
{
    char * strName = "MWSetVideoAutoHAlign";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bAuto = TRUE;
        int nRet = MWGetVideoAutoHAlign(g_hChannel[i], &bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoAutoHAlign(g_hChannel[i], bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoSamplingPhase()
{
    char * strName = "MWGetVideoSamplingPhase";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BYTE byValue = 0;
        int nRet = MWGetVideoSamplingPhase(g_hChannel[i], &byValue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoSamplingPhase()
{
    char * strName = "MWSetVideoSamplingPhase";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BYTE byValue = 0;
        int nRet = MWGetVideoSamplingPhase(g_hChannel[i], &byValue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoSamplingPhase(g_hChannel[i], byValue);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoSamplingPhaseAutoAdjust()
{
    char * strName = "MWGetVideoSamplingPhaseAutoAdjust";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bAuto = TRUE;
        int nRet = MWGetVideoSamplingPhaseAutoAdjust(g_hChannel[i], &bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoSamplingPhaseAutoAdjust()
{
    char * strName = "MWSetVideoSamplingPhaseAutoAdjust";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bAuto = TRUE;
        int nRet = MWGetVideoSamplingPhaseAutoAdjust(g_hChannel[i], &bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        nRet = MWSetVideoSamplingPhaseAutoAdjust(g_hChannel[i], bAuto);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetVideoTiming()
{
    char * strName = "MWSetVideoTiming";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan = TRUE;
        int nRet = MWSetInputSourceScan(g_hChannel[i], bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MWCAP_INPUT_SPECIFIC_STATUS inputStatus;
        memset(&inputStatus, 0, sizeof(MWCAP_INPUT_SPECIFIC_STATUS));
        nRet = MWGetInputSpecificStatus(g_hChannel[i], &inputStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        if (inputStatus.bValid != TRUE || INPUT_TYPE(inputStatus.dwVideoInputType) != MWCAP_VIDEO_INPUT_TYPE_VGA)
        {
            printResult("NO SIGNAL", strName);
            return;
        }

        MWCAP_VIDEO_TIMING videoTiming;
        videoTiming = inputStatus.vgaComponentStatus.videoTiming;
        nRet = MWSetVideoTiming(g_hChannel[i], videoTiming);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetVideoPreferredTimingArray()
{
    char * strName = "MWGetVideoPreferredTimingArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_TIMING videoTiming[32];
        LONG lSize = 32;
        int nRet = MWGetVideoPreferredTimingArray(g_hChannel[i], videoTiming, &lSize);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetCustomVideoTiming()
{
    char * strName = "MWSetCustomVideoTiming";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan = TRUE;
        int nRet = MWSetInputSourceScan(g_hChannel[i], bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MWCAP_INPUT_SPECIFIC_STATUS inputStatus;
        memset(&inputStatus, 0, sizeof(MWCAP_INPUT_SPECIFIC_STATUS));
        nRet = MWGetInputSpecificStatus(g_hChannel[i], &inputStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        if (inputStatus.bValid != TRUE || INPUT_TYPE(inputStatus.dwVideoInputType) != MWCAP_VIDEO_INPUT_TYPE_VGA)
        {
            printResult("NO SIGNAL", strName);
            return;
        }

        MWCAP_VIDEO_CUSTOM_TIMING videoTiming;
        videoTiming.syncInfo = inputStatus.vgaComponentStatus.syncInfo;
        videoTiming.videoTimingSettings = inputStatus.vgaComponentStatus.videoTimingSettings;

        nRet = MWSetCustomVideoTiming(g_hChannel[i], videoTiming);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetCustomVideoTimingsCount()
{
    char * strName = "MWGetCustomVideoTimingsCount";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCount = 0;
        int nRet = MWGetCustomVideoTimingsCount(g_hChannel[i], &dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetCustomVideoTimingsArray()
{
    char * strName = "MWGetCustomVideoTimingsArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_VIDEO_CUSTOM_TIMING videoTimings;
        memset(&videoTimings, 0, sizeof(MWCAP_VIDEO_CUSTOM_TIMING));

        DWORD dwCount = 1;
        int nRet = MWGetCustomVideoTimingsArray(g_hChannel[i], &videoTimings, &dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetCustomVideoTimingsArray()
{
    char * strName = "MWSetCustomVideoTimingsArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        BOOLEAN bScan = TRUE;
        int nRet = MWSetInputSourceScan(g_hChannel[i], bScan);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MWCAP_INPUT_SPECIFIC_STATUS inputStatus;
        memset(&inputStatus, 0, sizeof(MWCAP_INPUT_SPECIFIC_STATUS));
        nRet = MWGetInputSpecificStatus(g_hChannel[i], &inputStatus);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }

        MWCAP_VIDEO_CUSTOM_TIMING videoTiming;
        videoTiming.syncInfo = inputStatus.vgaComponentStatus.syncInfo;
        videoTiming.videoTimingSettings = inputStatus.vgaComponentStatus.videoTimingSettings;

        DWORD dwCount = 1;
        nRet = MWSetCustomVideoTimingsArray(g_hChannel[i], &videoTiming, dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}
void testMWGetCustomVideoResolutionsCount()
{
    char * strName = "MWGetCustomVideoResolutionsCount";

    for(i = 0; i < g_nChannelCount; i++)
    {
        DWORD dwCount = 0;
        int nRet = MWGetCustomVideoResolutionsCount(g_hChannel[i], &dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWGetCustomVideoResolutionsArray()
{
    char * strName = "MWGetCustomVideoResolutionsArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_SIZE resolutionSize[32];
        DWORD dwCount = 32;
        int nRet = MWGetCustomVideoResolutionsArray(g_hChannel[i], resolutionSize, &dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetCustomVideoResolutionsArray()
{
    char * strName = "MWSetCustomVideoResolutionsArray";

    for(i = 0; i < g_nChannelCount; i++)
    {
        MWCAP_SIZE resolutionSize;

        resolutionSize.cx = 1920;
        resolutionSize.cy = 1080;
        DWORD dwCount = 1;
        int nRet = MWSetCustomVideoResolutionsArray(g_hChannel[i], &resolutionSize, dwCount);
        if (nRet != 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }
    printResult("TRUE", strName);

    return;
}

//event
void testMWCreateEvent()
{
    char * strName = "MWCreateEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);
    MWCloseEvent(hEvent);

    return;
}

void testMWCloseEvent()
{
    char * strName = "MWCloseEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWSetEvent()
{
    char * strName = "MWSetEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWSetEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWClearEvent()
{
    char * strName = "MWClearEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWClearEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWIsSetEvent()
{
    char * strName = "MWIsSetEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWSetEvent(hEvent);
    if(nRet != 0)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWIsSetEvent(hEvent);
    if (nRet != 1)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWTryWaitEvent()
{
    char * strName = "MWTryWaitEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWTryWaitEvent(hEvent);
    if (nRet != FALSE)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWWaitEvent()
{
    char * strName = "MWWaitEvent";

    MWCAP_PTR hEvent = MWCreateEvent();
    if(hEvent <= 0)
    {
        printResult("FALSE", strName);
        return;
    }

    int nRet = MWWaitEvent(hEvent, 10);
    if (nRet != FALSE)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvent);
        return;
    }
    nRet = MWCloseEvent(hEvent);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        return;
    }
    printResult("TRUE", strName);

    return;
}

void testMWMultiWaitEvent()
{
    char * strName = "MWMultiWaitEvent";

    MWCAP_PTR hEvents[2];
    for(i = 0; i < 2; i++)
    {
        hEvents[i] = MWCreateEvent();
        if(hEvents[i] <= 0)
        {
            printResult("FALSE", strName);
            return;
        }
    }

    int nRet = MWSetEvent(hEvents[0]);
    if (nRet != 0)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvents[0]);
        MWCloseEvent(hEvents[1]);
        return;
    }
    nRet = MWMultiWaitEvent(hEvents, 2, 10);
    if (nRet <= 0)
    {
        printResult("FALSE", strName);
        MWCloseEvent(hEvents[0]);
        MWCloseEvent(hEvents[1]);
        return;
    }

    printResult("TRUE", strName);
    MWCloseEvent(hEvents[0]);
    MWCloseEvent(hEvents[1]);

    return;
}


#pragma pack(pop)
#endif /*_SETTING_H*/
