#include "audio-device-info.h"

PRO_CAPTURE_ALSA_DEVICE_INFO g_arrAudioDevicesInfo[MAX_ALSA_DEVICE_NUM] = {{{0}}};
int g_nAudioDevicesNum = 0;
static const char *g_V4l2SysPath = "/sys/class/video4linux";


int _FindPCISlotName(const char *upath, char *name)
{
    FILE *fp = NULL;
    char lineBuf[64];
    int ret = 0;

    fp = fopen(upath, "r");
    if (fp == NULL)
        return -1;

    while (1) {
        if (fgets(lineBuf, 64, fp) == NULL) {
            ret = -2;
            break;
        }

        if (strncmp(lineBuf, "PCI_SLOT_NAME=", sizeof("PCI_SLOT_NAME=")-1) == 0) {
            if (name != NULL)
                strcpy(name, lineBuf+sizeof("PCI_SLOT_NAME=")-1);
            ret = 0;
            break;
        }
    }

    return ret;
}

static int card_filter(const struct dirent *ent)
{
    if (0 == fnmatch("card*", ent->d_name, FNM_PATHNAME))
        return 1;

    return 0;
}


bool _RefreshAudioDevices2()
{
    char szDevicePath[32];
    char szSoundSysPath[128];
    char szSoundUeventPath[128];
    int v4l2Num;
    MW_RESULT mr;
    int ret;
    struct dirent **namelist;
    int cardNum;

    int nChannelNum = MWGetChannelCount();
    int i;

    memset(g_arrAudioDevicesInfo, 0, sizeof(g_arrAudioDevicesInfo));
    g_nAudioDevicesNum = 0;

    for (i = 0; i < nChannelNum; i++)
    {
        mr = MWGetDevicePath(i, szDevicePath);
        if (mr != MW_SUCCEEDED)
            return false;

        v4l2Num = -1;
        ret = sscanf(szDevicePath, "/dev/video%d", &v4l2Num);
        if (ret <= 0)
            return false;

        sprintf(szSoundSysPath, "%s/video%d/device/sound", g_V4l2SysPath, v4l2Num);
        ret = scandir(szSoundSysPath, &namelist, card_filter, alphasort);
        if (ret != 1) {
            if (ret > 0)
                free(namelist);
            return false;
        }

        cardNum = -1;
        ret = sscanf(namelist[0]->d_name, "card%d", &cardNum);
        if (ret <= 0) {
            free(namelist);
            return false;
        }

        sprintf(szSoundUeventPath, "%s/%s/device/uevent", szSoundSysPath, namelist[0]->d_name);
        ret = _FindPCISlotName(szSoundUeventPath, szDevicePath);
        if (ret < 0) {
            free(namelist);
            return false;
        }

        for (int j = 0; j < (int)strlen(szDevicePath); j++) {
            if (szDevicePath[j] == ':')
                szDevicePath[j] = '_';
            else if (szDevicePath[j] == ' '
                    || szDevicePath[j] == '\t'
                    || szDevicePath[j] == '\r'
                    || szDevicePath[j] == '\n') {
                szDevicePath[j] = '\0';
                break;
            }
        }

        sprintf(g_arrAudioDevicesInfo[g_nAudioDevicesNum].szHwName, "hw:%d,0", cardNum);
        sprintf(g_arrAudioDevicesInfo[g_nAudioDevicesNum].szPaName, "alsa_input.pci-%s.analog-stereo", szDevicePath);
        g_nAudioDevicesNum++;
    }

    return true;
}

bool RefreshAudioDevices()
{
    int nCard = -1;
    int rc;
    snd_ctl_t *hHandle;
    snd_ctl_card_info_t *info;
    snd_ctl_card_info_alloca(&info);


    char szHwCardName[MAX_HW_NAME_LEN];
    char szHwName[MAX_HW_NAME_LEN];
    char szPaName[MAX_PA_NAME_LEN];

    if (snd_card_next(&nCard) < 0 || nCard < 0)
        return false;

    while (nCard >= 0)
    {
        do
        {
            sprintf(szHwCardName, "hw:%d", nCard);
            if ((rc = snd_ctl_open(&hHandle, szHwCardName, 0)) < 0) {
                break;
            }
            if ((rc = snd_ctl_card_info(hHandle, info)) < 0) {
                snd_ctl_close(hHandle);
                break;
            }

            const char *szCardInfo = snd_ctl_card_info_get_name(info);
            if (strstr(szCardInfo, "Pro Capture") == NULL)
            {
                snd_ctl_close(hHandle);
                break;
            }

            sprintf(szHwName, "hw:%d,0", nCard);
            sprintf(szPaName, "alsa_input.%d.analog-stereo", nCard);
            strcpy(g_arrAudioDevicesInfo[g_nAudioDevicesNum].szHwName, szHwName);
            strcpy(g_arrAudioDevicesInfo[g_nAudioDevicesNum].szPaName, szPaName);
            g_nAudioDevicesNum++;

            snd_ctl_close(hHandle);
        }while(false);


        if (snd_card_next(&nCard) < 0)
            break;
    }

    return true;
}

int GetAudioDevicesCount()
{
    return g_nAudioDevicesNum;
}

void GetAudioHwNameByindex(int nIndex, char *szHwName)
{
    if (szHwName == NULL || nIndex >= g_nAudioDevicesNum)
        return;

    strncpy(szHwName, g_arrAudioDevicesInfo[nIndex].szHwName, MAX_HW_NAME_LEN);
    return;
}

void GetAudioPaNameByindex(int nIndex, char *szPaName)
{
    if (szPaName == NULL || nIndex >= g_nAudioDevicesNum)
        return;

    strncpy(szPaName, g_arrAudioDevicesInfo[nIndex].szPaName, MAX_PA_NAME_LEN);
    return;
}
