#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include "options.h"
#include "video-device-info.h"
#include "videorenderfb.h"
#include "videorenderxcb.h"
#include "videorendernull.h"

#define MAX_IMAGE_BUFFER_NUM 5


HCHANNEL g_hChannel = -1;
MWCAP_PTR g_hExitEvent = 0;


LONGLONG g_llTotalTime = 0LL;
DWORD g_dwFrameCount = 0;

VideoRenderBase *g_pVideoRenderer = NULL;


static void *video_capture_thread_proc(void *data)
{
    if (g_hChannel == -1)
        return NULL;

    // register event
    HTIMER hTimer = 0;
    MWCAP_PTR hTimerEvent = 0;
    MWCAP_PTR hCaptureEvent = 0;
    MWQueue<HANDLE64> *pImageQueue = new MWQueue<HANDLE64>;


    do
    {
        MW_RESULT mr;
        LONGLONG llCurTime, llStartTime, llNextTime, llCaptureTime;

        //allocate
        DWORD dwMinStride = FOURCC_CalcMinStride(g_dwColorSpace, g_nWidth, 4);
        DWORD dwImageSize = FOURCC_CalcImageSize(g_dwColorSpace, g_nWidth, g_nHeight, dwMinStride);


        for (int i = 0; i < MAX_IMAGE_BUFFER_NUM; i++)
        {
            HANDLE64 pbImage = (HANDLE64)(unsigned long) malloc(dwImageSize);
            if (pbImage != 0)
                pImageQueue->push(pbImage);
        }



        // register event
        hTimerEvent = MWCreateEvent();
        if (hTimerEvent == 0)
            break;

        hCaptureEvent = MWCreateEvent();
        if (hCaptureEvent == 0)
            break;

        hTimer = MWRegisterTimer(g_hChannel,hTimerEvent);
        if (hTimer == 0)
            break;

        MWStartVideoCapture(g_hChannel, hCaptureEvent);

        mr = MWGetDeviceTime(g_hChannel, &llCurTime);
        if (mr != MW_SUCCEEDED)
            break;
        llCaptureTime = llStartTime = llCurTime;
        llNextTime = 0;

        mr = MWScheduleTimer(g_hChannel, hTimer, llCaptureTime);
        MWCAP_PTR aEvents[] = {g_hExitEvent, hTimerEvent, hCaptureEvent};

        while(true)
        {
            DWORD dwRet = MWMultiWaitEvent(aEvents, 3, -1);
            if (dwRet <= 0)
            {
                printf("Error: Wait timer error %d\n", (DWORD)(10000000/g_fFrameRate));
                break;
            }

            if(dwRet & 0x01)
            {
                MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
               while(true)
               {
                   if (MWWaitEvent(hCaptureEvent, 200) <= 0)
                       break;
                   MWGetVideoCaptureStatus(g_hChannel, &captureStatus);
               }
               break;
            }

            if (dwRet & (0x01 << 1))
            {
                HANDLE64 pbItem;
                if (pImageQueue != NULL && pImageQueue->trypop(pbItem))
                {
                    mr = MWCaptureVideoFrameToVirtualAddressEx(g_hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, pbItem, dwImageSize,dwMinStride,
                                                               FOURCC_IsRGB(g_dwColorSpace), pbItem, g_dwColorSpace, g_nWidth, g_nHeight, 0, 0, 0, NULL,
                                                               0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE,
                                                               NULL, NULL, 0, 0, MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN,MWCAP_VIDEO_QUANTIZATION_UNKNOWN,
                                                               MWCAP_VIDEO_SATURATION_UNKNOWN
                                                               );

                    if(mr != MW_SUCCEEDED)
                    {
                        printf("Error:Capture video error\n");
                        break;
                    }
                }

                llCaptureTime += (DWORD)10000000/g_fFrameRate;
                mr = MWScheduleTimer(g_hChannel, hTimer, llCaptureTime);
            }

            if (dwRet & (0x01 << 2))
            {

                MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
                mr = MWGetVideoCaptureStatus(g_hChannel, &captureStatus);
                if (captureStatus.bFrameCompleted != 1)
                    continue;

                if (captureStatus.pvContext != 0 && pImageQueue != NULL)
                    pImageQueue->push(captureStatus.pvContext);

                g_dwFrameCount++;

                if (g_pVideoRenderer != NULL)
                {
                    g_pVideoRenderer->DisplayFrame((void *)captureStatus.pvContext, dwImageSize);
                }

                mr = MWGetDeviceTime(g_hChannel, &llCurTime);
                if (mr != MW_SUCCEEDED)
                    break;
                g_llTotalTime = llCurTime - llStartTime;
            }

            if (g_llTotalTime > llNextTime  && g_pVideoRenderer != NULL)
            {
                g_pVideoRenderer->DisplayFps((unsigned long long) g_dwFrameCount,
                                             1.0*10000000*g_dwFrameCount/g_llTotalTime);
                llNextTime += 10000000;
            }
        }

    }while(false);

    MWStopVideoCapture(g_hChannel);

    if (hCaptureEvent != 0)
    {
        MWCloseEvent(hCaptureEvent);
        hCaptureEvent = 0;
    }


    if (hTimer != 0)
    {
        MWUnregisterTimer(g_hChannel, hTimer);
        hTimer = 0;
    }

    if(hTimerEvent != 0)
    {
        MWCloseEvent(hTimerEvent);
        hTimerEvent = 0;
    }

    if (pImageQueue != NULL)
    {
        while (!pImageQueue->empty())
        {
            HANDLE64 item = pImageQueue->pop();
            if (item != 0)
            {
                free((void *)(unsigned long)item);
                item = 0;
            }
        }
    }

    return NULL;
}

static void ColorFormatToString(int nFormat, char *szFormat)
{
    if (szFormat == NULL)
        return;

    if (nFormat < 0 || nFormat > 5)
        return;

    switch(nFormat)
    {
    case 0:
        strcpy(szFormat, "auto");
        break;
    case 1:
        strcpy(szFormat, "RGB");
        break;
    case 2:
        strcpy(szFormat, "YUV BT.601");
        break;
    case 3:
        strcpy(szFormat, "YUV BT.709");
        break;
    case 4:
        strcpy(szFormat, "YUV BT.2020");
        break;
    case 5:
        strcpy(szFormat, "YUV BT.2020C");
        break;
    default:
        break;
    }

    return;
}

static void SetVideoColorFormat(int nFormat)
{
    if (g_hChannel == -1)
        return;

    if (nFormat == 5)
    {
        printf("Error: Color format \"bt2020c\" is not supported\n");
        printf("Error: Set color format to \"bt2020c\" error\n");
        return ;
    }

    MW_RESULT mr;

    mr = MWSetVideoInputColorFormat(g_hChannel, (MWCAP_VIDEO_COLOR_FORMAT)g_nInputColorFormat);

    char szFormat[64];
    ColorFormatToString(nFormat, szFormat);

    if (mr == MW_SUCCEEDED)
        printf("Set color format to \"%s\" successfully\n", szFormat);
    else
        printf("Error: Set color format to \"%s\" error\n", szFormat);

    return;
}

void QuantizationToString(int nFormat, char *szFormat)
{
    if(szFormat == NULL)
        return;

    if (nFormat < 0 || nFormat > 2)
        return;

    switch(nFormat)
    {
    case 0:
        strcpy(szFormat, "auto");
        break;
    case 1:
        strcpy(szFormat, "full");
        break;
    case 2:
        strcpy(szFormat, "limited");
        break;
    default:
        break;
    }

    return;
}

static void SetVideoQuantization(int nFormat)
{
    if (g_hChannel == -1)
        return;

    MW_RESULT mr;
    mr = MWSetVideoInputQuantizationRange(g_hChannel, (MWCAP_VIDEO_QUANTIZATION_RANGE)g_nInputQuantization);

    char szFormat[64];
    QuantizationToString(nFormat, szFormat);

    if (mr == MW_SUCCEEDED)
        printf("Set quantization to \"%s\" successfully\n", szFormat);
    else
        printf("Error: Set quantization to \"%s\" error\n", szFormat);
}

static void SetVideoAspect(int nX, int nY)
{
    if (g_hChannel == -1)
        return;

    MW_RESULT mr;
    if (nX == 0 || nY == 0)
        mr = MWSetVideoInputAspectRatio(g_hChannel, 0, 0);
    else
        mr = MWSetVideoInputAspectRatio(g_hChannel, nX, nY);

    char szFormat[64];
    if (nX == 0 || nY == 0)
        strcpy(szFormat,"auto");
    else
        sprintf(szFormat, "%d:%d", nX, nY);

    if (mr == MW_SUCCEEDED)
        printf("Set video aspect to \"%s\" successfully\n", szFormat);
    else
        printf("Error: Set video aspect to \"%s\" error\n", szFormat);
}



int main(int argc, char *argv[])
{
    MWCaptureInitInstance();
    MWRefreshDevice();

    OPTION_RESULT opr;
    int nDeviceCount = 0;

    do
    {
        opr = ParseCommandLine(argc, argv);

        switch (opr) {
        case OPTION_INVALID_PARAMS:
        {
            printf("Error:Invalid params \"%s\" for option %s\n",argv[optind -1], argv[optind-2]);
            return -1;
        }
            break;
        case OPTION_INVALID_OPTION:
        {
            printf("Error:Invalid options \"%s\"\n", argv[optind-2]);
            return -1;
        }
        case OPTION_NO_DEVICE_ARGUMENT:
        {
            printf("Error:Loss of essential option \"-d\"\n");
            return -1;
        }
        case OPTION_NOT_SUPPORTED_COLOR_SAPCE:
        {
            printf("xcb renderer only support \"I420\" color space\n");
            return -1;
        }
            break;
        default:
            break;
        }

        switch (g_nOutputRenderer)
        {
        case RENDERER_TYPE_XCB:
        {
            VideoRenderXCB *pRenderXCB = new VideoRenderXCB();
            pRenderXCB->Init(g_nWidth, g_nHeight);
            g_pVideoRenderer = pRenderXCB;
        }
            break;
        default:
            break;
        }

        if (g_pVideoRenderer != NULL)
        {
            g_pVideoRenderer->SetFrameFormat(g_nWidth, g_nHeight, g_dwColorSpace);
        }



        if ((g_dwCMD & (0x01 << OPTION_CMD_USAGE)) || g_dwCMD == 0)
        {
            PrintUsage();
            break;
        }
        if (g_dwCMD & (0x01 << OPTION_CMD_LIST))
        {
            SelectAllVideoDevices();
            PrintSelectedVideoDevicesInfo();
        }
        else
        {
            nDeviceCount = GetSelectDeviceCount();
            if (nDeviceCount == 0)
            {
                printf("Error: No Pro Capture device is found\n");
                break;
            }

            VIDEO_DEVICE_INFO deviceInfo;
            GetDeviceInfoByIndex(0, &deviceInfo);
            g_hChannel = MWOpenChannelByPath(deviceInfo.szDevicePath);
            if (g_hChannel == -1)
            {
                printf("Error: Open device %s error\n", deviceInfo.szDevicePath);
                break;
            }

            printf("Capture channel info \n"
                   "   %-13s : %d\n"
                   "   %-13s : %d\n"
                   "   %-13s : %s\n"
                   "   %-13s : %s\n\n",
                   "Board Index", deviceInfo.channelInfo.byBoardIndex,
                   "Channel Index", deviceInfo.channelInfo.byChannelIndex,
                   "Product Name", deviceInfo.channelInfo.szProductName,
                   "Serial Number", deviceInfo.channelInfo.szBoardSerialNo);

            if(g_dwCMD & (0x01 << OPTION_CMD_INPUT_COLOR_FORMAT))
            {
                SetVideoColorFormat(g_nInputColorFormat);
            }

            if(g_dwCMD & (0x01 << OPTION_CMD_INPUT_QUANTIZATION))
            {
                SetVideoQuantization(g_nInputQuantization);
            }

            if(g_dwCMD & (0x01 << OPTION_CMD_INPUT_ASPECT))
            {
                SetVideoAspect(g_nInputAspectX, g_nInputAspectY);
            }


            MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
            MWGetVideoSignalStatus(g_hChannel, &videoSignalStatus);

            if (videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
            {
                printf("Error: No video signal is locked\n");
                break;
            }
            else
            {
                double fps = (double)10000000LL / videoSignalStatus.dwFrameDuration;
                char szColorFormat[32], szQuantization[32];
                ColorFormatToString(videoSignalStatus.colorFormat, szColorFormat);
                QuantizationToString(videoSignalStatus.quantRange, szQuantization);

                printf("Input Signal Info\n"
                       "   %-13s : %d:%d\n"
                       "   %-13s : %s\n"
                       "   %-13s : %s\n"
                       "   %-13s : %d x %d\n"
                       "   %-13s : %.2f\n"
                       "   %-13s : %s\n\n",
                       "Aspect", videoSignalStatus.nAspectX, videoSignalStatus.nAspectY,
                       "Color format", szColorFormat,
                       "Quantization", szQuantization,
                       "Resolution", videoSignalStatus.cx, videoSignalStatus.cy,
                       "Frame rate", fps,
                       "Interlaced",videoSignalStatus.bInterlaced ? "YES" : "NO");
            }

            char szColorSpace[32];
            switch (g_dwColorSpace) {
            case MWFOURCC_YUYV:
                strcpy(szColorSpace, "YUYV");
                break;
            case MWFOURCC_RGB24:
                strcpy(szColorSpace, "RGB24");
                break;
            case MWFOURCC_I420:
                strcpy(szColorSpace, "I420");
                break;
            case MWFOURCC_NV12:
                strcpy(szColorSpace, "NV12");
                break;
            case MWFOURCC_RGBA:
                strcpy(szColorSpace, "RGB32");
                break;
            default:
                break;
            }
            printf("Output settings\n"
                   "   %-13s : %s\n"
                   "   %-13s : %dx%d\n"
                   "   %-13s : %.2f\n\n"
                   ,
                   "Color space", szColorSpace,
                   "Resolution", g_nWidth, g_nHeight,
                   "Frame rate", g_fFrameRate
                   );

            g_hExitEvent = MWCreateEvent();
            if (g_hExitEvent == 0)
            {
                printf("Error: Create exit event error\n");
                break;
            }


            pthread_t pThreadID = 0;
            if (pthread_create(&pThreadID, NULL, video_capture_thread_proc, NULL) !=  0)
            {
                printf("Error: Create video capture thread error\n");
                break;
            }

            if (g_nOutputRenderer != RENDERER_TYPE_NULL && g_pVideoRenderer != NULL)
            {
                g_pVideoRenderer->RunLoop(NULL, NULL);
                MWSetEvent(g_hExitEvent);
                pthread_join(pThreadID, NULL);

            }
            else
            {
                printf("Start capture video frame -- 0 s");
                fflush(stdout);


                for (int i = 0; i < (int)g_dwTime; i++)
                {
                    sleep(1);
                    printf("\rStart capture video frame -- %d s, total time: %.2f ms, total frames: %d, average fps: %.2f",
                           i+1, 1.0*g_llTotalTime/10000, g_dwFrameCount, 1.0*g_dwFrameCount*10000000/g_llTotalTime);
                    fflush(stdout);
                }
                printf("\n");

                MWSetEvent(g_hExitEvent);
                pthread_join(pThreadID, NULL);

                printf("Stop capture video frame\n");
                fflush(stdout);
            }



            printf("\nResult\n"
                   "   %-13s : %.2f ms\n"
                   "   %-13s : %d\n"
                   "   %-13s : %.2f fps\n"
                   ,
                   "Total time", 1.0*g_llTotalTime/10000,
                   "Total frames", g_dwFrameCount,
                   "Average fps", 1.0*g_dwFrameCount*10000000/g_llTotalTime);
        }

    }while(false);

    if (g_hChannel != -1)
    {
        MWCloseChannel(g_hChannel);
        g_hChannel = -1;
    }

    if (g_hExitEvent != 0)
    {
        MWCloseEvent(g_hExitEvent);
        g_hExitEvent = 0;
    }

    return 0;
}
