
#include <stdio.h>
#include <getopt.h>
#include <unistd.h>
#include <regex.h>
#include "options.h"
#include "common.h"
#include "video-device-info.h"
static const char short_options[] = "hld:o:c:f:r:i:t:";
static const struct option long_options[] =
{
    {"output-renderer",       required_argument, NULL, 'o'},
    {"color-space",           required_argument, NULL, 'c'},
    {"frame-rate",            required_argument, NULL, 'f'},
    {"resolution",            required_argument, NULL, 'r'},
    {"input-aspect",          required_argument, NULL, 'i'},
    {"input-color-format",    required_argument, NULL,  0 },
    {"input-quantization",    required_argument, NULL,  0 },
    {"capture-time",          required_argument, NULL, 't'},

    {NULL,                    no_argument,       NULL,  0 },
};

unsigned int g_dwCMD = 0;

int g_nWidth = 1024;
int g_nHeight = 720;
DWORD g_dwColorSpace = MWFOURCC_I420;
float g_fFrameRate = 25.00;
int g_nOutputRenderer = RENDERER_TYPE_NULL;

int g_nInputAspectX = 0;
int g_nInputAspectY = 0;
int g_nInputColorFormat = 'a';
int g_nInputQuantization = 'a';


DWORD g_dwTime = 10;

OPTION_RESULT ParseCommandLine(int argc, char *argv[])
{
    OPTION_RESULT opr = OPTION_OK;
    int nIdx;
    int c;
    while ((c = getopt_long(argc, argv, short_options, long_options, &nIdx)) != -1)
    {
        if (c == '?')
            return OPTION_INVALID_OPTION;

        switch (c)
        {
        case 0:
        {
            if(strcmp(long_options[nIdx].name, "input-color-format") == 0)
            {
                g_dwCMD |= (0x01 << OPTION_CMD_INPUT_COLOR_FORMAT);
                opr = ParseVideoColorFormatArgument(optarg);
            }
            if(strcmp(long_options[nIdx].name, "input-quantization") == 0)
            {
                g_dwCMD |= (0x01 << OPTION_CMD_INPUT_QUANTIZATION);
                opr = ParseVideoQuantizationArgument(optarg);
            }

        }
            break;
        case 'h':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_USAGE);
        }
            break;
        case 'l':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_LIST);
        }
            break;
        case 'd':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_CAPTURE);
            opr = ParseDeviceArgument(optarg);

        }
            break;
        case 'i':
        {
            g_dwCMD |= (0x01 << OPTION_CMD_INPUT_ASPECT);
            opr = ParseVideoAspectArgument(optarg);
        }
            break;
        case 'o':
        {
            opr = ParseOutputRendererArgument(optarg);
        }
            break;
        case 'c':
        {
            opr = ParseOutputColorSpace(optarg);
        }
            break;
        case 'f':
        {
            opr = ParseOutputFramerate(optarg);
        }
            break;
        case 'r':
        {
            opr = ParseOutputResolution(optarg);
        }
            break;
        case 't':
        {
            opr = ParseCaptureTimeArgument(optarg);
        }
            break;
        default:
            break;

        }

        if (opr != OPTION_OK)
            return opr;

    }

    if ((g_dwCMD & (0x01 << OPTION_CMD_CAPTURE)) == 0 &&
            (g_dwCMD & (0x01 << OPTION_CMD_USAGE)) == 0 &&
            (g_dwCMD & (0x01 << OPTION_CMD_LIST)) == 0 && g_dwCMD != 0)
         return OPTION_NO_DEVICE_ARGUMENT;

    if (g_nOutputRenderer != RENDERER_TYPE_NULL && g_dwColorSpace != MWFOURCC_I420)
        return OPTION_NOT_SUPPORTED_COLOR_SAPCE;


    return opr;

}

void PrintUsage()
{
    printf("USAGE\n"
           "  batch-mark [OPTIONS]\n\n"
           "DESCRIPTION\n"
           "  A command line tool for performance testing.\n\n"
           "OPTIONS\n"
           "  -h                                            print usage (Default option)\n"
           "  -l                                            list all capture device information\n\n"
           "  -d CAPTURE-DEVICE                             set capture device to be control, CAPTURE-DEVICE\n"
           "                                                can be V4L2 device path (eg. /dev/video0) or capture\n"
           "                                                device ID (Board ID:Channel ID, eg. 0:0)\n\n"
           "  -c --color-space FORMAT                       set ouput frame color space: FORMAT can be: yuyv | nv12\n"
           "                                                i420 | rgb24 | rgb32\n"
           "  -f --frame-rate FRAME_RATE                    set output frame rate (eg. 25, 30.00, 59.96)\n"
           "  -r --resolution INTxINT                       set output resolution (eg. 1024x720, 1920x1080), Range \n"
           "                                                from 1x1 to 2048x2048. If the height of resolution is a\n"
           "                                                odd number, benchmark will convert it to the nearest\n"
           "                                                even number\n\n"
           "  -i --input-aspect ASPECT-RATIO                set capture device input signal aspect. ASPECT-RATIO\n"
           "                                                can be a ratio(AspectX : AspectY eg. 16:9) or auto\n"
           "     --input-color-format FORMAT                set input color format. FORMAT can be: auto | rgb |\n"
           "                                                bt601 | bt709 | bt2020 | bt2020c\n"
           "     --input-quantization QUANTIZATION          set input quantization. QUANTIZATION can be: auto |\n"
           "                                                full | limited\n\n"
           "  -t --capture-time SECONDS                     set test time, unit: second(s)\n\n"
           "  -o --Output-renderer RENDERER                 set output renderer: null | xcb. If this option is\n"
           "                                                not null, the output color space can only be i420\n"
           );
}


OPTION_RESULT ParseDeviceArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (strchr(szArg, ':') == NULL)
    {
        if (!MatchReg(szArg, "^/dev/video[0-9]{1,2}+$"))
            return OPTION_INVALID_PARAMS;

        if (access(szArg, F_OK) != 0)
            return OPTION_NOT_EXIST_DEVICE;

        SelectVideoDevicesByPath(szArg);
    }
    else
    {
        if (!MatchReg(szArg, "^([0-9]+):([0-9]+)$"))
            return OPTION_INVALID_PARAMS;

        char *p;
        char *temp = strdup(szArg);
        char *temp1 = temp;
        const char *separator = ":";
        int nCount = 0;

        int nBoardIndex = -1;
        int nChannelIndex = -1;

        p = strtok(temp, separator);
        while(p != NULL)
        {
            switch(nCount)
            {
            case 0:
            {
                nBoardIndex = (BYTE)atoi(p);
                break;
            }
            case 1:
            {
                nChannelIndex = (BYTE)atoi(p);
                break;
            }
            default:
                break;
            }
            nCount++;

            p = strtok(NULL, separator);
        }

        free(temp1);
        temp1 = NULL;

        SelectVideoDevicesByID(nBoardIndex, nChannelIndex);
        if (GetSelectDeviceCount() == 0)
            return OPTION_ERROR_DEVICE_ID;
    }

    return OPTION_OK;
}


OPTION_RESULT ParseVideoAspectArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^((([0-9]+):([0-9]+)){1}|auto)$"))
        return OPTION_INVALID_PARAMS;

    if (strcmp(szArg, "auto") == 0)
    {
        g_nInputAspectX = 0;
        g_nInputAspectY = 0;
        return OPTION_OK;
    }

    char *p;
    char *temp = strdup(szArg);
    const char *separator = ":";
    int nCount = 0;

    p = strtok(temp, separator);
    while(p != NULL)
    {

        switch(nCount)
        {
        case 0:
        {
            g_nInputAspectX = (BYTE)atoi(p);
            break;
        }
        case 1:
        {
            g_nInputAspectY = (BYTE)atoi(p);
            break;
        }
        default:
            break;
        }
        nCount++;

        p = strtok(NULL, separator);
    }

    free(temp);

    return OPTION_OK;
}

OPTION_RESULT ParseVideoColorFormatArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|rgb|bt601|bt709|bt2020|bt2020c)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 0;
    else if (strcmp(szArg, "rgb") == 0)
        c = 1;
    else if (strcmp(szArg, "bt601") == 0)
        c = 2;
    else if (strcmp(szArg, "bt709") == 0)
        c = 3;
    else if (strcmp(szArg, "bt2020") == 0)
        c = 4;
    else if (strcmp(szArg, "bt2020c") == 0)
        c = 5;
    else
        c = 0;

    g_nInputColorFormat = c;

    return OPTION_OK;
}

OPTION_RESULT ParseVideoQuantizationArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(auto|full|limited)$"))
        return OPTION_INVALID_PARAMS;

    int c;
    if (strcmp(szArg, "auto") == 0)
        c = 0;
    else if (strcmp(szArg, "full") == 0)
        c = 1;
    else if (strcmp(szArg, "limited") == 0)
        c = 2;
    else
        c = 0;

    g_nInputQuantization = c;

    return OPTION_OK;
}

OPTION_RESULT ParseOutputRendererArgument (char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(null|xcb)$"))
        return OPTION_INVALID_PARAMS;

    if (strcmp(szArg, "null") == 0)
        g_nOutputRenderer = RENDERER_TYPE_NULL;
    else if (strcmp(szArg, "xcb") == 0)
        g_nOutputRenderer = RENDERER_TYPE_XCB;
    else
        g_nOutputRenderer = RENDERER_TYPE_NULL;

    return OPTION_OK;
}


OPTION_RESULT ParseOutputColorSpace(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(yuyv|nv12|i420|rgb24|rgb32)$"))
        return OPTION_INVALID_PARAMS;

    if (strcmp(szArg, "yuyv") == 0)
        g_dwColorSpace = MWFOURCC_YUYV;
    else if (strcmp(szArg, "nv12") == 0)
        g_dwColorSpace = MWFOURCC_NV12;
    else if (strcmp(szArg, "i420") == 0)
        g_dwColorSpace = MWFOURCC_I420;
    else if (strcmp(szArg, "rgb24") == 0)
        g_dwColorSpace = MWFOURCC_RGB24;
    else if (strcmp(szArg, "rgb32") == 0)
        g_dwColorSpace = MWFOURCC_RGBA;
    else
        g_dwColorSpace = MWFOURCC_YUYV;

    return OPTION_OK;
}

OPTION_RESULT ParseOutputFramerate(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^(([0-9]+[.]{1}[0-9]{1,2})|([1-9]{1}[0-9]*))$"))
        return OPTION_INVALID_PARAMS;

    g_fFrameRate = atof(szArg);

    return OPTION_OK;
}

OPTION_RESULT ParseOutputResolution(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([1-9][0-9]*x[1-9][0-9]*)$"))
        return OPTION_INVALID_PARAMS;

    char *p;
    char *temp = strdup(szArg);
    char *temp1 = temp;
    const char *separator = "x";

    p = strtok(temp, separator);
    if (p != NULL)
        g_nWidth = atoi(p);

    p = strtok(NULL, separator);
    if (p != NULL)
    {
        g_nHeight = atoi(p);
        g_nHeight = ((g_nHeight >> 1) << 1);
    }


    p = strtok(NULL, separator);
    if (p != NULL)
        return OPTION_INVALID_PARAMS;

    free(temp1);
    temp1 = NULL;

    if(g_nWidth > 2048 || g_nHeight > 2048)
        return OPTION_INVALID_PARAMS;

    return OPTION_OK;
}


OPTION_RESULT ParseCaptureTimeArgument(char *szArg)
{
    if (szArg == NULL)
        return OPTION_INVALID_PARAMS;

    if (!MatchReg(szArg, "^([1-9][0-9]*)$"))
        return OPTION_INVALID_PARAMS;

    g_dwTime = atoi(szArg);

    return OPTION_OK;
}











