#include <stdio.h>
#include "video-device-info.h"

VIDEO_DEVICE_INFO g_videoDeviceInfo[MAX_CHANNEL_COUNT] = {{{0},{0}}};
int g_nVideoDeviceNum = 0;

int SelectAllVideoDevices()
{
    int nDeviceNum = MWGetChannelCount();

    MW_RESULT mr;
    int nCount = 0;
    for(int i = 0; i < nDeviceNum; i++)
    {
        mr = MWGetDevicePath(i, g_videoDeviceInfo[nCount].szDevicePath);
        if (mr != MW_SUCCEEDED)
            continue;

        mr = MWGetChannelInfoByIndex(i, &g_videoDeviceInfo[nCount].channelInfo);
        if (mr != MW_SUCCEEDED)
            continue;

        nCount++;
    }

    g_nVideoDeviceNum = nCount;

    return g_nVideoDeviceNum;
}

int SelectVideoDevicesByID(int nBoardID, int nChannelID)
{
    if(nBoardID < 0 || nChannelID < 0)
        return 0;

    int nDeviceNum = MWGetChannelCount();

    MW_RESULT mr;
    int nCount = 0;
    for (int i = 0; i < nDeviceNum; i++)
    {
        MWCAP_CHANNEL_INFO channelInfo;
        mr = MWGetChannelInfoByIndex(i, &channelInfo);
        if (mr != MW_SUCCEEDED)
            continue;

        if (nBoardID != channelInfo.byBoardIndex || nChannelID != channelInfo.byChannelIndex)
            continue;

        mr = MWGetDevicePath(i, g_videoDeviceInfo[nCount].szDevicePath);
        if (mr != MW_SUCCEEDED)
            continue;

        g_videoDeviceInfo[nCount].channelInfo = channelInfo;
        nCount++;
    }

    g_nVideoDeviceNum = nCount;
    return nCount;
}

int SelectVideoDevicesByPath(char *szDevicePath)
{
    if (szDevicePath == NULL)
        return 0;

    int nDeviceNum = MWGetChannelCount();

    MW_RESULT mr;
    int nCount = 0;
    for (int i = 0; i < nDeviceNum; i++)
    {
        mr = MWGetDevicePath(i, g_videoDeviceInfo[nCount].szDevicePath);
        if (mr != MW_SUCCEEDED)
            continue;

        if (strcmp(szDevicePath, g_videoDeviceInfo[nCount].szDevicePath) != 0)
            continue;

        mr = MWGetChannelInfoByIndex(i, &g_videoDeviceInfo[nCount].channelInfo);
        if (mr != MW_SUCCEEDED)
            continue;

        nCount++;
    }

    g_nVideoDeviceNum = nCount;
    return nCount;
}

int GetSelectDeviceCount()
{
    return g_nVideoDeviceNum;
}

void GetDeviceInfoByIndex(int nIndex, VIDEO_DEVICE_INFO *deviceInfo)
{
    if (deviceInfo == NULL)
        return ;

    if (nIndex < 0 || nIndex >= g_nVideoDeviceNum)
        return;

    memcpy(deviceInfo, &g_videoDeviceInfo[nIndex], sizeof(VIDEO_DEVICE_INFO));

    return;
}

int GetDeviceIndexByPath(char *szDevicePath)
{
    MW_RESULT mr;
    int nChannelNum = MWGetChannelCount();

    int i;
    char szPath[MAX_CHANNEL_NAME_LEN];
    for (i = 0; i < nChannelNum; i++)
    {
        mr = MWGetDevicePath(i, szPath);
        if (mr != MW_SUCCEEDED)
            continue;

        if(strcmp(szDevicePath, szPath) == 0)
            return i;
    }

    return -1;
}

void PrintSelectedVideoDevicesInfo()
{
    printf("total: %d\n", g_nVideoDeviceNum);

    if (g_nVideoDeviceNum != 0)
        printf("%-15s %-15s %-15s %-15s %-15s %-30s\n",
               "device path", "firmware ver", "hardware ver", "driver ver", "alsa name","capture device Name");

    _RefreshAudioDevices2();

    for (int i = 0; i < g_nVideoDeviceNum; i++)
    {
        int nIndex = GetDeviceIndexByPath(g_videoDeviceInfo[i].szDevicePath);
        if(nIndex == -1)
            continue;

        char szHwName[MAX_HW_NAME_LEN];
        GetAudioHwNameByindex(nIndex, szHwName);

        char szChannelName[256] = {0};
        char szFirmwareVerCur[16] = {0};
        char szDriverVerCur[16] = {0};

        sprintf(szChannelName, "%02d:%02d %s",
                g_videoDeviceInfo[i].channelInfo.byBoardIndex,
                g_videoDeviceInfo[i].channelInfo.byChannelIndex,
                g_videoDeviceInfo[i].channelInfo.szProductName);
        sprintf(szFirmwareVerCur, "%d.%d",
                (DWORD)(g_videoDeviceInfo[i].channelInfo.dwFirmwareVersion >> 16) & 0xFFFF,
                (DWORD)(g_videoDeviceInfo[i].channelInfo.dwFirmwareVersion) & 0xFFFF);

        sprintf(szDriverVerCur, "%d.%d.%d",
                (DWORD)(g_videoDeviceInfo[i].channelInfo.dwDriverVersion >> 24) & 0xFF,
                (DWORD)(g_videoDeviceInfo[i].channelInfo.dwDriverVersion >> 16) & 0xFF,
                (DWORD)(g_videoDeviceInfo[i].channelInfo.dwDriverVersion) & 0xFFFF);

        printf("%-15s %-15s %-15c %-15s %-15s %-30s\n",
               g_videoDeviceInfo[i].szDevicePath,
               szFirmwareVerCur,
               g_videoDeviceInfo[i].channelInfo.chHardwareVersion,
               szDriverVerCur, szHwName, szChannelName);
    }
}

