////////////////////////////////////////////////////////////////////////////////
// CONFIDENTIAL and PROPRIETARY software of Magewell Electronics Co., Ltd.
// Copyright (c) 2011-2014 Magewell Electronics Co., Ltd. (Nanjing)
// All rights reserved.
// This copyright notice MUST be reproduced on all authorized copies.
////////////////////////////////////////////////////////////////////////////////
#ifndef VIDEORENDERFB_H
#define VIDEORENDERFB_H

#include "videorenderbase.h"

#ifdef __cplusplus
extern "C" {
#endif
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/mman.h>

#include <linux/fb.h>

struct pixelformat{
    char *name;
    struct fb_bitfield red;
    struct fb_bitfield green;
    struct fb_bitfield blue;
    struct fb_bitfield transp;
    int bpp;
    uint32_t mwfourcc;
};
#ifdef __cplusplus
}
#endif

class VideoRenderFB : public VideoRenderBase
{
public:
    VideoRenderFB();

    virtual ~VideoRenderFB();

    bool Init(const char *devname, int tnr);
    bool GetGeometry(int &width, int &height) {
        width = m_fbVar.xres;
        height = m_fbVar.yres;
        return (width != 0 && height != 0);
    }

    char *GetName() { return (char *)"fb"; }

    virtual uint32_t *GetSupportedFourcc(int &nCount);

    virtual bool SetFrameFormat(int nWidth, int nHeight, uint32_t dwColorSpace, bool bTry = false);

    virtual bool IsSupportImageBufType(VideoRenderImageBufType type) {
        if (type == VIDEO_RENDER_IMAGEBUF_VIRTUAL)
            return true;

        return false;
    }
    virtual VideoRenderImageBuf *AllocImageBuffer(VideoRenderImageBufType type);
    virtual void FreeImageBuffer(VideoRenderImageBuf *pImageBuf);

    virtual void RunLoop(ExitCleanCallback exit_call, void *data);

    virtual void DisplayFrame(VideoRenderImageBuf *imageBuf);
    virtual void DisplayFrame(void *buffer, size_t size);

    virtual void DisplayFps(unsigned long long frameCount, double fps);
private:

    int setmode(const struct pixelformat *pixf, int width, int height);

    int drawimage(unsigned char *buffer, size_t size);

private:
    int                             m_fbHandler;
    int                             m_ttyFd;
    int                             m_lastTv;

    struct fb_fix_screeninfo        m_fbFix;
    struct fb_var_screeninfo        m_fbVar;

    MWArray<struct pixelformat*, 16> m_arSupportFourcc;
    int                             m_nSupportFourccCount;

    unsigned char                   *m_fbMemAddr;
};

#endif // VIDEORENDERFB_H
