#include "captureexpress.h"
#include "videowidget.h"
//#include "videorender.h"
#include "capturethread.h"

int m_arrVideoResolution[30][2] = {
    {1920, 1080},
    {1680, 1050},
    {1600, 1200},
    {1440, 900},
    {1368, 768},
    {1280, 1024},
    {1280, 960},
    {1280, 800},
    {1280, 720},
    {1024, 768},
    {1024, 576},
    {960,  540},
    {856,  480},
    {800,  600},
    {768,  576},
    {720,  576},
    {720,  480},
    {640,  480},
    {640,  360},
    {512,  384},
    {384,  288},
    {352,  288},
    {352,  240},
    {320,  240},
    {176,  144},
    {176,  120},
};

double m_arrFrameRate[9] = {
    60.00,
    59.94,
    50.00,
    30.00,
    29.97,
    25.00,
    20.00,
    15.00,
    5.00
};

CCaptureExpress::CCaptureExpress(QWidget *parent)
    : QMainWindow(parent)
{

    m_bSnapshot = false;
    m_bCaptureVideo = false;
    m_TimerID = 0;
    m_nChannelNum = 0;
    m_bNoDisplay = false;
}

CCaptureExpress::~CCaptureExpress()
{
}

void CCaptureExpress::CreateWindow(char* argv)
{
    if(argv == NULL)
        m_bNoDisplay = false;
    else if(strcmp(argv, "-nodisplay") == 0)
        m_bNoDisplay = true;
    else
        m_bNoDisplay = false;

    //    m_App = new QApplication(argc,argv);
    //    m_btnFullScreen = new QPushButton(QIcon(":/icon/icon/fullscreen.png"), tr("Full Screen"), this);
    m_btnSnapshot = new QPushButton(QIcon(":/icon/icon/snapshot.png"), tr("Create snapshot"), this);
    m_labVideoChannel = new QLabel(tr("Video channel :"), this);
    m_labFrameRate = new QLabel(tr("Frame rate :"), this);
    m_labVideoResolution = new QLabel(tr("Video resolution :"), this);
    m_cmbVideoChannel = new QComboBox(this);
    m_cmbVideoResolution = new QComboBox(this);
    m_cmbFrameRate = new QComboBox(this);

    m_wndVideo = new CVideoWidget(this);
    m_wndVideo->CreateLayout();
    bool bRet = m_wndVideo->CreateRender();
    // bool bFormatEnable = true;
//    if (m_wndVideo->CreateRender() == false)
//    {
//        bFormatEnable = false;
//    }
    QPalette pal(Qt::black);
    m_wndVideo->setPalette(pal);
    m_wndVideo->setAutoFillBackground(true);
    m_labStatus = new QLabel(this);

    //set combobox
    int nChannelCount = SetCombox();
    bool bStartThread = (nChannelCount > 0 ? true : false);

    QHBoxLayout *pHlayoutTop = new QHBoxLayout;
    QHBoxLayout *pHlayoutBottom = new QHBoxLayout;
    QVBoxLayout *pVlayout = new QVBoxLayout;

    //Hlayout
    pHlayoutTop->addWidget(m_labVideoChannel);
    pHlayoutTop->addWidget(m_cmbVideoChannel);
    pHlayoutTop->addStretch();
    pHlayoutTop->addWidget(m_labVideoResolution);
    pHlayoutTop->addWidget(m_cmbVideoResolution);
    pHlayoutTop->addWidget(m_labFrameRate);
    pHlayoutTop->addWidget(m_cmbFrameRate);

    //    pHlayoutBottom->addWidget(m_btnFullScreen);
    pHlayoutBottom->addStretch();
    pHlayoutBottom->addWidget(m_btnSnapshot);

    //Vlayout
    pVlayout->addLayout(pHlayoutTop);
    pVlayout->addWidget(m_wndVideo);
    pVlayout->addLayout(pHlayoutBottom);

    //menubar and actions
    QMenuBar *pMenubar = this->menuBar();
    pMenubar->show();

    QMenu *pFile = pMenubar->addMenu(tr("File"));
    QAction *pActSnapshot = pFile->addAction(tr("Create snapshot"));
    QAction *pCutLine1 = pFile->addAction("");
    pCutLine1->setSeparator(true);
    pActSnapshot->setShortcut(QKeySequence("Ctrl+N"));
    QAction *pActOpenSnapshot = pFile->addAction(tr("Open snapshot directory"));
    QAction *pCutLine2 = pFile->addAction("");
    pCutLine2->setSeparator(true);
    pActOpenSnapshot->setShortcut(QKeySequence("Ctrl+M"));
    QAction *pActExit = pFile->addAction(tr("Exit"));
    pActExit->setShortcut(QKeySequence("Ctrl+X"));

    QMenu *pHelp = pMenubar->addMenu(tr("Help"));
    QAction *pAbout = pHelp->addAction("About");

    //statusbar
    m_barStatus = this->statusBar();
    m_barStatus->setStyleSheet("QStatusBar::item{border:0px}");
    m_barStatus->setSizeGripEnabled(true);
    m_barStatus->addWidget(m_labStatus);
    m_barStatus->show();

    //window size
    int nScreenWidth = QApplication::desktop()->width();
    int nScreenHeight = QApplication::desktop()->height();
    int cx = nScreenWidth / 2;
    int cy = nScreenHeight / 2;
    int x= cx / 2;
    int y= cy /2;
    this->setGeometry(x, y, cx, cy);
    this->setWindowTitle(tr("Magewell Capture Express V2.1 - Video"));
    this->setWindowIcon(QIcon(":/icon/icon/captureexpress.ico"));

    QWidget *wid = new QWidget(this);
    wid->setLayout(pVlayout);
    this->setCentralWidget(wid);

    QObject::connect(pActExit, SIGNAL(triggered(bool)), this, SLOT(onClickedActionExit(bool)));
    QObject::connect(pAbout, SIGNAL(triggered(bool)), this, SLOT(onClickedActionAbout(bool)));

   // if(bFormatEnable == false)
     //   return;

    QObject::connect(m_cmbVideoChannel, SIGNAL(currentIndexChanged(int)), this, SLOT(onChangedChannelIndex(int)));
    QObject::connect(m_cmbVideoResolution, SIGNAL(currentIndexChanged(int)), this, SLOT(onChangedResolutionIndex(int)));
    QObject::connect(m_cmbFrameRate, SIGNAL(currentIndexChanged(int)), this, SLOT(onChangedFrameRateIndex(int)));
    QObject::connect(m_btnSnapshot, SIGNAL(clicked(bool)), this, SLOT(onClickedBtnSnapshot(bool)));
    QObject::connect(pActSnapshot, SIGNAL(triggered(bool)), this, SLOT(onClickedBtnSnapshot(bool)));
    QObject::connect(pActOpenSnapshot, SIGNAL(triggered(bool)), this, SLOT(onClickedBtnOpenSnapshot(bool)));
    QObject::connect(m_labStatus, SIGNAL(linkActivated(QString)), this, SLOT(OpenSnapshotURL(QString)));

    if(bStartThread == true)
    {
        SetRefreshBackground(false);
        m_bCaptureVideo = CaptureThread();
    }
    else
    {
        SetRefreshBackground(true);
    }

    //add
    //SetRefreshBackground(true);//only test

    m_TimerID = this->startTimer(500);
}
//create thread
bool CCaptureExpress::CaptureThread()
{
    m_nChannelNum = 0;
    m_dFrameRate = 30.0;
    m_CaptureThread = new CCaptureThread;
    m_CaptureThread->SetCaptureFourcc(m_wndVideo->GetCaptureFourcc()); //for test
    m_CaptureThread->StartThread(0, 1920, 1080, m_dFrameRate, this, m_wndVideo->GetFrameBuffer());

    return true;
}

//void CCaptureExpress::OnCaptureCallback(VideoRenderImageBuf *pBuffer)
//{
//    if(m_bSnapshot == true)
//    {
//        m_CaptureThread->StopSnapShotThread();
//        m_CaptureThread->StartSnapShotThread();
//        m_bSnapshot = false;
//    }
//    if(m_bNoDisplay == false)
//    {
//        m_wndVideo->DisplayFrame(pBuffer);
//    }
//    else
//    {
//        SetRefreshBackground(true);
//    }
//}

void CCaptureExpress::OnCaptureCallback(MWCAP_PTR buffer, size_t size)
{
    if(m_bSnapshot == true)
    {
        m_CaptureThread->StopSnapShotThread();
        m_CaptureThread->StartSnapShotThread();
        m_bSnapshot = false;
    }
    if(m_bNoDisplay == false)
    {
        m_wndVideo->DisplayFrame((void*)(unsigned long)buffer, size);
    }
    else
    {
        SetRefreshBackground(true);
    }
}

void CCaptureExpress::OnSnapshotCallback(unsigned char *pImage)
{
    StatusRet signalStatus;
    m_CaptureThread->GetInputSignal(&signalStatus);
    int cx = signalStatus.cx;
    int cy = signalStatus.cy;

    QString strImageName = QDateTime::currentDateTime().toString("yy-MM-dd-hh:mm:ss.jpg");
    QString strImageDir = QString(QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).at(0)) + QString("/captureexpressv2.1");
    QString strImagePath = QString("%1/%2").arg(strImageDir).arg(strImageName);
    m_strShotImageName = QString("<a href=%1>%2</a>").arg(strImagePath).arg(strImageName);

    QDir dir;
    if(dir.exists(strImageDir) == false)
        dir.mkdir(strImageDir);

    //rgb to jpg
    struct jpeg_compress_struct jcs;
    struct jpeg_error_mgr jem;
    jcs.err = jpeg_std_error(&jem);
    jpeg_create_compress(&jcs);

    FILE *f = fopen(strImagePath.toLocal8Bit().data(),"wb");
    jpeg_stdio_dest(&jcs, f);

    jcs.image_width = cx;
    jcs.image_height = cy;
    jcs.input_components = 3;
    jcs.in_color_space = JCS_RGB;

    jpeg_set_defaults(&jcs);
    boolean bquality = TRUE;
    jpeg_set_quality(&jcs, 100, bquality);

    jpeg_start_compress(&jcs, bquality);
    JSAMPROW row_pointer[1];
    int row_stride;
    row_stride = jcs.image_width * 3;

    while(jcs.next_scanline < jcs.image_height)
    {
        row_pointer[0] = &pImage[jcs.next_scanline * row_stride];
        jpeg_write_scanlines(&jcs, row_pointer, 1);
    }

    jpeg_finish_compress(&jcs);
    fclose(f);
    jpeg_destroy_compress(&jcs);
}

int CCaptureExpress::SetCombox()
{
    //videoResolution comb box
    int i = 0;
    QString strResolution;
    while(m_arrVideoResolution[i][0] > 0)
    {
        strResolution = QString("%1 x %2").arg(m_arrVideoResolution[i][0]).arg(m_arrVideoResolution[i][1]);
        m_cmbVideoResolution->addItem(strResolution);
        i++;
    }

    //framerate comb box
    i = 0;
    QString strFrameRate;
    while(m_arrFrameRate[i] > 1)
    {
        strFrameRate = QString("%1").arg(m_arrFrameRate[i], 0, 'f', 2);
        m_cmbFrameRate->addItem(strFrameRate);
        i++;
    }
    m_cmbFrameRate->setCurrentIndex(3);

    int nChannelCount = 0;
    do
    {
        //channelinfo comb box
        bool nRes = MWCaptureInitInstance();
        if (nRes != true)
            break;

        MW_RESULT nRet = MWRefreshDevice();
        if (nRet != MW_SUCCEEDED)
            break;

        nChannelCount = MWGetChannelCount();
        QString strChannelName;
        MWCAP_CHANNEL_INFO channelInfo;
        for (int i = 0; i < nChannelCount; i++)
        {
            nRet = MWGetChannelInfoByIndex(i,&channelInfo);
            if(nRet == MW_SUCCEEDED)
            {
                strChannelName = QString("%1 - %2 %3").arg(channelInfo.byBoardIndex, 2, 10, QLatin1Char('0'))
                                                        .arg(channelInfo.byChannelIndex).arg(channelInfo.szProductName);
                m_cmbVideoChannel->addItem(strChannelName);
                m_lstChannelName<<strChannelName;
            }
        }
    }
    while(false);

    MWCaptureExitInstance();
    return nChannelCount;
}
//status options
void CCaptureExpress::GetStatus()
{
    StatusRet signalStatus;
    m_CaptureThread->GetInputSignal(&signalStatus);

    QString strAudioLPCM;
    QString strVideo;
    QString strAudio;

    //video status
    if(signalStatus.pVideo.state == MWCAP_VIDEO_SIGNAL_LOCKED)
    {
        double rate = 0;
        if(signalStatus.pVideo.bInterlaced == false)
        {
            rate = (double)10000000LL / signalStatus.pVideo.dwFrameDuration;
        }
        else
        {
            rate = ((double)10000000LL / signalStatus.pVideo.dwFrameDuration) * 2;
        }

        strVideo = QString("Input Video : Locked\t%1 x %2\t%3Hz ,\t").arg(signalStatus.pVideo.cx).arg(signalStatus.pVideo.cy).arg(rate, 0, 'f', 2);
    }
    else
        strVideo = QString("Input Video : No Signal ,\t");

    //audio status
    if(signalStatus.pAudio.wChannelValid > 0)
    {
        if(signalStatus.pAudio.bLPCM == true)
            strAudioLPCM = "LPCM";
        else
            strAudioLPCM = "";
        strAudio = QString("Input Audio : Valid  %1Hz  %2 bit  %3\t").arg(signalStatus.pAudio.dwSampleRate).arg(signalStatus.pAudio.cBitsPerSample).arg(strAudioLPCM);
    }
    else
        strAudio = QString("Input Audio : No Signal\t");

    //check image name and get the m_strStatus
    static int num = 0;
    if(m_strShotImageName.isNull() == false)
    {
        m_strStatus = QString("Image Name is : %1").arg(m_strShotImageName);
        num ++;
        if(num == 300)
        {
            m_strShotImageName.clear();
            num = 0;
        }
    }
    else
    {
        m_strStatus = QString("Current FPS : %1 ,\t%2 %3").arg(m_CaptureThread->GetFPS(), 0, 'f', 2).arg(strVideo).arg(strAudio);
        m_strStatus = GetElidedText(m_labStatus->font(), m_strStatus, (this->width() *2 / 3));
    }

    return;
}

//events
void CCaptureExpress::closeEvent(QCloseEvent *event)
{
    if(m_bCaptureVideo == true)
        m_CaptureThread->StopThread();
    if(m_TimerID > 0)
        this->killTimer(m_TimerID);
    QApplication::quit();
}

void CCaptureExpress::timerEvent(QTimerEvent *event)
{
    if(m_bCaptureVideo == true)
    {
        GetStatus();
        this->setWindowTitle(QString("Magewell Capture Express V2.1 - Video(%1)").arg(m_lstChannelName[m_nChannelNum]));
        m_labStatus->setText(m_strStatus);
    }
    else
    {
        this->setWindowTitle("Magewell Capture Express V2.1 - Video");
        this->killTimer(m_TimerID);
        m_TimerID = 0;
        m_barStatus->showMessage("No Device !", 3000);
    }
}

QString CCaptureExpress::GetElidedText(QFont font, QString strEdit, int nMaxWidth)
{
    QFontMetrics fontWidth(font);
    strEdit = fontWidth.elidedText(strEdit, Qt::ElideRight, nMaxWidth);

    return strEdit;
}

//slots
void CCaptureExpress::onChangedChannelIndex(int index)
{
    if (m_bCaptureVideo == true)
    {
        StatusRet signalStatus;
        m_CaptureThread->GetInputSignal(&signalStatus);
        int cx = signalStatus.cx;
        int cy = signalStatus.cy;

        m_nChannelNum = index;
        bool nRes = m_CaptureThread->StopThread();
        if(nRes != true)
            return;

        m_CaptureThread->StartThread(index, cx, cy, m_dFrameRate, this, m_wndVideo->GetFrameBuffer());
        m_strShotImageName.clear();
    }
}

void CCaptureExpress::onChangedResolutionIndex(int index)
{
    if(m_bCaptureVideo == true)
    {
        bool nRes = m_CaptureThread->StopThread();
        if(nRes != true)
            return;

        int cx = m_arrVideoResolution[index][0];
        int cy = m_arrVideoResolution[index][1];

        m_wndVideo->ResetBuffer(cx, cy);
        m_CaptureThread->StartThread(m_nChannelNum, cx, cy, m_dFrameRate, this, m_wndVideo->GetFrameBuffer());
        m_strShotImageName.clear();
    }
}

void CCaptureExpress::onChangedFrameRateIndex(int index)
{
    if(m_bCaptureVideo == true)
    {
        StatusRet signalStatus;
        m_CaptureThread->GetInputSignal(&signalStatus);
        int cx = signalStatus.cx;
        int cy = signalStatus.cy;
        m_dFrameRate = m_arrFrameRate[index];

        bool nRes = m_CaptureThread->StopThread();
        if(nRes != true)
            return;

        m_CaptureThread->StartThread(m_nChannelNum, cx, cy, m_dFrameRate, this, m_wndVideo->GetFrameBuffer());
        m_strShotImageName.clear();
    }

}

void CCaptureExpress::onClickedBtnSnapshot(bool bFlag)
{
    if(bFlag == false)
        m_bSnapshot = true;
}

void CCaptureExpress::onClickedBtnOpenSnapshot(bool bFlag)
{
    QString strImageDir = QString(QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).at(0)) + QString("/captureexpressv2.1");
    if(bFlag == false)
    {
        QDesktopServices::openUrl(QUrl::fromLocalFile(strImageDir));
    }
}

void CCaptureExpress::onClickedActionExit(bool bFlag)
{
    if (bFlag == false)
    {
        if(m_bCaptureVideo == true)
            m_CaptureThread->StopThread();
        QApplication::quit();
    }
}

void CCaptureExpress::onClickedActionAbout(bool bFlag)
{
    if (bFlag == false)
        QMessageBox::about(this, tr("About"), tr("Magewell Capture Express V2.1.2018\t\n"
                                                 "Nanjing Magewell Electronics Co., Ltd.\t\n"
                                                 "http://www.magewell.com\t\n"
                                                 "Copyright (c) 2011-2015. All rights reserved.\t\n"));
}

void CCaptureExpress::SetRefreshBackground(bool bFlag)
{
    if(bFlag == true)
        m_wndVideo->setUpdatesEnabled(true);
    else
        m_wndVideo->setUpdatesEnabled(false);
}

void CCaptureExpress::OpenSnapshotURL(QString strImagePath)
{
    QDesktopServices::openUrl(QUrl::fromLocalFile(strImagePath));
}
