#include "capturethread.h"

CCaptureThread::CCaptureThread(QObject *parent) : QObject(parent)
{
    m_bStartCapture = false;
    m_pid = 0;
    m_nSnapshotpid = 0;
    m_dwfourcc = MWFOURCC_YUY2;
    m_nIndex = 0;
    m_dFPS = 0;
    m_pCallback = NULL;
}

CCaptureThread::~CCaptureThread()
{
    //free(m_pImageBuffer);
    //m_pImageBuffer = NULL;
}

void CCaptureThread::CreateThread( )
{
    MWCAP_PTR hEvent = 0;
    MWCAP_PTR hTimerEvent = 0;
    MWCAP_PTR hTimer = 0;
    do
    {
        MW_RESULT nRet;
        hEvent = MWCreateEvent();
        if(hEvent <= 0)
            break;

        hTimerEvent = MWCreateEvent();
        if(hTimerEvent <= 0)
            break;

        hTimer = MWRegisterTimer(m_hChannel, hTimerEvent);
        if(hTimer <= 0)
            break;

        DWORD dwFourcc = m_dwfourcc;
        DWORD dwMinStride = FOURCC_CalcMinStride(dwFourcc, m_cx, 4);
        DWORD dwImageSize = FOURCC_CalcImageSize(dwFourcc, m_cx, m_cy, dwMinStride);
        HANDLE64 pImage = (HANDLE64)(unsigned long)malloc(dwImageSize);
        memset((unsigned char *)pImage, 0, dwImageSize);

        LONGLONG llFrameCount = 0;
        LONGLONG llStartTime = 0;
        LONGLONG llCurTime = 0;
        LONGLONG llTotaltime = 0;
        nRet = MWGetDeviceTime(m_hChannel, &llStartTime);
        if(nRet != MW_SUCCEEDED)
            break;

        LONGLONG llFrameTime = llStartTime;
        DWORD   dwFrameCount = 0;

        nRet = MWStartVideoCapture(m_hChannel, hEvent);
        if(nRet != MW_SUCCEEDED)
            break;

        while(m_bStartCapture)
        {
            nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hChannel, -1, (MWCAP_PTR)(long)pImage/*(MWCAP_PTR)(long)m_pImageBuffer->buf*/, dwImageSize, dwMinStride, 0, 0, dwFourcc, m_cx, m_cy,
                                                         0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                         MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
            if (nRet != MW_SUCCEEDED)
                break;

            int nRes = MWWaitEvent(hEvent, -1);
            if (nRes <= 0)
                break;

            MWCAP_VIDEO_CAPTURE_STATUS staus;
            nRet = MWGetVideoCaptureStatus(m_hChannel, &staus);
            if (nRet != MW_SUCCEEDED)
                break;

            if(m_pCallback != NULL)
                m_pCallback->OnCaptureCallback(pImage, dwImageSize);

            llFrameCount++;
            dwFrameCount ++;
            llTotaltime = llStartTime + llFrameCount * (LONGLONG)((double)10000000 / m_dFrameDuration);
            nRet = MWScheduleTimer(m_hChannel, hTimer, llTotaltime);
            if(nRet != MW_SUCCEEDED)
                break;

            nRes = MWWaitEvent(hTimerEvent,-1);
            if (nRes <= 0)
                break;

            nRet = MWGetDeviceTime(m_hChannel , &llCurTime);
            if(nRet != MW_SUCCEEDED)
                break;

            if (llCurTime - llFrameTime > 30000000LL) {
                dwFrameCount = 0;
                llFrameTime = llCurTime;
            }
            if (dwFrameCount % 10 == 0) {
                if(llCurTime - llFrameTime != 0)
                    m_dFPS = (double)(dwFrameCount * 10000000LL) / (llCurTime - llFrameTime);
            }
        }

        if(pImage != NULL)
            free((void *)pImage);

    }while(false);

    if(hTimer > 0)
        MWUnregisterTimer(m_hChannel, hTimer);

    if(hTimerEvent > 0)
        MWCloseEvent(hTimerEvent);

    if(hEvent > 0)
        MWCloseEvent(hEvent);
}


void CCaptureThread::StartThread(int nIndex, int cx, int cy, double dFrameDuration, ICaptureCallback *pCallback,
                                    VideoRenderImageBuf *pBuffer)
{
    do
    {
        m_nIndex = nIndex;
        m_bStartCapture = true;
        m_cx = cx;
        m_cy = cy;
       // m_pImageBuffer = pBuffer;
        m_pCallback = pCallback;
        m_dFrameDuration = dFrameDuration;

        MW_RESULT nRet;
        if(true != MWCaptureInitInstance())
            break;

        nRet = MWRefreshDevice();
        if(nRet != MW_SUCCEEDED)
            break;

        char channelPath[256];
        nRet = MWGetDevicePath(nIndex, channelPath);
        if(nRet != MW_SUCCEEDED)
            break;

        m_hChannel = MWOpenChannelByPath(channelPath);
        if(m_hChannel == -1)
            break;

        if(0 != pthread_create(&m_pid, NULL, Create, this))
            break;

        return;

    }while(false);

    m_hChannel = -1;
    return;
}

bool CCaptureThread::StopThread()
{
    m_bStartCapture = false;
    if(m_pid > 0)
    {
        if(0 != pthread_join(m_pid, NULL))
            return false;
    }
    if(-1 != m_hChannel)
        MWCloseChannel(m_hChannel);

    MWCaptureExitInstance();
    m_pid = 0;

    return true;
}

double CCaptureThread::GetFPS()
{
    return m_dFPS;
}

bool CCaptureThread::StartSnapShotThread()
{
    if(0 != pthread_create(&m_nSnapshotpid, NULL, CreateSnapShotThread, this))
        return false;

    return true;
}

void CCaptureThread::StopSnapShotThread()
{
    if(m_nSnapshotpid > 0)
    {
        if(0 != pthread_join(m_nSnapshotpid, NULL))
            return;
    }
    if(-1 != m_hSnapshotChannel)
        MWCloseChannel(m_hSnapshotChannel);

    m_nSnapshotpid = 0;

    return;
}

void CCaptureThread::SnapShotThreadProc()
{
    MWCAP_PTR hEvent = 0;
    MW_RESULT nRet = MW_SUCCEEDED;
    do
    {
        hEvent = MWCreateEvent();
        if(hEvent <= 0)
            break;

        char channelPath[256];
        nRet = MWGetDevicePath(m_nIndex, channelPath);
        if(nRet != MW_SUCCEEDED)
            break;

        m_hSnapshotChannel = MWOpenChannelByPath(channelPath);
        if(m_hSnapshotChannel == -1)
            break;

        DWORD dwFourcc = MWFOURCC_RGB24;
        DWORD dwMinStride = FOURCC_CalcMinStride(dwFourcc, m_cx, 4);
        DWORD dwImageSize = FOURCC_CalcImageSize(dwFourcc, m_cx, m_cy, dwMinStride);
        HANDLE64 pImage = (HANDLE64)(unsigned long)malloc(dwImageSize);
        memset((unsigned char *)pImage, 0, dwImageSize);

        nRet = MWStartVideoCapture(m_hSnapshotChannel, hEvent);
        if(nRet != MW_SUCCEEDED)
            break;

        nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hSnapshotChannel, -1, (MWCAP_PTR)(long)pImage, dwImageSize, dwMinStride, 0, 0, dwFourcc, m_cx, m_cy,
                                                     0, 0, 0, 0, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_CROPPING, 0, 0, 0, 0,
                                                     MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
        if (nRet != MW_SUCCEEDED)
            break;

        int nRes = MWWaitEvent(hEvent, -1);
        if (nRes <= 0)
            break;

        MWCAP_VIDEO_CAPTURE_STATUS staus;
        nRet = MWGetVideoCaptureStatus(m_hSnapshotChannel, &staus);
        if (nRet != MW_SUCCEEDED)
            break;

        m_pCallback->OnSnapshotCallback((unsigned char *)pImage);

        if(pImage != NULL)
            free((void *)pImage);

    }while(false);

    if(hEvent > 0)
        MWCloseEvent(hEvent);
}

void CCaptureThread::GetInputSignal(StatusRet *pStatus)
{
    if(-1 != m_hChannel)
    {
        MWGetVideoSignalStatus(m_hChannel, &(pStatus->pVideo));
        MWGetAudioSignalStatus(m_hChannel, &(pStatus->pAudio));
    }
    else
    {
        pStatus->pVideo.state = MWCAP_VIDEO_SIGNAL_NONE;
        pStatus->pAudio.wChannelValid = 0;
    }
    pStatus->cx = m_cx;
    pStatus->cy = m_cy;

    return;
}
