#include "cdlghdimi.h"
char g_szFileFormat[][32] =
{
    "mp3",
    "mp4",
    "aac",
    "wav",
    "wma",
    "png",
    "bmp",
    "jpg",
    "ttf",
    "txt",
    "conf",
    "dat",
    "sh",
    "bash",
    "rgb",
    "gz",
    "zip",
    "odp",
    "pptx",
    "log",
    "pdf",
    "asc",
    "gif",
    "xml",
    "ico",
    "bin"
};

CDlgHDIMI::CDlgHDIMI(QWidget *parent) : QWidget(parent)
{
    QGroupBox *pgpboxEdid = new QGroupBox("EDID", this);
    pgpboxEdid->setStyleSheet("QGroupBox::title{font-weight:bold}");
    m_editEDID = new QTextEdit(this);
    m_editEDID->setReadOnly(true);
    m_btnLoad = new QPushButton("Load ...", this);
    m_btnReset = new QPushButton("Reset ...", this);
    m_btnSave = new QPushButton("Save ...", this);

    QHBoxLayout *pHlayBtn = new QHBoxLayout;
    pHlayBtn->addStretch();
    pHlayBtn->addWidget(m_btnReset);
    pHlayBtn->addWidget(m_btnLoad);
    pHlayBtn->addWidget(m_btnSave);

    QVBoxLayout *pVlayEdid = new QVBoxLayout;
    pVlayEdid->addWidget(m_editEDID);
    pVlayEdid->addLayout(pHlayBtn);

    pgpboxEdid->setLayout(pVlayEdid);

    m_tabSub = new QTabWidget(this);
    m_lblAVI = new QLabel(m_tabSub);
    m_lblACP = new QLabel(m_tabSub);
    m_lblAudio = new QLabel(m_tabSub);
    m_lblSPD = new QLabel(m_tabSub);
    m_lblMS = new QLabel(m_tabSub);
    m_lblVS = new QLabel(m_tabSub);
    m_lblISRC1 = new QLabel(m_tabSub);
    m_lblISRC2 = new QLabel(m_tabSub);
    m_lblGamut = new QLabel(m_tabSub);

    m_lblAVI->setAlignment(Qt::AlignTop);
    m_lblACP->setAlignment(Qt::AlignTop);
    m_lblAudio->setAlignment(Qt::AlignTop);
    m_lblSPD->setAlignment(Qt::AlignTop);
    m_lblMS->setAlignment(Qt::AlignTop);
    m_lblVS->setAlignment(Qt::AlignTop);
    m_lblISRC1->setAlignment(Qt::AlignTop);
    m_lblISRC2->setAlignment(Qt::AlignTop);
    m_lblGamut->setAlignment(Qt::AlignTop);

    m_tabSub->addTab(m_lblAVI, "AVI");
    m_tabSub->addTab(m_lblAudio, "ACP");
    m_tabSub->addTab(m_lblSPD, "Audio");
    m_tabSub->addTab(m_lblMS, "SPD");
    m_tabSub->addTab(m_lblVS, "MS");
    m_tabSub->addTab(m_lblACP, "VS");
    m_tabSub->addTab(m_lblISRC1, "ISRC1");
    m_tabSub->addTab(m_lblISRC2, "ISRC2");
    m_tabSub->addTab(m_lblGamut, "Gamut");

    QVBoxLayout *pVlayMain = new QVBoxLayout;
    pVlayMain->addWidget(pgpboxEdid);
    pVlayMain->addWidget(m_tabSub);

    g_capture.AddNotify(this);
    m_shFile = new CGenerateSHFile(this);

    this->setLayout(pVlayMain);
    UpdateHDMI();

    QApplication::connect(m_btnLoad, SIGNAL(clicked(bool)), this, SLOT(onBtnLoadClicked()));
    QApplication::connect(m_btnReset, SIGNAL(clicked(bool)), this, SLOT(onBtnResetClicked()));
    QApplication::connect(m_btnSave, SIGNAL(clicked(bool)), this, SLOT(onBtnSaveClicked()));
    QApplication::connect(this, SIGNAL(sigHDMIInfoChanged()), this, SLOT(onHDMIInfoChanged()));
}

void CDlgHDIMI::DeviceChange()
{
    BYTE byEdid[1024];
    ULONG ulEdidSize = 1024;
    if(MW_SUCCEEDED != MWGetEDID(g_capture.GetVideoHandle(), byEdid, &ulEdidSize))
        return ;

    int width = 0;
    int height = 0;
    if(ulEdidSize <= 16)
    {
        width = ulEdidSize;
        height = 1;
    }
    else
    {
        width = 16;
        height = ((ulEdidSize % 16) == 0) ? (ulEdidSize / 16) : (ulEdidSize / 16 + 1);
    }

    char byHexEdid[1024] = {0};
    char *pEdidAdd = byHexEdid;
    BYTE temp = 0;
    for(int i = 0; i < height; i++)
    {
        sprintf(pEdidAdd, "%x", i);
        if(i < 16)
        {
            pEdidAdd ++;
            *pEdidAdd++ = '0';
        }

        else
            pEdidAdd += 2;

        *pEdidAdd ++ = ':';
        *pEdidAdd ++ = ' ';
        *pEdidAdd ++ = ' ';
        *pEdidAdd ++ = ' ';

        for(int j = 0; j < width; j++)
        {
            int nIndex = i * 16 + j;
            temp = byEdid[nIndex];
            if(temp < 16)
            {
                *pEdidAdd++  = '0';
                sprintf(pEdidAdd, "%x", temp);
                pEdidAdd ++;
            }
            else
            {
                sprintf(pEdidAdd, "%x", temp);
                pEdidAdd += 2;
            }

            *pEdidAdd++ = ' ';
        }

        *pEdidAdd++ = '\r';
        *pEdidAdd++ = '\n';
    }
    for(int i = 0; i < 1024; i ++)
    {
        byHexEdid[i] = toupper(byHexEdid[i]);
    }
    QString strEdid = QString("%1").arg(byHexEdid);

    m_editEDID->setText(strEdid);

}

void CDlgHDIMI::onHDMIInfoStatuChanged(unsigned long long ullStatusBits)
{
    emit sigHDMIInfoChanged();
}

QString CDlgHDIMI::UpdateHDMIInfo( MWCAP_HDMI_INFOFRAME_ID hdmiInfoID)
{
    QString strInfo = "N/A";

    DWORD dwAVI = 0;
    MW_RESULT nRet = MWGetHDMIInfoFrameValidFlag(g_capture.GetVideoHandle(), &dwAVI);

    if (MW_SUCCEEDED == nRet && (dwAVI & 0x01 << hdmiInfoID)) {
        HDMI_INFOFRAME_PACKET packet = {0};
        nRet = MWGetHDMIInfoFramePacket(g_capture.GetVideoHandle(), hdmiInfoID, &packet);

        if (MW_SUCCEEDED == nRet) {
            char szType[8] = {0};
            char szVersion[8] = {0};
            char szCheckSum[8] = {0};
            Num2HexStr(packet.header.byPacketType, szType);
            Num2HexStr(packet.header.byVersion, szVersion);
            Num2HexStr(packet.byChecksum, szCheckSum);

            //BYTE byEDID[1024] = {0};
            char szHexEDID [1024] = {0};
            char* pMove = szHexEDID;

            unsigned long lEDID = 1024;
            int nMove;
            BYTE byValue;
            for(int i = 0; i < packet.header.byLength; i += 8) {
                strcpy(pMove, "Data  ");
                pMove += 6;
                nMove = 2;
                if (i < 16)
                    *pMove++ = '0';
                sprintf(pMove, "%x", i);
                if (i < 16) {
                    nMove = 1;
                }
                pMove += nMove;
                *pMove++ = ':';
                *pMove++ = ' ';
                *pMove++ = ' ';
                *pMove++ = ' ';

                for (int j = i; j < i + 8 && j < packet.header.byLength; ++ j) {
                    nMove = 2;
                    byValue = packet.abyPayload[j];
                    if (byValue < 16) {
                        *pMove++ = '0';
                        nMove = 1;
                    }

                    sprintf(pMove, "%x", byValue);
                    pMove += nMove;

                    if (j < 15)
                        *pMove++ = ' ';
                }
                *pMove ++ = '\r';
                *pMove ++ = '\n';
            }
            for (int i = 0; i < 1024; ++ i)
                szHexEDID[i] = toupper(szHexEDID[i]);

            strInfo = QString("Type: 0x%1\nVersion: 0x%2\nLength: %3 Bytes\nChecksum: 0x%4\n%5").arg(szType).arg(szVersion).arg(packet.header.byLength).arg(szCheckSum).arg(szHexEDID);

        }
    }
    return strInfo;
}

void CDlgHDIMI::Num2HexStr( int nNum, char* pszBegin )
{
    if (nNum < 16) {
        pszBegin[0] = '0';
        ++ pszBegin;
    }

    sprintf(pszBegin, "%x", nNum);
}

bool CDlgHDIMI::GetSystemEdid()
{
    memset(m_bySysEdid, 0, 1024);
    m_ulEdidCount = 1024;
    if(MW_SUCCEEDED != MWGetEDID(g_capture.GetVideoHandle(), m_bySysEdid, &m_ulEdidCount))
        return false;

    return true;
}

//slots
void CDlgHDIMI::onBtnResetClicked()
{
    if(MW_SUCCEEDED != MWSetEDID(g_capture.GetVideoHandle(), m_bySysEdid, m_ulEdidCount))
        return;

    DeviceChange();
    return;
}

void CDlgHDIMI::onBtnLoadClicked()
{
    QString strFilePath = QFileDialog::getOpenFileName(this, "Open File", m_strFileDir,"bin(*.bin)", Q_NULLPTR,QFileDialog::DontUseNativeDialog);
    if(strFilePath.isEmpty() == true)
        return;

    QFile file(strFilePath);
    file.open(QIODevice::ReadOnly);

    QByteArray arrbyFileData = file.readAll();
    ULONG ulDataSize = arrbyFileData.size();
    file.close();

    MW_RESULT nRet;
    nRet = MWSetEDID(g_capture.GetVideoHandle(), (unsigned char *)arrbyFileData.data(), ulDataSize);
    if(nRet != MW_SUCCEEDED)
    {
        QMessageBox::warning(this, "HDMI", "Read EDID File Failed !");
    }
    else
    {
        DeviceChange();
    }
}

void CDlgHDIMI::onBtnSaveClicked()
{
    do
    {
        bool bHasSuffix = true;
        QString strFileName = "";
        strFileName = QFileDialog::getSaveFileName(this, "Save File", ".", "bin(*.bin*)", Q_NULLPTR,QFileDialog::DontUseNativeDialog );
        QFileInfo fileInfo(strFileName);
        if(strFileName.length() > 0)
        {
            m_strFileDir = fileInfo.absolutePath();
            QString strFileSuffi = fileInfo.completeSuffix();
            if(strFileSuffi.isEmpty() == true)
            {
                bHasSuffix = false;
                strFileName += ".bin";
            }
            else
            {
                bHasSuffix = true;
                int num = 0;
                int bFlag = false;
                while(QString(g_szFileFormat[num]).isEmpty() == false)
                {
                    QString strFileFmt = QString(g_szFileFormat[num]);
                    if(strFileSuffi.toLower() == strFileFmt)
                    {
                        bFlag = true;
                        break;
                    }
                    else
                    {
                        bFlag = false;
                        num++;
                    }
                }
                if(bFlag == false)
                    strFileName += ".bin";
            }

            QDir dir(m_strFileDir);
            dir.setFilter(QDir::Files | QDir::NoSymLinks);

            QFileInfoList fileList = dir.entryInfoList();
            QFileInfo tempInfo(strFileName);
            int i = 0;
            int BoxRet = QMessageBox::Yes;
            for(i = 0; i < fileList.size(); i++)
            {
                QFileInfo fileInfo = fileList.at(i);
                if(fileInfo.fileName() == tempInfo.fileName())
                {
                    if(bHasSuffix == false)
                    {
                        QString strText = QString("%1 has existed ! Do you want to replace it?").arg(tempInfo.fileName());
                        BoxRet = QMessageBox::warning(this, "HDMI", strText, QMessageBox::Yes, QMessageBox::No);
                        break;
                    }
                }
            }

            if(BoxRet == QMessageBox::Yes)
            {
                BYTE byEdid[1024];
                ULONG ulEdidSize = 1024;
                MW_RESULT nRet;
                nRet = MWGetEDID(g_capture.GetVideoHandle(), byEdid, &ulEdidSize);
                if(nRet != MW_SUCCEEDED)
                    break;

                if(strFileName.isEmpty())
                    break;

                FILE *fd = fopen(strFileName.toLocal8Bit().data(), "wb");
                if(fd == NULL)
                {
                    QMessageBox::warning(this, "HDMI", "You can't save file at this directory!");
                    break;
                }

                if (ulEdidSize != fwrite(byEdid, 1, ulEdidSize, fd))
                    break;

                fclose(fd);

                m_shFile->ModifyFileSaveEdid();
            }

            if(BoxRet != QMessageBox::Yes)
                m_strFileDir = ".";
        }
        else
            m_strFileDir = ".";

    }while(false);

    return;
}

void CDlgHDIMI::onHDMIInfoChanged()
{
    m_lblAVI->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_AVI));
    m_lblAudio->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_AUDIO));
    m_lblSPD->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_SPD));
    m_lblMS->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_MS));
    m_lblVS->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_VS));
    m_lblACP->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_ACP));
    m_lblISRC1->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_ISRC1));
    m_lblISRC2->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_ISRC2));
    m_lblGamut->setText(UpdateHDMIInfo(MWCAP_HDMI_INFOFRAME_ID_GAMUT));
}
