#include "cdlginput.h"

MWCAP_VIDEO_INPUT_TYPE m_szVideoInputType[8];
MWCAP_AUDIO_INPUT_TYPE m_szAudioInputType[8];

char g_wszSignalStatus[][32] = {
    "Not Present",
    "Unsupported",
    "Locking",
    "Locked"
};

char g_wszColorFormat[][32] = {
    "RGB",
    "YUV BT.601",
    "YUV BT.709",
    "YUV BT.2020",
    "YUV BT.2020C"
};

char g_wszQuantRange[][32] = {
    "Unknown",
    "Full range",
    "Limited range"
};

char g_wszSatRange[][32] = {
    "Unknown",
    "Full range",
    "Limited range",
    "Extended gamut"
};

CDlgInput::CDlgInput(QWidget *parent) : QWidget(parent)
{

    QLabel *plblVideoInput = new QLabel("Video input :", this);
    QLabel *plblAudioInput = new QLabel("Audio input :", this);

    m_cmbVideoInput = new QComboBox(this);
    m_cmbAudioInput = new QComboBox(this);

    m_ckbAutoScan = new QCheckBox("Auto scan", this);
    m_ckbLinkAv = new QCheckBox("Link with video", this);
    m_ckbAutoScan->setFocusPolicy(Qt::NoFocus);
    m_ckbLinkAv->setFocusPolicy(Qt::NoFocus);
//    m_ckbAutoScan->setStyleSheet("background-color:transparent");
//    m_ckbLinkAv->setStyleSheet("background-color:transparent");

    QGridLayout *playTop = new QGridLayout;
    playTop->setColumnStretch(0,1);
    playTop->setColumnStretch(1,2);
    playTop->setColumnStretch(2,1);

    int nIndex = 0;
    playTop->addWidget(plblVideoInput, nIndex, 0);
    playTop->addWidget(m_cmbVideoInput, nIndex, 1);
    playTop->addWidget(m_ckbAutoScan, nIndex, 2);

    ++nIndex;
    playTop->addWidget(plblAudioInput, nIndex, 0);
    playTop->addWidget(m_cmbAudioInput, nIndex, 1);
    playTop->addWidget(m_ckbLinkAv, nIndex, 2);

    QTabWidget *ptabWidget = new QTabWidget(this);
    m_tableVideoStatus = new QTableWidget(this);
    m_tableAudioStatus = new QTableWidget(this);
    m_tableSpecificStatus = new QTableWidget(this);
    m_tableVideoStatus->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableAudioStatus->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableSpecificStatus->setSelectionBehavior(QAbstractItemView::SelectRows);

    QStringList strlstTableHeaders;
    strlstTableHeaders << "Name" << "Value";

    m_tableVideoStatus->setColumnCount(2);
    m_tableVideoStatus->setColumnWidth(0, 200);
    m_tableVideoStatus->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableVideoStatus->horizontalHeader()->setStretchLastSection(true);
    m_tableVideoStatus->setHorizontalHeaderLabels(strlstTableHeaders);

    m_tableAudioStatus->setColumnCount(2);
    m_tableAudioStatus->setColumnWidth(0, 200);
    m_tableAudioStatus->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableAudioStatus->horizontalHeader()->setStretchLastSection(true);
    m_tableAudioStatus->setHorizontalHeaderLabels(strlstTableHeaders);

    m_tableSpecificStatus->setColumnCount(2);
    m_tableSpecificStatus->setColumnWidth(0, 200);
    m_tableSpecificStatus->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableSpecificStatus->horizontalHeader()->setStretchLastSection(true);
    m_tableSpecificStatus->setHorizontalHeaderLabels(strlstTableHeaders);

    ptabWidget->addTab(m_tableVideoStatus, "Video signal status");
    ptabWidget->addTab(m_tableAudioStatus, "Audio signal status");
    ptabWidget->addTab(m_tableSpecificStatus, "Input specific status");
    ptabWidget->setFocusPolicy(Qt::NoFocus);

    QVBoxLayout *playMain = new QVBoxLayout;
//    playMain->addLayout(playVideo);
//    playMain->addLayout(playAudio);
    playMain->addLayout(playTop);
    playMain->addWidget(ptabWidget);

    m_shFile = new CGenerateSHFile(this);
    g_capture.AddNotify(this);
    this->setLayout(playMain);

    UpdateInput();

}

void CDlgInput::UpdateInput()
{
//    CheckConfig();

    UpdateVideoStatus();
    UpdateAudioStatus();
    UpdateSpecificStatus();

    SetAVInputType();

//    QApplication::connect(m_cmbVideoInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoInputChanged(int)));
//    QApplication::connect(m_cmbAudioInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onAudioInputChanged(int)));
    QApplication::connect(m_ckbAutoScan, SIGNAL(stateChanged(int)), this, SLOT(onAutoScanChecked(int)));
    QApplication::connect(m_ckbLinkAv, SIGNAL(stateChanged(int)), this, SLOT(onLinkAVChecked(int)));
}

void CDlgInput::onInputStatuChanged(unsigned long long ullStatusBits)
{
    emit sigInputStatuChanged(ullStatusBits);
}

void CDlgInput::onInputSourceChanged(unsigned long long ullStatusBits)
{
    emit sigInputSourceChanged(ullStatusBits);
}

//private
void CDlgInput::UpdateVideoStatus()
{
    MWCAP_VIDEO_SIGNAL_STATUS videoSignalInfo;
    MW_RESULT nRet;
    nRet = MWGetVideoSignalStatus(g_capture.GetVideoHandle(), &videoSignalInfo);

    QList<TABLE_ITEM> lstTableItems;
    if(videoSignalInfo.state != MWCAP_VIDEO_SIGNAL_LOCKED)
    {
        AddItem(lstTableItems, "Signal status", g_wszSignalStatus[videoSignalInfo.state]);
    }
    else
    {
        AddItem(lstTableItems, "Signal status", g_wszSignalStatus[videoSignalInfo.state]);
        int nInteger;
        int nDecimal;
        GetVideoFPS(videoSignalInfo.dwFrameDuration, videoSignalInfo.bInterlaced, &nInteger, &nDecimal);
        AddItem(lstTableItems, "Resolution", QString("%1x%2%3, %4.%5Hz").arg(videoSignalInfo.cx).arg(videoSignalInfo.cy).arg(videoSignalInfo.bInterlaced ? "i":"p").arg(nInteger).arg(nDecimal));
        AddItem(lstTableItems, "Aspect ratio", QString("%1:%2").arg(videoSignalInfo.nAspectX).arg(videoSignalInfo.nAspectY));
        AddItem(lstTableItems, "Total scan size", QString("%1x%2pixels").arg(videoSignalInfo.cxTotal).arg(videoSignalInfo.cyTotal));
        AddItem(lstTableItems, "Active area offset", QString("X:%1, Y:%2").arg(videoSignalInfo.x).arg(videoSignalInfo.y));
//        AddItem(lstTableItems, "Signal status", g_wszSignalStatus[videoSignalInfo.state]);

        int nFormat = 0;
        switch(videoSignalInfo.colorFormat)
        {
        case MWCAP_VIDEO_COLOR_FORMAT_RGB:
            nFormat = 0;
            break;
        case MWCAP_VIDEO_COLOR_FORMAT_YUV601:
            nFormat = 1;
            break;
        case MWCAP_VIDEO_COLOR_FORMAT_YUV709:
            nFormat = 2;
            break;
        case MWCAP_VIDEO_COLOR_FORMAT_YUV2020:
            nFormat = 3;
            break;
        case MWCAP_VIDEO_COLOR_FORMAT_YUV2020C:
            nFormat = 4;
            break;
        }
        AddItem(lstTableItems, "Color format", QString("%1").arg(g_wszColorFormat[nFormat]));

        nFormat = 0;
        switch (videoSignalInfo.quantRange) {
        case MWCAP_VIDEO_QUANTIZATION_UNKNOWN:
            nFormat = 0;
            break;
        case MWCAP_VIDEO_QUANTIZATION_FULL:
            nFormat = 1;
            break;
        case MWCAP_VIDEO_QUANTIZATION_LIMITED:
            nFormat = 2;
            break;
        default:
            break;
        }
        AddItem(lstTableItems, "Quantization range", QString("%1").arg(g_wszQuantRange[nFormat]));

        nFormat = 0;
        switch (videoSignalInfo.quantRange) {
        case MWCAP_VIDEO_QUANTIZATION_UNKNOWN:
            nFormat = 0;
            break;
        case MWCAP_VIDEO_QUANTIZATION_FULL:
            nFormat = 1;
            break;
        case MWCAP_VIDEO_QUANTIZATION_LIMITED:
            nFormat = 2;
            break;
        default:
            break;
        }
        AddItem(lstTableItems, "Quantization range", QString("%1").arg(g_wszQuantRange[nFormat]));

        nFormat = 0;
        switch (videoSignalInfo.satRange) {
        case MWCAP_VIDEO_SATURATION_UNKNOWN:
            nFormat = 0;
            break;
        case MWCAP_VIDEO_SATURATION_FULL:
            nFormat = 1;
            break;
        case MWCAP_VIDEO_SATURATION_LIMITED:
            nFormat = 2;
            break;
        case MWCAP_VIDEO_SATURATION_EXTENDED_GAMUT:
            nFormat = 3;
            break;
        default:
            break;
        }
        AddItem(lstTableItems, "Saturation range", QString("%1").arg(g_wszSatRange[nFormat]));

    }

    int nTableItemCount = lstTableItems.size();
    m_tableVideoStatus->setRowCount(nTableItemCount);
    for(int i = 0; i < nTableItemCount; i++)
    {
        m_tableVideoStatus->setItem(i, 0, new QTableWidgetItem(lstTableItems.at(i).key));
        m_tableVideoStatus->setItem(i, 1, new QTableWidgetItem(lstTableItems.at(i).value));
    }
}

void CDlgInput::UpdateAudioStatus()
{
    MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
    MW_RESULT nRet;
    nRet = MWGetVideoSignalStatus(g_capture.GetVideoHandle(), &videoSignalStatus);

    QList<TABLE_ITEM>lstTableItems;
    if(videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
    {
        AddItem(lstTableItems, "Signal status", QString("%1").arg(g_wszSignalStatus[videoSignalStatus.state]));
    }
    else
    {
        MWCAP_AUDIO_SIGNAL_STATUS audioSignalStatus;
        MWGetAudioSignalStatus(g_capture.GetVideoHandle(), &audioSignalStatus);

        if (audioSignalStatus.wChannelValid > 0) {
            AddItem(lstTableItems, "Audio format", audioSignalStatus.bLPCM ? QString("%1 Hz, %2 bits, LPCM").arg(audioSignalStatus.dwSampleRate).arg(audioSignalStatus.cBitsPerSample) : QString("%1 Hz, %2 bits").arg(audioSignalStatus.dwSampleRate).arg(audioSignalStatus.cBitsPerSample));

            AddItem(lstTableItems, "Channel 1 & 2", audioSignalStatus.wChannelValid & 0x0001 ? "Preset" : "Not preset");
            AddItem(lstTableItems, "Channel 3 & 4", audioSignalStatus.wChannelValid & 0x0002 ? "Preset" : "Not preset");
            AddItem(lstTableItems, "Channel 5 & 6", audioSignalStatus.wChannelValid & 0x0004 ? "Preset" : "Not preset");
            AddItem(lstTableItems, "Channel 7 & 8", audioSignalStatus.wChannelValid & 0x0008 ? "Preset" : "Not preset");

            char abyData [1024] = {0};
            char* pMove = abyData;
            int nMove;
            BYTE byValue;
            for (int j = 0; j < 24; ++ j) {
                nMove = 2;
                byValue = audioSignalStatus.channelStatus.abyData[j];
                if (byValue < 16) {
                    *pMove++ = '0';
                    nMove = 1;
                }
//                itoa (byValue, pMove, 16);
                sprintf(pMove, "%x", byValue);
                pMove += nMove;
                *pMove++ = ' ';
            }
            AddItem(lstTableItems, "Channel status data", QString("%1").arg(abyData));

        }
        else {
            AddItem(lstTableItems, "Signal status", "Not present");
        }
    }

    m_tableAudioStatus->clearContents();
    m_tableAudioStatus->setRowCount(lstTableItems.size());
    for(int i = 0; i < lstTableItems.size(); i++)
    {
        m_tableAudioStatus->setItem(i, 0, new QTableWidgetItem(lstTableItems.at(i).key));
        m_tableAudioStatus->setItem(i, 1, new QTableWidgetItem(lstTableItems.at(i).value));
    }
}

void CDlgInput::UpdateSpecificStatus()
{
    MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
    MW_RESULT nRet;
    nRet = MWGetVideoSignalStatus(g_capture.GetVideoHandle(), &videoSignalStatus);

    QList<TABLE_ITEM>lstTableItems;

    if(videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
    {
        AddItem(lstTableItems, "Signal status", g_wszSignalStatus[videoSignalStatus.state]);
    }
    else
    {
        MWCAP_INPUT_SPECIFIC_STATUS inputSpecificStatus;
        nRet = MWGetInputSpecificStatus(g_capture.GetVideoHandle(), &inputSpecificStatus);

        QString strValue = " ";
        if(inputSpecificStatus.bValid)
        {
            AddItem(lstTableItems, "Signal status", "Present");
            if(inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_HDMI)
            {
                AddItem(lstTableItems, "Mode", QString("%1").arg(inputSpecificStatus.hdmiStatus.bHDMIMode ? "HDMI":"NO"));
                AddItem(lstTableItems, "HDCP encrypted", QString("%1").arg(inputSpecificStatus.hdmiStatus.bHDCP ? "YES":"NO"));
                AddItem(lstTableItems, "Color depth", QString("%1 Bits").arg(inputSpecificStatus.hdmiStatus.byBitDepth));

                switch (inputSpecificStatus.hdmiStatus.pixelEncoding) {
                case HDMI_ENCODING_RGB_444:
                    strValue = "R/G/B 4:4:4";
                    break;
                case HDMI_ENCODING_YUV_422:
                    strValue = "Y/U/V 4:2:2";
                    break;
                case HDMI_ENCODING_YUV_444:
                    strValue = "Y/U/V 4:4:4";
                    break;
                case HDMI_ENCODING_YUV_420:
                    strValue = "Y/U/V 4:2:0";
                    break;
                default:
                    break;
                }
                AddItem(lstTableItems, "Pix encoding", strValue);

                AddItem(lstTableItems, "VIC", QString("%1").arg(inputSpecificStatus.hdmiStatus.byVIC));
                AddItem(lstTableItems, "IT content", inputSpecificStatus.hdmiStatus.bITContent ? "True" : "False");
                AddItem(lstTableItems, "iming - scanning format", inputSpecificStatus.hdmiStatus.videoTiming.bInterlaced ? "Interlaced" : "Progressive");

                int nInteger;
                int nDecimal;
                GetVideoFPS(inputSpecificStatus.hdmiStatus.videoTiming.dwFrameDuration, inputSpecificStatus.hdmiStatus.videoTiming.bInterlaced, &nInteger, &nDecimal);
                AddItem(lstTableItems, "Timing - frame rate", QString("%1.%2 Hz").arg(nInteger).arg(nDecimal));

                AddItem(lstTableItems, "Timing - H total", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wHTotalWidth));
                AddItem(lstTableItems, "Timing - H active", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wHActive));
                AddItem(lstTableItems, "Timing - H front porch", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wHFrontPorch));
                AddItem(lstTableItems, "Timing - H sync width", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wHSyncWidth));
                AddItem(lstTableItems, "Timing - H back porch", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wHBackPorch));
                AddItem(lstTableItems, "Timing - H field 0 V total", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField0VTotalHeight));
                AddItem(lstTableItems, "Timing - H field 0 V active", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField0VActive));
                AddItem(lstTableItems, "Timing - H field 0 V front porch", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField0VFrontPorch));
                AddItem(lstTableItems, "Timing - H field 0 V sync width", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField0VSyncWidth));
                AddItem(lstTableItems, "Timing - H field 0 V back porch", QString("%1 Pixels").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField0VBackPorch));

                AddItem(lstTableItems, "Timing - H field 1 V total", QString("%1 Lines").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField1VTotalHeight));
                AddItem(lstTableItems, "Timing - H field 1 V active", QString("%1 Lines").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField1VActive));
                AddItem(lstTableItems, "Timing - H field 1 V front porch", QString("%1 Lines").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField1VFrontPorch));
                AddItem(lstTableItems, "Timing - H field 1 V sync width", QString("%1 Lines").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField1VSyncWidth));
                AddItem(lstTableItems, "Timing - H field 1 V back porch", QString("%1 Lines").arg(inputSpecificStatus.hdmiStatus.videoTiming.wField1VBackPorch));
            }
            else if(inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_VGA)
            {
                strValue = "";
                switch (inputSpecificStatus.vgaComponentStatus.syncInfo.bySyncType)
                {
                case VIDEO_SYNC_ALL:
                    strValue = "ALL";
                    break;
                case VIDEO_SYNC_HS_VS:
                    strValue = "HS_VS";
                    break;
                case VIDEO_SYNC_CS:
                    strValue = "CS";
                    break;
                case VIDEO_SYNC_EMBEDDED:
                    strValue = "Embendded";
                    break;
                }
                AddItem(lstTableItems, "Sync type", strValue);

                int nInteger;
                int nDecimal;
                GetVideoFPS(inputSpecificStatus.vgaComponentStatus.syncInfo.dwFrameDuration, inputSpecificStatus.vgaComponentStatus.syncInfo.bInterlaced, &nInteger, &nDecimal);
                AddItem(lstTableItems, "Timing - frame rate", QString("%1.%2 Hz").arg(nInteger).arg(nDecimal));

                AddItem(lstTableItems, "Scanning format", inputSpecificStatus.vgaComponentStatus.syncInfo.bInterlaced ? "Interlaced" : "Progressive");
                AddItem(lstTableItems, "VS width", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.syncInfo.wVSyncLineCount));
                AddItem(lstTableItems, "Total scan lines", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.syncInfo.wFrameLineCount));

                strValue = "";
                switch (inputSpecificStatus.vgaComponentStatus.videoTiming.dwType)
                {
                case MWCAP_VIDEO_TIMING_NONE:
                    strValue = "NONE";
                    break;
                case MWCAP_VIDEO_TIMING_LEGACY:
                    strValue = "LEGACY";
                    break;
                case MWCAP_VIDEO_TIMING_DMT:
                    strValue = "DMT";
                    break;
                case MWCAP_VIDEO_TIMING_CEA:
                    strValue = "CEA";
                    break;
                case MWCAP_VIDEO_TIMING_GTF:
                    strValue = "GTF";
                    break;
                case MWCAP_VIDEO_TIMING_CVT:
                    strValue = "CVT";
                    break;
                case MWCAP_VIDEO_TIMING_CVT_RB:
                    strValue = "CVT_RB";
                    break;
                case MWCAP_VIDEO_TIMING_FAILSAFE:
                    strValue = "FAILSAFE";
                    break;
                }
                AddItem(lstTableItems, "Timing - Type", strValue);

                AddItem(lstTableItems, "Timing - Pixel clock", QString("%1 Hz").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.dwPixelClock));
                AddItem(lstTableItems, "Timing - H active", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHActive));
                AddItem(lstTableItems, "Timing - H front porch", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHFrontPorch));
                AddItem(lstTableItems, "Timing - H sync width", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHSyncWidth));
                AddItem(lstTableItems, "Timing - H back proch", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHBackPorch));

                AddItem(lstTableItems, "Timing - V active", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVActive));
                AddItem(lstTableItems, "Timing - V front porch", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVFrontPorch));
                AddItem(lstTableItems, "Timing - V sync width", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVSyncWidth));
                AddItem(lstTableItems, "Timing - V back proch", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVBackPorch));
            }
            else if (inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_SDI) {
                strValue = "";
                switch (inputSpecificStatus.sdiStatus.sdiType)
                {
                case SDI_TYPE_SD:
                    strValue = "SD";
                    break;
                case SDI_TYPE_HD:
                    strValue = "HD";
                    break;
                case SDI_TYPE_3GA:
                    strValue = "3GA";
                    break;
                case SDI_TYPE_3GB_DL:
                    strValue = "3GB DL";
                    break;
                case SDI_TYPE_3GB_DS:
                    strValue = "3GB DS";
                    break;
                case SDI_TYPE_DL_CH1:
                    strValue = "DL CH1";
                    break;
                case SDI_TYPE_DL_CH2:
                    strValue = "DL CH2";
                    break;
                }
                AddItem(lstTableItems, "Type", strValue);

                strValue = "";
                switch (inputSpecificStatus.sdiStatus.sdiScanningFormat)
                {
                case SDI_SCANING_INTERLACED:
                    strValue = "Interlaced";
                    break;
                case SDI_SCANING_SEGMENTED_FRAME:
                    strValue = "Segmented frame";
                    break;
                case SDI_SCANING_PROGRESSIVE:
                    strValue = "Progressive";
                    break;
                }
                AddItem(lstTableItems, "Scanning format", strValue);

                strValue = "";
                switch (inputSpecificStatus.sdiStatus.sdiBitDepth)
                {
                case SDI_BIT_DEPTH_8BIT:
                    strValue = "8 Bits";
                    break;
                case SDI_BIT_DEPTH_10BIT:
                    strValue = "10 Bits";
                    break;
                case SDI_BIT_DEPTH_12BIT:
                    strValue = "12 Bits";
                    break;
                }
                AddItem(lstTableItems, "Color depth", strValue);

                strValue = "";
                switch (inputSpecificStatus.sdiStatus.sdiSamplingStruct)
                {
                case SDI_SAMPLING_422_YCbCr:
                    strValue = "Y/Cb/Cr, 4:2:2";
                    break;
                case SDI_SAMPLING_444_YCbCr:
                    strValue = "Y/Cb/Cr, 4:4:4";
                    break;
                case SDI_SAMPLING_444_RGB:
                    strValue = "R/G/B, 4:4:4";
                    break;
                case SDI_SAMPLING_420_YCbCr:
                    strValue = "4:2:0";
                    break;
                case SDI_SAMPLING_4224_YCbCrA:
                    strValue = "Y/Cb/Cr/A, 4:2:2:4";
                    break;
                case SDI_SAMPLING_4444_YCbCrA:
                    strValue = "Y/Cb/Cr/A, 4:4:4:4";
                    break;
                case SDI_SAMPLING_4444_RGBA:
                    strValue = "R/G/B/A, 4:4:4:4";
                    break;
                case SDI_SAMPLING_4224_YCbCrD:
                    strValue = "Y/Cb/Cr/D, 4:2:2:4";
                    break;
                case SDI_SAMPLING_4444_YCbCrD:
                    strValue = "Y/Cb/Cr/A, 4:4:4:4";
                    break;
                case SDI_SAMPLING_4444_RGBD:
                    strValue = "R/G/B/D, 4:4:4:4";
                    break;
                case SDI_SAMPLING_444_XYZ:
                    strValue = "X/Y/Z, 4:4:4";
                    break;
                }
                AddItem(lstTableItems, "Sampling struct", strValue);

            }
            else if (inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_COMPONENT) { //component
                switch (inputSpecificStatus.vgaComponentStatus.videoTiming.bySyncType)
                {
                case VIDEO_SYNC_ALL:
                    strValue = "All sync";
                    break;
                case VIDEO_SYNC_HS_VS:
                    strValue = "HS/VS sync";
                    break;
                case VIDEO_SYNC_CS:
                    strValue = "CS sync";
                    break;
                case VIDEO_SYNC_EMBEDDED:
                    strValue = "Embendded sync";
                    break;
                }
                AddItem(lstTableItems, "Sync type", strValue);

                int nBefore = 0, nAfter = 0;
                GetVideoFPS(inputSpecificStatus.vgaComponentStatus.syncInfo.dwFrameDuration, inputSpecificStatus.vgaComponentStatus.syncInfo.bInterlaced, &nBefore, &nAfter);
                AddItem(lstTableItems, "Frame rate", QString("%1.%2 Hz").arg(nBefore).arg(nAfter));

                AddItem(lstTableItems, "Scanning format", inputSpecificStatus.vgaComponentStatus.syncInfo.bInterlaced ? "Interlaced" : "Progressive");


                AddItem(lstTableItems, "VS width", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.syncInfo.wVSyncLineCount));
                AddItem(lstTableItems, "Total scan lines", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.syncInfo.wFrameLineCount));

                GetVideoFPS(inputSpecificStatus.hdmiStatus.videoTiming.dwFrameDuration, inputSpecificStatus.hdmiStatus.videoTiming.bInterlaced, &nBefore, &nAfter);
                AddItem(lstTableItems, "Embendded sync", QString("%1.%2 Hz").arg(nBefore).arg(nAfter));

                strValue = "";
                switch (inputSpecificStatus.vgaComponentStatus.videoTiming.dwType)
                {
                case MWCAP_VIDEO_TIMING_NONE:
                    strValue = "NONE";
                    break;
                case MWCAP_VIDEO_TIMING_LEGACY:
                    strValue = "LEGACY";
                    break;
                case MWCAP_VIDEO_TIMING_DMT:
                    strValue = "DMT";
                    break;
                case MWCAP_VIDEO_TIMING_CEA:
                    strValue = "CEA";
                    break;
                case MWCAP_VIDEO_TIMING_GTF:
                    strValue = "GTF";
                    break;
                case MWCAP_VIDEO_TIMING_CVT:
                    strValue = "CVT";
                    break;
                case MWCAP_VIDEO_TIMING_CVT_RB:
                    strValue = "CVT_RB";
                    break;
                case MWCAP_VIDEO_TIMING_FAILSAFE:
                    strValue = "FAILSAFE";
                    break;
                }
                AddItem(lstTableItems, "Timing - Type", strValue);

                AddItem(lstTableItems, "Timing - Pixel clock", QString("%1 Hz").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.dwPixelClock));
                AddItem(lstTableItems, "Timing - H active", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHActive));
                AddItem(lstTableItems, "Timing - H front porch", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHFrontPorch));
                AddItem(lstTableItems, "Timing - H sync width", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHSyncWidth));
                AddItem(lstTableItems, "Timing - H back proch", QString("%1 Pixels").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wHBackPorch));

                AddItem(lstTableItems, "Timing - V active", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVActive));
                AddItem(lstTableItems, "Timing - V front porch", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVFrontPorch));
                AddItem(lstTableItems, "Timing - V sync width", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVSyncWidth));
                AddItem(lstTableItems, "Timing - V back proch", QString("%1 Lines").arg(inputSpecificStatus.vgaComponentStatus.videoTiming.wVBackPorch));

            }
            else if (inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_CVBS) {//composite
                strValue = "";
                switch (inputSpecificStatus.cvbsYcStatus.standard)
                {
                case MWCAP_SD_VIDEO_NONE:
                    strValue = "NONE";
                    break;
                case MWCAP_SD_VIDEO_NTSC_M:
                    strValue = "NTSC-M";
                    break;
                case MWCAP_SD_VIDEO_NTSC_433:
                    strValue = "NTSC-433";
                    break;
                case MWCAP_SD_VIDEO_PAL_M:
                    strValue = "PAL-M";
                    break;
                case MWCAP_SD_VIDEO_PAL_60:
                    strValue = "PAL-60";
                    break;
                case MWCAP_SD_VIDEO_PAL_COMBN:
                    strValue = "PAL-C/O/M/B/N";
                    break;
                case MWCAP_SD_VIDEO_PAL_BGHID:
                    strValue = "PAL-B/G/H/I/D";
                    break;
                case MWCAP_SD_VIDEO_SECAM:
                    strValue = "SECAM";
                    break;
                case MWCAP_SD_VIDEO_SECAM_60:
                    strValue = "SECAM-60";
                    break;
                }
                AddItem(lstTableItems, "TV Standard", strValue);

                if (inputSpecificStatus.cvbsYcStatus.b50Hz) {
                    AddItem(lstTableItems, "Field rate", "50 Hz");

                }
            }
            else if (inputSpecificStatus.dwVideoInputType == MWCAP_VIDEO_INPUT_TYPE_YC) {
            }

        }
    }

    m_tableSpecificStatus->clearContents();
    m_tableSpecificStatus->setRowCount(lstTableItems.size());
    for(int i= 0; i < lstTableItems.size(); i++)
    {
        m_tableSpecificStatus->setItem(i, 0, new QTableWidgetItem(lstTableItems.at(i).key));
        m_tableSpecificStatus->setItem(i, 1, new QTableWidgetItem(lstTableItems.at(i).value));
    }

}

void CDlgInput::InputSourceChange()
{
    DWORD szdwVideoInputSources[8];
    DWORD cVideoBufferItems = 8;

    MWGetVideoInputSourceArray(g_capture.GetVideoHandle(), szdwVideoInputSources, &cVideoBufferItems);
    DWORD dwSelVideoInputSource = 0;
    MWGetVideoInputSource(g_capture.GetVideoHandle(), &dwSelVideoInputSource);

    int nSel = -1;
    for (DWORD i = 0; i < cVideoBufferItems; ++ i){
        if (dwSelVideoInputSource == szdwVideoInputSources[i]) {
            nSel = i;
            break;
        }
    }
    m_cmbVideoInput->setCurrentIndex(nSel);

    DWORD szdwAudioInputSources[8];
    DWORD cAudioBufferItems = 8;
    MWGetAudioInputSourceArray(g_capture.GetVideoHandle(), szdwAudioInputSources, &cAudioBufferItems);

    DWORD dwSelAudioInputSource = 0;
    MWGetAudioInputSource(g_capture.GetVideoHandle(), &dwSelAudioInputSource);
    nSel = -1;
    for (DWORD i = 0; i < cAudioBufferItems; ++ i){
        if (dwSelAudioInputSource == szdwAudioInputSources[i]) {
            nSel = i;
            break;
        }
    }
    m_cmbAudioInput->setCurrentIndex(nSel);

    UpdateVideoStatus();
    UpdateAudioStatus();
    UpdateSpecificStatus();
}


void CDlgInput::SetAVInputType()
{
    QApplication::disconnect(m_cmbVideoInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoInputChanged(int)));
    QApplication::disconnect(m_cmbAudioInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onAudioInputChanged(int)));

    m_cmbVideoInput->clear();
    m_cmbAudioInput->clear();

    DWORD szdwVideoInputSources[8];
    DWORD cVideoBufferItems = 8;
    MW_RESULT nRet;
    nRet = MWGetVideoInputSourceArray(g_capture.GetVideoHandle(), szdwVideoInputSources, &cVideoBufferItems);

    DWORD dwSelVideoInputSource = 0;
    nRet = MWGetVideoInputSource(g_capture.GetVideoHandle(), &dwSelVideoInputSource);

    char wszType[256] = {0};
    int nSel = -1;

    int nIndex;
    for (DWORD i = 0; i < cVideoBufferItems; ++ i){
        nIndex = -1;
        if (dwSelVideoInputSource == szdwVideoInputSources[i])
            nSel = i;

        GetVideoType(m_szVideoInputType[i], wszType, szdwVideoInputSources[i], nIndex);

        m_cmbVideoInput->addItem(QString("%1 input %2").arg(wszType).arg(nIndex), i);
    }
    m_cmbVideoInput->setCurrentIndex(nSel);

    DWORD szdwAudioInputSources[8];
    DWORD cAudioBufferItems = 8;
    if (MW_SUCCEEDED == MWGetAudioInputSourceArray(g_capture.GetVideoHandle(), szdwAudioInputSources, &cAudioBufferItems)) {

        DWORD dwSelAudioInputSource = 0;
        MWGetAudioInputSource(g_capture.GetVideoHandle(), &dwSelAudioInputSource);
        nSel = -1;
        for (DWORD i = 0; i < cAudioBufferItems; ++ i){
            nIndex = -1;
            if (dwSelAudioInputSource == szdwAudioInputSources[i])
                nSel = i;

            GetAudioType(m_szAudioInputType[i], wszType, szdwAudioInputSources[i], nIndex);

            m_cmbAudioInput->addItem(QString("%1 input %2").arg(wszType).arg(nIndex), i);
        }
        m_cmbAudioInput->setCurrentIndex(nSel);
    }

    BOOLEAN bAutoScan = 0;
    MWGetInputSourceScan(g_capture.GetVideoHandle(), &bAutoScan);
    m_ckbAutoScan->setChecked(bAutoScan);
    m_cmbVideoInput->setEnabled(!bAutoScan);

    BOOLEAN bLinkVideo = 0;
    MWGetAVInputSourceLink(g_capture.GetVideoHandle(), &bLinkVideo);
    m_ckbLinkAv->setChecked(bLinkVideo);
    m_cmbAudioInput->setEnabled(!bLinkVideo);

    QApplication::connect(m_cmbVideoInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoInputChanged(int)));
    QApplication::connect(m_cmbAudioInput, SIGNAL(currentIndexChanged(int)), this, SLOT(onAudioInputChanged(int)));

    QApplication::connect(this, SIGNAL(sigInputStatuChanged(unsigned long long)), this, SLOT(onInputStatuChangedMain(unsigned long long)));
    QApplication::connect(this, SIGNAL(sigInputSourceChanged(unsigned long long)), this, SLOT(onInputSourceChangedMain(unsigned long long)));

}

int CDlgInput::GetInputTypeIndex(int nValue, int nMin, int nMax)
{
    int nIndex = 0;
    for (int i = 0; i < 256; ++ i) {
        if (nMin + i == nValue) {
            nIndex = i;
            break;
        }
    }
    return nIndex;
}

void CDlgInput::GetVideoType( MWCAP_VIDEO_INPUT_TYPE& videoInputType, char* pszType, int nTypeValue, int& nIndex )
{
    int nInputTypeNone = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_NONE, 0);
    int nInputTypeHDMI = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_HDMI, 0);
    int nInputTypeVGA = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_VGA, 0);
    int nValueSDI = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_SDI, 0);
    int nInputTypeComponent = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_COMPONENT, 0);
    int nInputTypeCVBS = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_CVBS, 0);
    int nInputTypeYC = INPUT_SOURCE(MWCAP_VIDEO_INPUT_TYPE_YC, 0);

    if (nTypeValue >= nInputTypeNone && nTypeValue < nInputTypeHDMI) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeNone, nInputTypeHDMI);
        sprintf(pszType, "NONE");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_NONE;
    }
    else if (nTypeValue >= nInputTypeHDMI && nTypeValue < nInputTypeVGA) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeHDMI, nInputTypeVGA);
        sprintf(pszType, "HDMI");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_HDMI;
    }
    else if (nTypeValue >= nInputTypeVGA && nTypeValue < nValueSDI) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeVGA, nValueSDI);
        sprintf(pszType, "VGA");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_VGA;
    }
    else if (nTypeValue >= nValueSDI && nTypeValue < nInputTypeComponent) {
        nIndex = GetInputTypeIndex(nTypeValue, nValueSDI, nInputTypeComponent);
        sprintf(pszType, "SDI");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_SDI;
    }
    else if (nTypeValue >= nInputTypeComponent && nTypeValue < nInputTypeCVBS) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeComponent, nInputTypeCVBS);
        sprintf(pszType, "Component");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_COMPONENT;
    }
    else if (nTypeValue >= nInputTypeCVBS && nTypeValue < nInputTypeYC) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeCVBS, nInputTypeYC);
        sprintf(pszType, "Composite");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_CVBS;
    }
    else if (nTypeValue >= nInputTypeYC) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeYC, nInputTypeYC * 2);
        sprintf(pszType, "YC");
        videoInputType = MWCAP_VIDEO_INPUT_TYPE_YC;
    }
}

void CDlgInput::GetAudioType( MWCAP_AUDIO_INPUT_TYPE& audioInputType, char* pszType, int nTypeValue, int& nIndex )
{
    int nInputTypeNone = INPUT_SOURCE(MWCAP_AUDIO_INPUT_TYPE_NONE, 0);
    int nInputTypeHDMI = INPUT_SOURCE(MWCAP_AUDIO_INPUT_TYPE_HDMI, 0);
    int nInputTypeSDI = INPUT_SOURCE(MWCAP_AUDIO_INPUT_TYPE_SDI, 0);
    int nValueLine = INPUT_SOURCE(MWCAP_AUDIO_INPUT_TYPE_LINE_IN, 0);
    int nInputTypeMIC = INPUT_SOURCE(MWCAP_AUDIO_INPUT_TYPE_MIC_IN, 0);

    if (nTypeValue >= nInputTypeNone && nTypeValue < nInputTypeHDMI) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeNone, nInputTypeHDMI);
        sprintf(pszType, "NONE");
        audioInputType = MWCAP_AUDIO_INPUT_TYPE_NONE;
    }
    else if (nTypeValue >= nInputTypeHDMI && nTypeValue < nInputTypeSDI) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeHDMI, nInputTypeSDI);
        sprintf(pszType, "HDMI");
        audioInputType = MWCAP_AUDIO_INPUT_TYPE_HDMI;
    }
    else if (nTypeValue >= nInputTypeSDI && nTypeValue < nValueLine) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeSDI, nValueLine);
        sprintf(pszType, "SDI");
        audioInputType = MWCAP_AUDIO_INPUT_TYPE_SDI;
    }
    else if (nTypeValue >= nValueLine && nTypeValue < nInputTypeMIC) {
        nIndex = GetInputTypeIndex(nTypeValue, nValueLine, nInputTypeMIC);
        sprintf(pszType, "Line");
        audioInputType = MWCAP_AUDIO_INPUT_TYPE_LINE_IN;
    }
    else if (nTypeValue >= nInputTypeMIC) {
        nIndex = GetInputTypeIndex(nTypeValue, nInputTypeMIC, nInputTypeMIC * 2);
        sprintf(pszType, "MIC");
        audioInputType = MWCAP_AUDIO_INPUT_TYPE_MIC_IN;
    }
}

inline void CDlgInput::AddItem(QList<TABLE_ITEM> &lstTableItems, QString strKey, QString strValue)
{
    TABLE_ITEM item = {strKey, strValue};
    lstTableItems.append(item);
}

//slots
void CDlgInput::onVideoInputChanged(int nIndex)
{
    DWORD dwSelVideoInputSource = INPUT_SOURCE(m_szVideoInputType[nIndex], 0);
    MWSetVideoInputSource(g_capture.GetVideoHandle(), dwSelVideoInputSource);

    switch (m_szVideoInputType[nIndex]) {
    case MWCAP_VIDEO_INPUT_TYPE_HDMI:
       m_shFile->ModifyFileVideoInputType("hdmi");
        break;
    case MWCAP_VIDEO_INPUT_TYPE_VGA:
       m_shFile->ModifyFileVideoInputType("vga");
        break;
    case MWCAP_VIDEO_INPUT_TYPE_SDI:
       m_shFile->ModifyFileVideoInputType("sdi");
        break;
    case MWCAP_VIDEO_INPUT_TYPE_COMPONENT:
       m_shFile->ModifyFileVideoInputType("component");
        break;
    case MWCAP_VIDEO_INPUT_TYPE_CVBS:
       m_shFile->ModifyFileVideoInputType("cvbs");
        break;
    case MWCAP_VIDEO_INPUT_TYPE_YC:
       m_shFile->ModifyFileVideoInputType("yc");
        break;
    default:
        break;
    }
}

void CDlgInput::onAudioInputChanged(int nIndex)
{
    DWORD dwSelAudioInputSource = INPUT_SOURCE(m_szAudioInputType[nIndex], 0);
    MWSetAudioInputSource(g_capture.GetVideoHandle(), dwSelAudioInputSource);
    switch (m_szAudioInputType[nIndex]) {
    case MWCAP_AUDIO_INPUT_TYPE_HDMI:
       m_shFile->ModifyFileAudioInputType("hdmi");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_SDI:
       m_shFile->ModifyFileAudioInputType("sdi");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_LINE_IN:
       m_shFile->ModifyFileAudioInputType("line_in");
        break;
    case MWCAP_AUDIO_INPUT_TYPE_MIC_IN:
       m_shFile->ModifyFileAudioInputType("mic_in");
        break;
    default:
        break;
    }
}

void CDlgInput::onAutoScanChecked(int nStatus)
{
    bool bChecked = (nStatus == Qt::Checked) ? true : false;
    MWSetInputSourceScan(g_capture.GetVideoHandle(), bChecked);
    m_cmbVideoInput->setEnabled(!bChecked);

    if(bChecked == true)
    {
        m_shFile->ModifyFileAutoScan("true");

        MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
        MWGetVideoSignalStatus(g_capture.GetVideoHandle(), &videoSignalStatus);
        if(videoSignalStatus.state != MWCAP_VIDEO_SIGNAL_LOCKED)
        {
            MWSetInputSourceScan(g_capture.GetVideoHandle(), false);
            m_cmbVideoInput->setCurrentIndex(0);
        }
    }
    else
    {
        m_shFile->ModifyFileAutoScan("false");
    }
}

void CDlgInput::onLinkAVChecked(int nStatus)
{
    BOOLEAN bChecked = (nStatus == Qt::Checked) ? 1 : 0;
    MWSetAVInputSourceLink(g_capture.GetVideoHandle(), bChecked);
    m_cmbAudioInput->setEnabled(!bChecked);

    if(bChecked == 1)
    {
        m_shFile->ModifyFileLinkWithVideo("true");
    }
    else
    {
        m_shFile->ModifyFileLinkWithVideo("false");
    }
}

void CDlgInput::onInputStatuChangedMain(unsigned long long ullStatusBits)
{
    if ((ullStatusBits & MWCAP_NOTIFY_INPUT_SPECIFIC_CHANGE) != 0) {
        UpdateSpecificStatus();
    }

    if ((ullStatusBits & MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE) != 0) {
        UpdateVideoStatus();
    }

    if ((ullStatusBits & MWCAP_NOTIFY_AUDIO_SIGNAL_CHANGE) != 0) {
        UpdateAudioStatus();
    }
}

void CDlgInput::onInputSourceChangedMain(unsigned long long ullStatusBits)
{
    if ((ullStatusBits & MWCAP_NOTIFY_VIDEO_INPUT_SOURCE_CHANGE) != 0 || (ullStatusBits & MWCAP_NOTIFY_AUDIO_INPUT_SOURCE_CHANGE) != 0)
    {
        InputSourceChange();
    }
}
