#include "cvideochannel.h"
#include <QDebug>

CVideochannel::CVideochannel() /*: QWidget(parent)*/
{
    m_bCapture = false;
    m_hImageOSD = 0;
    m_strCurrentPath = QString("/dev/video0");
    m_nCurrentDeviceIndex = 0;
    m_dwFourcc = MWFOURCC_YUY2;
    memset(m_hChannel, -1, sizeof(m_hChannel));
    m_bOpenFlag = false;
}


bool CVideochannel::OpenVideoChannel(int nIndex)
{
    if(m_bOpenFlag == false)
    {
        MW_RESULT nRet = MW_SUCCEEDED;
        int nChannelCount = MWGetChannelCount();
        int i = 0;
        char path[256] = {0};
        do
        {
            for(i = 0; i < nChannelCount; i++)
            {
                nRet = MWGetDevicePath(i, path);
                if(nRet != MW_SUCCEEDED)
                {
                    m_strDevicePath[i] = QString("");
                    continue;
                }
                m_strDevicePath[i] = QString(path);

                m_hChannel[i] = MWOpenChannelByPath(path);
                if(m_hChannel[i] == -1)
                    continue;
            }
        }while(false);

        m_bOpenFlag = true;
        m_nCurrentDeviceIndex = nIndex;
        m_strCurrentPath = m_strDevicePath[nIndex];
        m_hCurrentChannel = m_hChannel[nIndex];
        if(m_hCurrentChannel == -1 || m_strCurrentPath == "")
            return false;
    }
    else
    {
        m_nCurrentDeviceIndex = nIndex;
        m_hCurrentChannel = m_hChannel[nIndex];
        m_strCurrentPath = m_strDevicePath[nIndex];
        if(m_hCurrentChannel == -1 || m_strCurrentPath == "")
            return false;
    }
    return true;
}

bool CVideochannel::Init()
{
    if(true != MWCaptureInitInstance())
        return false;

    MW_RESULT nRet;
    nRet = MWRefreshDevice();
    if(nRet != MW_SUCCEEDED)
        return false;

    return true;
}

void CVideochannel::Exit()
{
    int i = 0;
    int nChannelCount = MWGetChannelCount();
    for(i = 0; i < nChannelCount; i++)
    {
        if(m_hChannel[i] > 0)
        {
            MWCloseChannel(m_hChannel[i]);
            m_hChannel[i] = -1;
        }
    }

    MWCaptureExitInstance();
    return;
}

bool CVideochannel::StartCapture()
{
    m_bCapture = true;
    int nRet = pthread_create(&m_idCapture, NULL, ThreadCapture, this);

    return true;
}

void CVideochannel::StopCapture()
{
    if(m_bCapture == true)
        m_bCapture = false;

    if(m_idCapture > 0)
    {
        if(0 != pthread_join(m_idCapture, NULL))
            return;

        m_idCapture = -1;
    }

    return;
}

void CVideochannel::SetCallback(VideoRenderImageBuf *m_pVideoBuffer, ICaptureCallback *pCallback)
{
    m_Imagebuf = m_pVideoBuffer;
    m_Callback = pCallback;
}

QString CVideochannel::GetDeviceName(int nIndex)
{
    MWCAP_CHANNEL_INFO channelInfo;
    if(MW_SUCCEEDED != MWGetChannelInfoByIndex(nIndex, &channelInfo))
        return NULL;

    QString strDeviceName = QString("%1 %2 - %3").arg(channelInfo.szProductName).arg(channelInfo.byBoardIndex).arg(channelInfo.byChannelIndex);

    return strDeviceName;
}

//Amp Proc
void CVideochannel::SetVideoBrightness(int nValue)
{
    m_nBrightness = nValue;
}

void CVideochannel::SetVideoHue(int nValue)
{
    m_nHue = nValue;
}

void CVideochannel::SetVideoSaturation(int nValue)
{
    m_nSaturation = nValue;
}

void CVideochannel::SetVideoContract(int nValue)
{
    m_nContract = nValue;
}

void CVideochannel::SetAmpBrightness(int nValue)
{
    MWSetVideoBrightness(g_capture.GetVideoHandle(), nValue);
}

void CVideochannel::SetAmpHue(int nValue)
{
    MWSetVideoHue(g_capture.GetVideoHandle(), nValue);
}

void CVideochannel::SetAmpSaturation(int nValue)
{
    MWSetVideoSaturation(g_capture.GetVideoHandle(), nValue);
}

void CVideochannel::SetAmpContract(int nValue)
{
    MWSetVideoContrast(g_capture.GetVideoHandle(), nValue);
}

void CVideochannel::SetVideoAspectX(int nAspectX)
{
    m_nAspectX = nAspectX;
}

void CVideochannel::SetVideoAspectY(int nAspectY)
{
    m_nAspectY = nAspectY;
}

void CVideochannel::SetLowLatency(bool bLowLatency)
{
    m_bLowLatency = bLowLatency;
}

void CVideochannel::SetVideoCropRect(RECT *rectSrc)
{
    if (m_pRectSrc == NULL)
        m_pRectSrc = new RECT;
    memcpy(m_pRectSrc, rectSrc, sizeof(RECT));
}

void CVideochannel::SetVideoColorFormat(MWCAP_VIDEO_COLOR_FORMAT colorFormat) {
    m_colorFormat = colorFormat;
}

void CVideochannel::SetVideoQuant(MWCAP_VIDEO_QUANTIZATION_RANGE quantRange) {
    m_quantRange = quantRange;
}

void CVideochannel::SetVideoSaturation(MWCAP_VIDEO_SATURATION_RANGE satRange) {
    m_satRange = satRange;
}

void CVideochannel::SetVideoDeintMode(MWCAP_VIDEO_DEINTERLACE_MODE deinterlaceMode) {
    m_deinterlaceMode = deinterlaceMode;
}

void CVideochannel::SetVideoARMode(MWCAP_VIDEO_ASPECT_RATIO_CONVERT_MODE aspectRatioConvertMode) {
    m_aspectRatioConvertMode = aspectRatioConvertMode;
}

//osd
bool CVideochannel::UploadOSDImage(unsigned char* pngimage, WORD pngwidth, WORD pngheight)
{
    if (m_hImageOSD != 0) {
        long lRet = 0;
        MWCloseImage(m_hCurrentChannel, m_hImageOSD, &lRet);
        m_hImageOSD = 0;
    }

    m_hImageOSD = MWCreateImage(m_hCurrentChannel, m_cx, m_cy);
    if (m_hImageOSD == 0) {
        return false;
    }
    else {
        MW_RESULT nRet = MWUploadImageFromVirtualAddress(m_hCurrentChannel, m_hImageOSD, MWCAP_VIDEO_COLOR_FORMAT_YUV709, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN,
                                       0, 0, m_cx, m_cy, (MWCAP_PTR)pngimage, pngwidth * pngheight * 4, pngwidth * 4, pngwidth, pngheight, FALSE, TRUE, TRUE);
        if (nRet != MW_SUCCEEDED)
            return false;
        else
            return true;
    }
}

void CVideochannel::SetOSDImage(char *szPNGImagePath, bool bEnable)
{
    MWCAP_VIDEO_OSD_SETTINGS osdSettings;
    osdSettings.bEnable = bEnable;
    strcpy(osdSettings.szPNGFilePath, szPNGImagePath);
    MWSetVideoOSDSettings(m_hCurrentChannel, osdSettings);
}

void CVideochannel::SetEnableOSD(bool bImageOSD) {
    m_bImageOSD = bImageOSD;
}

//v4l2
void CVideochannel::GetOSDImage(char *szPNGImagePath, bool *bEnable)
{
    MWCAP_VIDEO_OSD_SETTINGS osdSettings = {0};
    MWGetVideoOSDSettings(m_hCurrentChannel, &osdSettings);
    *bEnable = osdSettings.bEnable;
    if (osdSettings.szPNGFilePath)
        memcpy(szPNGImagePath, osdSettings.szPNGFilePath, strlen(osdSettings.szPNGFilePath));
}

void CVideochannel::GetAMPBrightness(int *pnBrightness)
{
    MWGetVideoBrightness(m_hCurrentChannel, pnBrightness);
}

void CVideochannel::GetAMPContrast(int *pnContrast)
{
    MWGetVideoContrast(m_hCurrentChannel, pnContrast);
}

void CVideochannel::GetAMPHue(int *pnHue)
{
    MWGetVideoHue(m_hCurrentChannel, pnHue);
}

void CVideochannel::GetAMPSaturation(int *pnSaturation)
{
    MWGetVideoSaturation(m_hCurrentChannel, pnSaturation);
}

//protected
bool CVideochannel::DoVideoCapture()
{
    m_Callback->OnWndViewCallback(false);

    if (m_dwFourcc == -1)
    {
        m_Callback->OnWndViewCallback(true);
        return false;
    }

    m_cx = 1920;
    m_cy = 1080;
    DWORD dwFourcc = m_dwFourcc;
    DWORD dwMinStride = FOURCC_CalcMinStride(dwFourcc, m_cx, 4);
    DWORD dwImageSize = FOURCC_CalcImageSize(dwFourcc, m_cx, m_cy, dwMinStride);

    MW_RESULT nRet;
    MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
    MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
    MWCAP_VIDEO_SIGNAL_STATUS videoSignaleStatus;
    ULONGLONG ullStatusBits = 0;
    MWCAP_PTR hEventCapture;
    MWCAP_PTR hEventNotifyBuffering;
    MWCAP_PTR hEventNotifyBuffered;
    HNOTIFY hNotifyBuffering;
    HNOTIFY hNotifyBuffered;
    do
    {
        hEventCapture = MWCreateEvent();
        if(hEventCapture <= 0)
            break;

        hEventNotifyBuffering = MWCreateEvent();
        if(hEventNotifyBuffering <= 0)
            break;

        hEventNotifyBuffered = MWCreateEvent();
        if(hEventNotifyBuffered <= 0)
            break;

        hNotifyBuffering = MWRegisterNotify(m_hCurrentChannel, hEventNotifyBuffering,MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING |
                                            MWCAP_NOTIFY_VIDEO_INPUT_SOURCE_CHANGE |
                                            MWCAP_NOTIFY_AUDIO_INPUT_SOURCE_CHANGE |
                                            MWCAP_NOTIFY_INPUT_SPECIFIC_CHANGE |
                                            MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE |
                                            MWCAP_NOTIFY_AUDIO_SIGNAL_CHANGE | //end
                                            MWCAP_NOTIFY_VIDEO_SAMPLING_PHASE_CHANGE);
        if(hNotifyBuffering <= 0)
            break;

        hNotifyBuffered = MWRegisterNotify(m_hCurrentChannel, hEventNotifyBuffered, MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED |
                                           MWCAP_NOTIFY_VIDEO_INPUT_SOURCE_CHANGE |
                                           MWCAP_NOTIFY_AUDIO_INPUT_SOURCE_CHANGE |
                                           MWCAP_NOTIFY_INPUT_SPECIFIC_CHANGE |
                                           MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE |
                                           MWCAP_NOTIFY_AUDIO_SIGNAL_CHANGE | //end
                                           MWCAP_NOTIFY_VIDEO_SAMPLING_PHASE_CHANGE);

        if(hNotifyBuffered <= 0)
            break;

        nRet = MWStartVideoCapture(m_hCurrentChannel, hEventCapture);
        if(nRet != MW_SUCCEEDED)
            break;

        while(m_bCapture == true)
        {
            nRet = MWGetVideoSignalStatus(m_hCurrentChannel, &videoSignaleStatus);
            if(nRet != MW_SUCCEEDED)
                break;

            if(videoSignaleStatus.state == MWCAP_VIDEO_SIGNAL_LOCKED)
            {

                if(m_bLowLatency == true)
                {
                    MWWaitEvent(hEventNotifyBuffering, 300);

                    nRet = MWGetNotifyStatus(m_hCurrentChannel, hNotifyBuffering, &ullStatusBits);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    nRet = MWGetVideoBufferInfo(m_hCurrentChannel, &videoBufferInfo);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    nRet = MWGetVideoFrameInfo(m_hCurrentChannel, videoBufferInfo.iNewestBuffering, &videoFrameInfo);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    if(ullStatusBits == MWCAP_NOTIFY_VIDEO_FRAME_BUFFERING)
                    {
                        const RECT aOSDRects[] = {{0, 0, m_cx, m_cy}};
                        nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hCurrentChannel, videoBufferInfo.iNewestBuffering,
                                                                     (MWCAP_PTR)(long)m_Imagebuf->buf, dwImageSize, dwMinStride,
                                                                     0, 0, dwFourcc, m_cx, m_cy,0, NUM_PARTIAL_LINE,
                                                                     m_bImageOSD ? m_hImageOSD : 0, m_bImageOSD ? aOSDRects : 0, m_bImageOSD ? 1 : 0,
                                                                     m_nContract, m_nBrightness, m_nSaturation, m_nHue, m_deinterlaceMode, m_aspectRatioConvertMode,
                                                                     m_pRectSrc, 0, m_nAspectX, m_nAspectY,
                                                                     m_colorFormat, m_quantRange, m_satRange);
                        if(nRet != MW_SUCCEEDED)
                            break;

                        MWWaitEvent(hEventCapture, 300);

                        MWCAP_VIDEO_CAPTURE_STATUS videoCaptureStatus;
                        do
                        {
                            nRet = MWGetVideoCaptureStatus(m_hCurrentChannel, &videoCaptureStatus);
                        }while(nRet == MW_SUCCEEDED && videoCaptureStatus.bFrameCompleted == false);

                        if(nRet != MW_SUCCEEDED)
                            break;

                        m_Callback->OnCaptureCallback(m_Imagebuf);

                        LONGLONG llTime;
                        nRet = MWGetDeviceTime(m_hCurrentChannel, &llTime);
                        if(nRet != MW_SUCCEEDED)
                            break;
                    }
                }
                else
                {
                    MWWaitEvent(hEventNotifyBuffered, 300);

                    nRet = MWGetNotifyStatus(m_hCurrentChannel, hNotifyBuffered, &ullStatusBits);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    nRet = MWGetVideoBufferInfo(m_hCurrentChannel, &videoBufferInfo);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    nRet = MWGetVideoFrameInfo(m_hCurrentChannel, videoBufferInfo.iNewestBuffered, &videoFrameInfo);
                    if(nRet != MW_SUCCEEDED)
                        break;

                    if(ullStatusBits == MWCAP_NOTIFY_VIDEO_FRAME_BUFFERED)
                    {
                        const RECT aOSDRects[] = {{0, 0, m_cx, m_cy}};
                        nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hCurrentChannel, videoBufferInfo.iNewestBufferedFullFrame,
                                                                     (MWCAP_PTR)(long)m_Imagebuf->buf, dwImageSize, dwMinStride,
                                                                     0, 0, dwFourcc, m_cx, m_cy,0, 0,
                                                                     m_bImageOSD ? m_hImageOSD : 0, m_bImageOSD ? aOSDRects : 0, m_bImageOSD ? 1 : 0,
                                                                     m_nContract, m_nBrightness, m_nSaturation, m_nHue, m_deinterlaceMode, m_aspectRatioConvertMode,
                                                                     m_pRectSrc, 0, m_nAspectX, m_nAspectY,
                                                                     m_colorFormat, m_quantRange, m_satRange);
                        if(nRet != MW_SUCCEEDED)
                            break;

                        MWWaitEvent(hEventCapture, 300);

                        MWCAP_VIDEO_CAPTURE_STATUS videoCaptureStatus;
                        do
                        {
                            nRet = MWGetVideoCaptureStatus(m_hCurrentChannel, &videoCaptureStatus);
                        }while(nRet == MW_SUCCEEDED && videoCaptureStatus.bFrameCompleted == false);

                        if(nRet != MW_SUCCEEDED)
                            break;

                        m_Callback->OnCaptureCallback(m_Imagebuf);

                        LONGLONG llTime;
                        nRet = MWGetDeviceTime(m_hCurrentChannel, &llTime);
                        if(nRet != MW_SUCCEEDED)
                            break;
                    }
                }
            }
            else
            {
                MWWaitEvent(hEventNotifyBuffered, 300);

                const RECT aOSDRects[] = {{0, 0, m_cx, m_cy}};
                nRet = MWCaptureVideoFrameToVirtualAddressEx(m_hCurrentChannel, -1,
                                                             (MWCAP_PTR)(long)m_Imagebuf->buf, dwImageSize, dwMinStride,
                                                             0, 0, dwFourcc, m_cx, m_cy,0, 0,
                                                             m_bImageOSD ? m_hImageOSD : 0, m_bImageOSD ? aOSDRects : 0, m_bImageOSD ? 1 : 0,
                                                             m_nContract, m_nBrightness, m_nSaturation, m_nHue, m_deinterlaceMode, m_aspectRatioConvertMode,
                                                             m_pRectSrc, 0, m_nAspectX, m_nAspectY,
                                                             m_colorFormat, m_quantRange, m_satRange);
                if(nRet != MW_SUCCEEDED)
                    break;

                MWWaitEvent(hEventCapture, 300);

                MWCAP_VIDEO_CAPTURE_STATUS videoCaptureStatus;
                do
                {
                    nRet = MWGetVideoCaptureStatus(m_hCurrentChannel, &videoCaptureStatus);
                }while(nRet == MW_SUCCEEDED && videoCaptureStatus.bFrameCompleted == false);

                if(nRet != MW_SUCCEEDED)
                    break;

                m_Callback->OnCaptureCallback(m_Imagebuf);

                LONGLONG llTime;
                nRet = MWGetDeviceTime(m_hCurrentChannel, &llTime);
                if(nRet != MW_SUCCEEDED)
                    break;
            }

           MWWaitEvent(hEventNotifyBuffered, 300);

           nRet = MWGetNotifyStatus(m_hCurrentChannel, hNotifyBuffering, &ullStatusBits);
           if(nRet != MW_SUCCEEDED)
               break;

            if (ullStatusBits & MWCAP_NOTIFY_VIDEO_SAMPLING_PHASE_CHANGE) {
                for (DWORD i = 0; i < m_arrNotify.size(); ++ i) {
                    m_arrNotify[i]->onTimingStatuChanged(ullStatusBits);
                }
            }

            if (ullStatusBits & (MWCAP_NOTIFY_INPUT_SPECIFIC_CHANGE |
                                 MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE |
                                 MWCAP_NOTIFY_AUDIO_SIGNAL_CHANGE)) {
                for (DWORD i = 0; i < m_arrNotify.size(); ++ i) {
                    m_arrNotify[i]->onInputStatuChanged(ullStatusBits);

                }
            }

            if (ullStatusBits & (MWCAP_NOTIFY_VIDEO_INPUT_SOURCE_CHANGE |
                                 MWCAP_NOTIFY_VIDEO_SIGNAL_CHANGE |
                                 MWCAP_NOTIFY_AUDIO_INPUT_SOURCE_CHANGE)) {
                for (DWORD i = 0; i < m_arrNotify.size(); ++ i) {
                    m_arrNotify[i]->onInputSourceChanged(ullStatusBits);
                    m_arrNotify[i]->onTimingStatuChanged(ullStatusBits);
                    m_arrNotify[i]->onHDMIInfoStatuChanged(ullStatusBits);
                }
            }
        }

    }while(false);

    m_Callback->OnWndViewCallback(true);

    nRet = MWStopVideoCapture(m_hCurrentChannel);
    if(nRet != MW_SUCCEEDED)
        return false;

    nRet = MWUnregisterNotify(m_hCurrentChannel, hNotifyBuffering);
    if(nRet != MW_SUCCEEDED)
        return false;

    nRet = MWUnregisterNotify(m_hCurrentChannel, hNotifyBuffered);
    if(nRet != MW_SUCCEEDED)
        return false;

    nRet = MWCloseEvent(hEventCapture);
    if(nRet != MW_SUCCEEDED)
        return false;

    nRet = MWCloseEvent(hEventNotifyBuffering);
    if(nRet != MW_SUCCEEDED)
        return false;

    nRet = MWCloseEvent(hEventNotifyBuffered);
    if(nRet != MW_SUCCEEDED)
        return false;

    return true;
}
