#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "Ctabproperty.h"
CVideochannel g_capture;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->mainToolBar->hide();
    CheckSHFile();
    CreateLayout();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::CheckSHFile()
{
    QString strHomePath = QString(QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation).at(0));
    QString strCreatePath = strHomePath + "/deviceproperty";
    QDir dir;
    if(dir.exists(strCreatePath) == false)
        dir.mkdir(strCreatePath);

    int nChannleCount = MWGetChannelCount();
    for(int i = 0; i < nChannleCount; i++)
    {
        QString fileName = QString("%1/v4l2_control%2.sh").arg(strCreatePath).arg(i);
        QFile shFile(fileName);

        if(shFile.exists() == false)
        {
            if (!shFile.open(QIODevice::WriteOnly))
            {
                printf("Open v4l2_file file error\n");
                return;
            }

            QTextStream stream(&shFile);
            stream << "#!/bin/bash" <<"\n" \
                   << "DEVICE_PATH=/dev/video$DeviceNum"<<"\n"\
                   << "#OSD"<< "\n"\
                   << "#video input"<< "\n"\
                   << "ASPECTXINPUT=16"<< "\n"\
                   << "ASPECTYINPUT=9"<< "\n"\
                   << "ASPECTINPUT=$ASPECTXINPUT:$ASPECTYINPUT"<<"\n"\
                   << "COLORFMTINPUT=rgb"<< "\n"\
                   << "QUANTIZATIONINPUT=full"<< "\n"\
                   << "#input"<< "\n"\
                   << "VIDEOINPUTTYPE=vga"<< "\n"\
                   << "AUDIOINPUT=hdmi"<< "\n"\
                   << "AUTOSCAN=false"<< "\n"\
                   << "LINKWITHVIDEO=false"<< "\n"\
                   << "#timing"<< "\n"\
                   << "AUTO_SAMP=true"<< "\n"\
                   << "AUTO_HALIGN=true"<< "\n"\
                   << "PHASE=0"<< "\n"\
                   << "mwcap-control --video-input $VIDEOINPUTTYPE ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --audio-input $AUDIOINPUT ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --video-input-aspect $ASPECTINPUT ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --video-input-color-format $COLORFMTINPUT ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --video-input-quantization $QUANTIZATIONINPUT ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --video-auto-scan $AUTOSCAN ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --audio-link-with-video $LINKWITHVIDEO ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --edid-load $CurrentPath/devicePropertyEdid$DeviceNum.bin ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --timing-set $CurrentPath/timingSettings$DeviceNum.config ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --auto-sampl-phase $AUTO_SAMP ${DEVICE_PATH}"<<"\n"\
                   << "mwcap-control --auto-halign $AUTO_HALIGN ${DEVICE_PATH}"<<"\n"\
                   << "if [ false == $AUTO_SAMP ]"<<"\n"\
                   << "then"<<"\n"\
                   << "\tmwcap-control --set-phase $PHASE ${DEVICE_PATH}"<<"\n"\
                   << "fi"<<"\n";

            shFile.close();

            QString strCmd = QString("chmod +x %1").arg(fileName);
            system(strCmd.toLocal8Bit().data());
            strCmd = QString("sed -i '2a CurrentPath=%1' %2/v4l2_control%3.sh").arg(strCreatePath).arg(strCreatePath).arg(i);
            system(strCmd.toLocal8Bit().data());

            strCmd = QString("sed -i '1a DeviceNum=%1' %2/v4l2_control%3.sh").arg(i).arg(strCreatePath).arg(i);
            system(strCmd.toLocal8Bit().data());
        }
    }

    QFile initFile(QString("%1/v4l2.sh").arg(strCreatePath));
    if(false == initFile.exists())
    {
        if(!initFile.open(QIODevice::WriteOnly))
        {
            printf("Open v4l2.sh file error\n");
            return;
        }

        QTextStream stream(&initFile);
        stream << "#!/bin/bash" <<"\n" \
               << "APPLICATIONPATH="<<"\n"\
               << "num=0"<<"\n"\
               << "for ((i=0;i<num;i++))"<<"\n"\
               << "do"<<"\n"\
               << "sudo update-rc.d -f v4l2_control$i.sh remove 2>/dev/null\n"\
               << "sudo rm /etc/init.d/v4l2_control$i.sh 2>/dev/null\n"\
               << "sudo ln -s $APPLICATIONPATH/v4l2_control$i.sh /etc/init.d/ 2>/dev/null"<<"\n"\
               << "sudo update-rc.d v4l2_control$i.sh start 98 2 . 2>/dev/null"<<"\n"\
               << "done"<<"\n";

        initFile.close();


        QString strCmd = QString("sed -i '/APPLICATIONPATH=/c APPLICATIONPATH=%1' %2/v4l2.sh").arg(strCreatePath).arg(strCreatePath);
        system(strCmd.toLocal8Bit().data());

        strCmd = QString("sed -i '/num=/c num=%1' %2/v4l2.sh").arg(nChannleCount).arg(strCreatePath);
        system(strCmd.toLocal8Bit().data());

        strCmd = QString("chmod +x %1/v4l2.sh").arg(strCreatePath);
        system(strCmd.toLocal8Bit().data());
    }

    QFile file("/etc/init.d/v4l2_control0.sh");
    if(file.exists() == false)
    {
        QString strCmd = QString("pkexec %1/v4l2.sh").arg(strCreatePath);
        system(strCmd.toLocal8Bit().data());
    }
}

void MainWindow::OpenVideo(int nIndex)
{
    do
    {
        if(true != g_capture.OpenVideoChannel(nIndex))
        {
            m_wndView->setUpdatesEnabled(true);
            break;
        }

        if(true != g_capture.StartCapture())
        {
            m_wndView->setUpdatesEnabled(true);
            break;
        }

        m_wndView->setUpdatesEnabled(false);
        return;

    }while(false);

    m_wndView->setUpdatesEnabled(true);
    return;
}

void MainWindow::onInputSourceChanged(unsigned long long ullStatusBits)
{
    m_wndView->setStyleSheet("background-color:black");
}

//render
bool MainWindow::CreateRender()
{
    do
    {
        m_VideoRender = new VideoRenderXCB;
        bool bRes = m_VideoRender->Init(1920, 1080, m_wndView->winId());
        if(bRes != true)
            break;

        DWORD dwFourcc = MWFOURCC_YUY2;
        bRes = SupportFourcc(&dwFourcc);
        if(bRes == false)
            break;

        g_capture.SetVideoCaptureFourcc(dwFourcc);

        bRes = m_VideoRender->SetFrameFormat(1920, 1080, dwFourcc, false);
        if(bRes != true)
            break;

        m_pVideoBuffer = m_VideoRender->AllocImageBuffer(VIDEO_RENDER_IMAGEBUF_VIRTUAL);

        return true;
    }while(false);

    g_capture.SetVideoCaptureFourcc(-1);
    return false;

}

bool MainWindow::SupportFourcc(unsigned int *dwFourcc)
{
    int nCount = 0;
    uint32_t *pFourcc;
    pFourcc = m_VideoRender->GetSupportedFourcc(nCount);
    if (nCount <= 0)
        return false;
    else
    {
        *dwFourcc = *(pFourcc + 0);
        return true;
    }
}

void MainWindow::OnCaptureCallback(VideoRenderImageBuf *pBuffer)
{
    m_VideoRender->DisplayFrame(pBuffer);
}

void MainWindow::OnWndViewCallback(bool bUpdate)
{
    m_wndView->setUpdatesEnabled(bUpdate);
}

//protected:
void MainWindow::CreateLayout()
{
    QFile file("deviceProperty.conf");
    if(false == file.exists())
    {
        file.open(QIODevice::ReadWrite);
        file.close();
    }

    //create left widgets and start render
    m_wndView = new QWidget(this);
    m_wndView->setUpdatesEnabled(false);
    m_wndView->setStyleSheet("background-color:black");
    m_cmbDevice = new QComboBox(this);

    //get channel device name
    int nChannelCount = MWGetChannelCount();
    QString strDeviceName;
    for(int i = 0; i < nChannelCount; i++)
    {
        if((strDeviceName = g_capture.GetDeviceName(i)) == NULL ? 1 : 0 )
            continue;
        m_cmbDevice->addItem(strDeviceName);
    }

    //open default channel
    m_shFile = new CGenerateSHFile(this);
    bool bRet = CreateRender();
    if (bRet == true)
    {
        g_capture.SetCallback(m_pVideoBuffer, this);
        OpenVideo(0);
        m_shFile->ModifySHFileDeviceName(0);
    }
    else
    {
        OpenVideo(0);
        m_shFile->ModifySHFileDeviceName(0);
        m_wndView->setUpdatesEnabled(true);
    }

    //create right widgets and make layout
    m_tabProperty = new CTabProperty(this);

    QVBoxLayout *playLeft = new QVBoxLayout;
    playLeft->addWidget(m_cmbDevice);
    playLeft->addWidget(m_wndView);

    QHBoxLayout *pMainlayout = new QHBoxLayout;
    pMainlayout->addLayout(playLeft, 2);
    pMainlayout->addWidget(m_tabProperty, 1);

    QWidget *wid = new QWidget(this);
    wid->setLayout(pMainlayout);

    g_capture.AddNotify(this);

    this->setCentralWidget(wid);
    this->setGeometry(QApplication::desktop()->width() / 6, QApplication::desktop()->height() / 4,
                      QApplication::desktop()->width() * 2 / 3, QApplication::desktop()->height() / 2);

    this->setFixedSize(QApplication::desktop()->width() * 2 / 3, QApplication::desktop()->height() / 2);
    this->setWindowTitle("Device Property");

    QApplication::connect(m_cmbDevice, SIGNAL(currentIndexChanged(int)), this, SLOT(onChannelIndexChanged(int)));
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    m_shFile->ModifyFileSaveTimingSettings();
    g_capture.StopCapture();
    g_capture.Exit();
    QApplication::quit();
}

//slots
void MainWindow::onChannelIndexChanged(int nIndex)
{
    m_shFile->ModifyFileSaveTimingSettings();
    g_capture.StopCapture();
    OpenVideo(nIndex);
    m_wndView->setStyleSheet("background-color:black");
    m_tabProperty->VideoDeviceChanged();
    m_shFile->ModifySHFileDeviceName(nIndex);
}
